/*@ ----------------------------------------------------------------------------

 
 
                        This file is part of the class library
 
            SoPlex  --- the Sequential object-oriented simPlex
 
            (c) by      Roland Wunderling
                        Konarad Zuse Zentrum fuer Informationstechnik Berlin
                        Heilbronner Str. 10, 
                        D-10711 Berlin, Germany
 
        There is absolutely no right to redistribute this file without the
        explicite permission by the authour.
 
 
    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 


   Class #SPxWeightPR#

   Identification:
   $Id: spxweightpr.hh,v 1.1 1998/07/16 12:11:41 boehm Exp $

   Program history:
   $Log: spxweightpr.hh,v $
   Revision 1.1  1998/07/16 12:11:41  boehm
   *** empty log message ***

// Revision 1.6  1996/03/21  11:08:59  bzfwunde
// New Makefile
//
// Revision 1.5  1996/01/08  12:30:58  bzfwunde
// Moved to new non-GNU generic Makefile environment
//
// Revision 1.4  1995/11/21  16:28:10  bzfwunde
// introduced SUBDIR_INCLUDE
//
// Revision 1.3  1995/10/13  15:37:16  bzfwunde
// minor improvements
//
// Revision 1.2  1995/05/17  11:45:14  bzfwunde
// introduced SSVector
//
// Revision 1.1.1.1  1995/03/31  15:08:32  bzfwunde
// tested Version running with set packing
//

    ----------------------------------------------------------------------------
*/

#ifndef DEFspxweightpr		// prevent multiple includes
#define DEFspxweightpr

//@ ----------------------------------------------------------------------------
/*	\Section{Imports}
    Import required classes
 */
#ifndef	SUBDIR_INCLUDE

#include "spxpricer.hh"

#else 	// #SUBDIR_INCLUDE#

#include "spxpricer/spxpricer.hh"

#endif	// #SUBDIR_INCLUDE#


//@ ----------------------------------------------------------------------------
/* \Section{Class Declaration}
 */

/** weighted pricing. 
    Class #SPxWeightPR# is an implemantation class of #SPxPricer# that uses
    weights for columns and rows for selecting the Simplex pivots. The weights
    are computed by methods #computeCP()# and #computeRP()# which may be
    overridden by derived classes.

    The weights are interpreted as follows: The higher a value is, the more
    likely the corresponding row or column is set on one of its bounds.
 */
class SPxWeightPR : public SPxPricer
{
protected:
    DVector		cPenalty ;		// column penalties
    DVector		rPenalty ;		// row    penalties
    DVector		leavePenalty ;		// penalties for leaveing alg

    const double*	coPenalty ;
    const double*	penalty ;

    double		objlength ;		// length of objective vector.
	/// compute weights for columns. 
    virtual void	computeCP( int start, int end ) ;
	/// compute weights for rows. 
    virtual void	computeRP( int start, int end ) ;

    SoPlex*	thesolver ;
    double	theeps ;

public:
	/// 
    SoPlex*	solver() const				{ return thesolver ; }

	/// 
    double	epsilon() const				{ return theeps ; }
	/// 
    void	setEpsilon( double eps )		{ theeps = eps ; }

	/// 
    void	load( SoPlex* base ) ;

	/// 
    void	clear()					{ thesolver = 0 ; }

	/// 
    int		selectLeave() ;

	/// 
    SoPlex::Id	selectEnter() ;


	/// 
    virtual void	addedVecs  ( int n ) ;
	/// 
    virtual void	addedCoVecs( int n ) ;


	/// 
    virtual void	removedVec(int i) ;
	/// 
    virtual void	removedCoVecs(const int perm[]) ;
	/// 
    virtual void	removedVecs(const int perm[]) ;
	/// 
    virtual void	removedCoVec(int i) ;


	/// 
    void	setType( SoPlex::Type tp ) ;
	/// 
    void	setRep( SoPlex::Representation rep ) ;
	/// 
    void	left4(int n, SoPlex::Id id)
		{ (void)n ; (void)id ; }
	/// 
    void	entered4(SoPlex::Id id, int n)
		{ (void)n ; (void)id ; }

	/// 
    void	changeObj( const Vector& newObj )
		{ (void)newObj ; }
	/// 
    void	changeObj( int i, double newVal )
		{ (void)newVal ; (void)i ; }
	/// 
    void	changeLower( const Vector& newLower )
		{ (void)newLower ; }
	/// 
    void	changeLower( int i, double newLower )
		{ (void)i ; (void)newLower ; }
	/// 
    void	changeUpper( const Vector& newUpper )
		{ (void)newUpper ; }
	/// 
    void	changeUpper( int i, double newUpper )
		{ (void)i ; (void)newUpper ; }
	/// 
    void	changeLhs( const Vector& newLhs )
		{ (void)newLhs ; }
	/// 
    void	changeLhs( int i, double newLhs )
		{ (void)i ; (void)newLhs ; }
	/// 
    void	changeRhs( const Vector& newRhs )
		{ (void)newRhs ; }
	/// 
    void	changeRhs( int i, double newRhs )
		{ (void)i ; (void)newRhs ; }
	/// 
    void	changeRow( int i, const LPRow& newRow )
		{ (void)i ; (void)newRow ; }
	/// 
    void	changeCol( int i, const LPCol& newCol )
		{ (void)i ; (void)newCol ; }
	/// 
    void	changeElement( int i, int j, double val )
		{ (void)i ; (void)j ; (void)val ; }
	/// 
    void	changeSense( SoPlex::Sense sns )
		{ (void)sns ; }

	/// 
    int	isConsistent() const ;
} ;


#endif // #DEFspxweightpr#
