@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: boundbranchrule.w,v 4.11 2003/08/05 13:55:12 elf Exp $ @>

@q m i ABA_BOUNDBRANCHRULE @>

@ All member functions are defined in the file {\tt br|_|bound.cc}.

@(boundbranchrule.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "boundbranchrule.h"
#include "master.h"
#include "sub.h"
#include "fsvarstat.h"
#include "lpsub.h"
#else
#include "abacus/boundbranchrule.h"
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/fsvarstat.h"
#include "abacus/lpsub.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
  \BeginArg
  \docarg{|master|}{A pointer to the corresponding master of the optimization.}
  \docarg{|variable|}{The branching variable.}
  \docarg{|lBound|}{The lower bound of the branching variable.}
  \docarg{|uBound|}{The upper bound of the branching variable.}
  \EndArg
@q m e a @>

@(boundbranchrule.cc@>=
@q m b f @>
  ABA_BOUNDBRANCHRULE::ABA_BOUNDBRANCHRULE(ABA_MASTER *master, 
                                           int variable,
                                           double lBound, 
                                           double uBound) 
@q m e f @>
  : @/
    ABA_BRANCHRULE(master), @/
    variable_(variable), @/
    lBound_(lBound), @/
    uBound_(uBound)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(boundbranchrule.cc@>=
@q m b f @>
  ABA_BOUNDBRANCHRULE::~ABA_BOUNDBRANCHRULE()
@q m e f @>
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the |ABA_BOUNDBRANCHRULE| from an |ABA_MESSAGE|.

@q m e t @>

@q m b a @>
    \BeginArg
    \docarg
      {|master|}{A pointer to the corresponding master of the optimization.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(boundbranchrule.cc@>=
@q m b f @>
  ABA_BOUNDBRANCHRULE::ABA_BOUNDBRANCHRULE(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    ABA_BRANCHRULE(master, msg)
  {
    msg.unpack(variable_);
    msg.unpack(lBound_);
    msg.unpack(uBound_);
    msg.unpack(oldLpLBound_);
    msg.unpack(oldLpUBound_);
  }

@q m f pack @>
@ The virtual function |pack()| packs the data of the |ABA_BOUNDBRANCHRULE|
  in an |ABA_MESSAGE| object. The |pack()| function of the base class
  has to be called before the local data is packed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the |ABA_BOUNDBRANCHRULE| is packed.}
  \EndArg
@q m e a @>

@(boundbranchrule.cc@>=
@q m b f @>
  void ABA_BOUNDBRANCHRULE::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_BRANCHRULE::pack(msg);
    msg.pack(variable_);
    msg.pack(lBound_);
    msg.pack(uBound_);
    msg.pack(oldLpLBound_);
    msg.pack(oldLpUBound_);
  }

@q m f classId @>
@ The abstract virtual function |classId()| of |ABA_BRANCHRULE| has to be
  implemented in each derived subclass. It has to return an integer
  which identifies the class.

@q m b r @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e r @>

@(boundbranchrule.cc@>=
@q m b f @>
  int ABA_BOUNDBRANCHRULE::classId() const
@q m e f @>
  {
    return BOUNDBRANCHRULE_CLASSID;
  }


#endif


@q m f Output Operator @>
@ The output operator writes the branching variable together with its
  lower and upper bound to an output stream.
@q m e t @>

@q m b r @>
   \BeginReturn
   A reference to the output stream.
   \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The branch rule being output.}
  \EndArg
@q m e a @>

@(boundbranchrule.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_BOUNDBRANCHRULE &rhs)
@q m e f @>
  {
    return out << rhs.lBound_ << " <= x" << rhs.variable_ << " <= " << rhs.uBound_;
  }

@q m v extract @>
@ The function |extract()| modifies a
  subproblem by changing the lower and the upper bound of the
  branching variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the subproblem is successfully modified.}
  \docarg{|1|}{If the modification causes a contradiction.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The subproblem being modified.}
  \EndArg
@q m e a @>

@(boundbranchrule.cc@>=
@q m b f @>
  int ABA_BOUNDBRANCHRULE::extract(ABA_SUB *sub)
@q m e f @>
  {

    if (sub->fsVarStat(variable_)->fixedOrSet()) return 1;

    sub->lBound(variable_, lBound_);
    sub->uBound(variable_, uBound_);
    
    return 0;
  }

@q m v extract @>
@ The function |extract()| is overloaded to modify directly the linear
  programming relaxation. This required to evaluate the quality of
  a branching rule.
@q m e t @>

@(boundbranchrule.cc@>=
@q m b f @>
  void ABA_BOUNDBRANCHRULE::extract(ABA_LPSUB *lp)
@q m e f @>
  {
     oldLpLBound_ = lp->lBound(variable_);
     oldLpUBound_ = lp->uBound(variable_);

     lp->changeLBound(variable_, lBound_);
     lp->changeUBound(variable_, uBound_);
  }

@q m v unExtract @>
@q m e t @>
@ The function unExtract

@(boundbranchrule.cc@>=
@q m b f @>
  void ABA_BOUNDBRANCHRULE::unExtract(ABA_LPSUB *lp)
@q m e f @>
  {
    lp->changeLBound(variable_, oldLpLBound_);
    lp->changeUBound(variable_, oldLpUBound_);
  }

@q m f variable @>
@q m e t @>
@ The function |variable()|.

@q m b r @>
  \BeginReturn
  The number of the branching variable.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_BOUNDBRANCHRULE::variable() const
@q m e f @>


@q m f lBound @>
@q m e t @>
@ The function |lBound()|.

@q m b r @>
  \BeginReturn
  The lower bound of the branching variable.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_BOUNDBRANCHRULE::lBound() const
@q m e f @>


@q m f uBound @>
@q m e t @>
@ The function |uBound()|.

@q m b r @>
  \BeginReturn
  The upper bound of the branching variable.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_BOUNDBRANCHRULE::uBound() const
@q m e f @>


