@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: branchrule.w,v 4.10 2003/08/05 13:55:13 elf Exp $ @>

@q m i ABA_BRANCHRULE @>

@ All member functions are defined in the file {\tt branchrule.cc}.

@(branchrule.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "branchrule.h"
#include "master.h"
#else
#include "abacus/branchrule.h"
#include "abacus/master.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
   \BeginArg
   \docarg{|master|}{A pointer to the corresponding master of the optimization.}
   \EndArg
@q m e a @>

@(branchrule.cc@>=
@q m b f @>
  ABA_BRANCHRULE::ABA_BRANCHRULE(ABA_MASTER *master) 
@q m e f @>
  : @/
    master_(master)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(branchrule.cc@>=
@q m b f @>
  ABA_BRANCHRULE::~ABA_BRANCHRULE()
@q m e f @>
  { }

#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the |ABA_BRANCHRULE| from an |ABA_MESSAGE|.

@q m e t @>

@q m b a @>
    \BeginArg
    \docarg
      {|master|}{A pointer to the corresponding master of the optimization.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(branchrule.cc@>=
@q m b f @>
  ABA_BRANCHRULE::ABA_BRANCHRULE(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    master_(master)
  { }

@q m f pack @>
@ The function |pack()| packs the data of the |ABA_BRANCHRULE|
  in an |ABA_MESSAGE| object. This virtual function has to be redefined
  in a derived class if additional data should be packed. In this
  case the |pack()| function of the base class must be called
  by the |pack()| function of the derived class.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the |ABA_BRANCHRULE| is packed.}
  \EndArg
@q m e a @>

@(branchrule.cc@>=
@q m b f @>
  void ABA_BRANCHRULE::pack(ABA_MESSAGE &msg) const
@q m e f @>
  { }

#endif


@q m v branchOnSetVar @>
@ The virtual function |branchOnSetVar()| should indicate if the branching
  is performed by setting a binary variable. This is only required as in the
  current version of the GNU-compiler run time type information
  is not satisfactorily implemented.
@q m e t @>

  This function is currently required to determine global validity
  of Gomory cuts for general \mip s.

@q m b r @>
  \BeginReturn
  The default implementation
  returns always false. This function must be redefined in the
  class |ABA_SETBRANCHRULE|, where it has to return |true|.
  \EndReturn
@q m e r @>

@(branchrule.cc@>=
@q m b f @>
  bool ABA_BRANCHRULE::branchOnSetVar()
@q m e f @>
  {
    return false;
  }
  

@q m v extract @>
@ The function |extract()| modifies a subproblem by setting the branching
  variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the subproblem can be modified according to the
               branching rule.}
  \docarg{|1|}{If a contradiction occurs.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The subproblem being modified.}
  \EndArg
@q m e a @>

@q m b f @>
     virtual int extract(ABA_SUB *sub) = 0
@q m e f @>

@q m v extract @>
@ The virtual function |extract()| should modify the linear programming
  relaxation |lp| in
  order to determine the quality of the branching rule in a linear
  programming based branching rule selection. The default implementation
  does nothing except writing a warning to the error stream. If a
  derived concrete branching rule should be used in LP-based branching
  rule selection then this function has to be redefined.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|lp|}{A pointer to a the linear programming relaxtion of a
                a subproblem.}
  \EndArg
@q m e a @>

@(branchrule.cc@>=
@q m b f @>
  void ABA_BRANCHRULE::extract(ABA_LPSUB *lp)
@q m e f @>
  {
    master_->err() << "WARNING:" << endl;
    master_->err() << "  virtual dummy function extract() of base class ";
    master_->err() << "ABA_BRANCHRULE is called." << endl;
    master_->err() << "  Redefinition in derived class is missing." << endl;
  }

@q m v unExtract @>
@ The virtual function |unExtract()| should undo the modifictions of the
  linear programming relaxtion |lp|. This function has to be redefined
  in a derived class, if also |extract(ABA_LPSUB*)| is redefined there.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|lp|}{A pointer to a the linear programming relaxtion of a
                a subproblem.}
  \EndArg
@q m e a @>

@(branchrule.cc@>=
@q m b f @>
  void ABA_BRANCHRULE::unExtract(ABA_LPSUB *lp)
@q m e f @>
  {
    master_->err() << "WARNING:" << endl;
    master_->err() << "  virtual dummy function unExtract() of base class ";
    master_->err() << "ABA_BRANCHRULE is called." << endl;
    master_->err() << "  Redefinition in derived class is missing." << endl;
  }

@q m v initialize @>
@ The function initialize is a virtual dummy function doing nothing.
  It is called from the constructor of the subproblem and can be used
  to perform initializations of the branching rule that can be only
  done after the generation of the subproblem.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{A pointer to the subproblem that should be used for the
                 initialization.}
  \EndArg
@q m e a @>

@(branchrule.cc@>=
@q m b f @>
  void ABA_BRANCHRULE::initialize(ABA_SUB* sub)
@q m e f @>
  {}

