@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: column.w,v 4.10 2003/08/05 13:55:13 elf Exp $ @>

@q m i ABA_COLUMN  @>

@ All member functions are defined in the file {\tt column.cc}.

@(column.cc@>=

#ifdef ABACUS_OLD_INCLUDE
#include "column.h"
#include "global.h"
#else
#include "abacus/column.h"
#include "abacus/global.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@q m e t @>
@ A constructor.

@q m b a @>
  \BeginArg
  \docarg{|glob|} {A pointer to the corresponding global object.}
  \docarg{|obj|}{The objective function coefficient.}
  \docarg{|lb|} {The lower bound.}
  \docarg{|ub|} {The upper bound.}
  \docarg{|nnz|} {The number of nonzero elements stored in the
                    arrays |s| and |c|.}
  \docarg{|s|} {An array of the nonzero elements of the column.}
  \docarg{|c|} {An array of the nonzero coefficients associated with the
                  elements of |s|.}
  \EndArg
@q m e a @>

@(column.cc@>=
@q m b f @>   
  ABA_COLUMN::ABA_COLUMN(ABA_GLOBAL *glob, 
                         double obj, 
                         double lb, 
                         double ub,
                         int nnz, 
                         ABA_ARRAY<int> &s, 
                         ABA_ARRAY<double> &c)
@q m e f @>
  : @/
    ABA_SPARVEC(glob, nnz, s, c), @/
    obj_(obj), @/
    lBound_(lb), @/
    uBound_(ub)
  {  }

@q m f Constructor @>
@ Another constructor generating an uninitialized column.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \docarg{|maxNnz|}{The maximal number of nonzero elements that
                    can be stored in the row.}
  \EndArg
@q m e a @>

@(column.cc@>=
@q m b f @>
  ABA_COLUMN::ABA_COLUMN(ABA_GLOBAL *glob, int maxNnz) 
@q m e f @>
  : @/
    ABA_SPARVEC(glob, maxNnz)
  { }

@q m f Constructor @>
@ A constructor using a sparse vector for the initialization.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|} {A pointer to the corresponding global object.}
  \docarg{|obj|}{The objective function coefficient.}
  \docarg{|lb|} {The lower bound.}
  \docarg{|ub|} {The upper bound.}
  \docarg{|vec|} {A sparse vector storing the support and the coefficients
                    of the column.}
  \EndArg
@q m e a @>

@(column.cc@>=
@q m b f @>
  ABA_COLUMN::ABA_COLUMN(ABA_GLOBAL *glob, 
                         double obj, 
                         double lb, 
                         double ub,
                         ABA_SPARVEC &vec) 
@q m e f @>
  : @/
    ABA_SPARVEC(vec), @/
    obj_(obj), @/
    lBound_(lb), @/
    uBound_(ub)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(column.cc@>=
@q m b f @>
  ABA_COLUMN::~ABA_COLUMN()
@q m e f @>
  { }
  

#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>
@ The message constructor creates the column from an |ABA_MESSAGE|.

@q m b a @>
    \BeginArg
    \docarg
      {|glob|}{A pointer to the corresponding global object.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(column.cc@>=
@q m b f @>
  ABA_COLUMN::ABA_COLUMN(const ABA_GLOBAL *glob, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    ABA_SPARVEC(glob, msg)
  {
    msg.unpack(obj_);
    msg.unpack(lBound_);
    msg.unpack(uBound_);
  }


@q m f pack @>
@ The function |pack()| packs the data of the column in an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the column is packed.}
  \EndArg
@q m e a @>

@(column.cc@>=
@q m b f @>
  void ABA_COLUMN::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_SPARVEC::pack(msg);
    msg.pack(obj_);
    msg.pack(lBound_);
    msg.pack(uBound_);
  }

#endif


@q m f Output Operator @>
@q m e t @>
@ The output operator.

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The column being output.}
  \EndArg
@q m e a @>

@(column.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_COLUMN &rhs)
@q m e f @>
  {
    out << "objective function coefficient: " << rhs.obj() << endl;
    out << "bounds: " << rhs.lBound_ << " <= x <= " << rhs.uBound_ << endl;
    out << "nonzero rows of column :" << endl;

    for (int i = 0; i < rhs.nnz_; i++)
      out << 'r' << rhs.support_[i] << ": " << rhs.coeff_[i] << endl;

    return out;
  }

@q m f obj @>
@q m e t @>
@ The function |obj()|.

@q m b r @>
  \BeginReturn
  The objective function coefficient of the column.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_COLUMN::obj() const
@q m e f @>

@q m f obj @>
@ This version of the  function |obj()| sets the objective function 
  coefficient of the column.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|c|}{The new value of the objective function coefficient.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_COLUMN::obj(double c) 
@q m e f @>

@q m f lBound @>
@q m e t @>
@ The function |lBound()|.

@q m b r @>
  \BeginReturn
  The lower bound of the column.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_COLUMN::lBound() const
@q m e f @>
  
@q m f lBound @>
@ This version of the function |lBound()| sets the lower bound of the column.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|l|}{The new value of the lower bound.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_COLUMN::lBound(double l)
@q m e f @>

@q m f uBound @>
@q m e t @>
@ The function |uBound()|.

@q m b r @>
  \BeginReturn
  The upper bound of the column.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_COLUMN::uBound() const
@q m e f @>

@q m f uBound @>
@ This version of the function |uBound()| sets the upper bound of the column.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|u|}{The new value of the upper bound.}
  \EndArg
@q m e a @>
@q m b f @>
  void ABA_COLUMN::uBound(double u)
@q m e f @>

@q m f copy @>
@ The function |copy()| is very similar to the assignment operator,
  yet the columns do not have to be of equal size.
  A reallocation is performed if required.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|col|}{The column that is copied.}
  \EndArg
@q m e a @>

@(column.cc@>=
@q m b f @>
  void ABA_COLUMN::copy(const ABA_COLUMN &col)
@q m e f @>
  {
    ABA_SPARVEC::copy(col);
 
    obj_    = col.obj_;
    lBound_ = col.lBound_;
    uBound_ = col.uBound_;
  }
