@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: colvar.w,v 4.11 2003/08/05 13:55:14 elf Exp $ @>

@q m i ABA_COLVAR @>

@ All member functions are defined in the file {\tt colvar.cc}.

@(colvar.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "colvar.h"
#include "master.h"
#include "column.h"
#include "array.h"
#include "numcon.h"
#else
#include "abacus/colvar.h"
#include "abacus/master.h"
#include "abacus/column.h"
#include "abacus/array.h"
#include "abacus/numcon.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
  \BeginArg
  \docarg{|master|}{A pointer to the corresponding master of the optimization.}
  \docarg{|sub|}{A pointer to the subproblem associated with the
                   variable. This can be also the 0-pointer.}
  \docarg{|dynamic|}{If this argument is |true|, then the variable
                           can be removed from the active variable set
                           during the subproblem
                           optimization.}
  \docarg{|local|}{If this argument is |true|, then the constraint
                       is considered to be only locally valid. As a local
                       variable is associated with a subproblem, |sub|
                       must not be 0 if local is |true|.}
  \docarg{|lBound|}{The lower bound of the variable.}
  \docarg{|uBound|}{The upper bound of the variable.}
  \docarg{|varType|}{The type of the variable.}
  \docarg{|obj|}{The objective function coefficient of the variable.}
  \docarg{|nnz|}{The number of nonzero elements of the variable.}
  \docarg{|support|}{The array storing the constraints with the nonzero
                          coefficients.}
  \docarg{|coeff|}{The nonzero coefficients of the constraints stored
                        in |support|.}
  \EndArg

@q m e a @>

@(colvar.cc@>=
@q m b f @>
  ABA_COLVAR::ABA_COLVAR(ABA_MASTER *master, 
                         const ABA_SUB *sub, 
                         bool dynamic, 
                         bool local,
                         double lBound, 
                         double uBound,
                         ABA_VARTYPE::TYPE varType, 
                         double obj, 
                         int nnz, 
                         ABA_ARRAY<int> &support, 
                         ABA_ARRAY<double> &coeff)
@q m e f @>
  : @/
    ABA_VARIABLE(master, sub, dynamic, local, obj, lBound, uBound, varType), @/
    column_(master, obj, lBound, uBound, nnz, support, coeff)
  { }

@q m f Constructor @>
@ A constructor substituting |nnz|, |support|, and |coeff| of the previous
  constructor by an object of the class |ABA_SPARVEC|.
@q m e t @>

@(colvar.cc@>=
@q m b f @>
  ABA_COLVAR::ABA_COLVAR(ABA_MASTER *master, 
                         const ABA_SUB *sub, 
                         bool dynamic, 
                         bool local,
                         double lBound, 
                         double uBound,
                         ABA_VARTYPE::TYPE varType, 
                         double obj, 
                         ABA_SPARVEC &vector) 
@q m e f @>
  : @/
    ABA_VARIABLE(master, sub, dynamic, local, obj, lBound, uBound, varType), @/
    column_(master, obj, lBound, uBound, vector)
  { }
  
@q m v Destructor @>
@q m e t @>
@ The destructor.

@(colvar.cc@>=
@q m b f @>
  ABA_COLVAR::~ABA_COLVAR()
@q m e f @>
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the variable from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the variable is initialized.}
  \EndArg
@q m e a @>

@(colvar.cc@>=
@q m b f @>
  ABA_COLVAR::ABA_COLVAR(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    ABA_VARIABLE(master, msg), @/
    column_(master, msg)
  {  }

@q m f pack @>
@ The function |pack()| packs the data of the variable in an
  |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the variable is packed.}
  \EndArg
@q m e a @>

@(colvar.cc@>=
@q m b f @>
  void ABA_COLVAR::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_VARIABLE::pack(msg);
    column_.pack(msg);
  }

@q m f classId @>
@ The function |classId()| returns an integer which identifies the class.

@q m b r @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e r @>

@(colvar.cc@>=
@q m b f @>
  int ABA_COLVAR::classId() const
@q m e f @>
  {
    return COLVAR_CLASSID;
  }

#endif


@q m f Output Operator @>
@ The output operator writes the column representing the variable to 
  an output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The variable being output.}
  \EndArg
@q m e a @>

@(colvar.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_COLVAR &rhs)
@q m e f @>
  {
    return out << rhs.column_;
  }

@q m v print @>
@ The function |print()| writes the column representing the variable
  to an output stream.
  It redefines the virtual function |print()| of the base class
 |ABA_CONVAR|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \EndArg
@q m e a @>
 
@(colvar.cc@>=
@q m b f @>
  void ABA_COLVAR::print(ostream &out)
@q m e f @>
  {
    out << *this;
  }

@q m v coeff @>
@q m e t @>
@ The function |coeff()|.

@q m b r @>
  \BeginReturn
  The coefficient of the constraint |con|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|con|}{The constraint of which the coefficient is computed.
                 This must be a pointer to the class |ABA_NUMCON|.}
  \EndArg
@q m e a @>

@(colvar.cc@>=
@q m b f @>
  double ABA_COLVAR::coeff(ABA_CONSTRAINT *con)
@q m e f @>
  {
    return column_.origCoeff(((ABA_NUMCON*) con)->number());
  }

@q m f coeff @>
@ This version of the function |coeff()| computes the coefficient
  of a constraint with a given number.
@q m e t @>

@q m b r @>
  \BeginReturn
  The coefficient of constraint |i|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the constraint.}
  \EndArg
@q m e a @>

@(colvar.cc@>=
@q m b f @>
  double ABA_COLVAR::coeff(int i)
@q m e f @>
  {
    return column_.origCoeff(i);
  }

@q m f column @>
@q m e t @>
@ The function |column()|.

@q m b r @>
  \BeginReturn
  A pointer to the column representing the variable.
  \EndReturn
@q m e r @>

@(colvar.cc@>=
@q m b f @>
  ABA_COLUMN *ABA_COLVAR::column()
@q m e f @>
  {
    return &column_;
  }
