@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: conbranchrule.w,v 4.18 2003/08/05 13:55:14 elf Exp $ @>

@q m i ABA_CONBRANCHRULE @>

@ All member functions are defined in the file {\tt br|_|con.cc}.

@(conbranchrule.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "conbranchrule.h"
#include "active.h"
#include "master.h"
#include "sub.h"
#include "lpsub.h"
#else
#include "abacus/conbranchrule.h"
#include "abacus/active.h"
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/lpsub.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/parmaster.h"
#include "abacus/message.h"
#include "abacus/pool.h"
#include "abacus/poolslot.h"
#include "abacus/debug.h"
#endif

@q m f Constructor @>
@q m e t @>

@ The constructor.

@q m b t @>
  Note, the subproblem associated with the branching constraint will be 
  modified in the constructor of the subproblem generated with this 
  branching rule such that later the check for local validity of the 
  branching constraint is performed correcly.
@q m e t @>

@q m b a @>
   \BeginArg
   \docarg{|master|}{A pointer to the corresponding master of the optimization.}
   \docarg{|poolSlot|}{A pointer to the pool slot of the branching constraint.}
  \EndArg
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  ABA_CONBRANCHRULE::ABA_CONBRANCHRULE(
                         ABA_MASTER *master,
                         ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *poolSlot) 
@q m e f @>
  : @/
    ABA_BRANCHRULE(master), @/
    poolSlotRef_(poolSlot)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(conbranchrule.cc@>=
@q m b f @>
  ABA_CONBRANCHRULE::~ABA_CONBRANCHRULE()
@q m e f @>
  { }

#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the |ABA_CONBRANCHRULE| from an |ABA_MESSAGE|.

@q m e t @>

@q m b a @>
    \BeginArg
    \docarg
      {|master|}{A pointer to the corresponding master of the optimization.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  ABA_CONBRANCHRULE::ABA_CONBRANCHRULE(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    ABA_BRANCHRULE(master, msg), @/
    poolSlotRef_(master)
  {
    ABA_ID id(msg);
    char needConstraint;

    ABA_POOL<ABA_CONSTRAINT, ABA_VARIABLE> *pool = (ABA_POOL<ABA_CONSTRAINT, ABA_VARIABLE>*)
      master_->parmaster()->getPool(id.index());
    ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *ps = pool->findSlot(id);
    if (ps) {
      poolSlotRef_.slot(ps);
      needConstraint=0;
    }
    else
      needConstraint=1;

    // msg.clear();
    msg.pack(needConstraint);
    msg.send();         // send flag if constraint is missing

    msg.receive();      // receive constraint if needed
    if (needConstraint) {
      int classId;
      msg.unpack(classId);
      ABA_CONSTRAINT *con = (ABA_CONSTRAINT*) master_->unpackConVar(msg, classId);
      if (debug(DEBUG_MESSAGE_CONVAR)) {
        master_->out() << "DEBUG_MESSAGE_CONVAR: Constraint " << id << " (classId=" << classId
                       << ") of ABA_CONBRANCHRULE received." << endl;
      }
      ABA_POOL<ABA_CONSTRAINT, ABA_VARIABLE> *pool = (ABA_POOL<ABA_CONSTRAINT, ABA_VARIABLE>*)
        master_->parmaster()->getPool(id.index());
      ps = pool->insert(con);
      if (ps == 0) {
        master_->err() << "ABA_CONBRANCHRULE::ABA_CONBRANCHRULE(): no room"
                          " to insert constraint into pool." << endl;
        exit(Fatal);
      }
      ps->setIdentification(id);
      poolSlotRef_.slot(ps);
    }
  }

@q m f pack @>
@ The virtual function |pack()| packs the data of the |ABA_CONBRANCHRULE|
  in an |ABA_MESSAGE| object. The |pack()| function of the base class
  has to be called before the local data is packed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the |ABA_CONBRANCHRULE| is packed.}
  \EndArg
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  void ABA_CONBRANCHRULE::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_BRANCHRULE::pack(msg);
    if (poolSlotRef_.conVar() == 0) {           // removed conVar
      master_->err() << "ABA_CONBRANCHRULE::pack(): branching constraint not"
                        " available!" << endl;
      exit(Fatal);
    }

    ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *ps = poolSlotRef_.slot();
    if (!ps->getIdentification().isInitialized())
      ps->setNewIdentification();
    ps->getIdentification().pack(msg);

    msg.send();                 // send |ABA_ID| of constraint

    msg.receive();              // receive flag if constraint is missing
    char needConstraint;
    msg.unpack(needConstraint);
    // msg.clear();

    if (needConstraint) {
      ABA_CONSTRAINT *con = poolSlotRef_.conVar();
      if (debug(DEBUG_MESSAGE_CONVAR)) {
        master_->out() << "DEBUG_MESSAGE_CONVAR: sending Constraint " << ps->getIdentification()
                       << " (classId=" << con->classId()
                       << ") of ABA_CONBRANCHRULE..." << endl;
      }
      msg.pack(con->classId());
      con->pack(msg);
    }
  }

@q m f classId @>
@ The abstract virtual function |classId()| of |ABA_BRANCHRULE| has to be
  implemented in each derived subclass. It has to return an integer
  which identifies the class.

@q m b a @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  int ABA_CONBRANCHRULE::classId() const
@q m e f @>
  {
    return CONBRANCHRULE_CLASSID;
  }

#endif


@q m f Output Operator @>
@ The output operator writes the branching constraint on an output stream.
@q m e t @>

@q m b r @>
   \BeginReturn
   A reference to the output stream.
   \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The branch rule being output.}
  \EndArg
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_CONBRANCHRULE &rhs)
@q m e f @>
  {
    return out << rhs.poolSlotRef_;
  }


@q m v extract @>
@ The function |extract()| adds the branching constraint to the subproblem.
@q m e t @>
  Instead of adding it directly to the set of active constraints it is
  added to the cut buffer. 

@q m b r @>
  \BeginReturn
  Always 0, since there cannot be a contractiction.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The subproblem being modified.}
  \EndArg
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  int ABA_CONBRANCHRULE::extract(ABA_SUB *sub)
@q m e f @>
  {

    if (poolSlotRef_.conVar() == 0) {
      master_->err() << "ABA_CONBRANCHRULE::extract(): branching constraint not available" << endl;
      exit(Fatal);
    }

    if (sub->addBranchingConstraint(poolSlotRef_.slot())) {
      master_->err() << "ABA_CONBRANCHRULE::extract(): addition of branching constaint ";
      master_->err() << "to subproblem failed." << endl;
      exit(Fatal);
    }

    return 0;
  }

@q m v extract @>
@ The function |extract()| is overloaded to modify directly the linear
  programming relaxation. This required to evaluate the quality of
  a branching rule.
@q m e t @>

@(conbranchrule.cc@>=
@q m b f @>
  void ABA_CONBRANCHRULE::extract(ABA_LPSUB *lp)
@q m e f @>
  {
    ABA_BUFFER<ABA_CONSTRAINT*> newCon(master_, 1);

    newCon.push(poolSlotRef_.conVar());
    lp->addCons(newCon);
  }

@q m v unExtract @>
@ The function |unExtract()|.
@q m e t @>

@(conbranchrule.cc@>=
@q m b f @>
  void ABA_CONBRANCHRULE::unExtract(ABA_LPSUB *lp)
@q m e f @>
  {
    ABA_BUFFER<int> remove(master_, 1);

    remove.push(lp->nRow() - 1);

    @<pivot the slack variable associated with the removed row in@>;

    lp->removeCons(remove);
  }

@ @<pivot the slack variable associated with the removed row in@>=
  int status = lp->pivotSlackVariableIn(remove);

  if (status) {
    master_->err() << "WARNING: ";
    master_->err() << "ABA_CONBRANCHRULE::unExtract(): pivoting in ";
    master_->err() << "slack variable failed." << endl;
  }

@q m v initialize @>
@ The function initialize redefines the virtual function of the base 
  class |ABA_BRANCHRULE| in order to initialize the subproblem associated with
  the branching constraint.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{A pointer to the subproblem that is associated with
                 the branching constraint.}
  \EndArg
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  void ABA_CONBRANCHRULE::initialize(ABA_SUB* sub)
@q m e f @>
  {
    if (poolSlotRef_.conVar() == 0) {
      master_->err() << "ABA_CONBRANCHRULE::initialize(): ";
      master_->err() << "branching constraint not available" << endl;
      exit(Fatal);
    }

    poolSlotRef_.conVar()->sub(sub);
  }


@q m f constraint @>
@q m e t @>
@ The function |constraint()|.

@q m b a @>
  \BeginReturn
   A pointer to the branching constraint or a 0-pointer, if this
   constraint is not available.
  \EndReturn
@q m e a @>

@(conbranchrule.cc@>=
@q m b f @>
  ABA_CONSTRAINT *ABA_CONBRANCHRULE::constraint()
@q m e f @>
  {
    return poolSlotRef_.conVar();
  }
