@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: conclass.w,v 4.9 2003/08/05 13:55:14 elf Exp $ @>

@q m i ABA_CONCLASS @>

@ All member functions are defined in the file {\tt conclass.cc}.

@(conclass.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "conclass.h"
#include "master.h"
#else
#include "abacus/conclass.h"
#include "abacus/master.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@ The constructor initializes the constraint classification with
  \BeginArg
  \docarg
    {|master|}
    {A pointer to the corresponding master of the optimization.}
  \docarg
    {|allVarBinary|}
    {|true| if all variables are binary}
  \docarg
    {|trivial|}
    {|true| if it is a bound or a variable upper bound}
  \docarg
    {|bound|}
    {|true| if the constraint is a bound of the variable}
  \docarg
    {|varBound|}
    {|true| if the constraint is a variable upper bound}

@(conclass.cc@>=
  ABA_CONCLASS::ABA_CONCLASS(const ABA_MASTER *master, bool discrete,
                     bool allVarBinary, bool trivial,
                     bool bound, bool varBound) : @/
    master_(master), @/
    discrete_(discrete), @/
    allVarBinary_(allVarBinary), @/
    trivial_(trivial), @/
    bound_(bound), @/
    varBound_(varBound)
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the |ABA_CONCLASS| object from an |ABA_MESSAGE|.

@q m b a @>
    \BeginArg
    \docarg
      {|master|}{A pointer to the corresponding master of the optimization.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(conclass.cc@>=
@q m b f @>
  ABA_CONCLASS::ABA_CONCLASS(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    master_(master)
  {
    int bits = msg.unpackInt();
    discrete_     = ((bits&1)?true:false);
    allVarBinary_ = ((bits&2)?true:false);
    trivial_      = ((bits&4)?true:false);
    bound_        = ((bits&8)?true:false);
    varBound_     = ((bits&16)?true:false);
  }

@q m f pack @>
@ The function |pack()| packs the data of the |ABA_CONCLASS| in an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the |ABA_CONCLASS| is packed.}
  \EndArg
@q m e a @>

@(conclass.cc@>=
@q m b f @>
  void ABA_CONCLASS::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    msg.pack( discrete_ + 2*allVarBinary_ + 4*trivial_ +
              8*bound_ + 16*varBound_ );
  }

#endif


@ The output operator.

@(conclass.cc@>=
  ostream &operator<<(ostream &out, const ABA_CONCLASS &rhs)
  {
    bool classified = false;

    if (rhs.discrete_) {
      out << "discrete/";
      classified = true;
    }
    if (rhs.allVarBinary_) {
      out << "allVarBinary/";
      classified = true;
    }
    if (rhs.trivial_) {
      out << "trivial/";
      classified = true;
    }
    if (rhs.bound_) {
      out << "bound/";
      classified = true;
    }
    if (rhs.varBound_) {
      out << "variable bound/";
      classified = true;
    }

    if (!classified) out << "no classification ";
    
    return out;
  }

@ The function |allVarBinary()| returns |true| if all variables with
  nonzero coefficients of the constraint are binary.

  bool ABA_CONCLASS::allVarBinary() const

@ The function |trivial()| returns |true| if the constraint is a
  bound or a variable upper bound.

  bool ABA_CONCLASS::trivial() const
