@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@ The non-inline member functions are defined in the file {\tt constraint.cc}.
@q $Id: constraint.w,v 4.14 2003/08/05 13:55:14 elf Exp $ @>

@q m i ABA_CONSTRAINT @>

@(constraint.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "sub.h"
#include "constraint.h"
#include "row.h"
#include "conclass.h"
#include "active.h"
#else
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/constraint.h"
#include "abacus/row.h"
#include "abacus/conclass.h"
#include "abacus/active.h"
#endif

#include <math.h>

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
      \BeginArg
      \docarg{|master|} {A pointer to the corresponding master of the optimization.}
      \docarg{|sub|}{A pointer to the subproblem associated with the
                     constraint. This can be also the 0-pointer.}
      \docarg{|sense|} {The sense of the constraint.}
      \docarg{|rhs|}{The right hand side of the constraint.}
      \docarg{|dynamic|} {If this argument is |true|, then the constraint
                          can be removed from the active constraint set
                          during the cutting plane phase of the subproblem
                          optimization.}
      \docarg{|local|}{If this argument is |true|, then the constraint
                       is considered to be only locally valid. In this case
                       the argument |sub| must not be 0 as each locally
                       valid constraint is associated with a subproblem.
      }
      \docarg{|liftable|}{If this argument is |true|, then a lifting
                          procedure must be available, i.e., that the
                          coefficients of variables which have not been
                          active at generation time of the constraint
                          can be computed.}
      \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  ABA_CONSTRAINT::ABA_CONSTRAINT(ABA_MASTER *master, 
                                 const ABA_SUB *sub, 
                                 ABA_CSENSE::SENSE sense,
                                 double rhs, 
                                 bool dynamic,
                                 bool local, 
                                 bool liftable) 
@q m e f @>
    : @/
    ABA_CONVAR(master, sub, dynamic, local), @/
    sense_(master, sense), @/
    rhs_(rhs), @/
    conClass_(0), @/
    liftable_(liftable)
  {
    if (local && sub == 0) {
      master_->out() << "ABA_CONSTRAINT::ABA_CONSTRAINT(): ";
      master_->out() << "subtree of local item must not be 0" << endl;
      exit(Fatal);
    }
  }

@q m f Constructor @>
@ The following constructor initializes an empty constraint. This constructor
  is, e.g., useful if parallel separation is applied. In this case the
  constraint can be constructed and receive later its data by message
  passing.
@q m e t @>

@q m b a @>
      \BeginArg
      \docarg{|master|} {A pointer to the corresponding master of the optimization.}
      \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  ABA_CONSTRAINT::ABA_CONSTRAINT (ABA_MASTER *master) 
@q m e f @>
  : @/
    ABA_CONVAR(master, 0, true, true), @/
    sense_(master), @/
    conClass_(0)
  { }

@q m f Copy Constructor @>
@q m e t @>  
@ The copy constructor.

@q m b a @>
  \BeginArg
     \docarg{|rhs|}{The constraint being copied.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  ABA_CONSTRAINT::ABA_CONSTRAINT(const ABA_CONSTRAINT &rhs) 
@q m e f @>
    : @/
    ABA_CONVAR(rhs), @/
    sense_(rhs.sense_), @/
    rhs_(rhs.rhs_),
    conClass_(0),
    liftable_(rhs.liftable_)
  {
    if (rhs.conClass_) conClass_ = new ABA_CONCLASS(*(rhs.conClass_));
  }
    
@q m v Destructor @>
@q m e t @>
@ The destructor.

@(constraint.cc@>=
@q m b f @>
  ABA_CONSTRAINT::~ABA_CONSTRAINT()
@q m e f @>
  {
    delete conClass_;
  }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the constraint from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the constraint is initialized.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  ABA_CONSTRAINT::ABA_CONSTRAINT(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    ABA_CONVAR(master, msg), @/
    sense_(master, msg), @/
    conClass_(0)
  {
    msg.unpack(rhs_);
    int bits = msg.unpackInt();
    if (bits&1)
      conClass_ = new ABA_CONCLASS(master_, msg);
    liftable_ = ((bits&2)?true:false);
  }

@q m f pack @>
@ The function |pack()| packs the data of the constraint
  in an |ABA_MESSAGE| object. This virtual function has to be redefined
  in a derived class if additional data should be packed. In this
  case the |pack()| function of the base class must be called
  by the |pack()| function of the derived class.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the constraint is packed.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  void ABA_CONSTRAINT::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_CONVAR::pack(msg);

    sense_.pack(msg);
    msg.pack(rhs_);
    msg.pack( (conClass_?1:0) + (liftable_?2:0) );
    if (conClass_)
      conClass_->pack(msg);
  }

#endif


@q m f sense @>
@q m e t @>
@ The function |sense()|.

@q m b r @>
  \BeginReturn
    A pointer to the sense of the constraint.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CSENSE *ABA_CONSTRAINT::sense()
@q m e f @>

@q m v rhs @>
@q m e t @>
@ The virtual function |rhs()|.

@q m b r @>
  \BeginReturn
  The right hand side of the constraint.
  \EndReturn
@q m e r @>

@(constraint.cc@>=
@q m b f @>
  double ABA_CONSTRAINT::rhs()
@q m e f @>
  {
    return rhs_;
  }

@q m f liftable @>
@ The function |liftable()| checks if the constraint is liftable, i.e., if
  the coefficients of variables inactive at generation time of the constraint
  can be computed later.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the constraint can be lifted,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_CONSTRAINT::liftable() const
@q m e f @>
  
@q m v valid @>
@ The function |valid()| checks if the constraint is
  valid for the subproblem |sub|. Per default, this is the case
  if the constraint is globally valid, or the subproblem associated
  with the constraint is an ancestor of the subproblem sub in the
  enumeration tree.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the constraint is valid for the subproblem |sub|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The subproblem for which the validity is checked.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  bool ABA_CONSTRAINT::valid(ABA_SUB *sub)
@q m e f @>
  {
#ifdef ABACUSSAFE
    if (local_ && sub == 0) {
      master_->err() << "ABA_CONSTRAINT::valid(): cannot evaluate for locally ";
      master_->err() << "valid constraint with 0-pointer to subproblem.";
      exit(Fatal);
    }
#endif

    if (!local_ || sub_->ancestor(sub)) return true;
    else                                return false;
  }

@q m v genRow @>
@ The function |genRow()| generates the row format of the constraint
  associated with the variable set |var|.
  This function is declared virtual since faster constraint specific
  implementations might be desirable.
@q m e t @>

  All nonzero coefficients are added to the row format. Before
  we generate the coefficients we try to expand the constraint,
  afterwards it is compressed again.

@q m b r @>
  \BeginReturn  
  The number of nonzero elements in the row format |row|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|var|}{The variable set for which the row format should be
                   computed.}
  \docarg{|row|}{Stores the row format after calling this function.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  int ABA_CONSTRAINT::genRow(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *var,
                             ABA_ROW &row)
@q m e f @>
  {
    double eps      = master_->machineEps();
    double minusEps = -eps;
    int    n        = var->number();
    double c;

    _expand();
    
    for (int e = 0; e < n; e++) {
      c = coeff((*var)[e]);
      if (c > eps || c < minusEps) row.insert(e, c);
    }
    
    row.rhs(rhs());
    row.sense(sense_);
    _compress();
    return row.nnz();
  }

@q m v slack @>
@ The function |slack()| computes the slack of the vector |x| associated
  with the variable set |variables|.
@q m e t @>

@q m b r @>
  \BeginReturn
  The slack induced by the vector |x|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|variables|}{The variable set associated with the vector |x|.}
  \docarg{|x|}{The values of the variables.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  double ABA_CONSTRAINT::slack(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *variables,
                               double *x)
@q m e f @>
  {
    double eps      = master_->machineEps();
    double minusEps = -eps;
    double c;
    double xi;
    double lhs = 0.0;
    int    n   = variables->number();

    _expand();
    
    for (int i = 0; i < n; i++) {
      xi = x[i];
      if (xi > eps || xi < minusEps) {
        c = coeff((*variables)[i]);
        if (c > eps || c < minusEps)
          lhs += c * xi;
      }
    }

    _compress();

    return rhs() - lhs;
  }
  
@q m v violated @>  
@ The function |violated()| checks if a constraint is violated by
  a vector |x| associated with a variable set.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the constraint is violated,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|variables|}{The variables associated with the vector |x|.}
  \docarg{|x|}{The vector for which the violation is checked.}
  \docarg{|sl|}{If |sl| is nonzero, then |*sl| will store the value of the
                  violation, i.e., the slack.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>    
  bool ABA_CONSTRAINT::violated(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *variables,
                                double *x, 
                                double *sl)
@q m e f @>
  {
    double s = slack(variables, x);

    if (sl) *sl = s;
    
    return violated(s);
  }

@q m v violated @>
@ This version of function |violated()| checks for the violation given the
  slack of a vector.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the constraint
                    is an equation and the |slack| is nonzero, or 
                    if the constraint is a $\le$-inequality and the slack 
                    is negative, or the constraint is
                    a $\ge$-inequality and the slack is positive,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|slack|}{The slack of a vector.}
  \EndArg
@q m e a @>

@(constraint.cc@>=  
@q m b f @>
  bool ABA_CONSTRAINT::violated(double slack) const
@q m e f @>
  {
    switch (sense_.sense()) {
      case ABA_CSENSE::Equal:
        if (fabs(slack) > master_->eps()) return true;
        else                              return false;
      case ABA_CSENSE::Less:
        if (slack < -master_->eps()) return true;
        else                         return false;
      case ABA_CSENSE::Greater:
        if (slack > master_->eps()) return true;
        else                        return false;
      default: master_->err() << "ABA_CONSTRAINT::violated(): unknown sense" << endl;
               exit(Fatal);
               return false;  // to suppress compiler warning
    }
  }

@q m v voidLhsViolated @>
@ The function |voidLhsViolated()| can be called if after variable
  elimination the left hand side of the constraint has become void
  and the right hand side has been adapted to |newRhs|. Then this function
  checks if the constraint is violated.
@q m e t @>

@q m b r @>
  \BeginReturn
  If the value |newRhs| violates the sense of the constraint, i.e,
  it is |<|/|>|/|!=| 0 and the sense of the constraint is |>=|/|<=|/|=|,
  then we return the infeasibility mode (|TooLarge| or |TooSmall|),
  otherwise we return |Feasible|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|newRhs|}{The right hand side of the constraint after the
                    elimination of the variables.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  ABA_INFEASCON::INFEAS ABA_CONSTRAINT::voidLhsViolated(double newRhs) const
@q m e f @>
  {
    switch (sense_.sense()) {
      case ABA_CSENSE::Equal:
        if(newRhs > master_->eps())  return ABA_INFEASCON::TooLarge;
        if(newRhs < -master_->eps()) return ABA_INFEASCON::TooSmall;
        else                      return ABA_INFEASCON::Feasible;
      case ABA_CSENSE::Less:
        return newRhs < -master_->eps() ? ABA_INFEASCON::TooLarge : ABA_INFEASCON::Feasible;
      case ABA_CSENSE::Greater:
        return   newRhs > master_->eps() ? ABA_INFEASCON::TooSmall : ABA_INFEASCON::Feasible;
      default: master_->err() << "ABA_CONSTRAINT::voidLhsViolated(): unknown sense" << endl;
               exit(Fatal);
               return ABA_INFEASCON::Feasible;  // to suppress compiler warning
    }
  }

@q m f printRow @>
@ The function |printRow()| writes the row format of the constraint
  associated with the variable set |var| on an output stream.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|var|}{The variables for which the row format should be written.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  void ABA_CONSTRAINT::printRow(ostream &out, 
                                ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *var)
@q m e f @>
  {
    ABA_ROW row(master_, var->number());

    genRow(var, row);

    out << row;
  }

  
@q m v distance @>
@ The function |distance()|.
@q m e t @>

  The distance of a point $\overline{x}$ and a hyperplane
  $a^T x = \beta$ can be computed in the following way:
  Let $y$ be the intersection of the hyperplane $a^T x = \beta$
  and the line defined by $\overline{x}$ and the vector $a$.
  Then the distance $d$ is the length of the vector $||\overline{x} - y||$.

@q m b r @>
  \BeginReturn
  The Euclidean distance of the vector
  |x| associated with the variable set |actVar| to the hyperplane
  induced by the constraint.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{The point for which the distance should be computed.}
  \docarg{|actVar|}{The variables associated with |x|.}
  \EndArg
@q m e a @>

@(constraint.cc@>=
@q m b f @>
  double ABA_CONSTRAINT::distance(double *x, 
                                  ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *actVar)
@q m e f @>
  {

    ABA_ROW a(master_, actVar->number());

    int nnz = genRow(actVar, a);

    double ax = 0.0;

    for (int i = 0; i < nnz; i++)
      ax += a.coeff(i) * x[a.support(i)];

    return fabs((rhs() - ax)/a.norm());

  }

@ The function |classification()| returns a pointer to the classification
  of the constraint. If no classification is available then
  we try to classify the constraint. In this case |var| must not be
  a 0-pointer. Per default |var| is 0.

  A constraint classification can only be generated if the function
  |classify()| is redefined in a derived class.

@(constraint.cc@>=
  ABA_CONCLASS *ABA_CONSTRAINT::classification(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *var)
  {
    if (conClass_ == 0 || var) {
      if (var == 0) {
        master_->err() << "ABA_CONSTRAINT::classification(): Fatal error." << endl;
        master_->err() << "Neither classification nor variable set specified." << endl;
        exit(Fatal);
      }
      conClass_ = classify(var);
    }
    return conClass_;
  }

@ The default implementation of the function |classify()| returns a 0
  pointer.

@(constraint.cc@>=
  ABA_CONCLASS *ABA_CONSTRAINT::classify(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *var)
  {
    return 0;
  }


