@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: convar.w,v 4.15 2003/08/05 13:55:14 elf Exp $ @>

@q m i ABA_CONVAR @>

@ The non-inline member functions are defined in the file {\tt convar.cc}.

@(convar.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "sub.h"
#include "convar.h"
#include "active.h"
#else
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/convar.h"
#include "abacus/active.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#include "abacus/idmap.h"
#endif

@q m f Constructor @>
@q m e t @>

@ The constructor.

@q m b a @>
    \BeginArg
    \docarg
      {|master|}
      {A pointer to the corresponding master of the optimization.}
    \docarg
      {|sub|}
      {A pointer the subproblem the constraint/variable is associated with.
       If the item is not associated with any subproblem, then this
       can also be the 0-pointer.
      }
    \docarg
      {|dynamic|}
      {If this argument is |true|, then the constraint/variable can also be
       removed again from the set of active constraints/variables after it is
       added once.
      }
    \docarg
      {|local|}
      {If |local| is |true|, then the constraint/variable is only locally
       valid.
      }
    \EndArg
@q m e a @>

@(convar.cc@>=
@q m b f @>
  ABA_CONVAR::ABA_CONVAR(ABA_MASTER *master, const ABA_SUB *sub, bool dynamic, bool local) 
@q m e f @>
    : @/
    master_(master), @/
    sub_(sub), @/
    expanded_(false), @/
    nReferences_(0), @/
    dynamic_(dynamic), @/
    nActive_(0), @/
    nLocks_(0), @/
    local_(local)
  {
#ifdef ABACUS_PARALLEL
    if (local) {
      master_->err() << "ABA_CONVAR::ABA_CONVAR(): local constraints/variables"
                     << " are not suported in the parallel version of ABACUS."
                     << endl;
      exit(Fatal);
    }
#endif
  }

@q m v Destructor @>
@q m e t @>

@ The destructor.

@(convar.cc@>=
@q m b f @>
  ABA_CONVAR::~ABA_CONVAR()
@q m e f @>
  {
#ifdef ABACUSSAFE
    if (nActive_) {
      master_->err() << "ABA_CONVAR::~ABA_CONVAR(): constraint/variable still active: ";
      master_->err() << "counter = " << nActive_ << endl;
      exit(Fatal);
    }
    
    if (nLocks_) {
      master_->err() << "ABA_CONVAR::~ABA_CONVAR(): constraint/variable has still ";
      master_->err() << nLocks_ << " locks." << endl;
      exit(Fatal);
    }
#endif    
  }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the constraint/variable from an |ABA_MESSAGE|.

  A constraint/variable can only be constructed by the message constructor,
  if the associated subproblem of the constraint/variable already exists
  in the memory of the receiving processor. This is verified using the |ABA_ID| of
  the subproblem. The |sub_| pointer of the constraint/variable is then set
  to this subproblem.
@q m e t @>

@q m b a @>
    \BeginArg
    \docarg
      {|master|}{A pointer to the corresponding master of the optimization.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(convar.cc@>=
@q m b f @>
  ABA_CONVAR::ABA_CONVAR(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    master_(master), @/
    sub_(0), @/
    expanded_(false), @/
    nReferences_(0), @/
    nActive_(0), @/
    nLocks_(0) @/
  {
    int bits = msg.unpackInt();

    assert((bits&1)==0);             // |sub_| must be 0
    dynamic_  = ((bits&2)?true:false);
    local_    = ((bits&4)?true:false);
  }


@q m f pack @>
@ The function |pack()| packs the data of the constraint/variable
  in an |ABA_MESSAGE| object. This virtual function has to be redefined
  in a derived class if additional data should be packed. In this
  case the |pack()| function of the base class must be called
  by the |pack()| function of the derived class.

  The constraint/variable must be packed and unpacked in compressed
  format. Note, that it is not required for the constraint/variable
  to be in compressed format at the time when it is packed!
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the constraint/variable
                 is packed.}
  \EndArg
@q m e a @>

@(convar.cc@>=
@q m b f @>
  void ABA_CONVAR::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    msg.pack( ((sub_&&local_)?1:0) + 2*dynamic_ + 4*local_);

    if (sub_&&local_) {
      master_->err() << "ABA_CONVAR::pack(): constraint/variable must "
                        "be globally valid." << endl;
      exit(Fatal);
    }
  }

#endif


@q m f active @>
@ The function |active()| checks if the constraint/variable is active
  in at least one active subproblem. In the parallel implementation
  this can be more than one subproblem when multithreading occurs.
  Only those subproblems are taken into account which are related to the
  pool in which the constraint/variable is stored.

@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the constraint/variable is active,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_CONVAR::active() const
@q m e f @>

@ The function |activate()| must be called if the constraint/variable
  is added to the active formulation of an active subproblem.
  This function is only called within member functions of the class
  |ABA_SUB|.

  void ABA_CONVAR::activate()
  
@ The function |deactivate()| is the counterpart to |activate()|
  and is also called within members of the class |ABA_SUB| to indicate
  that the constraint/variable does not belong any more to the active
  formulation of an active subproblem.

@(convar.cc@>=
  void ABA_CONVAR::deactivate()
  {
#ifdef ABACUSSAFE
    if (nActive_ == 0) {
      master_->err() << "ABA_CONVAR::deactivate(): constraint/variable already non-active" << endl;
      exit(Fatal);
    }
#endif
    --nActive_;
  }
  
@ The function |nReferences()| returns the number of references to the
  pool slot (|ABA_POOLSLOTREF|) storing this constraint/variable.

  We require the bookkeeping of the references in order to determine
  if a constraint/variable can be deleted without causing any harm.

  int ABA_CONVAR::nReferences() const
  
@ The function |addReference()| indicates that there is a new reference to the
  pool slot storing this constraint/variable. The function is only
  called from members of the class |ABA_POOLSLOTREF|.

  void ABA_CONVAR::addReference()

@ The function |removeReference()| is the counterpart of the function
  |addReference()|
   and indicates the removal of a reference to this constraint. It
   is only called from members of the class |ABA_POOLSLOTREF|.

@(convar.cc@>=
  void ABA_CONVAR::removeReference()
  {
    if(--nReferences_ < 0) {
      master_->err() << "ABA_CONVAR::removeReference : reference counter negative" << endl;
      exit(Fatal);
    }
  }

@ If a constraint/variable has just been separated and added to
  the buffer of currently separated constraints/variables, then
  this item should not be removed before the buffer is emptied
  at the beginning of the next iteration. Hence, we provide a
  locking mechanism for constraints/variables by the following
  three functions.
  
@ The function |locked()| returns |true| if at least one lock
  is set on the constraint/variable, otherwise it returns |false|.

  bool ABA_CONVAR::locked() const

@ The function |lock()| adds an additional lock to the constraint/variable.

  void ABA_CONVAR::lock()

@ The function |unlock()| removes one lock from the constraint/variable.

@(convar.cc@>=
  void ABA_CONVAR::unlock()
  {
#ifdef ABACUSSAFE
    if (nLocks_ == 0) {
      master_->err() << "ABA_CONVAR::unlock(): constraint/variable not locked." << endl;
      exit(Fatal);
    }
#endif

    --nLocks_;
  }
  
@ The function |deletable()| returns |true| if the constraint/variable
  can be destructed. This is per default only possible if the
  reference counter is 0 and no lock is set.
  The function is declared virtual such
  that problem specific implementations are possible.

@(convar.cc@>=
  bool ABA_CONVAR::deletable() const
  {
    if (nReferences_ || nLocks_) return false;
    else                         return true;
  }

@q m f local @>
@q m e t @>
@ The function |local()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the constraint/variable  is only locally valid,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_CONVAR::local() const
@q m e f @>

@q m f global @>
@q m e t @>
@ The function |global()|.

@q m b r @>
  \BeginReturn 
  \docarg{|true|}{If the constraint/variable is globally valid,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_CONVAR::global() const
@q m e f @>

@q m f sub @>
@q m e t @>
@ The function |sub()|.

@q m b r @>
  \BeginReturn
  A pointer to the subproblem
  associated with the constraint/variable. Note, this can also
  be the 0-pointer.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_SUB *ABA_CONVAR::sub()
@q m e f @>

@q m f sub @>
@ This version of the function |sub()| associates a new subproblem
  with the constraint/variable.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The new subproblem associated with the constraint/variable.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_CONVAR::sub(ABA_SUB *sub)
@q m e f @>

@q m v dynamic @>
@q m e t @>    
@ The function |dynamic()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the constraint/variable can be also removed from the
                  set of active constraints/var\-i\-a\-bles
                  after it has been activated,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(convar.cc@>=
@q m b f @>
  bool ABA_CONVAR::dynamic() const
@q m e f @>
  {
    return dynamic_;
  }
  
@ Constraints/Variables often have to be stored in a format different
  from the format used in the linear program. One reason is to save
  memory and the other reason is that if constraints and/or variable
  sets are dynamic, then we require a format to compute the coefficients
  of later activated variables/constraints.

  The disadvantage of such a constraint format is that the computation
  of a single constraint coefficient can be very time consuming. Often
  it cannot be done in constant time. Hence we provide a mechanism
  which converts a constraint/variable to a format enabling efficient
  computation of coefficients. The following functions provide this
  feature.

@q m f expanded @>
@q m e t @>
@ The function |expanded()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the expanded format of a constraint/variable is 
                  available,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_CONVAR::expanded() const
@q m e f @>

@q m f \_expand @>
@ The function |\_expand()| tries to generate the expanded format of
  the constraint/variable. This will be only possible if the virtual
  function |expand()| is redefined for the specific constraint/variable.
@q m e t @>

@(convar.cc@>=
@q m b f @>
  void ABA_CONVAR::_expand()
@q m e f @>
  {
    if(expanded_) {
      master_->err() << "WARNING: ABA_CONVAR::_expand(): ";
      master_->err() << "constraint already expanded" << endl;
      return;
    }
    expand();
    expanded_ = true;
  }

@q m f \_compress @>
@ The function |\_compress()| removes the expanded format of the
  constraint/variable. This will be only possible if the virtual
  function |compress()| is redefined for the specific constraint/variable.
@q m e t @>

@(convar.cc@>=
@q m b f @>
  void ABA_CONVAR::_compress()
@q m e f @>
  {
    if(!expanded_) {
      master_->err() << "WARNING: ABA_CONVAR::_compress(): ";
      master_->err() << "constraint already compressed" << endl;
      return;
    }
    compress();
    expanded_ = false;
  }

@q m v expand @>
@ The default implementation of the function |expand()| is void.
  It should be redefined in derived classes.
@q m e t @>
  
@(convar.cc@>=
@q m b f @>
  void ABA_CONVAR::expand()
@q m e f @>
  { }

@q m v compress @>
@ Also the default implementation of the function |compress()| is void.
  It should be redefined in derived classes.
@q m e t @>

@(convar.cc@>=
@q m b f @>
  void ABA_CONVAR::compress()
@q m e f @>
  { }


@q m v print @>
@ The function writes the constraint/variable on the stream |out|.
  This function is used since the output operator cannot be declared
  virtual. The default implementation writes
  |"ABA_CONVAR::print() is only a dummy."| on the stream |out|.
  We do not declare this function pure virtual since it is not really
  required, mainly only for debugging. In this case a constraint/variable
  specific redefinition is strongly recommended.

  Normally, the implementation |out << *this| should be sufficient.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \EndArg
@q m e a @>

@(convar.cc@>=
@q m b f @>
  void ABA_CONVAR::print(ostream &out)
@q m e f @>
  {
    out << "ABA_CONVAR::print() is only a dummy." << endl;
  }

@q m v hashKey @>
@ The function |hashKey()| should provide a key for the
  constraint/variable that can be used to insert it into a hash table.
  As usual for hashing, it is not required that any two items have
  different keys.

  This function is required if the constraint/variable is stored in a
  pool of the class |ABA_NONDUPLPOOL|.
  
  The default implementation shows a warning and calls |exit()|.
  This function is not a pure virtual function because in the default
  version of \ABACUS\ it is not required.
@q m e t @>

  We do not use |double| as result type because typical problems in
  floating point arithmetic might give slightly different hash keys
  for two constraints that are equal from a mathematical point of
  view.

  The redundant return statement is required to suppress compiler warnings.

@q m b r @>
  \BeginReturn
  An integer providing a hash key for the constraint/variable.
  \EndReturn
@q m e r @>

@(convar.cc@>=
@q m b f @>
  unsigned ABA_CONVAR::hashKey()
@q m e f @>
  {
    master_->err() << "ABA_CONVAR::hashKey() must be defined in derived ";
    master_->err() << "class." << endl;
    exit(Fatal);
    return 0;
  }

@q m v name @>
@ The function |name()| should return the name of the
  constraint/variable. This function is required to emulate a simple
  run time type information (RTTI) that is still missing in \GPLUSPLUS/.
  This function will be removed as soon as RTTI is supported sufficiently.

  A user must take care that for each redefined version of this
  function in a derived class a unique name is returned. Otherwise
  fatal run time errors can occur. Therefore, we recommend to return
  always the name of the class.

  This function is required if the constraint/variable is stored in a
  pool of the class |ABA_NONDUPLPOOL|.
  
  The default implementation shows a warning and calls |exit()|.
  This function is not a pure virtual function because in the default
  version of \ABACUS\ it is not required.
@q m e t @>

  The redundant return statement is required to suppress compiler warnings.

@q m b r @>
  \BeginReturn
  The name of the constraint/variable.
  \EndReturn
@q m e r @>

@(convar.cc@>=
@q m b f @>
  const char *ABA_CONVAR::name()
@q m e f @>
  {
    master_->err() << "ABA_CONVAR::name() must be defined in derived ";
    master_->err() << "class." << endl;
    exit(Fatal);
    return 0;
  }

@q m v rank @>

@ The function should return a rank associated with the constraint/variable.
  The default implementation returns 0.
@q m e t @>

@q m b r @>
  \BeginReturn
  The rank of the constraint/variable.
  \EndReturn
@q m e r @>

@(convar.cc@>=
@q m b f @>
  double  ABA_CONVAR::rank()
@q m e f @>
  {
    return 0;
  }

@q m v equal @>
@ The function |equal()| should compare if the constraint/variable
  is identical (in a mathematical sense) with the constraint/variable
  |cv|. Using RTTI or its emulation provided by the function |name()|
  it is sufficient to implement this functions for
  constraints/variables of the same type.

  This function is required if the constraint/variable is stored in a
  pool of the class |ABA_NONDUPLPOOL|.
  
  The default implementation shows a warning and calls |exit()|.
  This function is not a pure virtual function because in the default
  version of \ABACUS\ it is not required.
@q m e t @>

  The redundant return statement is required to suppress compiler warnings.

@q m b r @>
  \BeginReturn 
  \docarg{|true|}{If the constraint/variable represented by this object
                  represents the same item as the constraint/variable
                  |cv|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|cv|}{The constraint/variable that should be compared with
                this object.}
  \EndArg
@q m e a @>

@(convar.cc@>=
@q m b f @>
  bool ABA_CONVAR::equal(ABA_CONVAR *cv)
@q m e f @>
  {
    master_->err() << "ABA_CONVAR::equal() must be defined in derived ";
    master_->err() << "class." << endl;
    exit(Fatal);
    return false;
  }

