@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: cplexif.w,v 4.37 2003/08/27 12:45:58 elf Exp $ @>

@q m i ABA_CPLEXIF @>

@ The member functions of the class |ABA_CPLEXIF| are implemented in the
  file |cplexif.cc|.

@(cplexif.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "lpmastercplex.h"
#include "cplexif.h"
#include "column.h"
#include "row.h"
#else
#include "abacus/master.h"
#include "abacus/lpmastercplex.h"
#include "abacus/lpsubcplex.h"
#include "abacus/cplexif.h"
#include "abacus/column.h"
#include "abacus/row.h"
#endif

#include <string.h>
#include <stdio.h>
#include <limits.h>

extern "C" {

#ifdef ABACUS_LP_CPLEX40

#include "cplex.h"

#ifdef ABACUS_SYS_WINNT
#include "calldll.h"
#endif

#else

#include "cpxdefs.inc"

#ifdef ABACUS_SYS_WINNT
#include "calldll.inc"
#endif

#endif
}

#ifdef ABACUS_SYS_POWERPLUS
extern "C" {
#include "/home/user/christof/abacus/include/math.h"
}
#else
#include <math.h>
#endif

extern "C" {
#include <unistd.h>
}

@<implementation of the members of |ABA_CPLEXIF|@>;


@ The initialization of the static members.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::nObjects_ = 0;
#ifdef ABACUS_LP_CPLEX40
  int ABA_CPLEXIF::nLivingObjects_ = 0;
  struct cpxenv *ABA_CPLEXIF::cplexEnv_ = 0;
#endif

@q m f Constructor @>
@ This constructor does not initialize the problem data of the linear
     program. It must be loaded later with the function |initialize()|.
@q m e t @>

@q m b a @>
     \BeginArg
     \docarg{|master|}{A pointer to the corresponding master of the
                     optimization.}
     \docarg{|nnzSurplus|}{Additional space for nonzero elements in the
                            constraint matrix of Cplex. Its default value
                            is 0.}
     \docarg{|relativeSurplus|}{If this argument is |true|, then the
                                additional number of nonzeros is relative
                                in percent to the initial number of nonzeros,
                                otherwise it is interpreted as an absolute
                                value. The default value is |true|.}
      \EndArg
@q m e a @>

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
  ABA_CPLEXIF::ABA_CPLEXIF(ABA_MASTER *master, 
                           double nnzSurplus, 
                           bool relativeSurplus) 
@q m e f @>
  : @/
    ABA_LP (master), @/
    nnzSurplus_(nnzSurplus), @/
    relativeSurplus_(relativeSurplus), @/
    cplexLp_(0), @/
    problemName_(0)
  {
    lpMasterCplex_ = master->lpMasterCplex();
    @<increment the object counters@>;
    @<open the Cplex environment@>;
    @<test the choice of ``infinity''@>;
    @<set the default pricing strategies@>;
    @<set the output level@>;
    @<suppress the output of the basis file@>;
  }

@ This counter provides IDs for the objects of the class |ABA_CPLEXIF| generated
  within one program. This is required for the generation of a unique
  name required by Cplex.
  
@<increment the object counters@>=
  ++nObjects_;
#ifdef ABACUS_LP_CPLEX40
  ++nLivingObjects_;
#endif

@ Starting with Cplex 4.0 an environment for each Cplex thread has to be
  opened.

@ @<open the Cplex environment@>=
#ifdef ABACUS_LP_CPLEX40
  if (nLivingObjects_ == 1) {
    int status;
    
    cplexEnv_ = CPXopenCPLEX(&status);

    if (status) {
      master_->err() << "ABA_CPLEXIF::ABA_CPLEXIF(): opening Cplex environment failed." << endl;
      master_->err() << "Return code of CPXopenCPLEX is " << status << "." << endl;
      exit(Fatal);
    }
  }
#endif

@ Cplex treats all values with absolute value greater than the
  |INFBOUND| (see {\tt cpxdefs.inc}) as infinity. If the value for infinity
  within this
  framework is smaller than |INFBOUND| we stop with an error message
  since unbounded variables might be treated differently.
  
@<test the choice of ``infinity''@>=
  if (master_->infinity() < CPX_INFBOUND) {
    master_->err() << "ABACUS infinity value = " << master_->infinity() << " is ";
    master_->err() << "less than Cplex infinity value = " << CPX_INFBOUND << "." <<  endl;
    exit(Fatal);
  }
  
@ @<set the default pricing strategies@>=
#ifdef TTT
  ::CPXsetintparam(cplexEnv_, CPX_PARAM_FASTMIP, 0);
#endif
  @<set the default primal pricing strategy@>;
  @<set the default dual pricing strategy@>;
  
@ @<set the default primal pricing strategy@>=
#ifdef ABACUS_LP_CPLEX40
  if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_PARTIAL")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_PPRIIND, CPX_PPRIIND_PARTIAL);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_AUTO")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_PPRIIND, CPX_PPRIIND_AUTO);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_DEVEX")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_PPRIIND, CPX_PPRIIND_DEVEX);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_STEEP") 
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_PPRIIND, CPX_PPRIIND_STEEP);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_STEEPQSTART")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_PPRIIND, CPX_PPRIIND_STEEPQSTART);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_FULL") 
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_PPRIIND, CPX_PPRIIND_FULL);
  else {
    master_->err() << "Unknown primal pricing strategy for Cplex: ";
    master_->err() << lpMasterCplex_->cplexPrimalPricing() << "." << endl;
    exit(Fatal);
  }
#else
  if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_PARTIAL")
    setppriind(CPX_PPRIIND_PARTIAL);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_AUTO")
    setppriind(CPX_PPRIIND_AUTO);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_DEVEX")
    setppriind(CPX_PPRIIND_DEVEX);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_STEEP") 
    setppriind(CPX_PPRIIND_STEEP);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_STEEPQSTART")
    setppriind(CPX_PPRIIND_STEEPQSTART);
  else if (lpMasterCplex_->cplexPrimalPricing() == "CPX_PPRIIND_FULL") 
    setppriind(CPX_PPRIIND_FULL);
  else {
    master_->err() << "Unknown primal pricing strategy for Cplex: ";
    master_->err() << lpMasterCplex_->cplexPrimalPricing() << "." << endl;
    exit(Fatal);
  }
#endif

@ @<set the default dual pricing strategy@>=
#ifdef ABACUS_LP_CPLEX40
  if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_AUTO")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_DPRIIND, CPX_DPRIIND_AUTO);
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_FULL")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_DPRIIND, CPX_DPRIIND_FULL);
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_STEEP") 
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_DPRIIND, CPX_DPRIIND_STEEP);
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_FULLSTEEP")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_DPRIIND, CPX_DPRIIND_FULLSTEEP);
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_STEEPQSTART")
    ::CPXsetintparam(cplexEnv_, CPX_PARAM_DPRIIND, CPX_DPRIIND_STEEPQSTART);
  else {
    master_->err() << "Unknown dual pricing strategy for Cplex: ";
    master_->err() << lpMasterCplex_->cplexPrimalPricing() << "." << endl;
    exit(Fatal);
  }
#else
  if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_AUTO") {
#ifndef ABACUS_LP_CPLEX30
    master_->err() << "Cplex dual simplex pricing strategy ";
    master_->err() << "CPX_DPRIIND_AUTO not available before Cplex 3.0." << endl;
    exit(Fatal);
#else    
    setdpriind(CPX_DPRIIND_AUTO);
#endif    
  }
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_FULL")
    setdpriind(CPX_DPRIIND_FULL);
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_STEEP") 
    setdpriind(CPX_DPRIIND_STEEP);
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_FULLSTEEP")
    setdpriind(CPX_DPRIIND_FULLSTEEP);
  else if (lpMasterCplex_->cplexDualPricing() == "CPX_DPRIIND_STEEPQSTART")
    setdpriind(CPX_DPRIIND_STEEPQSTART);
  else {
    master_->err() << "Unknown dual pricing strategy for Cplex: ";
    master_->err() << lpMasterCplex_->cplexPrimalPricing() << "." << endl;
    exit(Fatal);
  }
#endif

@ @<set the output level@>=
  iterationInformation(lpMasterCplex_->cplexOutputLevel());

@ We set the interval for writing the basis matrix as large as
  possible. The value |INT_MAX| has not been accepted by Cplex.
  We did not find a documentation on the maximal allowed value.

@<suppress the output of the basis file@>=
  int status;

#ifdef ABACUS_LP_CPLEX40
  status =   ::CPXsetintparam(cplexEnv_, CPX_PARAM_BASINTERVAL, INT_MAX/2);
#else
  int tooBig;
  int tooSmall;

  status = setbasintv(INT_MAX/2, &tooBig, &tooSmall);
#endif

  if (status) {
    master_->err() << "ABA_CPLEXIF::ABA_CPLEXIF: setting the interval for writing ";
    master_->err() << "the basis file failed." << endl;
    exit(Fatal);
  }

@q m f Constructor @>
@ A constructor with initialization.
@q m e t @>

@q m b a @>
      \BeginArg
      \docarg{|master|}{A pointer to the corresponding master of the optimization.}
      \docarg{|sense|}{The sense of the objective function.}
      \docarg{|nCol|}{The number of columns (variables).}
      \docarg{|maxCol|}{The maximal number of columns.}
      \docarg{|nRow|}{The number of rows.}
      \docarg{|maxRow|}{The maximal number of rows.}
      \docarg{|obj|}{An array with the objective function coefficients.}
      \docarg{|lb|}{An array with the lower bounds of the columns.}
      \docarg{|ub|}{An array with the upper bounds of the columns.}
      \docarg{|rows|}{An array storing the rows of the problem.}
      \docarg{|nnzSurplus|}{Additional space for nonzero elements in the
                            constraint matrix of Cplex.
                            Its default value is 0.}
     \docarg{|relativeSurplus|}{If this argument is |true|, then the
                                additional number of nonzeros is relative
                                in percent to the initial number of nonzeros,
                                otherwise it is interpreted as an absolute
                                value. The default value is |true|.}
      \EndArg
@q m e a @>

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
  ABA_CPLEXIF::ABA_CPLEXIF(ABA_MASTER *master, 
                           ABA_OPTSENSE sense, 
                           int nRow, 
                           int maxRow, 
                           int nCol, 
                           int maxCol,
                           ABA_ARRAY<double> &obj, 
                           ABA_ARRAY<double> &lb, 
                           ABA_ARRAY<double> &ub,
                           ABA_ARRAY<ABA_ROW*> &rows,
                           double nnzSurplus, 
                           bool relativeSurplus) 
@q m e f @>
: @/
    ABA_LP (master), @/
    nnzSurplus_(nnzSurplus), @/
    relativeSurplus_(relativeSurplus)
  {
    lpMasterCplex_ = master->lpMasterCplex();
    @<increment the object counters@>;
    @<test the choice of ``infinity''@>;
    @<set the default pricing strategies@>;
    @<set the output level@>;
     _initialize(sense, nRow, maxRow, nCol, maxCol, obj, lb, ub, rows);
  }


@q m v Destructor @>
@q m e t @>
@ The destructor.

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
  ABA_CPLEXIF::~ABA_CPLEXIF()
@q m e f @>
  {
    lpSolverTime_.start();
    
    if (cplexLp_) CPXfreeprob(cplexEnv_, &cplexLp_);  // CPLEX routine to free internal memory
    
#ifdef ABACUS_LP_CPLEX40
    if (--nLivingObjects_ == 0) {
      int status = CPXcloseCPLEX(&cplexEnv_);
      if (status) {
        master_->err() << "ABA_CPLEXIF::~ABA_CPLEXIF(): closing environment failed." << endl;
        master_->err() << "Return code of CPXcloseCPLEX() is " << status << "." << endl;
      }
    }
#else
    freeprob(&cplexLp_);  // CPLEX routine to free internal memory
#endif

    lpSolverTime_.stop();

    if (problemName_) {
        delete [] problemName_;
        delete [] xVal_;
        delete [] barXVal_;
        delete [] reco_;
        delete [] yVal_;
        delete [] slack_;
        delete [] cStat_;
        delete [] rStat_;
    }
  }

@ The function |_initialize()| implements the corresponding pure virtual
  function of the base class |LP| and loads the linear program
  defined by the following arguments to Cplex.
      \BeginArg
      \docarg{|sense|}{The sense of the objective function.}
      \docarg{|nCol|}{The number of columns (variables).}
      \docarg{|maxCol|}{The maximal number of columns.}
      \docarg{|nRow|}{The number of rows.}
      \docarg{|maxRow|}{The maximal number of rows.}
      \docarg{|obj|}{An array with the objective function coefficients.}
      \docarg{|lb|}{An array with the lower bounds of the columns.}
      \docarg{|ub|}{An array with the upper bounds of the columns.}
      \docarg{|rows|}{An array storing the rows of the problem.}
      \EndArg

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_initialize(ABA_OPTSENSE sense, int nRow, int maxRow,
                          int nCol, int maxCol,
                          ABA_ARRAY<double> &obj, ABA_ARRAY<double> &lBound,
                          ABA_ARRAY<double> &uBound, ABA_ARRAY<ABA_ROW*> &rows)
  {
    if (lpMasterCplex_->cplexHoldEnvironment_ &&
        lpMasterCplex_->cplexLicenseDummy_==0)
      lpMasterCplex_->cplexLicenseDummy_ = new ABA_CPLEXIF(master_);

    @<allocate the memory for CPLEX@>;
    @<copy the data to the CPLEX data structures@>;
    @<load the CPLEX data@>;

    delete [] objx;   
    delete [] rhsx;   
    delete [] senx;   
    delete [] matbeg; 
    delete [] matcnt; 
    delete [] matind; 
    delete [] matval; 
    delete [] bdl;
    delete [] bdu;
 
  }

@ Before we can allocate space for the constraint matrix, we have
  to determine its number of nonzero elements. We increase this
  number by |nnzSurplus_|\% or by |nnzSurplus_|
  either if this number should be interpreted as relative or absolute
  to provide additional space for adding
  rows and columns without performing a reallocation.

@<allocate the memory for CPLEX@>=
  marsz_ = maxRow;
  macsz_ = maxCol;

  int matsz = 0;

  for (int i = 0; i < nRow; i++)
    matsz += rows[i]->nnz();

  @<make sure that there is space for the dummy row@>;

  if (relativeSurplus_)
    matsz += ((int) (matsz * nnzSurplus_/100.0)) + 1;
  else
    matsz += (int) nnzSurplus_;

  problemName_ = new char[16];

  double* objx   = new double[macsz_];
  double* rhsx   = new double[marsz_];
  char*   senx   = new char[marsz_];
  int*    matbeg = new int[macsz_];
  int*    matcnt = new int[macsz_];
  int*    matind = new int[matsz];
  double* matval = new double[matsz];
  double* bdl    = new double[macsz_];
  double* bdu    = new double[macsz_];

  xVal_     = new double[macsz_];
  barXVal_  = new double[macsz_];
  reco_     = new double[macsz_];
  yVal_     = new double[marsz_];
  slack_    = new double[marsz_];
  cStat_    = new int[macsz_];
  rStat_    = new int[marsz_];

@ Since Cplex cannot optimize without constraints, we have to insert a
  dummy row. In this case we have to provide space for one row having
  one nonzero coefficient.

@<make sure that there is space for the dummy row@>=
  if (nRow == 0) {
    if (marsz_ == 0) marsz_ = 1;
    if (matsz  == 0) matsz  = 1;
  }

@ @<copy the data to the CPLEX data structures@>=
  @<initialize the name of the linear program@>;
  @<initialize bounds, objective function, right hand side, and the sense@>;
  @<copy the matrix@>;

@ Cplex requires a unique problem name for each linear program which must have
  at most 16 characters. Our name starts with the string |"aba"| followed
  by a dot, the process identification number (PID), and the number of the
  object of the class |CPLEX|. We suppose that the PID has
  at most 7 digits, the dots and the characters require another 5 digits,
  and one character has to be reserved for the terminating |'\n'|, then
  three digits can be used for the number of the instance.
  
@<initialize the name of the linear program@>=
  sprintf(problemName_, "%s.%ld.%d", "aba", getpid() % 10000000, nObjects_ % 1000);

@ @<initialize bounds, objective function, right hand side, and the sense@>=
  for(int i = 0; i < nCol; i++) {
    bdl[i]  = lBound[i];
    bdu[i]  = uBound[i];
    objx[i] = obj[i];
  }
  
  for(int c = 0; c < nRow; c++) {
    rhsx[c] = rows[c]->rhs();
    senx[c] = csense2cplex(rows[c]->sense());
  }

@ The constraint matrix has to be passed to Cplex in a column oriented
  format, whereas we assume---motivated by cutting plane algorithms---that
  the matrix is represented in a row oriented format.
  
@<copy the matrix@>=
  @<compute the columns of the constraint matrix@>;
  @<insert the columns into the constraint matrix of Cplex@>;
  
@ Afterwards we determine the number of nonzero entries for each column with
  the function |colsNnz()|, allocate sufficient memory, and finally compute
  the constraint matrix in the function |rows2cols()|.
  
@<compute the columns of the constraint matrix@>=
  ABA_ARRAY<int>      colSize(master_, nCol);
  ABA_ARRAY<ABA_SPARVEC*> cols(master_, nCol);

  colsNnz(nRow, rows, colSize);

  for (int i = 0; i < nCol; i++)
    cols[i] = new ABA_SPARVEC(master_, colSize[i]);
  
  rows2cols(nRow, rows, cols);
  
@ @<insert the columns into the constraint matrix of Cplex@>=
  ABA_SPARVEC *column;   // pointer to a column of the matrix
  int      cur = 0;  // current position when filling the CPLEX matrix

#ifdef ABACUS_NO_FOR_SCOPE  
  for(i = 0; i < nCol; i++) {
#else
  for(int i = 0; i < nCol; i++) {
#endif
    column      = cols[i];
    matcnt[i] = column->nnz();
    matbeg[i] = cur;
    
    for (int r = 0; r < matcnt[i]; r++) {
      matind[cur] = column->support(r);
      matval[cur] = column->coeff(r);
      ++cur;
    }
    delete column;
  }

@ @<load the CPLEX data@>=
  @<insert a dummy row for problems without rows@>;
  @<load the initial linear program to Cplex@>;
  @<remove the dummy row again@>;
  
@ For combinatorial optimization problems often the initially solved
  LP-relaxation is a problem having no side constraints, but bounded
  variables like $0 \le x_i \le 1$. Unfortunately such linear programs
  are not accepted by Cplex. In order to cheat this software we add
  a dummy row, call the function |loadprob()| and immediately remove
  this row afterwards. With this trick still the LP cannot be optimized
  by Cplex, however we can access the problem data and add
  constraints later. The optimization will be done explicitly by the
  function |noConOpti()|.

  This dummy row is of the form $x_0 \ge 0$.
  
@<insert a dummy row for problems without rows@>=
  bool noRows;
  if (nRow == 0) {
    nRow       =  1;
    rhsx[0]   = 0.0;
    senx[0]   = 'G';
    matcnt[0] = 1;
    matind[0] = 0;
    matval[0] = 1.0;
    noRows     = true;
  }
  else
    noRows = false;
    
@ Most parameters of the CPLEX function |loadprob()| are obvious from
  the declarations of members of the class |CPLEX| or its base class
  |LP|. All parameters with value |0| or |NULL| are optional parameters
  of CPLEX which we do not need.
  
@<load the initial linear program to Cplex@>=

 int status=0; 
 if (!cplexLp_) {
   lpSolverTime_.start();
   cplexLp_ = CPXcreateprob (cplexEnv_, &status , problemName_ );
   lpSolverTime_.stop();

   if (status) {
     master_->err() << "ABA_CPLEXIF::_initialize(): CPXcreateprob() failed" << endl;
     exit(Fatal);
   }
 }


lpSolverTime_.start();
status = CPXcopylp ( cplexEnv_, 
		     cplexLp_, 
		     nCol, 
                     nRow, 
		     optSense2cplex(sense), 
		     objx, 
		     rhsx, 
		     senx, 
		     matbeg,
                     matcnt, 
		     matind,
		     matval, 
		     bdl, 
		     bdu, 
		     NULL);
lpSolverTime_.stop();

if (status) {
    master_->err() << "ABA_CPLEXIF::_initialize(): CPXcopylp() failed" << endl;
    master_->err() << "  either you made an error with the arguments or" << endl;
    master_->err() << "  the license manager of your Cplex installation" << endl;
    master_->err() << "  makes trouble." << endl;
    exit(Fatal);
  }

@ @<remove the dummy row again@>=
  if (noRows) {
    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    int status = CPXdelrows(cplexEnv_, cplexLp_, 0, 0);
#else
    int status = delrows(cplexLp_, 0, 0);
#endif
    lpSolverTime_.stop();
    
    if (status) {
      master_->err() << "Deleting artificial row of Cplex failed." << endl;
      exit (Fatal);
    }
    nRow = 0;
  }

@ The function |_loadBasis()| loads a basis for Cplex.
  \BeginArg
  \docarg{|lpVarStat|}{An array storing the status of the columns.}
  \docarg{|lpVarStat|}{An array storing the status of the slack variables.}

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_loadBasis(ABA_ARRAY<ABA_LPVARSTAT::STATUS> &lpVarStat,
                           ABA_ARRAY<ABA_SLACKSTAT::STATUS> &slackStat)
  {
    int nCol = _nCol();
    for (int i = 0; i < nCol; i++)
      cStat_[i] = lpVarStat2cplex(lpVarStat[i]);

    int nRow = _nRow();
    for (int i = 0; i < nRow ; i++)
      rStat_[i] = slackStat2cplex(slackStat[i]);

    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX50
    int status= CPXcopybase(cplexEnv_,cplexLp_,cStat_,rStat_);
#elif ABACUS_LP_CPLEX40
    int status = CPXloadbase(cplexEnv_, cplexLp_, cStat_, rStat_);
#else
    int status = loadbase(cplexLp_, cStat_, rStat_);
#endif
    lpSolverTime_.stop();

    if (status)
      master_->err() << "Warning: ABA_CPLEXIF::loadBasis() call of loadbase failed" << endl;
  }    

@ The function |_getInfeas()| be can be called if the last linear program
  has been solved with the dual simplex method and is infeasible.
  In this case it computes the infeasible basic variable or constraint
  and the corresponding row |bInvRow| of the basis inverse.
  Either |infeasRow| or |infeasCol| is nonnegative. Then this number
  refers to an infeasible variable or slack variable, respectively.
  The function returns 0 if it is successful, 1 otherwise.

  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_getInfeas(int &infeasRow, int &infeasCol, double *bInvRow)
  {
    @<find the variable causing the infeasibility@>;
    @<find the corresponding basis index@>;
    @<determine the associated row of the basis inverse@>;
  }

@ @<find the variable causing the infeasibility@>=
  int status;

#ifdef ABACUS_LP_CPLEX40
  status = CPXgetijdiv (cplexEnv_, cplexLp_, &infeasRow, &infeasCol);
#else
  status = getijdiv (cplexLp_, &infeasRow, &infeasCol);
#endif
  if (status) {
    master_->err() << "ABA_CPLEXIF::_getInfeas(): getijdiv() failed" << endl;
    exit (Fatal);
  }

@ @<find the corresponding basis index@>=
  int basisIndex;

#ifdef ABACUS_LP_CPLEX40
  status = CPXgetijrow(cplexEnv_, cplexLp_, infeasRow, infeasCol, &basisIndex);
#else
  status = getijrow(cplexLp_, infeasRow, infeasCol, &basisIndex);
#endif

  if (status) {
    master_->err() << "ABA_CPLEXIF::_getInfeas(): getijrow() failed" << endl;
    exit (Fatal);
  }

@ @<determine the associated row of the basis inverse@>=
#ifdef ABACUS_LP_CPLEX40
  status = CPXbinvrow(cplexEnv_, cplexLp_, basisIndex, bInvRow);
#else
  status = binvrow(cplexLp_, basisIndex, bInvRow);
#endif
  if (status) {
    master_->err() << "ABA_CPLEXIF::_getInfeas(): binvrow() failed" << endl;
    exit (Fatal);
  }

  return 0;

@ The function |_remRows()| removes the rows listed in |ind|.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_remRows(ABA_BUFFER<int> &ind)
  {
    int maxRow = _maxRow();

    int *marked = new int[maxRow];
    for (int i = 0; i < maxRow; i++) marked[i] = 0;

    const int n = ind.number();
#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < n; i++) marked[ind[i]] = 1;
#else
    for (int i = 0; i < n; i++) marked[ind[i]] = 1;
#endif

    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    int status = CPXdelsetrows(cplexEnv_, cplexLp_, marked);
#else
    int status = delsetrows(cplexLp_, marked);
#endif

    lpSolverTime_.stop();

    if (status) {
      master_->err() << "CPLEX: cplex function delsetrows failed" << endl;
      exit(Fatal);
    }

    delete [] marked;
  }
  
@ The function |_addRows()| adds the |rows| to the linear program.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_addRows(ABA_BUFFER<ABA_ROW*> &rows)
  {
    @<|ABA_CPLEXIF::_addRows()|: local variables@>;
    @<|ABA_CPLEXIF::_addRows()|: count number of nonzeros and allocate memory@>;
    @<fill up the data structure for adding rows in CPLEX@>;
    @<has CPLEX enough memory for adding rows?@>;
    @<add the rows to CPLEX@>;
    @<|ABA_CPLEXIF::_addRows()| free allocated memory@>;
  }
  
@ @<|ABA_CPLEXIF::_addRows()|: local variables@>=
    double    *rhs;      // right hand side of added rows
    char      *sense;    // sense of added rows
    int       *rmatbeg;  // first index of each added row in sparse format
    int       *rmatind;  // support of added rows
    double    *rmatval;  // coefficients of added rows
    int        r,i;      // loop indices
    int        status;   // return value of Cplex callable library functions

    int nzcnt = 0;              // total number of nonzeros of all added rows

@ @<|ABA_CPLEXIF::_addRows()|: count number of nonzeros and allocate memory@>=
    const int rowsNumber = rows.number();

    for(r = 0; r < rowsNumber; r++) nzcnt += rows[r]->nnz();
    
    rhs     = new double[rowsNumber];
    sense   = new char[rowsNumber];
    rmatbeg = new int[rowsNumber];
    rmatind = new int[nzcnt];
    rmatval = new double[nzcnt];

@ @<|ABA_CPLEXIF::_addRows()| free allocated memory@>=
    delete [] rhs;
    delete [] sense;
    delete [] rmatbeg;
    delete [] rmatind;
    delete [] rmatval;

@ @<fill up the data structure for adding rows in CPLEX@>=
    nzcnt = 0;
    for(r = 0; r < rowsNumber; r++) {
      rmatbeg[r] = nzcnt;
      const int rNnz = rows[r]->nnz();
      for(i = 0; i < rNnz; i++) {
        rmatind[nzcnt] = rows[r]->support(i);
        rmatval[nzcnt] = rows[r]->coeff(i);
        nzcnt++;
      }
      rhs[r]   = rows[r]->rhs();
      sense[r] = csense2cplex(rows[r]->sense());
    }  

@ For Cplex 2.2 a reallocation is performed if |addrows()| fails.

@<has CPLEX enough memory for adding rows?@>=
// WATCH OUT
if ( marsz_ < CPXgetnumrows(cplexEnv_, cplexLp_) + rows.number()) {
  int newMarSz = (int) (1.1*CPXgetnumrows(cplexEnv_, cplexLp_) + rows.number()) + 1;
  _rowRealloc(newMarSz);
}  


@ For Cplex 3.0 and CplexBarrier a necessary reallocation has been
  performed already earlier. Hence, if the function fails, we have
  made something wrong.
  
@<add the rows to CPLEX@>=
  lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
  status = CPXaddrows(cplexEnv_, cplexLp_, 0, rows.number(), nzcnt, rhs, sense,
                      rmatbeg, rmatind, rmatval, NULL, NULL);
#else
  status = ::addrows(cplexLp_, 0, rows.number(), nzcnt, rhs, sense,
                     rmatbeg, rmatind, rmatval, NULL, NULL);
#endif
  lpSolverTime_.stop();


  if(status) {
    master_->out() << "ABA_CPLEXIF::_addRows() failed" << endl;
    exit (Fatal);
  }


@ The function |_remCols()| removes the columns listed in |vars|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_remCols(ABA_BUFFER<int> &vars)
  {
    int max = maxCol();
    
    int *remove = new int[max];
    for (int i = 0; i  < max; i++) remove[i] = 0;

    const int varsNumber = vars.number();
#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < varsNumber; i++) remove[vars[i]] = 1;
#else
    for (int i = 0; i < varsNumber; i++) remove[vars[i]] = 1;
#endif
    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    int status = CPXdelsetcols(cplexEnv_, cplexLp_, remove);
#else
    int status = delsetcols(cplexLp_, remove);
#endif

    lpSolverTime_.stop();

    if (status) {
      master_->err() << "CPLEX: cplex function delsetcols() failed" << endl;
      exit(Fatal);
    }

    delete [] remove;
  }
  
@ The function |_addCols()| adds the columns |newCols| to the linear program.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_addCols(ABA_BUFFER<ABA_COLUMN*> &newCols)
  {
    @<|ABA_CPLEXIF::_addCols()|: local variables@>;
    @<|ABA_CPLEXIF::_addCols()|: count nonzeros and allocate memory@>;
    @<fill up internal CPLEX data structure for adding columns@>;
    @<has CPLEX enough memory for adding columns?@>;
    @<add the new columns to CPLEX@>;
    @<|ABA_CPLEXIF::_addCols()|: free allocated memory@>;
  }
    
@ @<|ABA_CPLEXIF::_addCols()|: local variables@>=
    int     nzcnt;    // number of nonzero elements in added columns
    double *objx;     // objective function coefficients of added columns
    int    *cmatbeg;  // first index of added column in sparse format
    int    *cmatind;  // support of added columns
    double *cmatval;  // nonzero coefficients of added columns
    double *bdl;      // lower bounds on added columns
    double *bdu;      // upper bounds on added columns
    int     status;   // return status of Cplex callable library functions
    int i,j;          // loop indices
    
    int nAdd = newCols.number();  // number of added columns

@ For Cplex 2.2 the reallocation is performed if the addition of columns fails.

@<has CPLEX enough memory for adding columns?@>=
// WATCH OUT 
if ( macsz_ < CPXgetnumcols(cplexEnv_, cplexLp_) + newCols.number()) {
  int newMacSz = (int) (1.1*CPXgetnumcols(cplexEnv_, cplexLp_) + newCols.number()) + 1;
  _colRealloc(newMacSz);
}


@ @<|ABA_CPLEXIF::_addCols()|: count nonzeros and allocate memory@>=
    nzcnt = 0;
    for (i = 0; i < nAdd; i++) nzcnt += newCols[i]->nnz();

    objx    = new double[nAdd];
    cmatbeg = new int[nAdd];
    cmatind = new int[nzcnt];
    cmatval = new double[nzcnt];
    bdl     = new double[nAdd];
    bdu     = new double[nAdd];

@ @<|ABA_CPLEXIF::_addCols()|: free allocated memory@>=
    delete [] objx;
    delete [] cmatbeg;
    delete [] cmatind;
    delete [] cmatval;
    delete [] bdl;
    delete [] bdu;

@ @<fill up internal CPLEX data structure for adding columns@>=
    nzcnt = 0;
    ABA_COLUMN *col;
    for (i = 0; i < nAdd; i++) {
      col     = newCols[i];
      objx[i] = col->obj();
      bdl[i]  = col->lBound();
      bdu[i]  = col->uBound();
      cmatbeg[i] = nzcnt;
      const int colNnz = col->nnz();

      for (j = 0; j < colNnz; j++) {
        cmatind[nzcnt] = col->support(j);
        cmatval[nzcnt] = col->coeff(j);
        ++nzcnt;
      }
    }

@ @<add the new columns to CPLEX@>=
  lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
  status = CPXaddcols(cplexEnv_, cplexLp_, nAdd, nzcnt, objx, cmatbeg, cmatind,
                      cmatval, bdl, bdu, NULL);
#else
  status = ::addcols(cplexLp_, nAdd, nzcnt, objx, cmatbeg, cmatind,
                     cmatval, bdl, bdu, NULL);
#endif

  lpSolverTime_.stop();

  if(status) {
    master_->out() << "ABA_CPLEXIF::_addCols() failed" << endl;
    exit (Fatal);
  }



@ The function |changeRhs()| sets the right hand side of the linear
  program to |newRhs|. This array must have at least length of the
  number of rows.
  This function implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_changeRhs(ABA_ARRAY<double> &newRhs)
  {
    int status;
    
    lpSolverTime_.start();

    int nRow = _nRow();

    for (int i = 0; i < nRow; i++) {
#ifdef ABACUS_LP_CPLEX40
      status = CPXchgcoef(cplexEnv_, cplexLp_, i, -1, newRhs[i]);
#else
      status = chgcoef(cplexLp_, i, -1, newRhs[i]);
#endif      
      if (status) {
        master_->err() << "ABA_CPLEXIF::changeRhs(): chgcoeff() return error" << endl;
        exit(Fatal);
      }
    }

    lpSolverTime_.stop();

  }
  
@ The function |_changeLBound()| sets the lower bound of column |i| to
  |newLb|.
  It implements the pure virtual function of the base class |LP|.
                      
@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_changeLBound(int i, double newLb)
  {
    int    status;
    int    index[1] = {i};
    char   lu[1]   = {'L'};
    double bd[1]   = {newLb};

    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    status = CPXchgbds(cplexEnv_, cplexLp_, 1, index, lu, bd);
#else
    status = chgbds(cplexLp_, 1, index, lu, bd);
#endif

    lpSolverTime_.stop();

    if (status) {
      master_->err() << "ABA_CPLEXIF::_changeLBound: chgbds() failed" << endl;
      exit (Fatal);
    }
  }

@ The function |_changeUBound()| sets the upper bound of column |i| to
  |newLb|.
  It implements the pure virtual function of the base class |LP|.
                      
@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_changeUBound(int i, double newUb)
  {
    int    index[1] = {i};
    char   lu[1]    = {'U'};
    double bd[1]    = {newUb};

    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    int status = CPXchgbds(cplexEnv_, cplexLp_, 1, index, lu, bd);
#else
    int status = chgbds(cplexLp_, 1, index, lu, bd);
#endif

    lpSolverTime_.stop();

    if (status) {
      master_->err() << "ABA_CPLEXIF::_changeUBound: chgbds() failed" << endl;
      exit (Fatal);
    }
  }

@ The function |_pivotSlackVariableIn()| pivots the slack variables
  stored in the buffer |rows| into the basis. This function defines
  the pure virtual function of the base class |LP|.

  \BeginReturn
  \docarg{|0|}{All variables could be pivoted in,}
  \docarg{|1|}{otherwise.}
  \EndArg

  \BeginArg
  \docarg{|rows|}{The numbers of the slack variables that should be
                  pivoted in.}
  \EndArg

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_pivotSlackVariableIn(ABA_BUFFER<int> &rows)
  {
     int status;
     const int rowsNumber = rows.number();

     int *slacks = new int[rowsNumber];

     for (int i = 0; i < rowsNumber; i++)
       slacks[i] = rows[i];

#ifdef ABACUS_LP_CPLEX40
     status = CPXpivotin(cplexEnv_, cplexLp_, slacks, rowsNumber);
#else 
     status = pivotin(cplexLp_, slacks, rowsNumber);
#endif

     delete [] slacks;

     return status;
  }
  
@q m f iterationInformation @>
@ The function |iterationInformation()| emulates the Cplex function
  |setitfoind()| and |setscr\_ind(1)|.

  Note: The  output of Cplex is not compatible with the |ABA_OSTREAM| class of
  this framework and is always written to |stdout|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|level|}{If |level| is 0,
    then no iteration information is output, if |level| is 1 then
    iteration information is output after every refactorization, and
    if |level| is 2, then this information is output every iteration.}
  \EndArg
@q m e a @>

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
  void ABA_CPLEXIF::iterationInformation(int level)
@q m e f @>
  {
#ifndef ABACUS_LP_CPLEX40
    int tooSmall;
    int tooBig;
#endif
    int status;
    
    if (level) {
#ifdef ABACUS_LP_CPLEX40
      status = ::CPXsetintparam (cplexEnv_, CPX_PARAM_SCRIND, 1);
      if (status) {
        master_->err() << "ABA_CPLEXIF::iterationInformation(): ";
        master_->err() << "calling CPXsetintparam() failed." << endl;
        master_->err() << "Return value is " << status << "." << endl;
        exit(Fatal);
      }
#else
      setscr_ind(1);
#endif
    }
    else {
#ifdef ABACUS_LP_CPLEX40
      status = ::CPXsetintparam (cplexEnv_, CPX_PARAM_SCRIND, 0);
      if (status) {
        master_->err() << "ABA_CPLEXIF::iterationInformation(): ";
        master_->err() << "calling CPXsetintparam() failed." << endl;
        master_->err() << "Return value is " << status << "." << endl;
        exit(Fatal);
      }
#else
      setscr_ind(0);
#endif
    }
#ifdef ABACUS_LP_CPLEX40
    status = ::CPXsetintparam(cplexEnv_, CPX_PARAM_SIMDISPLAY, level);
    if (status) {
        master_->err() << "ABA_CPLEXIF::iterationInformation(): ";
        master_->err() << "calling CPXsetintparam() failed." << endl;
        master_->err() << "Return value is " << status << "." << endl;
        exit(Fatal);
    }
#else
    status = setitfoind(level, &tooSmall, &tooBig);
    if (status) {
      master_->err() << "ABA_CPLEXIF::iterationInformation(): calling setitfoind() failed" << endl;
      exit(Fatal);
    }
#endif
  }

@ The function |_primalSimplex()| calls the primal simplex method.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_LP::OPTSTAT ABA_CPLEXIF::_primalSimplex()
  {
#ifdef CPLEX_DEBUG
    interfaceCheck();
#endif
    if (nRow() == 0)
      return noConOpti();
    else {
      @<perform the primal simplex method of Cplex@>;
      @<check for presolver statuses (primal simplex)@>;
      @<are there phase 1 iterations of the primal simplex?@>;
      @<get the solution of the primal simplex method of Cplex@>;
    }
  }

@ @<perform the primal simplex method of Cplex@>=
  int status;

  lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX50
  status= CPXprimopt(cplexEnv_,cplexLp_);

#elif ABACUS_LP_CPLEX40
  status = CPXoptimize(cplexEnv_, cplexLp_);
#else
  status = ::optimize(cplexLp_);
#endif  

  lpSolverTime_.stop();

@ If the presolver found unboundedness or infeasibility then no
  solution information is available. In this case we have to to turn
  off the aggregator and presolver and optimize again in order to find
  out if the problem is either infeasible or unbounded. Afterwards, we
  turn these features on again for the solutions of further linear programs.

@<check for presolver statuses (primal simplex)@>=
  if (status == CPXERR_PRESLV_INForUNBD) {
    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_AGGIND, CPX_OFF);
    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_PREIND, CPX_OFF);
    
#ifdef ABACUS_LP_CPLEX50
    status= CPXprimopt(cplexEnv_,cplexLp_);
#else
    status = CPXoptimize(cplexEnv_, cplexLp_);
#endif

    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_AGGIND, CPX_ON);
    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_PREIND, CPX_ON);
#else
    (void) setpreind(0);
    (void) setaggind(0);

    status = ::optimize(cplexLp_);

    (void) setpreind(1);
    (void) setaggind(1);
#endif  

    lpSolverTime_.stop();
  } else  if (status) {
     master_->err() 
       << "ABA_CPLEXIF::_primalSimplex(): CPXprimopt() failed" 
       <<", return status is" << status << endl;
     exit(Fatal);
   }


@ @<are there phase 1 iterations of the primal simplex?@>=
  int phaseOneIterations;

#ifdef ABACUS_LP_CPLEX50
  phaseOneIterations = CPXgetphase1cnt(cplexEnv_,cplexLp_);
#elif ABACUS_LP_CPLEX40
  phaseOneIterations = CPXgetitci(cplexEnv_, cplexLp_);
#else
  phaseOneIterations = getitci(cplexLp_);
#endif

  if (phaseOneIterations) {
    master_->err() << "Warning: Cplex required " << phaseOneIterations;
    master_->err() << " phase I iterations" << endl;
  }

@ @<get the solution of the primal simplex method of Cplex@>=
    lpSolverTime_.start();
#ifdef ABACUS_LP_CPLEX40
    status = CPXgetstat(cplexEnv_, cplexLp_);
#elif ABACUS_LP_CPLEX30
    status = getstat(cplexLp_);
#else
    solution(cplexLp_, &status, 0, 0, 0, 0, 0);
#endif

    lpSolverTime_.stop();

    getSol();

    switch (status) {
#ifdef ABACUS_LP_CPLEX80
      case CPX_STAT_OPTIMAL:    return Optimal;
      case CPX_STAT_INFEASIBLE: return Infeasible;
      case CPX_STAT_UNBOUNDED:  return Unbounded;
      case CPX_STAT_OPTIMAL_INFEAS: 
#else
      case CPX_OPTIMAL:    return Optimal;
      case CPX_INFEASIBLE: return Infeasible;
      case CPX_UNBOUNDED:  return Unbounded;
      case CPX_OPTIMAL_INFEAS: 
#endif
              master_->err() << "ABA_CPLEXIF::_primalSimplex(): ";
              master_->err() << "Warning Cplex returnd status ";
              master_->err() << "CPX_OPTIMAL_INFEAS." <<  endl;
              master_->err() << "The LP-solution might be incorrect ";
              master_->err() << "due to numerical problems." << endl;
              master_->err() << "ABACUS assumes value is correct!" << endl;
              return Optimal;
      default:             master_->err() << "ABA_CPLEXIF::_primalSimplex(): status is ";
                           master_->err() << status << endl;
                           return Error;
    }

@ The function |_dualSimplex()| calls the dual simplex method.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_LP::OPTSTAT ABA_CPLEXIF::_dualSimplex()
  {
#ifdef CPLEX_DEBUG
    interfaceCheck();
#endif
    if (nRow() == 0)
      return noConOpti();
    else {
      @<perform the dual simplex method of Cplex@>;
      @<check for presolver statuses (dual simplex)@>;
      @<are there phase 1 iterations of the dual simplex?@>;
      @<get the solution of the dual simplex method of Cplex@>;
    }
  }
  
@ @<perform the dual simplex method of Cplex@>=
  int status;

  lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
  status = CPXdualopt(cplexEnv_, cplexLp_);
#else
  status = dualopt(cplexLp_);
#endif

  lpSolverTime_.stop();

@ If the presolver found unboundedness or infeasibility then no
  solution information is available. In this case we have to to turn
  off the aggregator and presolver and optimize again in order to find
  out if the problem is either infeasible or unbounded. Afterwards, we
  turn these features on again for the solutions of further linear programs.

@<check for presolver statuses (dual simplex)@>=
  if (status == CPXERR_PRESLV_INForUNBD) {
    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_AGGIND, CPX_OFF);
    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_PREIND, CPX_OFF);
    
    status = CPXdualopt(cplexEnv_, cplexLp_);

    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_AGGIND, CPX_ON);
    (void) ::CPXsetintparam(cplexEnv_, CPX_PARAM_PREIND, CPX_ON);
#else
    (void) setpreind(0);
    (void) setaggind(0);

    status = ::dualopt(cplexLp_);

    (void) setpreind(1);
    (void) setaggind(1);
#endif  

    lpSolverTime_.stop();
  } else  if (status) {
     master_->err() 
       << "ABA_CPLEXIF::_dualSimplex(): CPXdualopt() failed" 
       <<", return status is" << status << endl;
     exit(Fatal);
   }

@ @<are there phase 1 iterations of the dual simplex?@>=
  int phaseOneIterations;

#ifdef ABACUS_LP_CPLEX50
  phaseOneIterations= CPXgetphase1cnt(cplexEnv_,cplexLp_);
#elif ABACUS_LP_CPLEX40
  phaseOneIterations = CPXgetitci(cplexEnv_, cplexLp_);
#else
  phaseOneIterations = getitci(cplexLp_);
#endif

  if (phaseOneIterations) {
    master_->err() << "Warning: Cplex required " << phaseOneIterations;
    master_->err() << " phase I iterations" << endl;
  }

@ If |getstat(cplexLp_)| returns the status |Unbounded| (|Infeasible|) after
  the |LP|   has been solved with |dualopt(cplexLp_)|, then this means that the
  |LP| is dual unbounded (infeasible), i.e. primal infeasible (unbounded).
  For details we refer to [Cpl94], pp. A4--A5).

  For the status |CPX_OPTIMAL_INFEAS| the Cplex manual says ``problem
  optimal with unscaled infeasibilities''. We suppose that there are
  some numerical difficulties, but continue the optimization like for
  the status |CPX_OPTIMAL|.

@<get the solution of the dual simplex method of Cplex@>=
  lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
  status = CPXgetstat(cplexEnv_, cplexLp_);
#elif ABACUS_LP_CPLEX30
  status = getstat(cplexLp_);
#else
  solution(cplexLp_, &status, 0, 0, 0, 0, 0);
#endif

  
  lpSolverTime_.stop();

  getSol();

  switch (status) {
#ifdef ABACUS_LP_CPLEX80
    case CPX_STAT_OPTIMAL:    return Optimal;
    case CPX_STAT_UNBOUNDED:  return Unbounded;
    case CPX_STAT_INFEASIBLE: return Infeasible;
    case CPX_STAT_OPTIMAL_INFEAS:
#else
    case CPX_OPTIMAL:    return Optimal;
    case CPX_INFEASIBLE: return Unbounded;
    case CPX_UNBOUNDED:  return Infeasible;
    case CPX_OPTIMAL_INFEAS:
#endif
      master_->err() << "ABA_CPLEXIF::_dualSimplex(): status CPX_OPTIMAL_INFEAS ";
      master_->err() << "ignored, continue optimization" << endl;
      return Optimal;
    default:             master_->err() << "ABA_CPLEXIF::_dualSimplex(): status is ";
                         master_->err() << status << endl;
                          return Error;
  }


@ The function |_barrier()| calls the barrier method.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_LP::OPTSTAT ABA_CPLEXIF::_barrier(bool doCrossover)
  {
    if (nRow() == 0)
      return noConOpti();
    else {
      @<perform the barrier method of Cplex@>;
      @<get the solution of the barrier method of Cplex@>;
    }
  }

@ Cplex Version < 5.0:
  We have to cheat Cplex in order to call the barrier method. We observed
  that if Cplex can find a basis then it uses the simplex method although
  the function |hybbaropt()| is called. Therefore, we first call
  |baropt()| and then |hybbaropt()|   to cross over to a basis.
  
@<perform the barrier method of Cplex@>=
#ifdef ABACUS_LP_CPLEX22
  master_->err() << "Sorry, Cplex 2.2 implements no barrier method." << endl;
  master_->err() << "Taking primal simplex method instead." << endl;
  return _primalSimplex();
#else

  int status;

  lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
  status = CPXbaropt(cplexEnv_, cplexLp_);
  @<extract the barrier solution before crossing over@>;
  if (doCrossover)
    status = CPXhybbaropt(cplexEnv_, cplexLp_, 'o');
#else
  status = baropt(cplexLp_);
  @<extract the barrier solution before crossing over@>;
  if (doCrossover)
    status = hybbaropt(cplexLp_);
#endif

@ The barrier solution can be useful for applying special fixing
  criteria of the variables. Therefore we extract it here before we
  proceed with crossing over to basis.

@<extract the barrier solution before crossing over@>=
#ifdef ABACUS_LP_CPLEX40
    status = CPXsolution (cplexEnv_, cplexLp_, 0, 0, barXVal_, 0, 0, 0);
#else
    status = solution (cplexLp_, 0, 0, barXVal_, 0, 0, 0);
#endif

    if (status) 
      barXValStatus_ = Missing;
    else
      barXValStatus_ = Available;

@ If the barrier method does neither find a primal and dual feasible solution
  nor the |LP| is unbounded, then we invoke as recommended in
  [Cpl94], page A5, the primal method for an enhanced infeasibility
  analysis.

@<get the solution of the barrier method of Cplex@>=
#ifdef ABACUS_LP_CPLEX40
  status = CPXgetstat(cplexEnv_, cplexLp_);
#elif ABACUS_LP_CPLEX30
  status = getstat(cplexLp_);
#else
  solution(cplexLp_, &status, 0, 0, 0, 0, 0);
#endif

  lpSolverTime_.stop();

  getSol(doCrossover);

  switch (status) {
#ifdef ABACUS_LP_CPLEX80
    case CPX_STAT_OPTIMAL:                 return Optimal;
    case CPX_STAT_NUM_BEST:                return Optimal;
    case CPX_STAT_OPTIMAL_FACE_UNBOUNDED:  return Unbounded;
#else
    case CPX_OPTIMAL:                 return Optimal;
    case CPX_NUM_BEST_FEAS:           return Optimal;
    case CPX_OPTIMAL_FACE_UNBOUNDED:  return Unbounded;
#endif
    default:                               return _primalSimplex();
  }
#endif

@ The function |noConOpti()| emulates an optimization if no rows
  are loaded to Cplex, i.e., the linear program is only optimized over
  the bounds on the columns. The function returns either |Optimal|,
  |Infeasible|, or |Unbounded|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_LP::OPTSTAT ABA_CPLEXIF::noConOpti()
  {
    @<|ABA_CPLEXIF::noConOpti()|: local variables@>;

    master_->out() << "CPLEX: emulation of optimization without rows" << endl;

    value_ = 0.0;

    if (sense().max()) {
      @<maximize without rows@>;
    }
    else {
      @<minimize without rows@>;
    }
    
    xValStatus_ = Available;
    barXValStatus_ = recoStatus_= yValStatus_ = slackStatus_ = basisStatus_ = Missing;

    @<determine the basis status of the variables@>;
    
    return Optimal;
  }

@ @<|ABA_CPLEXIF::noConOpti()|: local variables@>=
    double  ub;
    double  lb;

@ In a maximization problems all columns with positive objective
  function coefficient go to the upper bound, the other columns
  go to the lower bound.
  
@<maximize without rows@>=
  int nCol = _nCol();
  for (int i = 0; i < nCol; i++) {
    if (_obj(i) > 0.0) {
      @<move the column to the upper bound to maximize@>;
    }
    else {
      @<move the column to the lower bound to maximize@>;
    }
  }

@ @<move the column to the upper bound to maximize@>=
  ub = _uBound(i);
  if (ub < _lBound(i)) {
    optStat_ = Infeasible;
    return Infeasible;
  }
  else if (master_->isInfinity(ub)) {
    optStat_ = Unbounded;
    value_   = master_->infinity();
    return Unbounded;
  }
  else {
    xVal_[i] = ub;
    value_  += _obj(i) * ub;
  }

@ @<move the column to the lower bound to maximize@>=
  lb = _lBound(i);
  if (lb > _uBound(i)) {
    optStat_ = Infeasible;
    return Infeasible;
  }
  else if (master_->isMinusInfinity(lb)) { 
    optStat_ = Unbounded;
    value_   = master_->infinity();
    return Unbounded;
  }
  else {
    xVal_[i] = lb;
    value_  += _obj(i) * lb;
  }

@ In a minimization problems all columns with negative objective
  function coefficient go to the upper bound, the other columns
  go to the lower bound.
  
@<minimize without rows@>=
  int nCol = _nCol();
  for (int i = 0; i < nCol; i++) {
    if (_obj(i) < 0.0) {
      @<move the column to the upper bound to minimize@>;
    }
    else {
      @<move the column to the lower bound to minimize@>;
    }
  }

@ @<move the column to the upper bound to minimize@>=
  ub = _uBound(i);
  if (ub < _lBound(i)) {
    optStat_ = Infeasible;
    return Infeasible;
  }
  else if (master_->isInfinity(ub)) {
    optStat_ = Unbounded;
    value_   = -master_->infinity();
    return Unbounded;
  }
  else {
    xVal_[i] = ub;
    value_  += _obj(i) * ub;
  }

@ @<move the column to the lower bound to minimize@>=
  lb = _lBound(i);
  if (lb > _uBound(i)) {
    optStat_ = Infeasible;
    return Infeasible;
  }
  if (master_->isMinusInfinity(lb)) {
    optStat_ = Unbounded;
    value_   = -master_->infinity();
    return Unbounded;
  }
  else {
    xVal_[i] = lb;
    value_  += _obj(i) * lb;
  }

@ The status of the columns is either {\tt AtLowerBound} (0) or
  {\tt AtUpperBound} (2).
  
@<determine the basis status of the variables@>=
  double eps = master_->eps();
      
  int nCol = _nCol();
  for (int i = 0; i < nCol; i++)
    if (xVal_[i] < _lBound(i) + eps)
      cStat_[i] = 0;
    else
      cStat_[i] = 2;

  basisStatus_ = Available;
  

@q m f setppriind @>
@ The function |setppriind()| emulates the Cplex function with the
  same name.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|priind|}{The primal pricing strategy. See your Cplex manual
                    for the possible values.}
  \EndArg
@q m e a @>

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
  void ABA_CPLEXIF::setppriind(int priind)
@q m e f @>
  {
    int status;

#ifdef ABACUS_LP_CPLEX40
    status = ::CPXsetintparam(cplexEnv_, CPX_PARAM_PPRIIND, priind);
#else
    int tooSmall;
    int tooBig;
    status = ::setppriind(priind,&tooSmall, &tooBig);
#endif

    if (status) {
      master_->err() << "ABA_CPLEXIF::setppriind(" << priind << "): unknown strategy" << endl;
      exit (Fatal);
    }
  }
  
@q m f setdpriind @>
@ The function |setdpriind()| emulates the Cplex function with the
  same name.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|priind|}{The dual pricing strategy. See your Cplex manual
                    for the possible values.}
  \EndArg
@q m e a @>

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
  void ABA_CPLEXIF::setdpriind(int priind)
@q m e f @>
  {
    int status;

#ifdef ABACUS_LP_CPLEX40
    status = ::CPXsetintparam(cplexEnv_, CPX_PARAM_DPRIIND, priind);
#else
    int tooSmall;
    int tooBig;
    status = ::setdpriind(priind , &tooSmall, &tooBig);
#endif

    if (status) {
      master_->err() << "ABA_CPLEXIF::setdpriind(" << priind << "): unknown strategy" << endl;
      exit (Fatal);
    }
  }

@q m f CPXgetdblparam @>
@ The function |CPXgetdblparam()| emulates the Cplex function with the
  same name that obtains the current value of a Cplex parameter of
  type |double|.

  This function is only available in Cplex versions 4.0 or newer.
@q m e t @>

@q m b r @>
  \BeginReturn
   If the value of the parameter can be obtained 0 is returned, 
   otherwise a nonzero value.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|whichParam|}{The name of the parameter (see your Cplex
                        manual).}
  \docarg{|value|}{Holds the parameter value in case of a successful
                   call.}
  \EndArg
@q m e a @>


@<implementation of the members of |ABA_CPLEXIF|@>=
#ifdef ABACUS_LP_CPLEX40
@q m b f @>
  int ABA_CPLEXIF::CPXgetdblparam(int whichParam, double *value)
@q m e f @>
  {
    return ::CPXgetdblparam(cplexEnv_, whichParam, value);
  }
#endif

@q m f CPXsetdblparam @>
@ The function |CPXsetdblparam()| emulates the Cplex function with the
  same name that sets the value of a Cplex parameter of type |double|.

  This function is only available in Cplex versions 4.0 or newer.
@q m e t @>

@q m b r @>
  \BeginReturn
   If the value of the parameter can be set 0 is returned, 
   otherwise a nonzero value.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|whichParam|}{The name of the parameter (see your Cplex
                        manual).}
  \docarg{|value|}{The new value of the parameter.}
  \EndArg
@q m e a @>


@<implementation of the members of |ABA_CPLEXIF|@>=
#ifdef ABACUS_LP_CPLEX40
@q m b f @>
  int ABA_CPLEXIF::CPXsetdblparam(int whichParam, double value)
@q m e f @>
  {
    return ::CPXsetdblparam(cplexEnv_, whichParam, value);
  }
#endif

@q m f CPXgetintparam @>
@ The function |CPXgetintparam()| emulates the Cplex function with the
  same name that obtains the current value of a Cplex parameter of
  type |int|.

  This function is only available in Cplex versions 4.0 or newer.
@q m e t @>

@q m b r @>
  \BeginReturn
   If the value of the parameter can be obtained 0 is returned, 
   otherwise a nonzero value.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|whichParam|}{The name of the parameter (see your Cplex
                        manual).}
  \docarg{|value|}{Holds the parameter value in case of a successful
                   call.}
  \EndArg
@q m e a @>


@<implementation of the members of |ABA_CPLEXIF|@>=
#ifdef ABACUS_LP_CPLEX40
@q m b f @>
  int ABA_CPLEXIF::CPXgetintparam(int whichParam, int *value)
@q m e f @>
  {
    return ::CPXgetintparam(cplexEnv_, whichParam, value);
  }
#endif

@q m f CPXsetintparam @>
@ The function |CPXsetintparam()| emulates the Cplex function with the
  same name that sets the value of a Cplex parameter of type |int|.

  This function is only available in Cplex versions 4.0 or newer.
@q m e t @>

@q m b r @>
  \BeginReturn
   If the value of the parameter can be set 0 is returned, 
   otherwise a nonzero value.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|whichParam|}{The name of the parameter (see your Cplex
                        manual).}
  \docarg{|value|}{The new value of the parameter.}
  \EndArg
@q m e a @>


@<implementation of the members of |ABA_CPLEXIF|@>=
#ifdef ABACUS_LP_CPLEX40
@q m b f @>
  int ABA_CPLEXIF::CPXsetintparam(int whichParam, int value)
@q m e f @>
  {
    return ::CPXsetintparam(cplexEnv_, whichParam, value);
  }
#endif

@q m f cplexLp @>
@ The function |cplexLp()| should be used carefully
  because it breaks the class principle. 
  In particular, it should not be used to to modify the internal data
  structures of Cplex.
@q m e t @>
 However, we require this
 direct access for the separators in the mixed integer optimizer.

@q m b r @>
  \BeginReturn
  A pointer to the internal problem
  representation of Cplex. 
  \EndReturn
@q m e r @>

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
   struct cpxlp *ABA_CPLEXIF::cplexLp()
@q m e f @>
   {
     return cplexLp_;
   }

@q m f cplexEnv @>
@ The function |cplexEnv()| is only available in Cplex Version 4.0 and newer.
  Like the function |cplexLp()| this function should be used very carefully.
  In particular, it should not be used to to modify the internal data
  structures of Cplex.
@q m e t @>

@q m b r @>
  \BeginReturn
  A pointer to the cplex environment.
  \EndReturn
@q m e r @>

@<implementation of the members of |ABA_CPLEXIF|@>=
#ifdef ABACUS_LP_CPLEX40
@q m b f @>
  struct cpxenv *ABA_CPLEXIF::cplexEnv()
@q m e f @>
  {
    return cplexEnv_;
  }
#endif

@q m f print @>
@ The function |print()| writes the program loaded by Cplex to
  the standard output. This function is especially useful for debugging,
  but breaks the stream concept of \ABACUS.
@q m e t @>

  The constant |L_tmpnam| und the function |tmpnam()| are declared
  in |<stdio.h>| and can be used to generate unique names. The Cplex
  function |lpwrite()| writes the problem to file which is then
  written to the output.

@<implementation of the members of |ABA_CPLEXIF|@>=
@q m b f @>
  void ABA_CPLEXIF::print()
@q m e f @>
  {
    char s[13]="abacusXXXXXX";

    int fd=mkstemp(s);
    close(fd);

#ifdef ABACUS_LP_CPLEX40
    CPXlpwrite (cplexEnv_, cplexLp_, s);
#else
    lpwrite (cplexLp_, s);
#endif
    char *systemCall = new char[13+10];

    strcpy(systemCall, "cat ");
    strcat(systemCall, s);
    system(systemCall);

    strcpy(systemCall, "rm -f ");
    strcat(systemCall, s);
    system(systemCall);

    delete [] systemCall;
  }

@ The function |_sense()| returns the sense of the optimization.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_OPTSENSE ABA_CPLEXIF::_sense() const
  {
    ABA_OPTSENSE s;

#ifdef ABACUS_LP_CPLEX40
    int cplexSense = CPXgetobjsen(cplexEnv_, cplexLp_);
#else
    int cplexSense = getobjsen(cplexLp_);
#endif

    if (cplexSense == 1)
      s.sense(ABA_OPTSENSE::Min);
    else
      s.sense(ABA_OPTSENSE::Max);

    return s;
  }


@ This version of the  function |_sense()| changes the sense of the
  optimization. 
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_sense(const ABA_OPTSENSE &newSense)
  {
#ifdef ABACUS_LP_CPLEX40
    CPXchgobjsen(cplexEnv_, cplexLp_, optSense2cplex(newSense));
#else
    chgobjsen(cplexLp_, optSense2cplex(newSense));
#endif
  }

@ The function |_nRow()| returns the number of rows of the linear program.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_nRow() const
  {
#ifdef ABACUS_LP_CPLEX40  
    return CPXgetnumrows(cplexEnv_, cplexLp_);
#else
    return getmar(cplexLp_);
#endif
  }

@ The function |_maxRow()| returns the maximal number of rows of the linear
  program.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_maxRow() const
  {
    return marsz_;
  }

@ The function |_nCol()| returns the number of columns of the linear
  program.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_nCol() const
  {
#ifdef ABACUS_LP_CPLEX40
    return CPXgetnumcols(cplexEnv_, cplexLp_);
#else
    return getmac(cplexLp_);
#endif
  }

@ The function |_maxCol()| returns the maximal number of columns of the linear
  program.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_maxCol() const
  {
    return macsz_;
  }

@ The function |_obj()| returns the objective function coefficient of
  column |i|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_obj(int i) const
  {
    double obj;

#ifdef ABACUS_LP_CPLEX40
    int status = CPXgetobj(cplexEnv_, cplexLp_, &obj, i, i);
#else
    int status = getobj(cplexLp_, &obj, i, i);
#endif

    if (status) {
      master_->err() << "ABA_CPLEXIF::_obj(" << i << "): ";
      master_->err() << "getobj() returned status " << status << endl;
      exit(Fatal);
    }

    return obj;
  }

@ The function |_lBound()| returns the lower bound of column |i|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_lBound(int i) const
  {
    double lb;

#ifdef ABACUS_LP_CPLEX40
    int status = CPXgetlb(cplexEnv_, cplexLp_, &lb, i, i);
#else
    int status = getbdl(cplexLp_, &lb, i, i);
#endif

    if (status) {
      master_->err() << "ABA_CPLEXIF::_lBound(" << i << "): ";
#ifdef ABACUS_LP_CPLEX40
      master_->err() << "getlb() returned status " << status << endl;
#else
      master_->err() << "getbdl() returned status " << status << endl;
#endif
      exit(Fatal);
    }

    return lb;
  }

@ The function |_uBound()| returns the upper bound of column |i|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_uBound(int i) const
  {
    double ub;
#ifdef ABACUS_LP_CPLEX40
    int status = CPXgetub(cplexEnv_, cplexLp_, &ub, i, i);
#else
    int status = getbdu(cplexLp_, &ub, i, i);
#endif
    if (status) {
      master_->err() << "ABA_CPLEXIF::_uBound(" << i << "): ";
#ifdef ABACUS_LP_CPLEX40
      master_->err() << "getbdu() returned status " << status << endl;
#else
      master_->err() << "getbdu() returned status " << status << endl;
#endif
      exit(Fatal);
    }

    return ub;
  }

@ The function |_rhs()| returns the right hand side of row |i|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_rhs(int i) const
  {
    double r;

#ifdef ABACUS_LP_CPLEX40
    int status = CPXgetrhs(cplexEnv_, cplexLp_, &r, i, i);
#else
    int status = getrhs(cplexLp_, &r, i, i);
#endif

    if (status) {
      master_->err() << "ABA_CPLEXIF::_getrhs(" << i << "): ";
      master_->err() << "getrhs() returned status " << status << endl;
      exit(Fatal);
    }

    return r;
  }

@ The function |_row()| stores a copy of row |i| in |r|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_row(int i, ABA_ROW &r) const
  {
    @<get row |i| from Cplex@>;
    @<insert the data into |r|@>;
    @<|ABA_CPLEXIF::_row()|: clean up@>;
  }

@ Cplex provides only the function |getrows()| for retrieving several rows.
  Therefore, we set up data structures for storing only one row.
  
@<get row |i| from Cplex@>=
  int     nzcnt;
  int    *rmatbeg = new int[2];
#ifdef ABACUS_LP_CPLEX40
  int    *rmatind = new int[CPXgetnumcols(cplexEnv_, cplexLp_)];
  double *rmatval = new double[CPXgetnumcols(cplexEnv_, cplexLp_)];
#else
  int    *rmatind = new int[getmac(cplexLp_)];
  double *rmatval = new double[getmac(cplexLp_)];
#endif
  int     surplus;

  int status = CPXgetrows(cplexEnv_, cplexLp_, &nzcnt, rmatbeg, rmatind, 
                          rmatval, CPXgetnumcols(cplexEnv_, cplexLp_), 
                          &surplus, i, i);

  char sense;
  CPXgetsense (cplexEnv_, cplexLp_, &sense, i, i);

  if (status) {
    master_->err() << "ABA_CPLEXIF::_row() failed" << endl;
    exit(Fatal);
  }
  
@ @<insert the data into |r|@>=
  r.clear();
    
  for (int j = 0; j < nzcnt; j++)
    r.insert(rmatind[j], rmatval[j]);

  r.sense(cplex2csense(sense));  
  r.rhs(_rhs(i));

@ @<|ABA_CPLEXIF::_row()|: clean up@>=
  delete [] rmatbeg;
  delete [] rmatind;
  delete [] rmatval;

@ The function |_nnz()| returns the number of nonzero elements in the
  constraint matrix (not including the right hand side).
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_nnz() const
  {
#ifdef ABACUS_LP_CPLEX40
    return CPXgetnumnz(cplexEnv_, cplexLp_);
#else
    return getmat(cplexLp_);
#endif
  }

@ The function |_value()| returns the optimum value of the linear program.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_value() const
  {
    return value_;
  }

@ The function |_barXVal()| returns the value of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_barXVal(int i)
  {
    return barXVal_[i];
  }

@ The function |_reco()| returns the reduced cost of the column |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_reco(int i)
  {
    return reco_[i];
  }


@ The function |_slack()| returns the value of the slack column of the
  row  |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_slack(int i)
  {
    return slack_[i];
  }


@ The function |getSol()| extracts the solution, i.e., the value,
  the status, the values of the variables, slack variables, and dual
  variables, the reduced costs, and the statuses of the variables
  and slack variables form the internal Cplex data structure.

  If there are no rows in the matrix, then the solution has
  been retrieved already in the function |noConOpti()|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::getSol(bool getBasis)
  {
    lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
    int status = CPXsolution (cplexEnv_, cplexLp_, 0, &value_, xVal_, yVal_,
                              slack_, reco_);
#else
    int status = solution (cplexLp_, 0, &value_, xVal_, yVal_,
                           slack_, reco_);
#endif
    lpSolverTime_.stop();

    if (status) 
      xValStatus_ = recoStatus_= yValStatus_ = slackStatus_ = Missing;
    else
      xValStatus_ = recoStatus_= yValStatus_ = slackStatus_ = Available;

    if(getBasis){

        lpSolverTime_.start();

#ifdef ABACUS_LP_CPLEX40
        status = CPXgetbase(cplexEnv_, cplexLp_, cStat_, rStat_);
#else
        status = ::getbase(cplexLp_, cStat_, rStat_);
#endif
        lpSolverTime_.stop();

        if (status)
            basisStatus_ = Missing;
        else
            basisStatus_ = Available;
    }else
        basisStatus_=Missing;
  }

@ The function |optSense2cplex()| returns the |sense| of the objective
  function in the internal representation of Cplex associated with
  the linear program.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::optSense2cplex(ABA_OPTSENSE sense) const
  {
    if(sense.min()) return  1;
    else            return -1;
  }
  

  

@ If a reallocation is performed in the base class |LP|, we
  reinitialize the internal data structure. Actually this
  reinitialization is redundant since it would be performed
  automatically if |addRows()| or |addCols()| fail. However,
  to be consistent, and if a reallocation is performed to
  decrease the size of the arrays we call |reinitialize()|.

@ The function |_rowRealloc()| reallocates the internal memory such that
  |newSize| rows can be stored.
  It implements the corresponding pure virtual function of the base
  class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
void ABA_CPLEXIF::_rowRealloc(int newSize)
{
  int nRows = _nRow();

  if ( nRows >= newSize ) {
    master_->err() << "ABA_CPLEXIF::_rowRealloc : new Dimesion smaller than the old one" 
                   << endl;
    exit(Fatal);
  }
  
  double* newYVal  = new double[newSize];
  double* newSlack = new double[newSize];
  int*    newRStat = new int[newSize];
  
  
  for(int i=0; i<nRows; i++){
    newYVal[i]  = yVal_[i];
    newSlack[i] = slack_[i];
    newRStat[i] = rStat_[i];
  }
  
  delete [] yVal_;
  delete [] slack_;
  delete [] rStat_;
  
  yVal_=newYVal;
  slack_=newSlack;
  rStat_=newRStat;
  
  marsz_= newSize;

}

@ The function |_colRealloc()| reallocates the internal memory such that
  |newSize| columns can be stored.
  It implements the corresponding pure virtual function of the base
  class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  void ABA_CPLEXIF::_colRealloc(int newSize)
  {
  int nCols = _nCol();

  if ( nCols >= newSize ) {
    master_->err() << "ABA_CPLEXIF::_colRealloc : new Dimesion smaller than the old one" 
                   << endl;
    exit(Fatal);
  }
  
  double* newXVal    = new double[newSize];
  double* newBarXVal = new double[newSize];
  double* newReco    = new double[newSize];
  int*    newCStat   = new int[newSize];
    
  for(int i=0; i<nCols; i++){
    newXVal[i]    = xVal_[i];
    newBarXVal[i] = barXVal_[i];
    newReco[i]    = reco_[i];
    newCStat[i]   = cStat_[i];
  }
  
  delete [] xVal_;
  delete [] barXVal_;
  delete [] reco_;
  delete [] cStat_;
  
  xVal_    = newXVal;
  barXVal_ = newBarXVal;
  reco_    = newReco;
  cStat_   = newCStat;
  
  macsz_= newSize;

}



@ The function |_setSimplexIterationLimit()| changes the iteration
  limit of the Simplex algorithm.
  This function defines a pure virtual function of the base class |LP|.

  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be set,}
  \docarg{|1|}{otherwise.}
  \EndReturn

  \BeginArg
  \docarg{|limit|}{The new value of the iteration limit.}
   \EndArg

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_setSimplexIterationLimit(int limit)
  {
#ifdef ABACUS_LP_CPLEX40
    return ::CPXsetintparam(cplexEnv_, CPX_PARAM_ITLIM, limit);
#else
    int tooSmall;
    int tooBig;

    return setitlim(limit, &tooSmall, &tooBig);
#endif
  }

@ The function |_getSimplexIterationLimit()| defines a pure virtual
 function of the base class |LP|.

  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be get,}
  \docarg{|1|}{otherwise.} 
  \EndReturn

  \BeginArg
  \docarg{|limit|}{Stores the iteration limit if the  return value is 0.}
  \EndArg

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::_getSimplexIterationLimit(int &limit)
  {
#ifdef ABACUS_LP_CPLEX40
    return ::CPXgetintparam(cplexEnv_, CPX_PARAM_ITLIM, &limit);
#else
     getitlim(&limit);
     return 0;
#endif
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |_xVal()| returns the value of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_xVal(int i)
  {
    return xVal_[i];
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |cplex2lpVarStat()| returns the variable status
  of ABACUS associated with the internal Cplex variable status |stat|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_LPVARSTAT::STATUS ABA_CPLEXIF::cplex2lpVarStat(int stat) const
  {
    switch (stat) {
      case 0:  return ABA_LPVARSTAT::AtLowerBound;
      case 1:  return ABA_LPVARSTAT::Basic;
      case 2:  return ABA_LPVARSTAT::AtUpperBound;
      case 3:  return ABA_LPVARSTAT::NonBasicFree;
      default: master_->err() << "ABA_CPLEXIF::cplex2lpVarStat(" << stat << "): ";
               master_->err() << "unknown status" << endl;
               exit(Fatal);
               return ABA_LPVARSTAT::AtLowerBound; // to suppress compiler warning
    }
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |slackStat2Cplex()| returns the slack variable status of
  Cplex associated with |stat|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::slackStat2cplex(ABA_SLACKSTAT::STATUS stat) const
  {
    switch(stat) {
      case ABA_SLACKSTAT::NonBasicZero:    return 0;
      case ABA_SLACKSTAT::Unknown:;
      case ABA_SLACKSTAT::Basic:           return 1;
      case ABA_SLACKSTAT::NonBasicNonZero: return 2;
      default: master_->err() << "ABA_CPLEXIF::slackStat2cplex(" << (int) stat << "): ";
               master_->err() << " corresponding CPLEX status unknown" << endl;
               exit (Fatal);
               return -1; // to suppress compiler warning
    }
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |cplex2slackStat()| returns the slack variable status
  associated with the internal Cplex slack variable status |stat|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_SLACKSTAT::STATUS ABA_CPLEXIF::cplex2slackStat(int stat) const
  {
    switch (stat) {
      case 0:  return ABA_SLACKSTAT::NonBasicZero;
      case 1:  return ABA_SLACKSTAT::Basic;
      case 2:  return ABA_SLACKSTAT::NonBasicNonZero;
      default: master_->err() << "ABA_CPLEXIF::cplex2slackStat(" << stat << "): ";
               master_->err() << "unknown status" << endl;
               exit(Fatal);
               return ABA_SLACKSTAT::NonBasicZero; // to suppress compiler warning
    }
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |_lpVarStat()| returns the status of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_LPVARSTAT::STATUS ABA_CPLEXIF::_lpVarStat(int i)
  {
    return cplex2lpVarStat(cStat_[i]);
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |_slackStat()| returns the status of the slack column |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_SLACKSTAT::STATUS ABA_CPLEXIF::_slackStat(int i)
  {
    return cplex2slackStat(rStat_[i]);
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |csense2cplex()| returns the |sense| of a row
  in the internal representation of Cplex associated with |sense|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  char ABA_CPLEXIF::csense2cplex(ABA_CSENSE *sense) const
  {
    switch(sense->sense()) {
      case ABA_CSENSE::Less:    return 'L';
      case ABA_CSENSE::Equal:   return 'E';
      case ABA_CSENSE::Greater: return 'G';
      default: master_->err() << "ABA_CPLEXIF::csense2cplex(" << *sense << "):";
               master_->err() << "unknown sense" << endl;
               exit(Fatal);
               return 'G';  // to suppress compiler warning
    }
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |cplex2csense()| converts the Cplex representation of the
  row sense to the \ABACUS\ representation.

@<implementation of the members of |ABA_CPLEXIF|@>=
  ABA_CSENSE::SENSE ABA_CPLEXIF::cplex2csense(char sense) const
  {
    switch(sense) {
      case 'L': return ABA_CSENSE::Less;
      case 'E': return ABA_CSENSE::Equal;
      case 'G': return ABA_CSENSE::Greater;
      default: master_->err() << "ABA_CPLEXIF::cplex2csense(" << sense << ")" << endl;
               master_->err() << "unknown sense" << endl;
               exit(Fatal);
               return ABA_CSENSE::Less; // to suppress compiler warning
    }
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/


@ The function |lpVarStat2cplex()| returns the variable status
  of Cplex associated with |stat|.

@<implementation of the members of |ABA_CPLEXIF|@>=
  int ABA_CPLEXIF::lpVarStat2cplex(ABA_LPVARSTAT::STATUS stat) const
  {
    switch (stat) {
      case ABA_LPVARSTAT::AtLowerBound: return 0;
      case ABA_LPVARSTAT::Basic:        return 1;
      case ABA_LPVARSTAT::AtUpperBound: return 2;
      case ABA_LPVARSTAT::NonBasicFree: return 3;
      case ABA_LPVARSTAT::Unknown:      return 0;
      default: master_->err() << "ABA_CPLEXIF::lpVarStat2cplex(" << (int) stat << "): ";
               master_->err() << "unknown status" << endl;
               exit(Fatal);
               return -1; // to suppress compiler warning
    }
  }

  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ The function |_yVal()| returns the value of the dual column of the
  row |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<implementation of the members of |ABA_CPLEXIF|@>=
  double ABA_CPLEXIF::_yVal(int i)
{
  return yVal_[i];
}


  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/
  /************************************************************/

@ INTERFACE CHECK

@<implementation of the members of |ABA_CPLEXIF|@>=

double ABA_CPLEXIF::trueSubLB(int i)
{
  const ABA_SUB* sub = ((ABA_LPSUBCPLEX*) this)->sub();
  return sub->fsVarStat(i)->fixedOrSet()?elimVal(i):sub->lBound(i);
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

double ABA_CPLEXIF::trueSubUB(int i)
{
  const ABA_SUB* sub = ((ABA_LPSUBCPLEX*) this)->sub();
  return sub->fsVarStat(i)->fixedOrSet()?elimVal(i):sub->uBound(i);
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

double ABA_CPLEXIF::elimVal(int i) const
{
    const ABA_SUB* sub = ((ABA_LPSUBCPLEX*) this)->sub();
    switch (sub->fsVarStat(i)->status()) {
    case ABA_FSVARSTAT::SetToLowerBound:   return sub->lBound(i);
    case ABA_FSVARSTAT::FixedToLowerBound: return sub->variable(i)->lBound();
    case ABA_FSVARSTAT::SetToUpperBound:   return sub->uBound(i);
    case ABA_FSVARSTAT::FixedToUpperBound: return sub->variable(i)->uBound();
    case ABA_FSVARSTAT::Set:               return sub->fsVarStat(i)->value();
    case ABA_FSVARSTAT::Fixed:             return sub->variable(i)->fsVarStat()->value();
      
    default: master_->err() << "ABA_LPSUB::elimVal():";
      master_->err() << " variable neither fixed nor set" << endl;
      exit(Fatal);
               return 0.0;   // to suppress compiler warning
    }
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

bool ABA_CPLEXIF::interfaceCheck()
{

  const ABA_SUB* sub = ((ABA_LPSUBCPLEX*) this)->sub();
  
  master_->out() 
    << "------------------------------------------------------------" << endl;
  master_->out() << "comparing CPLEX data with sub problem data ..." << endl;
  master_->out() 
    << "------------------------------------------------------------" << endl;

  //
  // check problem dimensions
  //
  int numRowCPLEX = CPXgetnumrows(cplexEnv_, cplexLp_);
  int numRowSUB   = sub->nCon();
  int numColCPLEX = CPXgetnumcols(cplexEnv_, cplexLp_);
  int numColSUB   = sub->nVar();
  master_->out() << "number of cols  : " << numColCPLEX << " / " << numColSUB;
  if (numColSUB == numColCPLEX) {
    master_->out() << " OK" << endl;
  } else {
    master_->out() << " <----(ERROR)" << endl;
    exit(Fatal);
  }
  master_->out() << "number of rows  : " << numRowCPLEX << " / " << numRowSUB;
  if (numRowSUB == numRowCPLEX) {
    master_->out() << " OK" << endl;
  } else {
    master_->out() << " <----(ERROR)" << endl;
    exit(Fatal);
  }

  //
  // Comparing ABACUS and CPLEX LP Matrix
  //
  master_->out() << "checking cons   : "; 
  for(int i=0; i<numRowSUB; i++){
    ABA_ROW  cplexRow(master_, numColCPLEX);
    ABA_ROW  abacusRow(master_, numColSUB);
    _row(i,cplexRow);
    sub->constraint(i)->genRow(sub->actVar(), abacusRow);
    
    // check sense
    //
    if(abacusRow.sense()->sense()!=cplexRow.sense()->sense()){
      master_->out() << " row " << i 
		     << " : wrong sense <----(ERROR)" << endl;
      exit(Fatal);
    }
    
    // check RHS
    //
    if(abacusRow.rhs()!=cplexRow.rhs()){
      master_->out() << " row " << i 
		     << " : wrong RHS <----(ERROR)" << endl;
      exit(Fatal);
    }
    
    // check dimension
    //
    if( abacusRow.nnz() == cplexRow.nnz()) {      
      for(int j=0; j<abacusRow.nnz(); j++){
	// check coefficients 
	//
	if ((abacusRow.support(j)!=cplexRow.support(j))
	    || (abacusRow.coeff(j)!=cplexRow.coeff(j))) 
	  {
	    master_->out() << " row " << i 
			   << " : wrong coeff <----(ERROR)" << endl;
	    exit(Fatal);
	  } 
      }
    } else {
      master_->out() << " row " << i 
		     << " : different size  <----(ERROR)" << endl;
      exit(Fatal);
    }
  }
  master_->out() << " OK" << endl;
  
  // check Bounds
  //
  master_->out() << "checking bounds : ";   
  for(int i=0; i<numColCPLEX; i++){
    if (_uBound(i)!=trueSubUB(i)){
      master_->out() << " col " << i 
		     << " : different upper bound " << endl
		     << _uBound(i) << "/" <<  sub->uBound(i) 
		     << " (cplex/abacus) <----(ERROR)" << endl;
      exit(Fatal);
    }
    if (_lBound(i)!=trueSubLB(i)){
      master_->out() << " col " << i 
		     << " : different lower bound " << endl
		     << _lBound(i) << "/" <<  sub->lBound(i) 
		     << " (cplex/abacus) <----(ERROR)" << endl;
      exit(Fatal);
    }
  }
  master_->out() << " OK" << endl;
  master_->out() 
    << "------------------------------------------------------------" 
    << endl <<  endl ;
  
  return true;
}


