@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: cputimer.w,v 4.12 2003/08/05 13:55:15 elf Exp $ @>

@q m i ABA_CPUTIMER @>

@ The member functions are defined in the file {\tt cputimer.cc}.

@(cputimer.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "cputimer.h"
#else
#include "abacus/cputimer.h"
#endif

#ifdef ABACUS_SYS_POWERPLUS
#include <sys/time.h>
#elif defined ABACUS_COMPILER_VISUAL_CPP 
#include <time.h>
#else
extern "C"
{
#include <time.h>
#include <sys/times.h>
#include <limits.h>
#include <unistd.h>
#endif
}

@ For the POWERPLUS and UNIX systems we have to initialize some static
  members.

@(cputimer.cc@>=
#ifdef ABACUS_SYS_POWERPLUS
unsigned ABA_CPUTIMER::lastTime_ = 0;
unsigned ABA_CPUTIMER::overflowTime_ = 0;
#endif

#if defined ABACUS_SYS_SOLARIS || ABACUS_SYS_LINUX

long ABA_CPUTIMER::clk_tck_ = sysconf(_SC_CLK_TCK);
#endif


@q m f Constructor @>
@ After the application of the constructor the timer is not running,
  i.e., to measure time it has to be started explicitly.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to a global object.}
  \EndArg
@q m e a @>

@(cputimer.cc@>=
@q m b f @>
  ABA_CPUTIMER::ABA_CPUTIMER(ABA_GLOBAL *glob) 
@q m e f @>
  : @/
    ABA_TIMER(glob)
  { }

@q m f Constructor @>
@ This constructor initializes the total time of the timer.
  The timer is not running, too.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to a global object.}
  \docarg{|centiSeconds|}{The intial value of the total time in
                          $1\over 100$ seconds.}
  \EndArg
@q m e a @>

@(cputimer.cc@>=
@q m b f @>
  ABA_CPUTIMER::ABA_CPUTIMER(ABA_GLOBAL *glob, long centiSeconds) 
@q m e f @>
  : @/
    ABA_TIMER(glob, centiSeconds)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(cputimer.cc@>=
@q m b f @>
  ABA_CPUTIMER::~ABA_CPUTIMER()
@q m e f @>
  { }
  
@ The function |theTime()| returns the used cpu time in $1\over100$ seconds
  since the start of the program.
  This function redefines the pure virtual function of the base
  class |ABA_TIMER|.

  Since |CLOCKS_PER_SEC| can be 1000000 
  the standard library function |clock()| returns negative values
  after about
  than 35 minutes. Hence we measure the cpu time with the function
  |times| which is common on \UNIX/ systems, although not defined
  in the \CEE/-ANSI-standard.

  For the POWERPLUS system Thomas Christof provided the measurement of
  the cpu-time. The problem is that about every minute there is an
  overflow of the function |TimeNow()|. In this case this result is
  less than |lastTime_| and we have to increase |overflowTime_|.
  This timing is only correct if at least every 70 minutes the
  function |theTime()| is called.
  
@(cputimer.cc@>=
  long ABA_CPUTIMER::theTime() const
  {
#ifdef ABACUS_SYS_POWERPLUS
    @<compute the time for power plus@>;
#elif defined ABACUS_COMPILER_VISUAL_CPP
    @<compute the time for Visual C++@>;
#else
    @<compute the time for Unix systems@>;
#endif
  }

@ @<compute the time for power plus@>=
  unsigned time=(((unsigned) TimeNow())*100)/CLOCK_TICK;
  if(time < lastTime_) {
    overflowTime_+=(unsigned)((((double)(~(unsigned)0))*100)/(double)CLOCK_TICK);
  }
  lastTime_=time;
  return (long) (overflowTime_ + time);


@ @<compute the time for Visual C++@>=
  return (clock()*100)/CLOCKS_PER_SEC;

@ @<compute the time for Unix systems@>=
  struct tms now;
 
  times (&now);
  return (long) (now.tms_utime*100)/clk_tck_; 

