@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: csense.w,v 4.11 2003/08/05 13:55:16 elf Exp $ @>

@q m i ABA_CSENSE @>

@ All member functions are defined in the file {\tt csense.cc}.

@(csense.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "csense.h"
#include "global.h"
#else
#include "abacus/csense.h"
#include "abacus/global.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@ If the default constructor is used, the sense is undefined.
@q m e t @>

@q m b a @>
       \BeginArg
       \docarg{|glob|}{A pointer to the corresponding global object.}
       \EndArg
@q m e a @>

@(csense.cc@>=
@q m b f @>
  ABA_CSENSE::ABA_CSENSE(ABA_GLOBAL *glob) 
@q m e f @>
  : @/
    glob_(glob)
  { }

@q m f Constructor @>
@ This constructor initializes the sense.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \docarg{|s|}{The sense.}
  \EndArg
@q m e a @>

@(csense.cc@>=
@q m b f @>
  ABA_CSENSE::ABA_CSENSE(ABA_GLOBAL *glob, SENSE s) 
@q m e f @>
  : @/
    glob_(glob), @/
    sense_(s)
  { }

@q m f Constructor @>
@ With this constructor the sense of the constraint can also be
       initialized with a single letter.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \docarg{|s|}{A character representing the sense:
               {\tt E} or {\tt e} stand
               for |Equal|, {\tt G} and {\tt g} stand for |Greater|, and
               {\tt L} or {\tt l} stand for |Less|.}
  \EndArg
@q m e a @>

@(csense.cc@>=
@q m b f @>
  ABA_CSENSE::ABA_CSENSE(ABA_GLOBAL *glob, char s) 
@q m e f @>
  : @/
    glob_(glob)
  {
    switch (s) {
      case 'l':
      case 'L': sense_ = Less;
                break;
      case 'e':
      case 'E': sense_ = Equal;
                break;
      case 'g':
      case 'G': sense_ = Greater;
                break;
      default: glob_->err() << "ABA_CSENSE::ABA_CSENSE(): unknown argument " << s << endl;
               exit(Fatal);
               break;
    }
  }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the |ABA_CSENSE| from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the sense is initialized.}
  \EndArg
@q m e a @>

@(csense.cc@>=
@q m b f @>
  ABA_CSENSE::ABA_CSENSE(ABA_GLOBAL *glob, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    glob_(glob)
  {
    sense_ = (SENSE) msg.unpackInt();
  }

@q m f pack @>
@ The function |pack()| packs the data of the sense in an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the sense is packed.}
  \EndArg
@q m e a @>

@(csense.cc@>=
@q m b f @>
  void ABA_CSENSE::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    msg.pack((int)sense_);
  }

#endif

      
@q m f Output Operator @>
@ The output operator writes the sense on an output stream
  in the form |<=|, |=|, or |>=|.
@q m e t @>

@q m b r @>
  \BeginReturn
  The output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The sense being output.}
  \EndArg
@q m e a @>

@(csense.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_CSENSE &rhs)
@q m e f @>
  {
    switch (rhs.sense_) {
      case ABA_CSENSE::Less:    out << "<=";
                            break;
      case ABA_CSENSE::Equal:   out << '=';
                            break;
      case ABA_CSENSE::Greater: out << ">=";
                            break;
    }
    return out;
  }

@q m f Assignment Operator @>
@ The default assignment operator is overloaded such that also the
  enumeration |SENSE| can be used on the right hand side.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the sense.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|rhs|}{The new sense.}
  \EndArg
@q m e a @>

@q m b f @>
  const ABA_CSENSE &ABA_CSENSE::operator=(SENSE rhs)
@q m e f @>

@q m f sense @>
@q m e t @>
@ The function |sense()|.

@q m b r @>
  \BeginReturn
  The sense of the constraint.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CSENSE::SENSE ABA_CSENSE::sense() const
@q m e f @>

@q m f sense @>
@ This overloaded version of |sense()| changes the sense of the
  constraint.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|s|}{The new sense.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_CSENSE::sense(SENSE s) 
@q m e f @>

@q m f sense @>
@ The sense can  also be changed by a character as in the
  constructor |ABA_CSENSE(ABA_GLOBAL *glob, char s)|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|s|}{The new sense.}
  \EndArg
@q m e a @>

@(csense.cc@>=
@q m b f @>
  void ABA_CSENSE::sense(char s)
@q m e f @>
  {
    switch (s) {
      case 'l':
      case 'L': sense_ = Less;
                break;
      case 'e':
      case 'E': sense_ = Equal;
                break;
      case 'g':
      case 'G': sense_ = Greater;
                break;
      default: glob_->err() << "ABA_CSENSE::sense(): unknown argument " << s << "." << endl;
               exit(Fatal);
               break;
    }
  }
