@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q by Max B\"ohm @>
@q $Id: dualbound.w,v 4.4 2003/08/05 13:55:17 elf Exp $ @>

@q m i ABA_DUALBOUND @>

@ The member functions are defined in the file {\tt dualbound.cc}.

@(dualbound.cc@>=
#include "abacus/dualbound.h"
#include "abacus/global.h"

@q m f Constructor @>

@ The constructor.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \EndArg
@q m e a @>

@(dualbound.cc@>=
@q m b f @>
  ABA_DUALBOUND::ABA_DUALBOUND(ABA_GLOBAL *glob)
@q m e f @>
  : @/
    glob_(glob), @/
    bounds_(0)
  {
  }

@q m f Destructor @>
@q m e t @>
@ The destructor deletes the allocated memory.

@(dualbound.cc@>=
@q m b f @>
  ABA_DUALBOUND::~ABA_DUALBOUND()
@q m e f @>
  {
    delete[] bounds_;
  }

@q m f initialize @>

@ This function initializes the set of dual bounds.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|n|}{The object can hold up to n dual bounds.}
  \docarg{|minIsBest|}{If this parameter is true the
  function |best()| returns the minimum of the dual bounds,
  otherwise it returns the maximum.}
  \EndArg
@q m e a @>

@(dualbound.cc@>=
@q m b f @>
  void ABA_DUALBOUND::initialize(int n, bool minIsBest)
@q m e f @>
  {
    minIsBest_ = minIsBest;
    set_.initialize(n);
    delete[] bounds_;
    bounds_ = new @[double[n]@];
    updateBestAndWorst();
  }

@q m f insert @>
@q m e t @>
@ The function |insert()| inserts a dual bound in the set.
  If a bound with the same index already exists in the set
  the value of the bound is updated.

@q m b a @>
  \BeginArg
  \docarg{|i|}{The index of the dual bound (0..n-1).}
  \docarg{|d|}{The dual bound.}
  \EndArg
@q m e a @>

@(dualbound.cc@>=
@q m b f @>
  void ABA_DUALBOUND::insert(int i, double d)
@q m e f @>
  {
    if (set_.exists(i) && (bounds_[i]==best_ || bounds_[i]==worst_)) {
      bounds_[i] = d;
      updateBestAndWorst();
    }
    else {
      set_.insert(i);
      bounds_[i] = d;
      if (minIsBest_) {
        if (d < best_) { best_ = d; bestIndex_ = i; }
        if (d > worst_) worst_ = d;
      }
      else {
        if (d > best_) { best_ = d; bestIndex_ = i; }
        if (d < worst_) worst_ = d;
      }
    }
  }

@ The function |remove()| removes a dual bound from the set.

@q m b a @>
  \BeginArg
  \docarg{|i|}{The index of the dual bound (0..n-1).}
  \EndArg
@q m e a @>

@(dualbound.cc@>=
@q m b f @>
  void ABA_DUALBOUND::remove(int i)
@q m e f @>
  {
    if (set_.exists(i)) {
      set_.remove(i);
      if (bounds_[i]==best_ || bounds_[i]==worst_)
        updateBestAndWorst();
    }
  }

@ The function |better()|.

@q m b a @>
  \BeginArg
  \docarg{|i|}{The index of the dual bound (0..n-1).}
  \docarg{|d|}{The value of the dual bound to be tested.}
  \EndArg
@q m e a @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{if |d| is a new dual bound or |d| is better than the
                  previous dual bound at index |i|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(dualbound.cc@>=
@q m b f @>
  bool ABA_DUALBOUND::better(int i, double d) const
@q m e f @>
  {
    if (set_.exists(i)) {
      // test, if the new bound is better than the old one
      double oldBound = bounds_[i];

      if (minIsBest_) {
        if (d <= oldBound)
          return false;
      }
      else {
        if (d >= oldBound)
          return false;
      }
    }
    return true;
  }


@ The function |best()|. Returns the best dual bound (minimum
  or maximum) of all dual bounds in the set.

@q m b a @>
  \BeginArg
  \docarg{index}{An pointer to an integer to which the index of the
                 best dual bound should be stored. This is an optional
                 parameter.}
  \EndArg
@q m e a @>

@q m b r @>
  \BeginReturn
  The best dual bound.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA\_DUALBOUND::best(int *index) const
@q m e f @>


@ This version of the function |best()| returns the best dual bound (minimum
  or maximum) of all dual bounds in the set and a specified dual bound.

@q m b a @>
  \BeginArg
  \docarg{|d|}{An additional dual bound.}
  \EndArg
@q m e a @>

@q m b r @>
  \BeginReturn
  The best dual bound.
  \EndReturn
@q m e r @>

@(dualbound.cc@>=
@q m b f @>
  double ABA_DUALBOUND::best(double d) const
@q m e f @>
  {
    if (minIsBest_)
      return d < best_ ? d : best_;
    else
      return d > best_ ? d : best_;
  }

@ The function |worst()|. Returns the worst dual bound (minimum
  or maximum) of all dual bounds in the set.

@q m b r @>
  \BeginReturn
  The worst dual bound.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA\_DUALBOUND::worst() const
@q m e f @>


@ The function |updateBestAndWorst()| updates the variables |best_|, |worst_|
  and |bestIndex_|.

@(dualbound.cc@>=
  void ABA_DUALBOUND::updateBestAndWorst()
  {
    bestIndex_ = -1;
    if (minIsBest_) {
      int count = set_.count();
      best_ = glob_->infinity();
      worst_ = -glob_->infinity();
      for (int i = 0; i < count; i++) {
        double d = bounds_[set_.elem(i)];
        if (d < best_) { best_ = d; bestIndex_ = set_.elem(i); }
        if (d > worst_) worst_ = d;
      }
    }
    else {
      int count = set_.count();
      best_ = -glob_->infinity();
      worst_ = glob_->infinity();
      for (int i = 0; i < count; i++) {
        double d = bounds_[set_.elem(i)];
        if (d > best_) { best_ = d; bestIndex_ = set_.elem(i); }
        if (d < worst_) worst_ = d;
      }
    }
  }

