@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: fsvarstat.w,v 4.11 2003/08/05 13:55:18 elf Exp $ @>

@q m i ABA_FSVARSTAT @>

@ All member functions are defined in the file {\tt fsvarstat.cc}.

@(fsvarstat.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "global.h"
#include "fsvarstat.h"
#else
#include "abacus/global.h"
#include "abacus/fsvarstat.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@ This constructor initializes  the status as |Free|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to a global object.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_FSVARSTAT::ABA_FSVARSTAT(ABA_GLOBAL *glob) 
@q m e f @>

@q m f Constructor @>
@ This constructor initializes the status explicitely.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to a global object.}
  \docarg{|status|}{The initial status that must neither be |Fixed| nor |Set|.
                    For these two statuses the next constructor has to be
                    used.}
  \EndArg
@q m e a @>

@(fsvarstat.cc@>=
@q m b f @>
  ABA_FSVARSTAT::ABA_FSVARSTAT(ABA_GLOBAL *glob, STATUS status) 
@q m e f @>
   : @/
    glob_(glob), @/
    status_(status), @/
    value_(0.0)
  {
    if (status == Fixed || status == Set) {
      glob_->err() << "ABA_FSVARSTAT::ABA_FSVARSTAT(): ";
      glob_->err() << "value to set/fix missing" << endl;
      exit (Fatal);
    }
  }

@q m f Constructor @>
@ This constructor initializes the status explicitely to |Fixed| or |Set|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to a global object.}
  \docarg{|status|}{The initial status that must be |Fixed| or |Set|.}
  \docarg{|value|}{The value associated with the status |Fixed| or |Set|.}
  \EndArg
@q m e a @>

@(fsvarstat.cc@>=  
@q m b f @>
  ABA_FSVARSTAT::ABA_FSVARSTAT(ABA_GLOBAL *glob, STATUS status, double value) 
@q m e f @>
  : @/
    glob_(glob), @/
    status_(status), @/
    value_(value)
  {
    if (status != Fixed && status != Set) {
      glob_->err() << "ABA_FSVARSTAT::ABA_FSVARSTAT(): wrong status for this constructor" << endl;
      exit (Fatal);
    }
  }

@q m f Constructor @>
@ This constructor makes a copy.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|fsVarStat|}{The status is initialized with a copy of |*fsVarStat|.}
  \EndArg
@q m e a @>

@(fsvarstat.cc@>=
@q m b f @>
  ABA_FSVARSTAT::ABA_FSVARSTAT(ABA_FSVARSTAT *fsVarStat) 
@q m e f @>
  : @/
    glob_(fsVarStat->glob_), @/
    status_(fsVarStat->status_), @/
    value_(fsVarStat->value_)
  { }

#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the status from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
    \BeginArg
    \docarg{|glob|}{A pointer to a global object.}
    \docarg{|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(fsvarstat.cc@>=
@q m b f @>
  ABA_FSVARSTAT::ABA_FSVARSTAT(const ABA_GLOBAL *glob, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    glob_(glob)
  {
    status_ = (STATUS) msg.unpackInt();
    msg.unpack(value_);
  }

@q m f pack @>
@ The function |pack()| packs the data of the status in an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the status is packed.}
  \EndArg
@q m e a @>

@(fsvarstat.cc@>=
@q m b f @>
  void ABA_FSVARSTAT::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    msg.pack((int)status_);
    msg.pack(value_);
  }

#endif


@q m f Output Operator @>
@ The output operator writes the status and, if the status is
  |Fixed| or |Set|, also its value on an output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The variable status being output.}
  \EndArg
@q m e a @>

@(fsvarstat.cc@>=
@q m b f @>
  ostream &operator<<(ostream& out, const ABA_FSVARSTAT &rhs)
@q m e f @>
  {
    switch (rhs.status_) {
      case ABA_FSVARSTAT::Free:              out << "Free";
                                      break;
      case ABA_FSVARSTAT::SetToLowerBound:   out << "SetToLowerBound";
                                      break;
      case ABA_FSVARSTAT::Set:               out << "Set to " << rhs.value_;
                                      break;
      case ABA_FSVARSTAT::SetToUpperBound:   out << "SetToUpperBound";
                                      break;
      case ABA_FSVARSTAT::FixedToLowerBound: out << "FixedToLowerBound";
                                      break;
      case ABA_FSVARSTAT::Fixed:             out << "Fixed to "<< rhs.value_;
                                      break;
      case ABA_FSVARSTAT::FixedToUpperBound: out << "FixedToUpperBound";
                                       break;
      default: rhs.glob_->err() << "unknown status" << endl;
               rhs.exit(rhs.Fatal);
               break;
    }
    return out;
  }

@q m f status @>
@q m e t @>
@ The function |status()|.

@q m b r @>
  \BeginReturn
  The status of fixing or setting.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_FSVARSTAT::STATUS ABA_FSVARSTAT::status() const
@q m e f @>

@q m f status @>
@ This version of the function |status()| assigns a new status. For
  specifying also a value in case of the statuses |Fixed| or |Set| the
  next version of this function can be use.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|stat|}{The new status.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_FSVARSTAT::status(STATUS stat)
@q m e f @>

@q m f status @>
@ This version of the function |status()| can assign a new status also
  for the statuses |Fixed| and |Set|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|stat|}{The new status.}
  \docarg{|val|}{A value associated with the new status.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_FSVARSTAT::status(STATUS stat, double val)
@q m e f @>

@q m f status @>
@ A version of |status()| for assigning the status of an other object of
  the class |ABA_FSVARSTAT|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|stat|}{A pointer to the object that status and value is copied.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_FSVARSTAT::status(const ABA_FSVARSTAT *stat)
@q m e f @>

@q m f value @>
@q m e t @>
@ The function |value()|.

@q m b r @>
  \BeginReturn
  The value of fixing or setting if the variable has status |Fixed| or |Set|.
  \EndReturn
@q m e r @>fd

@q m b f @>
  double ABA_FSVARSTAT::value() const
@q m e f @>

@q m f value @>
@ This version of |value()| assigns a new value of fixing or setting.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|val|}{The new value.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_FSVARSTAT::value(double val)
@q m e f @>

@q m f fixed @>
@q m e t @>
@ The function |fixed()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the status is |FixedToLowerBound|, |Fixed|, or
                  |FixedToUpperBound|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(fsvarstat.cc@>=
@q m b f @>
  bool ABA_FSVARSTAT::fixed() const
@q m e f @>
  {
    switch (status_) {
      case FixedToLowerBound:
      case FixedToUpperBound:
      case Fixed:             return true;
      default:                return false;
    }
  }

@q m f set @>
@q m e t @>
@ The function |set()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the status is |SetToLowerBound|, |Set|, or
                  |SetToUpperBound|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(fsvarstat.cc@>=
@q m b f @>
  bool ABA_FSVARSTAT::set() const
@q m e f @>
  {
    switch (status_) {
      case SetToLowerBound:
      case SetToUpperBound:
      case Set:             return true;
      default:              return false;
    }
  }

@q m f fixedOrSet @>
@q m e t @>
@ The function |fixedOrSet()|.

@q m b r @>
  \BeginReturn
  \docarg{|false|}{If the status is |Free|,}
  \docarg{|true|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_FSVARSTAT::fixedOrSet() const
@q m e f @>

@q m f contradiction @>
@ We say there is a contradiction between two status if they are
fixed/set to different bounds or values. However, two statuses are not
contradiction if one of them is ``fixed'' and the other one is
``set'', if this fixing/setting refers to the same bound or value.
@q m e t @>
@ The function |contradiction()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If there is a contradiction between the status of this
                object and |fsVarStat|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|fsVarStat|}{A pointer to the status with which contradiction is
                       is tested.}
  \EndArg
@q m e a @>

@(fsvarstat.cc@>=
@q m b f @>
  bool ABA_FSVARSTAT::contradiction(ABA_FSVARSTAT *fsVarStat) const
@q m e f @>
  {
    STATUS status = fsVarStat->status();
    
    switch (status) {
      case Set:
      case Fixed: return contradiction(status, fsVarStat->value());
      default   : return contradiction(status);
    }
  }

@q m f contradiction @>
@ Another version of the function |contradiction()|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If there is a contradiction between the status of this
                    object and (|status|,|value|),}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|status|}{The status with which contradiction is checked.}
  \docarg{|value|}{The value with which contradiction is checked.
                   The default value of |value| is 0.}
  \EndArg
@q m e a @>

@(fsvarstat.cc@>=
@q m b f @>
  bool ABA_FSVARSTAT::contradiction(STATUS status, double value) const
@q m e f @>
  {

    switch (status_) {
      case SetToLowerBound:
      case FixedToLowerBound: switch (status) {
                                case SetToUpperBound:
                                case FixedToUpperBound:
                                case Set:
                                case Fixed:               return true;
                                
                                default:                  return false;
      }

      case SetToUpperBound:
      case FixedToUpperBound: switch (status) {
                                case SetToLowerBound:
                                case FixedToLowerBound:
                                case Set:
                                case Fixed:               return true;
                                
                                default:                  return false;
      }
      case Fixed:
      case Set:  switch (status) {
                   case Fixed:
                   case Set: if (glob_->equal(value_, value)) return true;
                             else                             return false;
                   default:  break;
      }
      default: return false;
    }
  }

