@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: global.w,v 4.22 2003/08/05 13:55:18 elf Exp $ @>

@q m i ABA_GLOBAL @>

@ All member functions are defined in the file {\tt global.cc}.

@(global.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "global.h"
#else
#include "abacus/global.h"
#endif


#ifdef ABACUS_SYS_POWERPLUS
extern "C" {
#include "/home/user/christof/abacus/include/math.h"
}
#else
#include <math.h>
#endif
#include <limits.h>
#include <stdio.h>
#include <string.h>

@q m f Constructor @>
@ The constructor initializes our filtered output and error stream
  with the standard output stream |cout|  and the standard error stream
  |cerr|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|eps|}{The zero-tolerance used within all member functions of
                   objects which have a pointer to this global object
                   (default value |1.0e-4|).}
  \docarg{|machineEps|}{The machine dependent zero tolerance
                           (default value |1.0e-7|).}
  \docarg{|infinity|}{All values greater than |infinity| are
                        regarded as ``infinite big'', all values less than
                        |-infinity| are regarded as ``infinite small''
                        (default value |1.0e30|).}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  ABA_GLOBAL::ABA_GLOBAL(double eps, double machineEps, double infinity) 
@q m e f @>
    : @/
    out_(cout), @/
    err_(cerr), @/
    eps_(eps), @/
    machineEps_(machineEps), @/
    infinity_(infinity), @/
    paramTable_(this, 100)
  {
    int tabSize = 4;
    
    tab_ = new char[tabSize+1];
    for (int i = 0; i <= tabSize; i++) tab_[i] = ' ';
    tab_[tabSize] = '\0';
  }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(global.cc@>=
@q m b f @>
  ABA_GLOBAL::~ABA_GLOBAL()
@q m e f @>
  {
    delete [] tab_;
  }


@q m f Output Operator @>
@ The output operator writes some of the data members to an ouput stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The object being output.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_GLOBAL &rhs)
@q m e f @>
  {
    out << "zero tolerance:         " << rhs.eps_ << endl;
    out << "machine zero tolerance: " << rhs.machineEps_ << endl;
    out << "infinity:               " << rhs.infinity_ << endl;
    return out;
  }

@q m v out @>
@ The function out() returns a reference to the output stream
  associated with this global object after writing |nTab| (default value 0)
  tabulators on this stream. This tabulator is not the normal
  tabulator but consists of four blanks.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the global output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|nTab|}{The number of tabulators which should be written to the
                  global output stream. The default value is 0.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  ABA_OSTREAM& ABA_GLOBAL::out(int nTab) /*const*/
@q m e f @>
  {
    for (int i = 0; i < nTab; i++) out_ << tab_;
    return out_;
  }

@q m v err @>
@ The function |err()| behaves like the function |out()| except that
  the global error stream is used instead of the global output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the global error stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|nTab|}{The number of tabulators which should be written to the
                  global error stream. The default value is 0.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  ABA_OSTREAM& ABA_GLOBAL::err(int nTab) /*const*/
@q m e f @>
  {
    for (int i = 0; i < nTab; i++) err_ << tab_;
    return err_;
  }

@q m f eps @>
@q m e t @>
@ The function |eps()|.

@q m b r @>
  \BeginReturn
  The zero tolerance.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_GLOBAL::eps() const
@q m e f @>

@q m f eps @>
@ This version of the function |eps()| sets the zero tolerance.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|e|}{The new value of the zero tolerance.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_GLOBAL::eps(double e)
@q m e f @>

@q m f machineEps @>
@ The function |machineEps()| provides a machine dependent zero
  tolerance.
  The machine dependent zero tolerance is used, e.g., to test if
  a floating point value is 0.
  This value is usually less than |eps()|, which provides, e.g.,
  a safety tolerance if a constraint is violated.
@q m e t @>

@q m b r @>
  \BeginReturn
  The machine dependent zero tolerance.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_GLOBAL::machineEps() const
@q m e f @>

@q m f machineEps @>
@ This version of the function |machineEps()| sets the machine
  dependent zero tolerance.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|e|}{The new value of the machine dependent zero tolerance.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_GLOBAL::machineEps(double e)
@q m e f @>

@q m f infinity @>
@ The function |infinity()| provides a floating point  value of 
  ``infinite'' size. Especially, we assume that |-infinity()| is
  the lower and |infinity()| is the upper bound of an unbounded variable
  in the linear program. 
@q m e t @>

@q m b r @>
  \BeginReturn
  A very large floating point number. The default value of |infinity()| is
  |1.0e30|.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_GLOBAL::infinity() const
@q m e f @>

@q m f infinity @>
@ This version of the function |infinity()| sets the ``infinite value''.
@q m e t @>
  
@q m b a @>
  \BeginArg
  \docarg{|x|}{The new value representing ``infinity''.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_GLOBAL::infinity(double x)
@q m e f @>

@q m f isInfinity @>
@q m e t @>
@ The function |isInfinity()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |x| is regarded as ``infinite'' large,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{The value compared with ``infinity''.}
  \EndArg
@q m e a @>

@q m b f @>
  bool ABA_GLOBAL::isInfinity(double x) const
@q m e f @>
  
@q m f isMinusInfinity @>
@q m e t @>
@ The function |isMinusInfinity()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |x| is regarded as infinite small;}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{The value compared with ``minus infinity''.}
  \EndArg
@q m e a @>

@q m b f @>
  bool ABA_GLOBAL::isMinusInfinity(double x) const
@q m e f @>

@q m f equal @>
@q m e t @>
@ The function |equal()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the absolute difference of |x| and |y| is less than
                  the machine dependent zero tolerance,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{The first value being compared.}
  \docarg{|y|}{The second value being compared.}
  \EndArg
@q m e a @>

@q m b f @>
  bool ABA_GLOBAL::equal(double x, double y) const
@q m e f @>

@q m f isInteger @>
@q m e t @>
@ The function |isInteger()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the value |x| differs at most by the machine
                  dependent zero tolerance from an integer value,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_GLOBAL::isInteger(double x) const
@q m e f @>

@q m f isInteger @>
@q m e t @>
@ The function |isInteger()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the value |x| differs at most by |eps|
                  from an integer value,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(global.cc@>=
@q m b f @>
  bool ABA_GLOBAL::isInteger(double x, double eps) const
@q m e f @>
  {
     double frac = fracPart(x);

     if ((frac > eps) &&
         (frac < 1.0 - eps))
       return false;    
     else
       return true;
  }

@q m v enter @>
@ The virtual function |enter()| displays the string {\tt ENTER>}
  on the global output stream and waits for a character on
  the input stream |in|, e.g., a keystroke if |in == cin|.
@q m e t @>

@q m b r @>
  \BeginReturn
  The character read from the input stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|in|}{The input stream the character should be read from.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  char ABA_GLOBAL::enter(istream &in) 
@q m e f @>
  {
    char c;
    char d;

    out() << endl << "ENTER>" << flush;
    
    c = (char) in.get();
    d = c;
    while (d != '\n') d = (char) in.get();

    return c;

  }

@q m f insertParameter @>
@ The  |insertParameters()| inserts a parameter in the parameter table.
If the parameter already is in the table, the value is overwritten.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|name|}{The name of the parameter.}
  \docarg{|value|}{The literal value of the parameter.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::insertParameter(const char *name, const char *value)
@q m e f @>
  {
    if(!name||!value) {
      err() << "ABA_GLOBAL:insertParameter(): both arguments must ";
      err() << "be non-zero pointers" << endl;
      exit(Fatal);
    }
    ABA_STRING stName(this,name);
    ABA_STRING stValue(this,value);
    paramTable_.overWrite(stName, stValue);
  }

@q m f readParameters @>
@ The function |readParameters()| opens the parameter file |fileName|,
  reads all parameters, and inserts them in the parameter table.
  A parameter file may have at most 1024 characters per line.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|fileName|}{The name of the parameter file.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::readParameters(const char *fileName)
@q m e f @>
  {
    @<|ABA_GLOBAL::readParameters()|: local variables@>;
    @<open the parameter file |fileName|@>;
    @<read the parameter file line by line@>;
  }

@ Setting the maximal number of characters per line to 1024 is not very
  nice, but should usually not generate too much trouble. We might remove
  this restrictions in a future release. The buffers for the strings
  have space for one additional character, the terminating |'\0'|.
  
@<|ABA_GLOBAL::readParameters()|: local variables@>=
  const int maxCharPerLine = 1025;
  char      line[maxCharPerLine];
  char      nameBuf[maxCharPerLine];
  char      valueBuf[maxCharPerLine];
  ABA_STRING    name(this);
  ABA_STRING    value(this);
  int       status;

@ @<open the parameter file |fileName|@>=
//CHANGED  ifstream paramFile(fileName, ios::in | ios::nocreate);
  ifstream paramFile(fileName, ios::in);

  if (!paramFile) {
    err() << "ABA_GLOBAL::readParameters(): opening file " << fileName << " failed" << endl;
    exit(Fatal);
  }

@ Lines in a parameter file starting with |'#'| are comments and hence they
  are skipped. Every other non-void line must contain two strings. The
  first one is the name of the parameter, the second one its value.

  The function |sscanf()| returns the number of read arguments. Empty lines
  for which |sscanf()| returns 0 are ignored.
  
@<read the parameter file line by line@>=
  while (!paramFile.eof()) {
    paramFile.getline(line, maxCharPerLine);
    if (line[0] == '#') continue;
    status = sscanf(line, "%s%s", nameBuf, valueBuf);
    if (status == 1) {
      err() << "ABA_GLOBAL::readParameters(" << fileName << "): ";
      err() << "value missing for parameter " << nameBuf << endl;
      exit(Fatal);
    }
    else if (status == 2) {
      name = nameBuf;
      value = valueBuf;
      paramTable_.overWrite(name, value);
    }
  }

@q m f  assignParameter  @>
@ The function |assignParameter()| searches for the parameter |name| in
  the parameter table. If no parameter |name| is found and no default value
  of the parameter is given, the program terminates with an error messages.
  The program terminates also with an error message if the value of a 
  parameter is not within a specified feasible region.
  Depending on the type of the parameter, a feasible region can be an 
  interval (specified by |minVal| and |maxVal|) or can be given by
  a set of feasible settings (given by a number |nFeasible| and a pointer
  |feasible| to the feasible values. 

  This function is overloaded in two ways. First, this function is defined 
  for different types of  the argument |parameter|,  second, for each such 
  type we have both versions, with and without a default value of
  the parameter.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|param|}{The variable |parameter| receives the value of the
                      parameter.}
  \docarg{|name|}{The name of the parameter.}
  \docarg{|minVal|}{The  value of the parameter is considered as infeasible
                     if it is less than |minVal|.}
  \docarg{|maxVal|}{The  value of the parameter is considered as infeasible
                     if it is larger than |maxVal|.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(int &param, const char *name, 
                                   int minVal, int maxVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      err() << "ABA_GLOBAL::assignParameter(): parameter ";
      err() << name << " not found in parameter table." << endl;
      exit(Fatal);
    }
    if(param<minVal||param>maxVal) {
      err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
      err() << " is out of range" << endl;
      err() << "value: " << param << "," ;
      err() << " feasible range: " << minVal << " ... " << maxVal << endl;
      exit(Fatal);
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(unsigned &param, const char *name, 
                                   unsigned minVal, unsigned maxVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      err() << "ABA_GLOBAL::assignParameter(): parameter ";
      err() << name << " not found in parameter table." << endl;
      exit(Fatal);
    }
    if(param<minVal||param>maxVal) {
      err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
      err() << " is out of range" << endl;
      err() << "value: " << param << "," ;
      err() << " feasible range: " << minVal << " ... " << maxVal << endl;
      exit(Fatal);
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(double &param, const char *name, 
                                   double minVal, double maxVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      err() << "ABA_GLOBAL::assignParameter(): parameter ";
      err() << name << " not found in parameter table." << endl;
      exit(Fatal);
    }
    if(param<minVal||param>maxVal) {
      err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
      err() << " is out of range" << endl;
      err() << "value: " << param << "," ;
      err() << " feasible range: " << minVal << " ... " << maxVal << endl;
      exit(Fatal);
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(bool &param, const char *name)
@q m e f @>
  {   
    if(getParameter(name, param)){
      err() << "ABA_GLOBAL::assignParameter(): parameter ";
      err() << name << " not found in parameter table." << endl;
      exit(Fatal);
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|param|}{The variable |parameter| receives the value of the
                      parameter.}
  \docarg{|name|}{The name of the parameter.}
  \docarg{|nFeasible|}{The number of feasible settings. If |nFeasible| is
                       equal to zero, then all values are allowed. 0
                       is the default value.}
  \docarg{|feasible|}{If |nFeasible| is greater zero, the this are
       the settings for the parameter to be considered as
                        feasible. Must be an array of |nFeasible| strings.}
  \EndArg
@q m e a @>


@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(ABA_STRING &param, const char *name, 
                                   unsigned nFeasible, const char *feasible[])
@q m e f @>
  {
    if(getParameter(name, param)){
      err() << "ABA_GLOBAL::assignParameter(): parameter ";
      err() << name << " not found in parameter table." << endl;
      exit(Fatal);
    }
    if(nFeasible){
      unsigned i;
      for(i=0;i<nFeasible;i++) {
        ABA_STRING stFeas(this,feasible[i]);
        if(param==stFeas)
          break;
      }
      if(i==nFeasible) {
        err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
        err() << " is not feasible." << endl;
        err() << "value of parameter: " << param  << endl;     
        err() << "feasible settings : ";
        for(i=0;i<nFeasible;i++) {
          if(i>0)
            err() << "                    ";
          err() << feasible[i];
          err() << endl;
        }         
        exit(Fatal);
      }
    }
  }



@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|param|}{The variable |param| receives the value of the
                      parameter.}
  \docarg{|name|}{The name of the parameter.}
  \docarg{|feasible|}{A string consisting of all feasible characters. 
           If |feasible| is zero, then all characters are allowed.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(char &param, const char *name, 
                                   const char *feasible)
@q m e f @>
  {
    if(getParameter(name, param)){
      err() << "ABA_GLOBAL::assignParameter(): parameter ";
      err() << name << " not found in parameter table." << endl;
      exit(Fatal);
    }
    if(feasible){
      int i,len;
      ABA_STRING stFeasible(this,feasible);
      len=stFeasible.size();
      for(i=0;i<len;i++) 
        if(stFeasible[i]==param)
          break;
      if(feasible&&i==len) {
        err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
        err() << " is not feasible." << endl;
        err() << "value: " << param << ",";
        err() << "feasible settings: " << feasible << endl;
        exit(Fatal);
      }
    }
  }


@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|parameter|}{The variable |parameter| receives the value of the
                      parameter.}
  \docarg{|name|}{The name of the parameter.}
  \docarg{|minVal|}{The  value of the parameter is considered as infeasible
                     if it is less than |minVal|.}
  \docarg{|maxVal|}{The  value of the parameter is considered as infeasible
                     if it is larger than |maxVal|.}
  \docarg{|defVal|}{ The default value that is used when the paramter is 
                       not found in the parameter table.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(int &param, const char *name, 
                                   int minVal, int maxVal, 
                                   int defVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      param=defVal;
    }
    if(param<minVal||param>maxVal) {
      err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
      err() << " is out of range" << endl;
      err() << "value: " << param << "," ;
      err() << " feasible range: " << minVal << " ... " << maxVal << endl;
      exit(Fatal);
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(unsigned &param, const char *name, 
                                   unsigned minVal, unsigned maxVal,
                                   unsigned defVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      param=defVal;
    }
    if(param<minVal||param>maxVal) {
      err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
      err() << " is out of range" << endl;
      err() << "value: " << param << "," ;
      err() << " feasible range: " << minVal << " ... " << maxVal << endl;
      exit(Fatal);
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(double &param, const char *name, 
                                   double minVal, double maxVal,
                                   double defVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      param=defVal;
    }
    if(param<minVal||param>maxVal) {
      err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
      err() << " is out of range" << endl;
      err() << "value: " << param << "," ;
      err() << " feasible range: " << minVal << " ... " << maxVal << endl;
      exit(Fatal);
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(bool &param, const char *name, 
                                   bool defVal)
@q m e f @>
  {   
    if(getParameter(name, param)){
      param=defVal;
    }
  }

@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|parameter|}{The variable |parameter| receives the value of the
                      parameter.}
  \docarg{|name|}{The name of the parameter.}
  \docarg{|nFeasible|}{The number of feasible settings. If |nFeasible| is
                       equal to zero, then all settings are allowed.}
  \docarg{|feasible|}{The settings for the parameter to be considered as
                        feasible. Must be an array of |nFeasible| strings.}
  \docarg{|defVal|}{The default value that is used when the paramter is 
                       not found in the parameter table.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(ABA_STRING &param, const char *name, 
                                   unsigned nFeasible, const char *feasible[],
                                   const char *defVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      param=defVal;
    }
    if(nFeasible){
      unsigned i;
      for(i=0;i<nFeasible;i++) {
        ABA_STRING stFeas(this,feasible[i]);
        if(param==stFeas)
          break;
      }
      if(i==nFeasible) {
        err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
        err() << " is not feasible." << endl;
        err() << "value of parameter: " << param <<   endl;     
        err() << "feasible settings : ";
        for(i=0;i<nFeasible;i++) {
          if(i>0)
            err() << "                    ";
          err() << feasible[i];
          err() << endl;
        }         
        exit(Fatal);
      }
    }
  }
 
@q m f  assignParameter  @>
@ See \ref{ABA?GLOBAL::assignParameter} for description.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|param|}{The variable |param| receives the value of the
                      parameter.}
  \docarg{|name|}{The name of the parameter.}
  \docarg{|feasible|}{A string containing all feasible settings. 
   If |feasible| is zero, then all settings are allowed.}  
    \docarg{|defVal|}{The default value that is used when the paramter is 
                       not found in the parameter table.}
  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  void ABA_GLOBAL::assignParameter(char &param, const char *name, 
                                   const char* feasible, char defVal)
@q m e f @>
  {
    if(getParameter(name, param)){
      param=defVal;
    }
    if(feasible){
      int i;
      ABA_STRING stFeasible(this,feasible);
      int len=stFeasible.size();
      for(i=0;i<len;i++) 
      if(stFeasible[i]==param)
        break;
      if(i==len) {
        err() << "ABA_GLOBAL::assignParameter(): parameter " << name;
        err() << " is not feasible." << endl;
        err() << "value: " << param << ",";
        err() << "feasible settings: " << feasible << endl;
        exit(Fatal);
      }
    }
  }
@q m f findParameter @>
@ The function |findParameter()| searches for the parameter |name| in
  the parameter table. If no parameter |name| is found 
  the program terminates with an error messages.
  The program terminates also with an error message if the value of a 
  parameter is not within a given list of  feasible settings.
  This function is overloaded and can be used for different types of 
  parameters such as integer valued, char valued and string parameters.
@q m e t @>
@q m b r @>
  \BeginReturn
  The index of the matched feasible setting.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|name|}{The name of the parameter.}
  \docarg{|nFeasible|}{The number of feasible settings.}
  \docarg{|feasible|}{The settings for the parameter to be considered as
                        feasible. Must be an array of |nFeasible| strings.}

  \EndArg
@q m e a @>

@(global.cc@>=
@q m b f @>
  int ABA_GLOBAL::findParameter(const char *name,
                                unsigned nFeasible,
                                const int *feasible)
@q m e f @>
  {
    unsigned i;
    int param;
    assignParameter(param,name,INT_MIN,INT_MAX);
    for(i=0;i<nFeasible;i++) 
      if(feasible[i]==param)
        break;
    if(i==nFeasible) {
      err() << "ABA_GLOBAL::findParameter(): parameter " << name;
      err() << " is not feasible." << endl;
      err() << "value of parameter: " << param <<   endl;     
      err() << "feasible settings : ";
      for(i=0;i<nFeasible;i++) 
        err() << feasible[i];
      err() << endl;
      exit(Fatal);      
    }
    return i;
  }

@q m f  findParameter  @>
@ See \ref{ABA?GLOBAL::findParameter} for description.
@q m e t @>

@(global.cc@>=
@q m b f @>
  int ABA_GLOBAL::findParameter(const char *name,
                                unsigned nFeasible,
                                const char *feasible[])
@q m e f @>
  {
    unsigned i;
    ABA_STRING param(this);
    assignParameter(param,name);
    for(i=0;i<nFeasible;i++) {
      ABA_STRING stFeas(this,feasible[i]);      
      if(stFeas==param)
        break;
    }
    if(i==nFeasible) {
      err() << "ABA_GLOBAL::findParameter(): parameter " << name;
      err() << " is not feasible." << endl;
      err() << "value of parameter: " << param <<   endl;     
      err() << "feasible settings : ";
      for(i=0;i<nFeasible;i++) {
        if(i>0)
          err() << "                    ";
        err() << feasible[i];
        err() << endl;
      }   
      exit(Fatal);      
    }
    return i;
  }

@q m f  findParameter  @>
@ See \ref{ABA?GLOBAL::findParameter} for description.
@q m e t @>

@(global.cc@>=
@q m b f @>
  int ABA_GLOBAL::findParameter(const char *name,
                                const char *feasible)
@q m e f @>
  {

    if(!feasible){
      err() << "ABA_GLOBAL::findParameter(const char*, const char*): " <<
        "second argument must be non-zero." << endl;
      exit(Fatal);
    }
    int i;
    char param;
    assignParameter(param,name);
    int len=strlen(feasible);
    for(i=0;i<len;i++) {
      if(feasible[i]==param)
        break;
    }
    if(i==len) {
      err() << "ABA_GLOBAL::findParameter(): parameter " << name;
      err() << " is not feasible." << endl;
      err() << "value of parameter: " << param <<   endl;     
      err() << "feasible settings : " << feasible << endl;
      exit(Fatal);      
    }
    return i;
  }


@q m f getParameter @>
@ The function |getParameter()| searches for the parameter |name| in
  the parameter table. 
  This function is overloaded for different types of the argument |parameter|.
  See also the functions |assignParameter| and |findParameter| with 
  enhanced functionality. 
@q m e t @>
@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the parameter is found,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|name|}{The name of the parameter.}
  \docarg{|parameter|}{The variable |parameter| receives the value of the
                      parameter, if the function returns 1, otherwise it
                      is undefined.}
  \EndArg
@q m e a @>

  The function |find()| returns the a pointer to the string storing the
  value of the parameter. Then we have to convert this string to the
  type of |parameter|.

@(global.cc@>=
@q m b f @>
  int ABA_GLOBAL::getParameter(const char *name, int &parameter)
@q m e f @>
  {
    ABA_STRING  nameString(this, name);
    
    ABA_STRING *s = paramTable_.find(nameString);
    if  (s == 0)
      return 1;
    else {
      parameter = s->ascii2int();
      return 0;
    }
  }

@q m f getParameter @>
@q m e t @>

@q m b f @>
  int ABA_GLOBAL::getParameter(const char *name, unsigned int &parameter)
@q m e f @>
  {
    ABA_STRING  nameString(this, name);
    
    ABA_STRING *s = paramTable_.find(nameString);
    if  (s == 0)
      return 1;
    else {
      parameter = s->ascii2unsignedint();
      return 0;
    }
  }
  
@q m f getParameter @>
@q m e t @>

@q m b f @>
  int ABA_GLOBAL::getParameter(const char *name, double &parameter)
@q m e f @>

  {
    ABA_STRING  nameString(this, name);

    ABA_STRING *s = paramTable_.find(nameString);
    if  (s == 0)
      return 1;
    else {
      parameter = s->ascii2double();
      return 0;
    }
  }
  
@q m f getParameter @>
@q m e t @>

@q m b f @>
  int ABA_GLOBAL::getParameter(const char *name, ABA_STRING &parameter)
@q m e f @>
  {
    ABA_STRING  nameString(this, name);

    ABA_STRING *s = paramTable_.find(nameString);
    if  (s == 0)
      return 1;
    else {
      parameter = *s;
      return 0;
    }
  }
  
@q m f getParameter @>
@q m e t @>

@q m b f @>
  int ABA_GLOBAL::getParameter(const char *name, bool &parameter)
@q m e f @>
  {
    ABA_STRING  nameString(this, name);

    ABA_STRING *s = paramTable_.find(nameString);
    if  (s == 0)
      return 1;
    else {
      parameter = s->ascii2bool();
      return 0;
    }
  }

@q m f getParameter @>
@q m e t @>

@q m b f @>
  int ABA_GLOBAL::getParameter(const char *name, char &parameter)
@q m e f @>
  {
    ABA_STRING  nameString(this, name);

    ABA_STRING *s = paramTable_.find(nameString);
    if  (s == 0)
      return 1;
    else {
      parameter = (s->string())[0];
      return 0;
    }
  }
