@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: history.w,v 4.12 2003/08/05 13:55:18 elf Exp $ @>

@q m i ABA_HISTORY @>

@ The member and friend functions are implemented in the file {\tt history.cc}.

@(history.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "history.h"
#include "sub.h"
#include "master.h"
#else
#include "abacus/history.h"
#include "abacus/sub.h"
#include "abacus/master.h"
#endif

@q m f Constructor @>
@ The constructor initializes a history table with 100 possible entries.
  If this number is exceeded an automatic reallocation is performed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|master|}{A pointer to the corresponding master of the optimization.}
  \EndArg
@q m e a @>

@(history.cc@>=
@q m b f @>
  ABA_HISTORY::ABA_HISTORY(ABA_MASTER *master) 
@q m e f @>
  : @/
    master_(master), @/
    primalBound_(master, 100), @/
    dualBound_(master, 100), @/
    time_(master, 100), @/
    n_(0)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.
 
@(history.cc@>=
@q m b f @>
  ABA_HISTORY::~ABA_HISTORY()
@q m e f @>
  {
  }

@q m f Output Operator @>
@q m e t @>
@ The output operator.

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The solution history being output.}
  \EndArg
@q m e a @>

@(history.cc@>=
#include <math.h>

@q m b f @>
  ostream& operator<<(ostream& out, const ABA_HISTORY &rhs)
@q m e f @>
  {
    @<|ABA_HISTORY|: local variables of the output operator@>;
    @<|ABA_HISTORY|: constants for formatting output@>;
    
    out << "Solution History" << endl << endl;
    
    if (rhs.n_) {
      @<headline of history table@>;
      @<has the optimum solution been proved?@>;
      @<output the history table@>;
    }
    else
      out << "no solution history available" << endl;
    return out;
  }

@ @<|ABA_HISTORY|: local variables of the output operator@>=
    double    guarantee; // guarantee of a solution in the history
    double    quality;   // quality of a solution in the history
    bool      opt;       // |true| if problem has been solved to optimality
    bool      qAvailable;   // |true| if quality can be given at this point of history
    bool      gAvailable;   // |true| if guarantee can be given at this point of history
    bool      feasible;  // |true| if feasible solution available at this point of history
    int       last;      // index of last entry in history table
    double    ub;        // upper bound
    double    lb;        // lower bound
    double    optVal;    // value of the optimum solution
    long      min;       // total minutes
    long      rmin;      // total minutes modulo 60
    long      rsec;      // total seconds modulo 60

    const double eps      = rhs.master_->eps();
    const double infinity = rhs.master_->infinity();
    const bool   max      = rhs.master_->optSense()->max();

@ @<|ABA_HISTORY|: constants for formatting output@>=
    const int w1 = 10;   // width of first column
    const int w2 = 13;   // width of second column
    const int w3 = 13;   // width of third column
    const int w4 = 12;   // width of forth column
    const int w5 = 12;   // width of fifth column
    const int w6 = 12;   // width of sixth column

@ If the optimum solution has been proved we can also output the
  quality of every feasible solution in the history table.

@<has the optimum solution been proved?@>=
    
  last = rhs.n_ - 1;

  if (fabs(rhs.primalBound_[last] - rhs.dualBound_[last]) < eps) {
    opt    = true;
    optVal = rhs.primalBound_[last];
  }
  else opt = false;
     
@ @<headline of history table@>=
  out << setw(w1) << "Solution";

  if (rhs.master_->optSense()->max()) {
    out << setw(w2) << "Feas. Sol.";
    out << setw(w3) << "Upper Bound";
  }
  else {
    out << setw(w2) << "Lower Bound";
    out << setw(w3) << "Feas. Sol.";
  }

  out << setw(w4) << "Guarantee";
  out << setw(w5) << "Quality";
  out << setw(w6) << "Time";
  out << endl;  

@ @<output the history table@>=
  for(int i = 0; i <= last; i++) {
    @<determine guarantee and quality@>;
    
    out << setw(w1) << i;

    out << setw(w2) << lb;
    out << setw(w3) << ub;

    if (gAvailable) out << setw(w4-1) << guarantee << "%";
    else         out << setw(w4)   << "---";
    if (qAvailable) out << setw(w5-1) << quality << "%";
    else         out << setw(w5)   << "---";
        
    @<output the history time@>;
    out << endl;
  }

@ @<determine guarantee and quality@>=
  @<determine the upper and the lower bound@>;
  @<determine if a feasible solutions is currently available@>;
  @<determine the guarantee@>;
  @<determine the quality@>;

@ In a maximization problem the lower bounds are the primal feasible solutions
  und the upper bounds the dual feasible solutions, in a minimization
  problem this is vice versa.

@<determine the upper and the lower bound@>=
  if (max) {
    ub = rhs.dualBound_[i];
    lb = rhs.primalBound_[i];
  }
  else {
    lb = rhs.dualBound_[i];
    ub = rhs.primalBound_[i];
  }

@ A feasible solution is available if the lower bound is not minus
  infinity for maximization problems, and the upper bound not infinity
  for minimization problems, respectively.
  
@<determine if a feasible solutions is currently available@>=
  if (max)
    feasible = (lb == -infinity ? false : true);
  else
    feasible = (ub == infinity ? false : true);
  
@ A guarantee can only be given if a feasible solution
  is available, i.e., the lower bound is not minus infinity for minimization
  problems, or the upper bound is not infinity for maximization problems,
  respectively, and the lower bound is either nonzero or equals the upper
  bound.

@<determine the guarantee@>=
  if (feasible) {
    if (max)
      gAvailable = (ub == infinity ? false : true);
    else 
      gAvailable = (lb == -infinity ? false : true);

    if (gAvailable) {
      if (fabs(lb) > eps)          guarantee = fabs((ub - lb)/lb*100.0);
      else if(fabs(ub - lb) < eps) guarantee = 0.0;
      else gAvailable = false;
    }
  }
  else gAvailable = false;

  

@ The quality can be only determined if the optimum solution has been
  found and at the current point of the history a feasible solution is
  available. Moreover, the nominator in the expression for the guarantee
  computation must either be nonzero, or the denominator must be zero.

@<determine the quality@>=
  if (opt && feasible) {
    if (max) {
      if (fabs(lb) > eps) {
        qAvailable = true;
        quality = fabs((optVal - lb)/lb*100.0);
      }
      else if (fabs(optVal - lb) < eps) {
        qAvailable = true;
        quality = 0.0;
      }
      else qAvailable = false;
    }
    else {
      if (fabs(optVal) > eps) {
        qAvailable = true;
        quality = fabs((ub - optVal)/optVal*100.0);
      }
      else if (fabs(ub - optVal) < eps) {
        qAvailable = true;
        quality= 0.0;
      }
      else qAvailable = false;
    }

  }
  else qAvailable = false;
  
@ The time is recorded in seconds and is transformed to the
  format {\tt hh:mm:ss}. Up to 999 hours the output will be nice.
  
@<output the history time@>=
    rsec = rhs.time_[i]%60;
    min  = rhs.time_[i]/60;
    rmin = min%60;

    out << setw(w6-9) << "";  // to get the correct width
    out << setw(3) << min/60 << ":";
    if(rmin < 10) out << '0';
    out << rmin << ':';
    if(rsec < 10) out << '0';
    out << rsec;
  

@q m f update @>
@ The function |update()| adds an additional line to the history table,
  primal bound, dual bound, and the time are taken from the
  corresponding master object. 
  The history table is automatically reallocated if necessary.
  
  Usually an explicit call to this function from an application class
  is not required since |update()| is automatically called if
  a new global primal or dual bound is found.
@q m e t @>

@(history.cc@>=
@q m b f @>
  void ABA_HISTORY::update()
@q m e f @>
  {
    if (n_ == size()) realloc();

    dualBound_[n_]   = master_->dualBound();
    primalBound_[n_] = master_->primalBound();
    time_[n_]        = master_->totalTime()->seconds();;
    ++n_;
  }

@ The function |size()| returns the length of the history table.

  int ABA_HISTORY::size() const

@ The function |realloc()| enlarges the history table by 100 components.

@(history.cc@>=
  void ABA_HISTORY::realloc()
  {
    int newSize = size() + 100;
 
    primalBound_.realloc(newSize);
    dualBound_.realloc(newSize);
    time_.realloc(newSize);
  }
