@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q by Max B\"ohm @>
@q $Id: id.w,v 4.3 2003/08/05 13:55:19 elf Exp $ @>

@q m i ABA_ID @>

@ The member functions are implemented in the file {\tt id.cc}.


@(id.cc@>=
#include "abacus/id.h"
#include "abacus/message.h"


@q m f Constructor @>
@ The constructor. |sequence_| will be set to 0 indicating that the
  |ABA_ID| is not initialized.
@q m e t @>

@(id.cc@>=
@q m b f @>
  ABA_ID::ABA_ID()
@q m e f @>
  : @/
    sequence_(0)
  {
  }


@q m f Constructor @>
@ The message constructor.
@q m e t @>

@q m b a @>
    \BeginArg
    \docarg{|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(id.cc@>=
@q m b f @>
  ABA_ID::ABA_ID(ABA_MESSAGE &msg)
@q m e f @>
  {
    msg.unpack(sequence_);
    msg.unpack(proc_);
    msg.unpack(index_);
  }


@q m f pack @>
@ The function |pack()| packs the |ABA_ID| in an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the |ABA_ID| is packed.}
  \EndArg
@q m e a @>

@(id.cc@>=
@q m b f @>
  void ABA_ID::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    msg.pack(sequence_);
    msg.pack(proc_);
    msg.pack(index_);
  }


@q m f Output Operator @>
@q m e t @>
@ The output operator.

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|id|}{The |ABA_ID| being output.}
  \EndArg
@q m e a @>

@(id.cc@>=
@q m b f @>
  ostream& operator<<(ostream &out, const ABA_ID &id)
@q m e f @>
  {
    if (id.sequence_ == 0)
      return out << "(uninitialized id)";
    else
      return out << '(' << id.sequence_ << ',' << id.proc_ << ','
                 << id.index_ << ')';
  }


@q m f Comparison Operator @>
@q m e t @>
@ The comparison operator.

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If both |ABA_ID| objects are not equal,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|lhs|}{The left hand side of the comparison.}
  \docarg{|rhs|}{The right hand side of the comparison.}
  \EndArg
@q m e a @>

@(id.cc@>=
@q m b f @>
  int operator==(const ABA_ID &lhs, const ABA_ID &rhs)
@q m e f @>
  {
    return lhs.sequence_ == rhs.sequence_ && lhs.proc_ == rhs.proc_ &&
           lhs.index_ == rhs.index_;
  }

@q m f initialize @>
@ The function |initialize()| sets the processor number and the sequence
  number.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|sequence|}{The sequence number of the |ABA_ID|.}
  \docarg{|proc|}{The processor number of the |ABA_ID|.}
  \docarg{|index|}{If a constraint/variable is identified, then this
                  optional parameter determines the index of the pool
                  in which the constraint/variable is stored.}
  \EndArg
@q m e a @>

@(id.cc@>=
@q m b f @>
  void ABA_ID::initialize(unsigned long sequence, int proc, int index)
@q m e f @>
  {
    sequence_ = sequence;
    proc_ = proc;
    index_ = index;
  }

@q m f uninitialize @>
@ The function |uninitialize()| sets the |ABA_ID| to the uninitialized state.
@q m e t @>

@(id.cc@>=
@q m b f @>
  void ABA_ID::uninitialize()
@q m e f @>
  {
    sequence_ = 0;
  }

@q m f isInitialized @>
@ The function |isInitialized()| tests if the |ABA_ID| was initialized.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|false|}{If the |ABA_ID| is not initialized.}
  \docarg{|true|}{If the |ABA_ID| is initialized.}
  \EndReturn
@q m e r @>

@(id.cc@>=
@q m b f @>
  bool ABA_ID::isInitialized() const
@q m e f @>
  {
    return sequence_ != 0;
  }



