@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: infeascon.w,v 4.10 2003/08/05 13:55:19 elf Exp $ @>

@q m i ABA_INFEASCON @>

@ The member functions are defined in the file {\tt infeascon.cc}.

@(infeascon.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "infeascon.h"
#include "convar.h"
#include "variable.h"
#include "constraint.h"
#include "master.h"
#else
#include "abacus/infeascon.h"
#include "abacus/convar.h"
#include "abacus/variable.h"
#include "abacus/constraint.h"
#include "abacus/master.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
  \BeginArg
  \docarg{|master|}{A pointer to the corresponding master of the optimization.}
  \docarg{|con|}{The infeasible constraint.}
  \docarg{|inf|}{The way of infeasibility.}
  \EndArg
@q m e a @>

@(infeascon.cc@>=
@q m b f @>
  ABA_INFEASCON::ABA_INFEASCON(ABA_MASTER *master, ABA_CONSTRAINT *con, INFEAS inf) 
@q m e f @>
  : @/
    master_(master), @/
    constraint_(con), @/
    infeas_(inf)
  {  }

@q m f constraint @>
@q m e t @>
@ The function |constraint()|.

@q m b r @>
  \BeginReturn
  A pointer to the infeasible constraint.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CONSTRAINT *ABA_INFEASCON::constraint()
@q m e f @>

@q m f infeas @>
@q m e t @>
@ The function |infeas()|.

@q m b r @>
  \BeginReturn
  The way of infeasibility of the constraint.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_INFEASCON::INFEAS ABA_INFEASCON::infeas() const
@q m e f @>
  
@q m f goodVar @>
@q m e t @>
@ The function |goodVar()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the variable |v| might reduce the infeasibility,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|v|}{A variable for which we test if its addition might reduce
               infeasibility.}
  \EndArg
@q m e a @>

@(infeascon.cc@>=
@q m b f @>
  bool ABA_INFEASCON::goodVar(ABA_VARIABLE *v)
@q m e f @>
  {
    double eps = master_->machineEps();
    
#ifdef ABACUS_NO_BOOL
    bool pos;
    bool neg;
 
    if (v->uBound() > eps)  pos = true;
    else                    pos = false;
    if (v->lBound() < -eps) neg = true;
    else                    neg = false;
#else
    bool pos = v->uBound() > eps;
    bool neg = v->lBound() < -eps;
#endif
    double c = constraint_->coeff(v);
    
    if (infeas_ == TooSmall) {
      if (c > eps && pos)  return true;
      if (c < -eps && neg) return true;
      return false;
    }
    else if (infeas_ == TooLarge) {
      if (c > eps && neg)  return true;
      if (c < -eps && pos) return true;
      return false;
    }
    else {
      master_->err() << "ABA_INFEASCON::goodVar(): constraint is feasible" << endl;
      exit(Fatal);
      return false;  // to suppress compiler warnings
    }
  }
