@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q by Max B\"ohm @>
@q $Id: intset.w,v 4.4 2003/08/05 13:55:19 elf Exp $ @>

@q m i ABA_INTSET @>

@ The member functions are defined in the file {\tt intset.cc}.

  The set is implemented by the arrays |uninitializedMap_[]| and
  |position_[]|. The following holds for each element e stored in
  the set.

  a) $0 \le |uninitializedMap_[e]| < |count_|$

  b) $|position_[uninitializedMap_[e]]| = e$

  It is not required to initialize the arrays.

@(intset.cc@>=
#include "abacus/intset.h"

@q m f Constructor @>

@ The constructor. An empty set is created. $O(n)$ bytes of memory are
  allocated for the internal representation of the data structure.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|n|}{The created object can store subsets of $\{0,\dots,n-1\}$.}
  \EndArg
@q m e a @>

@(intset.cc@>=
@q m b f @>
  ABA_INTSET::ABA_INTSET()
@q m e f @>
  :
    uninitializedMap_(0), @/
    position_(0), @/
    count_(0)
  {
  }

@q m f Destructor @>
@q m e t @>
@ The destructor deletes the allocated memory.

@(intset.cc@>=
@q m b f @>
  ABA_INTSET::~ABA_INTSET()
@q m e f @>
  {
    delete[] uninitializedMap_;
    delete[] position_;
  }

@q m f initialize @>

@ An empty set is initialized. $O(n)$ bytes of memory are
  allocated for the internal representation of the data structure.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|n|}{The initialized object can store subsets of $\{0,\dots,n-1\}$.}
  \EndArg
@q m e a @>

@(intset.cc@>=
@q m b f @>
  void ABA_INTSET::initialize(int n)
@q m e f @>
  {
    delete[] uninitializedMap_;
    delete[] position_;
    uninitializedMap_ = new @[int[n]@];
    position_ = new @[int[n]@];
    count_ = 0;
  }

@q m f exists @>
@q m e t @>
@ The function |exists()| tests if an integer is contained in the set.

@(intset.cc@>=
@q m b f @>
  bool ABA_INTSET::exists(int elem) const
@q m e f @>
  {
    int i = uninitializedMap_[elem];
    return (unsigned)i < (unsigned)count_ && position_[i]==elem;
  }

@q m f insert @>
@q m e t @>
@ The function |insert()| inserts an element in the set. The element
  is not inserted twice if it is already contained in the set.

@q m b a @>
  \BeginArg
  \docarg{|elem|}{The element to be inserted.}
  \EndArg
@q m e a @>

@(intset.cc@>=
@q m b f @>
  void ABA_INTSET::insert(int elem)
@q m e f @>
  {
    int i = uninitializedMap_[elem];
    if ( (unsigned)i < (unsigned)count_ && position_[i]==elem)
      return;
    // elem is not contained in the set. It is inserted by the next two lines.
    uninitializedMap_[elem] = count_;
    position_[count_++] = elem;
  }

@q m f remove @>
@q m e t @>
@ The function |remove()| removes an element from the set if it is contained
  in it.

@q m b a @>
  \BeginArg
  \docarg{|elem|}{The element to be removed.}
  \EndArg
@q m e a @>

@(intset.cc@>=
@q m b f @>
  void ABA_INTSET::remove(int elem)
@q m e f @>
  {
    int i = uninitializedMap_[elem];
    if ( (unsigned)i < (unsigned)count_ && position_[i]==elem) {
      // elem is contained in the set. We have to remove it.
      if (--count_ != i) {
        int help = position_[count_];
        position_[i] = help;
        uninitializedMap_[help] = i;
      }
    }
  }

@q m f count @>
@q m e t @>
@ The function |count()|. This function and the function |elem()| can be used
  to iterate over the elements of the set.

@q m b r @>
  \BeginReturn
  {The number of elements contained in the set.}
  \EndReturn
@q m e r @>

@(intset.cc@>=
@q m b f @>
  int ABA_INTSET::count() const
@q m e f @>
  {
    return count_;
  }

@q m f elem @>
@q m e t @>
@ The function |elem()| is used to access the elements contained
  in the set.

@q m b a @>
  \BeginArg
  \docarg{|index|}{The index of the element to be returned. This must
                   be a non negative integer less than |number()|.}
  \EndArg
@q m e a @>

@q m b r @>
  \BeginReturn
  {The element referenced by the given index.}
  \EndReturn
@q m e r @>

@(intset.cc@>=
@q m b f @>
  int ABA_INTSET::elem(int index) const
@q m e f @>
  {
    return position_[index];
  }

