@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: lp.w,v 4.16 2003/08/07 11:33:17 elf Exp $ @>

@q m i ABA_LP @>

@ The member functions of the class |ABA_LP| are implemented in the
  file {\tt lp.cc}.

@(lp.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "lp.h"
#include "sparvec.h"
#include "row.h"
#include "column.h"
#include "lpvarstat.h"
#include "slackstat.h"
#else
#include "abacus/master.h"
#include "abacus/lp.h"
#include "abacus/sparvec.h"
#include "abacus/row.h"
#include "abacus/column.h"
#include "abacus/lpvarstat.h"
#include "abacus/slackstat.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
    \BeginArg
    \docarg{|master|}{A pointer to the corresponding master
                      of the optimization.}
    \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  ABA_LP::ABA_LP (ABA_MASTER *master)
@q m e f @>
  : @/
    master_(master), @/
    optStat_(Unoptimized), @/
    xValStatus_(Missing), @/
    barXValStatus_(Missing), @/
    yValStatus_(Missing), @/
    recoStatus_(Missing), @/
    slackStatus_(Missing), @/
    basisStatus_(Missing), @/
    nOpt_(0), @/
    lpSolverTime_(master)
  { }
  
@q m v Destructor @>
@q m e t @>
@ The destructor.

@(lp.cc@>=
@q m b f @>
  ABA_LP::~ABA_LP()
@q m e f @>
  { }

@q m f initialize @>
@ The function |initialize()| loads the linear program defined
   by its arguments.
@q m e t @>

   We do not perform the initialization via arguments of a constructor,
   since for the most frequent application of linear programs within
   \ABACUS, the solution of the linear programming relaxations in
   the subproblems, the problem data is preprocessed before it is
   loaded. Only after the preprocessing in the constructor of the derived
   class, we can call |initialize()|.

   Of course, it would be possible to provide an extra constructor
   with automatic initialization if required.

@q m b a @>
      \BeginArg
      \docarg{|sense|}{The sense of the objective function.}
      \docarg{|nCol|}{The number of columns (variables).}
      \docarg{|maxCol|}{The maximal number of columns.}
      \docarg{|nRow|}{The number of rows.}
      \docarg{|maxRow|}{The maximal number of rows.}
      \docarg{|obj|}{An array with the objective function coefficients.}
      \docarg{|lb|}{An array with the lower bounds of the columns.}
      \docarg{|ub|}{An array with the upper bounds of the columns.}
      \docarg{|rows|}{An array storing the rows of the problem.}
      \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::initialize(ABA_OPTSENSE sense, 
                          int nRow, 
                          int maxRow,
                          int nCol, 
                          int maxCol,
                          ABA_ARRAY<double> &obj, 
                          ABA_ARRAY<double> &lBound,
                          ABA_ARRAY<double> &uBound, 
                          ABA_ARRAY<ABA_ROW*> &rows)
@q m e f @>
  {
    _initialize(sense, nRow, maxRow, nCol, maxCol, obj, lBound, uBound, rows);
  }

@q m f initialize @>
@ This version of the function |initialize()| performs like its
  previous version, but also initializes the basis with the
  arguments:
@q m e t @>

@q m b a @>
     \BeginArg
     \docarg{|lpVarStat|}{An array storing the status of the columns.}
     \docarg{|slackStat|}{An array storing the status of the slack variables.}
     \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::initialize(ABA_OPTSENSE sense, 
                          int nRow, 
                          int maxRow,
                          int nCol, 
                          int maxCol,
                          ABA_ARRAY<double> &obj, 
                          ABA_ARRAY<double> &lBound,
                          ABA_ARRAY<double> &uBound, 
                          ABA_ARRAY<ABA_ROW*> &rows,
                          ABA_ARRAY<ABA_LPVARSTAT::STATUS> &lpVarStat,
                          ABA_ARRAY<ABA_SLACKSTAT::STATUS> &slackStat)
@q m e f @>
  {
    _initialize(sense, nRow, maxRow, nCol, maxCol, obj, lBound, uBound, rows);
    ABA_LP::loadBasis(lpVarStat, slackStat);
  }

@q m v loadBasis @>
@ The function |loadBasis()| loads a new basis for the linear program.
@q m e t @>

@q m b a @>
     \BeginArg
     \docarg{|lpVarStat|}{An array storing the status of the columns.}
     \docarg{|slackStat|}{An array storing the status of the slack variables.}
     \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
   void ABA_LP::loadBasis(ABA_ARRAY<ABA_LPVARSTAT::STATUS> &lpVarStat,
                          ABA_ARRAY<ABA_SLACKSTAT::STATUS> &slackStat)
@q m e f @>
   {
     _loadBasis(lpVarStat, slackStat);
   }

@q m v optimize @>
@ The function |optimize()| performs the optimization of the linear
  program.
@q m e t @>

@q m b r @>
  \BeginReturn
  The status of the optimization.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|method|}{The method with which the optimization is performed.}
  \EndArg
@q m e a @>
  
@(lp.cc@>=
@q m b f @>
  ABA_LP::OPTSTAT ABA_LP::optimize(METHOD method)
@q m e f @>
  {
    if (nCol() == 0) {
      master_->err() << "ABA_LP::optimize(): cannot optimize ";
      master_->err() << "(number of columns is 0)" << endl;
      exit (Fatal);
    }

    ++nOpt_;

    switch (method) {
      case Primal:  optStat_ = _primalSimplex();
                    break;
      case Dual:    optStat_ = _dualSimplex();
                    break;
      case BarrierAndCrossover: optStat_ = _barrier(true);
                    break;
      case BarrierNoCrossover: optStat_ = _barrier(false);
                    break;
    }

    return optStat_;
  }

@q m f remRows @>
@ The function |remRows()| removes rows of the linear program.
@q m e t @>
  The function |initPostOpt()|
  resets the optimization status and the availability statuses of
  the solution.

@q m b a @>
  \BeginArg
  \docarg{|ind|}{The numbers of the rows that should be removed.}
  \EndArg
@q m e a @>
@(lp.cc@>=
@q m b f @>
  void ABA_LP::remRows(ABA_BUFFER<int> &ind)
@q m e f @>
  {

    initPostOpt();

    _remRows(ind);
  }

@ The function |initPostOpt()| must be called after each modification
   of the linear program. It resets the optimization status and the
   availability status of the solution.

@(lp.cc@>=
  void ABA_LP::initPostOpt()
  {
    optStat_     = Unoptimized;
    xValStatus_  = barXValStatus_ = recoStatus_  = Missing;
    slackStatus_ = yValStatus_    = basisStatus_ = Missing;
  }

@q m f addRows @>
@ The function |addRows()| adds rows to the linear program.
  If the new number of rows exceeds the maximal number of rows a reallocation
  is performed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newRows|}{The rows that should be added to the linear program.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::addRows(ABA_BUFFER<ABA_ROW*> &newRows)
@q m e f @>
  {
    
    if(nRow() + newRows.number() > maxRow())
      rowRealloc(nRow() + newRows.number());

    initPostOpt();
    
    _addRows(newRows);

  }

@q m f rowRealloc @>
@ The function |rowRealloc()| performs a reallocation of the row space
  of the linear program.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newSize|}{The new maximal number of rows of the linear program.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::rowRealloc(int newSize)
@q m e f @>
  {
    _rowRealloc(newSize);
  }

@q m f remCols @>
@ The function |remCols()| removes  columns from the linear program.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|cols|}{The numbers of the columns that should be removed.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::remCols(ABA_BUFFER <int> &cols)
@q m e f @>
  {
    initPostOpt();
    _remCols(cols);
  }

@q m f addCols @>
@ The function |addCols()| adds columns to the linear
  program. If the new number of columns exceeds the maximal number
  of columns a reallocation is performed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newCols|}{The new columns that are added.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::addCols(ABA_BUFFER<ABA_COLUMN*> &newCols)
@q m e f @>
  {

    if (nCol() + newCols.number() > maxCol())
      colRealloc(nCol() + newCols.number());

    initPostOpt();

    _addCols(newCols);
  }

@q m f colRealloc @>
@ The function |colRealloc()| performs a reallocation of the column space
  of the linear program.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newSize|}{The new maximal number of columns of the linear program.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::colRealloc(int newSize)
@q m e f @>
  {
    _colRealloc(newSize);
  }

@q m f changeRhs @>
@ The function |changeRhs()| changes the complete right hand side of the
  linear program.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newRhs|}{The new right hand side of the rows.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::changeRhs(ABA_ARRAY<double> &newRhs)
@q m e f @>
  {
    initPostOpt();
    
    _changeRhs(newRhs);
  }

@q m v changeLBound @>
@ The function |changeLBound()|changes the lower bound of a single column.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The column.}
  \docarg{|newLb|}{The new lower bound of the column.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::changeLBound(int i, double newLb)
@q m e f @>
  {
#ifdef ABACUSSAFE
    colRangeCheck(i);
#endif    

    initPostOpt();
    
    _changeLBound(i, newLb);
  }

@q m v changeUBound @>
@ The function |changeUBound()| changes the upper bound of a single column.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The column.}
  \docarg{|newUb|}{The new upper bound of the column.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::changeUBound(int i, double newUb)
@q m e f @>
  {
#ifdef ABACUSSAFE
    colRangeCheck(i);
#endif    

    initPostOpt();
    
    _changeUBound(i, newUb);
  }

@q m v pivotSlackVariableIn @>
@ The function |pivotSlackVariableIn()| pivots the slack variables
  stored in the buffer |rows| into the basis.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{All variables could be pivoted in,}
  \docarg{|1|}{otherwise.}
  \EndArg
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|rows|}{The numbers of the slack variables that should be
                  pivoted in.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  int ABA_LP::pivotSlackVariableIn(ABA_BUFFER<int> &rows)
@q m e f @>
  {
    initPostOpt();

    return _pivotSlackVariableIn(rows);
  }

@q m f sense @>
@q m e t @>
@ The function |sense()|.

@q m b r @>
  \BeginReturn
  The optimization sense of the linear program.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_OPTSENSE ABA_LP::sense() const
@q m e f @>

@q m f sense @>
@q m e t @>
@ The function |sense()| changes the sense of the optimization.

@q m b r @>
  \BeginArg
  \docarg{|newSense|}{The new sense of the optimization.}
  \EndArg
@q m e r @>

@q m b f @>
  void ABA_LP::sense(const ABA_OPTSENSE &newSense)
@q m e f @>

@q m f nRow @>
@q m e t @>
@ The function |nRow()|.

@q m b r @>
  \BeginReturn
  The current number of rows.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_LP::nRow() const
@q m e f @>

@q m f maxRow @>
@q m e t @>
@ The function |maxRow()|.

@q m b r @>
  \BeginReturn
  The maximal number of rows which
  can currently be stored. However, more rows can be added to the LP
  since in this case the automatic reallocation is executed.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_LP::maxRow() const
@q m e f @>

@q m f nCol @>
@q m e t @>
@ The function |nCol()|.

@q m b r @>
  \BeginReturn
  The current number of columns.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_LP::nCol() const
@q m e f @>

@q m f maxCol @>
@q m e t @>
@ The function |maxCol()|.

@q m b r @>
  \BeginReturn
  The  maximal number of columns which
  can currently be stored. However, more columns can be added to the LP
  since in this case the automatic reallocation is executed or
  a reallocation can be performed by an explicit call to |colRealloc()|.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_LP::maxCol() const
@q m e f @>

@q m f nnz @>
@q m e t @>
@ The function |nnz()|.

@q m b r @>
  \BeginReturn
  The number of nonzero elements in the
  constraint matrix. To be more exact, this is the number of elements
  stored in the sparse representation of the constraint matrix.
  If a (redundant) coefficient with value zero is stored, this element
  is also counted.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_LP::nnz() const
@q m e f @>

@q m f obj @>
@q m e t @>
@ The function |obj()|.

@q m b r @>
  \BeginReturn
   The objective function coefficient of column |i|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{A column.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::obj(int i) const
@q m e f @>

@q m f lBound @>
@q m e t @>
@ The function |lBound()|.

@q m b r @>
  \BeginReturn
  The lower bound of column |i|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{A column.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::lBound(int i) const
@q m e f @>

@q m f uBound @>
@q m e t @>
@ The function |uBound()|.

@q m b r @>
  \BeginReturn
   The upper bound of column |i|.
   \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{A column.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::uBound(int i) const
@q m e f @>

@q m f row @>
@ The function |row()| retrieves a row from the linear program.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the row.}
  \docarg{|r|}{Stores the row after calling this function.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_LP::row(int i, ABA_ROW &r) const
@q m e f @>

@q m f rhs @>
@q m e t @>
@ The function |rhs()|.

@q m b r @>
  \BeginReturn
  The right hand side of the |i|-th row.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the row.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::rhs(int i) const
@q m e f @>

@q m v value @>
@q m e t @>
@ The function |value()|.

@q m b r @>
  \BeginReturn
  The solution of the |ABA_LP| after the optimization has been performed.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_LP::value() const
@q m e f @>

@q m v xVal @>
@q m e t @>
@ The function |xVal()|.

@q m b r @>
  \BeginReturn
  The $x$-value of column |i| in the solution of the linear program.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a column.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::xVal(int i)
@q m e f @>

@q m v barXVal @>
@q m e t @>
@ The function |barXVal()|.

@q m b r @>
  \BeginReturn
  The $x$-value of column |i| in the barrier solution before crossing
  over to a basis solution of the linear program.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a column.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::barXVal(int i)
@q m e f @>

@q m v reco @>
@q m e t @>
@ The function |reco()|.

@q m b r @>
  \BeginReturn
  The reduced cost of column |i|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a column.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::reco(int i)
@q m e f @>

@q m v yVal @>
@q m e t @>
@ The function |yVal()|.

@q m b r @>
  \BeginReturn
  The dual variable of row |c|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|c|}{The number of a row.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::yVal(int c)
@q m e f @>

@q m v slack @>
@q m e t @>
@ The function |slack()|.

@q m b r @>
  \BeginReturn
  The value of the slack variable of row |c|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|c|}{The number of a row.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_LP::slack(int c)
@q m e f @>

@q m f xValStatus @>
@q m e t @>
@ The function |xValStatus()|.

@q m b r @>
  \BeginReturn
  \docarg{|Available|}{If the $x$-values can be retrieved,}
  \docarg{|Missing|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_LP::SOLSTAT ABA_LP::xValStatus() const
@q m e f @>

@q m f barXValStatus @>
@q m e t @>
@ The function |barXValStatus()|.

@q m b r @>
  \BeginReturn
  \docarg{|Available|}{If the $x$-values of the barrier solution
                       before cross over can be retrieved,}
  \docarg{|Missing|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_LP::SOLSTAT ABA_LP::barXValStatus() const
@q m e f @>

@q m f recoStatus @>
@q m e t @>
@ The function |recoStatus()|.

@q m b r @>
  \BeginReturn
  \docarg{|Available|}{If the reduced costs can be retrieved,}
  \docarg{|Missing|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_LP::SOLSTAT ABA_LP::recoStatus() const
@q m e f @>

@q m f yValStatus @>
@q m e t @>
@ The function |yValStatus()|.

@q m b r @>
  \BeginReturn
  \docarg{|Available|}{If the dual variables can be retrieved,}
  \docarg{|Missing|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_LP::SOLSTAT ABA_LP::yValStatus() const
@q m e f @>

@q m f slackStatus @>
@q m e t @>
@ The function |slackStatus()|.

@q m b r @>
  \BeginReturn
  \docarg{|Available|}{If the status of the  slack variables can be retrieved,}
  \docarg{|Missing|}{otherwise.}
  \EndReturn
@q m e r @>
  
@q m b f @>
  ABA_LP::SOLSTAT ABA_LP::slackStatus() const
@q m e f @>

@q m f basisStatus @>
@q m e t @>
@ The function |basisStatus()|.

@q m b r @>
  \BeginReturn
  \docarg{|Available|}{If the status of the columns can be retrieved,}
  \docarg{|Missing|}{otherwise.}
  \EndReturn
@q m e r @>
  
@q m b f @>
  ABA_LP::SOLSTAT ABA_LP::basisStatus() const
@q m e f @>

@q m f nOpt @>
@q m e t @>
@ The function |nOpt()|.

@q m b r @>
  \BeginReturn
  The number of optimizations of the |ABA_LP|.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_LP::nOpt() const
@q m e f @>
  
@q m v infeasible @>
@q m e t @>
@ The function |infeasible()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the optimization status is |Infeasible|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_LP::infeasible() const
@q m e f @>

@q m v getInfeas @>
@ The function |getInfeas()| can be called if the last linear program
  has been solved with the dual simplex method and is infeasible and
  all inactive variables price out correctly. Then, the basis is dual
  feasible, but primal infeasible, i.e., some variables or slack variables
  violate their bounds.
  In this case the function |getInfeas()| determines an infeasible variable
  or slack variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{On success,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|infeasRow|}{Holds after the execution the number of an infeasible
                         slack variable, or $-1$.}
    \docarg{|infeasVar|}{Holds after the execution the number of an infeasible
                         column, or $-1$.}
    \docarg{|bInvRow|}{Holds after the execution the row of the basis inverse
                       corresponding to the infeasible column or
                       slack variable, which
                       is always a basic variable.}
  If |getInfeas()| is successful, then either |infeasRow| or |infeasVar| is
  $-1$ and the other argument holds the nonnegative number of the infeasible
  variable.
  \EndArg                      
@q m e a @>


@(lp.cc@>=
@q m b f @>
  int ABA_LP::getInfeas(int &infeasRow, int &infeasCol, double *bInvRow)
@q m e f @>
  {
    return _getInfeas(infeasRow, infeasCol, bInvRow);
  }

@q m v lpVarStat @>
@q m e t @>
@ The function |lpVarStat()|.

@q m b r @>
  \BeginReturn
  The status of column |i| in the linear program.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a column.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_LPVARSTAT::STATUS ABA_LP::lpVarStat(int i)
@q m e f @>

@q m v slackStat @>
@q m e t @>
@ The function |slackStat()|.

@q m b r @>
  \BeginReturn
  The  status of the  slack variable of row |i|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a row.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_SLACKSTAT::STATUS ABA_LP::slackStat(int i)
@q m e f @>

@q m f colsNnz @>
@ The function |colsNnz()| computes the number of nonzero
  elements in each column of a given set of rows.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|nRow|}{The number of rows.}
  \docarg{|rows|}{The array storing the rows.}
  \docarg{|nnz|}{An array of length at least the number of columns
                    of the linear program which will
                    hold the number of nonzero elements of each column.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::colsNnz(int nRow, ABA_ARRAY<ABA_ROW*> &rows, ABA_ARRAY<int> &nnz)
@q m e f @>
  {
    ABA_ROW *row;
    int i, r;
    int rowNnz;

    nnz.set(0);
    
    for(r = 0; r < nRow; r++) {  
      row = rows[r];
      rowNnz = row->nnz();
      
      for(i = 0; i < rowNnz; i++)
        nnz[row->support(i)]++;
    }
  }
  
@q m f rows2cols @>
@ The function |rows2cols()| computes the columnwise
  representation of the row matrix.
@q m e t @>

@q m b a @>
    \BeginArg
     \docarg{|nRow|}{The number of rows.}
     \docarg{|rows|}{The array storing the rows.}
     \docarg{|cols|}{An array holding pointers to sparse vectors which
                     will contain the columnwise representation of
                     the constraint matrix defined by |rows|. The
                     length of this array must be at least the number
                     of columns. The elements of the array must not be
                     0-pointers. Sparse vectors of sufficient length
                     should be allocated before the function is called.
                     The size of these sparse vectors can be
                     determined with the function |colsNnz()|.}
    \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::rows2cols(int nRow, 
                         ABA_ARRAY<ABA_ROW*> &rows, 
                         ABA_ARRAY<ABA_SPARVEC*> &cols)
@q m e f @>
  {
    ABA_ROW *row;
    int  i,r;

    for(r = 0; r < nRow; r++) {
      row = rows[r];

      const int rowNnz = row->nnz();

      for(i = 0; i < rowNnz; i++)
        cols[row->support(i)]->insert(r, row->coeff(i));
    }
  }

@q m f rowRangeCheck @>
@ The function |rowRangeCheck()| terminates the program if there
  is no row with index |r|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|r|}{The number of a row of the linear program.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::rowRangeCheck(int r) const
@q m e f @>
  {
    if (r < 0 || nRow() <= r) {
      master_->err() << "ABA_LP::rowRangeCheck(" << r << "): range of rows ";
      master_->err() << "0 ... " << nRow()-1 << " violated." << endl;
      exit(Fatal);
    }
  }

@q m f colRangeCheck @>
@ The function |colRangeCheck()| terminates the program if there
  is no column with index |i|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a column.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  void ABA_LP::colRangeCheck(int i) const
@q m e f @>
  {
    if (i < 0 || nCol() <= i) {
      master_->err() << "ABA_LP::colRangeCheck(" << i << "): range of columns ";
      master_->err() << "0 ... " << nCol()-1 << " violated." << endl;
      exit(Fatal);
    }
  }

@q m f Output Operator @>
@ The output operator writes the objective function, followed by
  the constraints, the bounds on the columns and the solution values
  (if available) to an output stream.
@q m e t @>
  Every ten output columns we perform a line break for better
  readability. This has also the advantage that LP-solvers with
  an input function requiring a limited length of a line (e.g., Cplex
  255 characters) have a higher chance to read a file generated by
  this output operator.
@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The linear program being output.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  ostream &operator<<(ostream& out, const ABA_LP& rhs)
@q m e f @>
  {
    @<|ABA_LP|: |operator<<|: local variables@>;
    @<output the objective function@>;
    out << "s.t." << endl;
    @<output the constraints@>;
    @<output the bounds@>;
    out << "End" << endl;
    @<output the solution of the linear program@>;

    return out;
  }

@ @<|ABA_LP|: |operator<<|: local variables@>=
  char sign;  // sign of a coefficient
  double c;   // coefficient
  const double eps = rhs.master_->machineEps();
    
@ The objective function is written in the form
  {\tt min 3.1 x0 + 4 x2}. Variables with coefficient 0 are not output.
  We also suppress the output of a $+$ before the first coefficient
  and the output of coefficients with value $1.0$.
  
@<output the objective function@>=
  out << rhs.sense() << ' ';

  for (int i = 0; i < rhs.nCol(); i++) {
    c = rhs.obj(i);
    if (c < -eps || c > eps) {  // do not output variables with coefficient 0
      if (c < 0.0) {
        sign = '-';
        c = -c;
      }
      else sign = '+';
  
      if (i > 0 || sign == '-') out << sign << ' ';  // do not write a |+| before x0
      if (c < 1.0 - eps || 1.0 + eps < c)  // do not write coefficient 1.0
        out << c << ' ';
      out << 'x' << i << ' ';
    }
    if (i && !(i%10)) out << endl;
  }
  out << endl;

@ The constraints of the |ABA_LP| are output row by row.
@<output the constraints@>=
  ABA_ROW row(rhs.master_, rhs.nCol());

#ifdef ABACUS_NO_FOR_SCOPE  
  for(i = 0; i < rhs.nRow(); i++) {
#else
  for(int i = 0; i < rhs.nRow(); i++) {
#endif
    rhs.row(i, row);
    out << "(" << i << "): " << row << endl;
  }
  
@ The bounds are written in the form {\tt 0 <= x0 <= 2.3}.

@<output the bounds@>=
  out << "Bounds" << endl;
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < rhs.nCol(); i++)
#else
  for (int i = 0; i < rhs.nCol(); i++)
#endif
    out << rhs.lBound(i) << " <= x" << i << " <= " << rhs.uBound(i) << endl;


@ Finally the status of optimization of the |ABA_LP| is output, together
  with the value of the optimal solution if it is available.

@<output the solution of the linear program@>=
  switch(rhs.optStat_) {
    case ABA_LP::Unoptimized: out << "No solution available";
                          break;
    case ABA_LP::Error: out << "Optimization failed";
                    break;
    case ABA_LP::Optimal: out << "Optimum value: " << rhs.value();
                      break;
    case ABA_LP::Feasible: out << "Primal feasible value: " << rhs.value();
                      break;
    case ABA_LP::Infeasible: out << "Problem primal infeasible";
                         break;
    case ABA_LP::Unbounded: out << "Problem unbounded";
                        break;
    default: rhs.master_->err() << "operator<<(ABA_OSTREAM&, const ABA_LP&):";
#if defined ABACUS_COMPILER_VISUAL_CPP
             rhs.master_->err() << "Unknown ABA_LP::Status: " << (int) rhs.optStat_ << endl;
#else
             rhs.master_->err() << "Unknown ABA_LP::Status: " << rhs.optStat_ << endl;
#endif
             exit(ABA_ABACUSROOT::Fatal);
  }
  out << endl;

@q m f writeBasisMatrix @>
@ The function |writeBasisMatrix()| writes the complete basis 
  of an optimal linear program to a file. 
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If a basis is available and could be written,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|fileName|}{The name of the file the basis is written to.}
  \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  int ABA_LP::writeBasisMatrix(const char *fileName)
@q m e f @>
  {
    if (optStat_ != Optimal || slackStatus_ == Missing || 
        basisStatus_ == Missing) return 1;

    @<open the file for writing the basis@>;
    @<mark the basic variables@>;
    @<write the basis row by row@>;
    
    return 0;
  }

@ @<open the file for writing the basis@>=
  ofstream file(fileName);
  if (!file) return 0;

@ @<mark the basic variables@>=
  @<mark the basic structural variables@>;
  @<mark the basic slack variables@>;
  @<check the number of the basic variables@>;

@ @<mark the basic structural variables@>=
  ABA_ARRAY<bool> basicCol(master_, nCol());
  ABA_ARRAY<int>  basisIndexCol(master_, nCol());
  int nBasic = 0;

  for (int i = 0; i < nCol(); i++)
    if (lpVarStat(i) == ABA_LPVARSTAT::Basic) {
      basicCol[i] = true;
      basisIndexCol[i] = nBasic;
      nBasic++;
    }
    else
      basicCol[i] = false;

@ @<mark the basic slack variables@>=
  ABA_ARRAY<int>  basisIndexRow(master_, nRow());
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRow(); i++) 
#else
  for (int i = 0; i < nRow(); i++) 
#endif
    if (slackStat(i) == ABA_SLACKSTAT::Basic) {
      basisIndexRow[i] = nBasic;
      nBasic++;
    }

@ @<check the number of the basic variables@>=
  if (nBasic != nRow()) {
    master_->err() << "number of basic variables " << nBasic << " != ";
    master_->err() << "number of rows " << nRow() << endl;
    exit(Fatal);
  }

@ @<write the basis row by row@>=
  file << nRow() << endl;

  ABA_ROW           sparseRow(master_, nCol());

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRow(); i++) {
#else
  for (int i = 0; i < nRow(); i++) {
#endif
    row(i, sparseRow);
    int nBasicInRow = 0;
    const int sparseRowNnz = sparseRow.nnz();

    for (int j = 0; j < sparseRowNnz; j++)
      if (basicCol[sparseRow.support(j)]) nBasicInRow++;
    if (slackStat(i) == ABA_SLACKSTAT::Basic) nBasicInRow++;

    file << i << ' ' << nBasicInRow << ' ';
#ifdef ABACUS_NO_FOR_SCOPE
    for (j = 0; j < sparseRowNnz; j++)
#else
    for (int j = 0; j < sparseRowNnz; j++)
#endif
      if (basicCol[sparseRow.support(j)])  {
        file << basisIndexCol[sparseRow.support(j)] << ' ';
        file << sparseRow.coeff(j) << ' ';
      }
    if (slackStat(i) == ABA_SLACKSTAT::Basic) 
      file << basisIndexRow[i] << " 1";
    file << endl;
  }

@q m f setSimplexIterationLimit @>
@ The function |setSimplexIterationLimit()| changes the iteration
  limit of the Simplex algorithm.
@q m e t @>

@q m b r @>
  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be set,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|limit|}{The new value of the iteration limit.}
   \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  int ABA_LP::setSimplexIterationLimit(int limit)
@q m e f @>
  {
    return _setSimplexIterationLimit(limit);
  }

@q m f getSimplexIterationLimit @>
@ The function |getSimplexIterationLimit()|.
@q m e t @>

@q m b r @>
  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be get,}
  \docarg{|1|}{otherwise.} 
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|limit|}{Stores the iteration limit if the  return value is 0.}
   \EndArg
@q m e a @>

@(lp.cc@>=
@q m b f @>
  int ABA_LP::getSimplexIterationLimit(int &limit)
@q m e f @>
  {
    return _getSimplexIterationLimit(limit);
  }

