@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: lpif.w,v 4.11 2003/08/05 13:55:21 elf Exp $ @>

@ The file |lpif.cc| contains all code fragments of the basic
  \ABACUS\ library which depend on the preprocessor definition of the
  supported LP solvers. Only this file needs to be recompiled
  if the LP solver definitions change.

  Currently the following definitions are supported:
  
  For Cplex:
  ABACUS_LP_CPLEX22, ABACUS_LP_CPLEX30, ABACUS_LP_CPLEX40, ABACUS_LP_CPLEX50,
  ABACUS_LP_CPLEX60, ABACUS_LP_CPLEX65, ABACUS_LP_CPLEX80

  For Soplex:
  ABACUS_LP_SOPLEX.

  For Xpress:
  ABACUS_LP_XPRESS.


@(lpif.cc@>=
#include "abacus/master.h"
#include "abacus/sub.h"

#if defined ABACUS_LP_CPLEX22 || defined ABACUS_LP_CPLEX30 || defined ABACUS_LP_CPLEX40
#define ABACUS_LP_CPLEX
#elif defined ABACUS_LP_CPLEX50 || ABACUS_LP_CPLEX60|| ABACUS_LP_CPLEX65
#define ABACUS_LP_CPLEX
#elif defined ABACUS_LP_CPLEX80
#define ABACUS_LP_CPLEX
#endif

#ifdef ABACUS_LP_CPLEX
#include "abacus/lpmastercplex.h"
#include "abacus/lpsubcplex.h"
#endif
#ifdef ABACUS_LP_SOPLEX
#include "abacus/lpmastersoplex.h"
#include "abacus/lpsubsoplex.h"
#endif
#ifdef ABACUS_LP_XPRESS
#include "abacus/lpmasterxpress.h"
#include "abacus/lpsubxpress.h"
#endif



@ The function |ABA_SUB::generateLp()|.

@(lpif.cc@>=
ABA_LPSUB *ABA_SUB::generateLp()
  {
    switch (master_->defaultLpSolver()) {
#ifdef ABACUS_LP_CPLEX
      case ABA_MASTER::Cplex:
        return new @,@[ABA_LPSUBCPLEX(master_, this)@];
#endif
#ifdef ABACUS_LP_SOPLEX
      case ABA_MASTER::Soplex:
        return new @,@[ABA_LPSUBSOPLEX(master_, this)@];
#endif
#ifdef ABACUS_LP_XPRESS
      case ABA_MASTER::Xpress:
        return new @,@[ABA_LPSUBXPRESS(master_, this)@];
#endif
      default:
        master_->err() << "Error: ABACUS library not compiled for ";
        master_->err() << "selected LP-Solver." << endl;
        exit(Fatal);
    }
    return 0;           // to supress compiler warning
  }


@ The function |ABA_MASTER::_createLpMasters()|.

@(lpif.cc@>=
void ABA_MASTER::_createLpMasters()
{
#ifdef ABACUS_LP_CPLEX
  lpMasterCplex_ = new ABA_LPMASTERCPLEX(this);
#endif
#ifdef ABACUS_LP_SOPLEX
  lpMasterSoplex_ = new ABA_LPMASTERSOPLEX(this);
#endif
#ifdef ABACUS_LP_XPRESS
  lpMasterXpress_ = new ABA_LPMASTERXPRESS(this);
#endif
}

@ The function |ABA_MASTER::_deleteLpMasters()|.

@(lpif.cc@>=
void ABA_MASTER::_deleteLpMasters()
{
#ifdef ABACUS_LP_CPLEX
  delete lpMasterCplex_;
#endif
#ifdef ABACUS_LP_SOPLEX
  delete lpMasterSoplex_;
#endif
#ifdef ABACUS_LP_XPRESS
  delete lpMasterXpress_;
#endif
}

@ The function |ABA_MASTER::_initializeLpParameters()|.

@(lpif.cc@>=
void ABA_MASTER::_initializeLpParameters()
{
#ifdef ABACUS_LP_CPLEX
  lpMasterCplex_->initializeLpParameters();
#endif
#ifdef ABACUS_LP_SOPLEX
  lpMasterSoplex_->initializeLpParameters();
#endif
#ifdef ABACUS_LP_XPRESS
  lpMasterXpress_->initializeLpParameters();
#endif
}

@ The function |ABA_MASTER::_setDefaultLpParameters()|.

@(lpif.cc@>=
void ABA_MASTER::_setDefaultLpParameters()
{
#ifdef ABACUS_LP_CPLEX
  lpMasterCplex_->setDefaultLpParameters();
#endif
#ifdef ABACUS_LP_SOPLEX
  lpMasterSoplex_->setDefaultLpParameters();
#endif
#ifdef ABACUS_LP_XPRESS
  lpMasterXpress_->setDefaultLpParameters();
#endif
}

@ The function |ABA_MASTER::_printLpParameters()|.

@(lpif.cc@>=
void ABA_MASTER::_printLpParameters()
{
#ifdef ABACUS_LP_CPLEX
  lpMasterCplex_->printLpParameters();
#endif
#ifdef ABACUS_LP_SOPLEX
  lpMasterSoplex_->printLpParameters();
#endif
#ifdef ABACUS_LP_XPRESS
  lpMasterXpress_->printLpParameters();
#endif
}

@ The function |ABA_MASTER::_outputLpStatistics()|.

@(lpif.cc@>=
void ABA_MASTER::_outputLpStatistics()
{
#ifdef ABACUS_LP_CPLEX
  lpMasterCplex_->outputLpStatistics();
#endif
#ifdef ABACUS_LP_SOPLEX
  lpMasterSoplex_->outputLpStatistics();
#endif
#ifdef ABACUS_LP_XPRESS
  lpMasterXpress_->outputLpStatistics();
#endif
}

