@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: lpmastercplex.w,v 4.10 2003/08/05 13:55:21 elf Exp $ @>

@q m i ABA_LPMASTERCPLEX @>

@ The member functions are defined in the file {\tt lpmastercplex.cc}.

@(lpmastercplex.cc@>=

#if defined(ABACUS_SYS_WINNT) && defined(ABACUS_LP_CPLEX50)
#define _CPLEX50_DLL
#endif

#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "lpmastercplex.h"
#include "cplexif.h"
#else
#include "abacus/master.h"
#include "abacus/lpmastercplex.h"
#include "abacus/cplexif.h"
#endif

#ifdef _CPLEX50_DLL
extern "C" {
#include "cplex.h"
#include "calldll.h"
}
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.
@q m b a @>
  \BeginArg
     \docarg{|master|}{The master of the optimization.}
  \EndArg
@q m e a @>

@(lpmastercplex.cc@>=
@q m b f @>
  ABA_LPMASTERCPLEX::ABA_LPMASTERCPLEX(ABA_MASTER *master)
@q m e f @>
    : @/
    ABA_LPMASTER(master), @/
    cplexPrimalPricing_(master, "CPX_PPRIIND_AUTO"), @/
    cplexDualPricing_(master, "CPX_DPRIIND_STEEP"), @/
    cplexOutputLevel_(0), @/
    nCplexReinitializations_(0), @/
    cplexHoldEnvironment_(true), @/
    cplexLicenseDummy_(0)
  {
#ifdef _CPLEX50_DLL
    int status = OpenCPLEXDLL();
    if ( status ) {
      master->err() << "Failed to initialize Cplex DLL, status " << status << endl;
      exit(Fatal);
    }
#endif
  }


@q m f Destructor @>
@q m e t @>
@ The destructor.

@(lpmastercplex.cc@>=
@q m b f @>
  ABA_LPMASTERCPLEX::~ABA_LPMASTERCPLEX()
@q m e f @>
  {
    delete cplexLicenseDummy_;
#ifdef _CPLEX50_DLL
   CloseCPLEXDLL ();
#endif
  }


@q m v initializeLpParameters @>
@ The function |initializeLpParameters()| initializes the LP solver
  specific Parameters.
@q m e t @>

@(lpmastercplex.cc@>=
@q m b f @>
  void ABA_LPMASTERCPLEX::initializeLpParameters()
@q m e f @>
  {
    master_->assignParameter(cplexPrimalPricing_, "CplexPrimalPricing");
    master_->assignParameter(cplexDualPricing_, "CplexDualPricing");
    master_->assignParameter(cplexOutputLevel_, "CplexOutputLevel", 0, 2);
    master_->assignParameter(cplexHoldEnvironment_, "CplexHoldEnvironment");
  }


@q m v setDefaultLpParameters @>
@ The function |setDefaultLpParameters()| sets default values of the LP solver
  specific Parameters.
@q m e t @>

@(lpmastercplex.cc@>=
@q m b f @>
  void ABA_LPMASTERCPLEX::setDefaultLpParameters()
@q m e f @>
  {
    master_->insertParameter("CplexPrimalPricing", "CPX_PPRIIND_AUTO");
    master_->insertParameter("CplexDualPricing", "CPX_DPRIIND_STEEP");
    master_->insertParameter("CplexOutputLevel", "0");
    master_->insertParameter("CplexHoldEnvironment", "true");
  }


@q m v printLpParameters @>
@ The function printLpParameters()| prints the settings of the LP solver
  specific Parameters.
@q m e t @>

@(lpmastercplex.cc@>=
@q m b f @>
  void ABA_LPMASTERCPLEX::printLpParameters()
@q m e f @>
  {
    master_->out() << "  Primal pricing method of Cplex         : ";
    master_->out() << cplexPrimalPricing_ << endl;
    master_->out() << "  Dual pricing method of Cplex           : ";
    master_->out() << cplexDualPricing_ << endl;
    master_->out() << "  Output level of Cplex                  : ";
    master_->out() << cplexOutputLevel_ << endl;
    master_->out() << "  Hold Cplex environment                 : ";
    master_->out() << onOff(cplexHoldEnvironment_) << endl;
  }


@q m v outputLpStatistics @>
@ The function outputLpStatistics()| prints LP solver
  specific Statistics.
@q m e t @>

@(lpmastercplex.cc@>=
@q m b f @>
  void ABA_LPMASTERCPLEX::outputLpStatistics()
@q m e f @>
  {
    const int w = 6;

    master_->out() << "  Number of Cplex Reinitializations : ";
    master_->out() << setWidth(w) <<  nCplexReinitializations_ << endl;
  }


@ The function |countCplexReinitialization()| increments the
  counter for the reinitializations of the LP-solver Cplex.

@(lpmastercplex.cc@>=
  void ABA_LPMASTERCPLEX::countCplexReinitialization()
  {
    ++nCplexReinitializations_;
  }

@q m f cplexPrimalPricing @>
@q m e t @>
@ The function |cplexPrimalPricing()|.

@q m b r @>
  \BeginReturn
  The primal pricing strategy for Cplex (see your Cplex manual).
  \EndReturn
@q m e r @>

@(lpmastercplex.cc@>=
@q m b f @>
  ABA_STRING& ABA_LPMASTERCPLEX::cplexPrimalPricing()
@q m e f @>
  {
    return cplexPrimalPricing_;
  }

@q m f cplexPrimalPricing @>
@ The function |cplexPrimalPricing()| changes the pricing method of
  the primal simplex algorithm of the LP-solver Cplex. 

  Note, this
  function does not automatically change the pricing strategy of already
  constructed objects of the class |ABA_CPLEXIF| or the derived class
  |ABA_LPSUBCPLEX|. Only the pricing strategy of objects constructed after
  this function call is changed. In order to change the primal pricing
  strategy of ``living'' objects of the class |ABA_CPLEXIF|, use the 
  function |ABA_CPLEXIF::setppriind()|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|method|}{A string with the new primal pricing method.
                    Consult your Cplex manual for valid strategies.}
  \EndArg
@q m e a @>

@(lpmastercplex.cc@>=
@q m b f @>
  void ABA_LPMASTERCPLEX::cplexPrimalPricing(const char *method)
@q m e f @>
  {
    cplexPrimalPricing_ = method;
  }

@q m f cplexDualPricing @>
@q m e t @>
@ The function |cplexDualPricing()|

@q m b r @>
  \BeginReturn
  The dual pricing strategy for Cplex (see your Cplex manual).
  \EndReturn
@q m e r @>

@(lpmastercplex.cc@>=
@q m b f @>
  ABA_STRING& ABA_LPMASTERCPLEX::cplexDualPricing()
@q m e f @>
  {
    return cplexDualPricing_;
  }

@q m f cplexDualPricing @>
@ The function |cplexDualPricing()| changes the pricing method of
  the dual simplex algorithm of the LP-solver Cplex. 

  Note, this
  function does not automatically change the pricing strategy of already
  constructed objects of the class |ABA_CPLEXIF| or the derived class
  |ABA_LPSUBCPLEX|. Only the pricing strategy of objects constructed after
  this function call is changed. In order to change the dual pricing
  strategy of ``living'' objects of the class |ABA_CPLEXIF|, use the 
  function |ABA_CPLEXIF::setdpriind()|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|method|}{A string with the new dual pricing method.
                    Consult your Cplex manual for valid strategies.}
  \EndArg
@q m e a @>

@(lpmastercplex.cc@>=
@q m b f @>
  void ABA_LPMASTERCPLEX::cplexDualPricing(const char *method)
@q m e f @>
  {
    cplexDualPricing_ = method;
  }

@q m f cplexOutputLevel @>
@q m e t @>
@ The function |cplexOutputLevel()|.

@q m b r @>
  \BeginReturn
  The output level of Cplex (0: no ouput of Cplex, 1: a line
                             of output every refactorization, 
                             2: A line of output every iteration).
  \EndReturn
@q m e r @>

@(lpmastercplex.cc@>=
@q m b f @>
  int ABA_LPMASTERCPLEX::cplexOutputLevel() const
@q m e f @>
  {
    return cplexOutputLevel_;
  }

@q m f cplexOutputLevel @>
@ The function |cplexOutputLevel()| sets the amount of output produced
  by the LP-Solver Cplex.

  Note, this
  function does not automatically change the amount of output of already
  constructed objects of the class |ABA_CPLEXIF| or the derived class
  |ABA_LPSUBCPLEX|. Only the output of objects constructed after
  this function call is changed. In order to change the amount of 
  output of ``living'' objects of the class |ABA_CPLEXIF|, use the 
  function |ABA_CPLEXIF::iterationInformation()|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|level|}{If |0| no output is generated, if |1| output every
                   refactorization is generated, if |2| output every
                   iteration is generated.}
  \EndArg
@q m e a @>

@(lpmastercplex.cc@>=
@q m b f @>
  void ABA_LPMASTERCPLEX::cplexOutputLevel(int level)
@q m e f @>
  {
    cplexOutputLevel_ = level;
  }

