@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: lpsub.w,v 4.12 2003/08/05 13:55:22 elf Exp $ @>

@q m i ABA_LPSUB @>

@ The member functions are implemented in the file {\tt lpsub.cc}.

@(lpsub.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "lpsub.h"
#include "infeascon.h"
#include "row.h"
#include "constraint.h"
#include "sub.h"
#include "master.h"
#include "lpvarstat.h"
#include "variable.h"
#include "slackstat.h"
#include "column.h"
#else
#include "abacus/lpsub.h"
#include "abacus/infeascon.h"
#include "abacus/row.h"
#include "abacus/constraint.h"
#include "abacus/sub.h"
#include "abacus/master.h"
#include "abacus/lpvarstat.h"
#include "abacus/variable.h"
#include "abacus/slackstat.h"
#include "abacus/column.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
   \BeginArg
   \docarg{|master|}{A pointer to the corresponding master of the optimization.}
   \docarg{|sub|}{The subproblem of which the LP-relaxation is
                  solved.}
   \EndArg
@q m e a @>

@(lpsub.cc@>=  
@q m b f @>
  ABA_LPSUB::ABA_LPSUB (ABA_MASTER *master, const ABA_SUB *sub) 
@q m e f @>
  : @/
    ABA_LP(master), @/
    sub_(sub), @/
    orig2lp_(master, sub->maxVar()), @/
    lp2orig_(master, sub->maxVar()),@/
    infeasCons_(master, sub->maxCon())
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor deletes the components of |infeasCons_| since they might
  have been allocated in the constructor and |ABA_SUB::initializeLp()| deletes
  after having tried to add variables restoring feasibility immediately
  |ABA_LPSUB|. Afterwards the constructor of |ABA_LPSUB| is called again.

@(lpsub.cc@>=
@q m b f @>
  ABA_LPSUB::~ABA_LPSUB()
@q m e f @>
  {
    const int nInfeasCons = infeasCons_.number();

    for(int c = 0; c < nInfeasCons; c++) delete infeasCons_[c];
  }

@q m v initialize @>
@ The function |initialize()| has to be called in the constructor
  of the class derived from this class and from a class implementing
  an LP-solver. This function will pass the linear program of
  the associated subproblem to the solver.
@q m e t @>

@(lpsub.cc@>=
@q m b f @>
  void ABA_LPSUB::initialize()
@q m e f @>
  {
    @<|ABA_LPSUB::initialize()|: local variables@>;
    @<generate the row format of the active constraints@>;
    @<eliminate set and fixed variables and initialize the columns@>;
    @<initialize the LP-solver and clean up@>;
  }

@ @<|ABA_LPSUB::initialize()|: local variables@>=
  ABA_ARRAY<double> obj(master_, sub_->nVar());
  ABA_ARRAY<double> lBound(master_, sub_->nVar());
  ABA_ARRAY<double> uBound(master_, sub_->nVar());
  ABA_ARRAY<ABA_ROW*>   rows(master_, sub_->nCon());
    
  ABA_ARRAY<ABA_LPVARSTAT::STATUS> lpVarStat(master_, sub_->nVar());
  ABA_ARRAY<ABA_SLACKSTAT::STATUS> slackStat(master_, sub_->nCon());

  ABA_ROW   row(master_, sub_->nVar());                // buffer to store generated row
  int         conNnz;                          // number of nonzeros of constraint |c|
  int         c;                               // loop index

@ After the generation of the row format we allocate a row of
  the correct length and make a copy in order to safe memory.
  
@<generate the row format of the active constraints@>=
  int nRow = 0;

  const int nCon = sub_->nCon();
  
  for (c = 0; c < nCon; c++) {
    conNnz  = sub_->constraint(c)->genRow(sub_->actVar(), row);
    rows[nRow] = new ABA_ROW(master_, conNnz);
    rows[nRow]->copy(row);
    slackStat[nRow] = sub_->slackStat(c)->status();
    ++nRow;
    row.clear();
  }

@ @<eliminate set and fixed variables and initialize the columns@>=
  ABA_VARIABLE   *v;                           // pointer to variable of subproblem
  ABA_ARRAY<bool> marked(master_, sub_->nVar(), false);  // |true| if variable can be eliminated

  nOrigVar_ = sub_->nVar();
  valueAdd_ = 0.0;

  @<|ABA_LPSUB|: mark variables to eliminate, build objective function and bounds@>;
  @<|ABA_LPSUB|: update the constraints@>;


@ We mark all variables which can be eliminated, add them to the
  |ABA_BUFFER| |delVar|, compute the mappings from the original variable
  set to the actual variable set in the |LP|, and vice versa, and
  determine the correction term for the LP-value.

  If all variables can be eliminated then we do not eliminate the last
  variable for simpification. Otherwise it would be necessary to load
  an problem with 0 variables to the LP-solver which is, e.g., for
  Cplex not possible. Although the emulation of the optimization would
  still be simple, but extra work would have to be performed if later
  constraints were added.
  
@<|ABA_LPSUB|: mark variables to eliminate, ...@>=
    const int nVar = sub_->nVar();
    int nCol = 0;
    for (int i = 0; i < nVar; i++) {
      v = sub_->variable(i);
      if(sub_->fsVarStat(i)->fixedOrSet()) {
        if (eliminable(i) && (nCol || (i != sub_->nVar() - 1))) {
          @<eliminate variable |i| from the LP@>;
        }
        else {
          @<fix variable |i| in the LP@>;
        }
      }
      else {
        @<add variable |i| to the LP@>;
      }
    }
    
@ @<eliminate variable |i| from the LP@>=
  marked[i]  = true;
  valueAdd_  += v->obj() * elimVal(i);
  orig2lp_[i] = -1;
    
@ As variable |i| could not be eliminated we set both its upper and lower
  bound to the value it is fixed or set to.
  
@<fix variable |i| in the LP@>=
  orig2lp_[i]     = nCol;
  lp2orig_[nCol]  = i;
  obj[nCol]       = v->obj();
  lBound[nCol]    = elimVal(i);
  uBound[nCol]    = elimVal(i);
  lpVarStat[nCol] = sub_->lpVarStat(i)->status();
  ++nCol;

@ @<add variable |i| to the LP@>=
  orig2lp_[i]     = nCol;
  lp2orig_[nCol]  = i;
  obj[nCol]       = v->obj();
  lBound[nCol]    = sub_->lBound(i);
  uBound[nCol]    = sub_->uBound(i);
  lpVarStat[nCol] = sub_->lpVarStat(i)->status();
  ++nCol;

@ If all active variables of a constraint are eliminated then
  its left hand side is void (implicitly 0), but its right hand side
  can be nonzero. Depending on the sense of the constraint it can be
  infeasible.
  If the elimination of variables from constraints causes an infeasible
  |LP|, the constraint is memorized in |infeasCons_|.
  
@<|ABA_LPSUB|: update the constraints@>=
  ABA_BUFFER<int> delVar(master_, sub_->nVar()); // buffer of deletable components of row format
  double            rhsDelta;  // correction of right hand side due to eliminations
  ABA_INFEASCON::INFEAS infeas;    // infeasibility mode (TooLarge, TooSmall)

  for (c = 0; c < nCon; c++) {
    @<eliminate the variables from the constraint@>;
    @<check if the constraint is now infeasible@>;
  }

@ @<eliminate the variables from the constraint@>=
  delVar.clear();
  rhsDelta = 0.0;
  const int rNnz = rows[c]->nnz();
  for(int i = 0; i < rNnz; i++)
    if(marked[rows[c]->support(i)]) {
      delVar.push(i);
      rhsDelta += rows[c]->coeff(i)*elimVal(rows[c]->support(i));
    }

  rows[c]->delInd(delVar, rhsDelta);

@ @<check if the constraint is now infeasible@>=
  if (rows[c]->nnz() == 0) {
    infeas = sub_->constraint(c)->voidLhsViolated(rows[c]->rhs());
    if (infeas != ABA_INFEASCON::Feasible)
      infeasCons_.push(new ABA_INFEASCON(master_, sub_->constraint(c), infeas));
  }
  rows[c]->rename(orig2lp_);

@ @<initialize the LP-solver and clean up@>=
  ABA_LP::initialize(*master_->optSense(), nRow, sub_->maxCon(), nCol,
                     sub_->maxVar(), obj, lBound, uBound, rows,
                     lpVarStat, slackStat);

  for (c = 0; c < nCon; c++)
    delete rows[c];

@ The function |constraint2row()| generates the row format of the
  constraint |cons| and stores it in |rows|.

@(lpsub.cc@>=
  void ABA_LPSUB::constraint2row(ABA_BUFFER<ABA_CONSTRAINT*> &cons,
                                 ABA_BUFFER<ABA_ROW*> &rows)
  {                              
    int   conNnz;                      // number of nonzero elements in constraint
    ABA_ROW   rowBuf(master_, sub_->nVar());  // dummy to generate row
    ABA_ROW  *row;                         // pointer to the new row

    const int nCons = cons.number();
    
    for (int c = 0; c < nCons; c++) {
      conNnz = cons[c]->genRow(sub_->actVar(), rowBuf);
      row = new ABA_ROW(master_, conNnz);
      row->copy(rowBuf);
      rows.push(row);
      rowBuf.clear();
    }
  }
    
@ The function |eliminable()| returns |true| if the function can be 
  eliminated.
  This function may be only applied to variables which are fixed or set!
  It is sufficient for turning off any variable elimination to return always
  |false| by this function.

@(lpsub.cc@>=
  bool ABA_LPSUB::eliminable(int i) const
  {
    if (master_->eliminateFixedSet()) {
#ifdef ABACUS_NO_BOOL
      if (sub_->lpVarStat(i)->basic()) return false;
      else                             return true;
#else
      return !sub_->lpVarStat(i)->basic();
#endif
    }
    else
      return false;
  }

@ The function |eliminated()| returns |true| if the variable
  |i| is actually eliminated from the |LP|. This function can
  give different results than the function |eliminate(i)| since
  the condition to eliminate a variable might have become |true|
  after the |LP| has been set up.

  bool ABA_LPSUB::eliminated(int i) const
  
@ The function |elimVal()| returns the value the variable |i| to
  which it is fixed or set to.

  The value of an eliminated variable is defined by the bound to which
  it is fixed or set.
  There is no reason to distinguish between |sub_| and |master_|
  in the |switch| statement, since both values should be equal.

@(lpsub.cc@>=
  double ABA_LPSUB::elimVal(int i) const
  {
    switch (sub_->fsVarStat(i)->status()) {
      case ABA_FSVARSTAT::SetToLowerBound:   return sub_->lBound(i);
      case ABA_FSVARSTAT::FixedToLowerBound: return sub_->variable(i)->lBound();
      case ABA_FSVARSTAT::SetToUpperBound:   return sub_->uBound(i);
      case ABA_FSVARSTAT::FixedToUpperBound: return sub_->variable(i)->uBound();
      case ABA_FSVARSTAT::Set:               return sub_->fsVarStat(i)->value();
      case ABA_FSVARSTAT::Fixed:             return sub_->variable(i)->fsVarStat()->value();
      
      default: master_->err() << "ABA_LPSUB::elimVal():";
               master_->err() << " variable neither fixed nor set" << endl;
               exit(Fatal);
               return 0.0;   // to suppress compiler warning
    }
  }

@ The function |elimVal()| returns the value a variable is fixed or set
  to.
     \BeginArg
     \docarg{|fsVarStat|}{A pointer to the status of the variable.}
     \docarg{|lb|}{The lower bound of the variable.}
     \docarg{|ub|}{The upper bound of the variable.}

@(lpsub.cc@>=
   double ABA_LPSUB::elimVal(ABA_FSVARSTAT *stat, double lb, double ub) const
   {
     switch (stat->status()) {
       case ABA_FSVARSTAT::SetToLowerBound:   return lb;
       case ABA_FSVARSTAT::FixedToLowerBound: return lb;
       case ABA_FSVARSTAT::SetToUpperBound:   return ub;
       case ABA_FSVARSTAT::FixedToUpperBound: return ub;
       case ABA_FSVARSTAT::Set:               return stat->value();
       case ABA_FSVARSTAT::Fixed:             return stat->value();
       
       default: master_->err()  << "ABA_LPSUB::elimVal():";
                master_->err()  << " variable neither fixed nor set"  << endl;
                exit(Fatal);
               return 0.0;   // to suppress compiler warning
     }
   }

@ The function |optimize()| performs the optimization of the linear
  program with method |method|.
  This function redefines a virtual function of the base class |LP|.

  We have to reimplement |optimize()| since there might be infeasible
  constraints.
  If a linear program turns out to be infeasible but has not been
  solved with the dual simplex method we solve it again to
  find a dual feasible basis which can be used to determine
  inactive variables restoring feasibility.
  Before the optimization can be performed the infeasible constraints
  must be removed with the function |_initMakeFeas()|, then the
  |LP| should be deleted and reconstructed. This is done by the function
  |solveLp()| in the cutting plane algorithm of the class |ABA_SUB|.

@(lpsub.cc@>=
  ABA_LP::OPTSTAT ABA_LPSUB::optimize(METHOD method)
  {
    OPTSTAT status;
    
    if (infeasCons_.number()) {
      master_->err() << "ABA_LPSUB::optimize(): there are infeasible constraints" << endl;
      exit (Fatal);
      return Error;  // to suppress compiler warnings
    }
    else {
      status = ABA_LP::optimize(method);
      if (status == Infeasible && method != Dual) return optimize(Dual);
      else                                        return status;
    }
  }
  
@ The function |removeCons()| removes all constraints listed in the
  buffer |ind| from the linear program.

@(lpsub.cc@>=
  void ABA_LPSUB::removeCons(ABA_BUFFER<int> &ind)
  {
    ABA_LP::remRows(ind);
  }
  
@ The function |removeVars()| removes the variables with names given in
  |vars| from the linear program.

@(lpsub.cc@>=
  void ABA_LPSUB::removeVars(ABA_BUFFER<int> &vars)
  {
    @<|ABA_LPSUB::removeVars()|: local variables@>;
    @<|ABA_LPSUB::removeVars()|: update the number of original variables@>;
    @<divide removed variables in eliminated and non-eliminated ones@>;
    @<adapt the right hand side if eliminated variables are removed@>;
    @<remove the non-eliminated variables@>;
    @<update mappings of original variables and LP variables@>;
  }
       
@ @<|ABA_LPSUB::removeVars()|: local variables@>=
    ABA_BUFFER<int>       lpVars(master_, vars.number());   // indices in |LP| of removed variables
    ABA_ARRAY<double>     rhsDelta(master_, sub_->nCon(), 0.0);  // changes of right hand side
    int               lpName;                        // name of variable in the |LP|
    double            coeff;
    ABA_VARIABLE         *v;
    bool              modifyRhs = false;
    double            eps = master_->eps();

@ @<|ABA_LPSUB::removeVars()|: update the number of original variables@>=
  int oldNOrigVar = nOrigVar_;

  nOrigVar_ -= vars.number();
  
@ If a removed variable has earlier been eliminated from the LP, then
  we might have to adapt the right hand side again, if earlier
  the elimination changed the right and side. Otherwise,
  we add the variable to the buffer |lpVars| in order to remove
  it explicitly later.
  
@<divide removed variables in eliminated and non-eliminated ones@>=
  const int nVars = vars.number();

  for (int i = 0; i < nVars; i++) {
    lpName = orig2lp_[vars[i]];
    if (lpName == -1) {
      @<remove eliminated variable@>;
    }
    else lpVars.push(lpName);
  }

@ @<remove eliminated variable@>=
  valueAdd_  += sub_->variable(i)->obj() * elimVal(i);

  const int nCon = sub_->nCon();

  v = sub_->variable(i);
  for (int c = 0; c < nCon; c++) {
    coeff = sub_->constraint(c)->coeff(v);
    if (fabs(coeff) > eps) {
      rhsDelta[c] += coeff * elimVal(i);
      modifyRhs    = true;
    }
  }

@ @<adapt the right hand side if eliminated variables are removed@>=
  if (modifyRhs) {
    ABA_ARRAY<double> newRhs(master_, sub_->nCon());
    const int nCon = sub_->nCon();

    for (int c = 0; c < nCon; c++)
      newRhs[c] = rhs(c) - rhsDelta[c];
    ABA_LP::changeRhs(newRhs);
  }

@ Here, we also should check for constraints getting a void left hand side
  and becoming infeasible. However, on the one hand this is computationally
  expensive (using the member function |row()|)
  as most LP-solvers (as, e.g., Cplex) work in a column oriented
  form, and second, if immediately afterwards variables are added then
  the linear program could become again feasible.

  Moreover, if only inequalities with void left hand side become infeasible,
  then these infeasibilities are recognized by the LP-solver and resolved
  in |makeFeas()|. Only equations can cause some trouble as there is no
  slack variable.

  Therefore,  unfortunately, taking care that no equation becomes infeasible has to be
  left to the user.
  
@<remove the non-eliminated variables@>=
  ABA_LP::remCols(lpVars);

@ @<update mappings of original variables and LP variables@>=
  @<sort the variables being removed@>;
  @<update mapping of original variables to LP variables@>;
  @<update mapping of LP variables to original variables@>;

@ @<sort the variables being removed@>=
  @<check if sorting is required@>;
  @<if yes, sort the variables@>;

@ @<check if sorting is required@>=
  bool unordered = false;

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVars - 1; i++)
#else
  for (int i = 0; i < nVars - 1; i++)
#endif
    if (vars[i] > vars[i+1]) {
      unordered = true;
      break;
    }

@ @<if yes, sort the variables@>=
  ABA_BUFFER<int> varsSorted(master_, oldNOrigVar);

  if (unordered) {
    ABA_ARRAY<bool> marked(master_, oldNOrigVar, false);

#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < nVars; i++)
#else
    for (int i = 0; i < nVars; i++)
#endif
      marked[vars[i]] = true;

#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < oldNOrigVar; i++)
#else
    for (int i = 0; i < oldNOrigVar; i++)
#endif
      if (marked[i]) 
        varsSorted.push(i);
  }
  else 
    for (int i = 0; i < nVars; i++)
      varsSorted.push(vars[i]);

@ In order to update the mapping of the original variables to the LP-variables 
  we have to eliminate the removed variables from the array |orig2lp_| by a
  leftshift. Moreover, if the variable |i| is not removed then we have to
  reduce |orig2lp_| by the number of variables that have been removed with a 
  index than |i| that have not been eliminated.

@<update mapping of original variables to LP variables@>=
  int current = varsSorted[0];
  int nNotEliminatedRemoved = 0;

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVars - 1; i++) {
#else
  for (int i = 0; i < nVars - 1; i++) {
#endif
    if (orig2lp_[varsSorted[i]] != -1)
      nNotEliminatedRemoved++;

    const int last = varsSorted[i+1];
    for(int j = varsSorted[i]+1; j < last; j++)
      if (orig2lp_[j] == -1)
        orig2lp_[current++] = -1;
      else
        orig2lp_[current++] = orig2lp_[j] - nNotEliminatedRemoved;
  }

  if (orig2lp_[varsSorted[nVars-1]] != -1)
      nNotEliminatedRemoved++;

  for (int j = varsSorted[nVars - 1] + 1; j < oldNOrigVar; j++)
    if (orig2lp_[j] == -1)
      orig2lp_[current++] = -1;
    else
      orig2lp_[current++] = orig2lp_[j] - nNotEliminatedRemoved;
  

@ Since |orig2lp_| is updated already we can update the reverse
  mapping |lp2orig_| in a straight forward way by scanning |orig2lp_|.
 
@<update mapping of LP variables to original variables@>=
  int nVarLp = 0;

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nOrigVar_; i++)
#else
  for (int i = 0; i < nOrigVar_; i++)
#endif
    if (orig2lp_[i] != -1) 
      lp2orig_[nVarLp++] = i;

@ The function |addCons()| adds the constraints |newCons| to the linear
  program.

@(lpsub.cc@>=
  void ABA_LPSUB::addCons(ABA_BUFFER<ABA_CONSTRAINT*> &newCons)
  {
    @<|ABA_LPSUB::addCons()|: local variables@>;

    constraint2row(newCons, newRows);

    @<eliminate variables in added constraints@>;
    
    ABA_LP::addRows(newRows);

    for (int i = 0; i < newRows.number(); i++)
      delete newRows[i];
  }

@ @<|ABA_LPSUB::addCons()|: local variables@>=
  ABA_BUFFER<ABA_ROW*> newRows(master_, newCons.number());  // the new constraints in row format
  ABA_BUFFER<int> delVar(master_, sub_->nVar()); // buffer of deletable components of row format
  double            rhsDelta;  // correction of right hand side
  ABA_INFEASCON::INFEAS infeas;    // infeasibility mode (TooLarge, TooSmall)

  ABA_ROW*         nr;

@ Also the elimination of variables in an added constraint might
  cause a void left hand side (interpreted as 0) violated the right hand
  side of the constraint. These infeasible constraints are recognized,
  but the resolution is currently not implemented.

@<eliminate variables in added constraints@>=
  const int nNewRows = newRows.number();

  for (int c = 0; c < nNewRows; c++) {
    @<eliminate variables in constraint |c|@>;
    @<check if constraint |c| has become infeasible@>;
  }

@ @<eliminate variables in constraint |c|@>=
  delVar.clear();
  rhsDelta = 0.0;
  nr       = newRows[c];
  const int nrNnz = nr->nnz();
  for(int i = 0; i < nrNnz; i++)
    if(eliminated(nr->support(i))) {
      delVar.push(i);
      rhsDelta += nr->coeff(i)*elimVal(nr->support(i));
    }
    nr->delInd(delVar,rhsDelta);
    nr->rename(orig2lp_);

@ @<check if constraint |c| has become infeasible@>=
  if(nr->nnz() == 0) {
    infeas = newCons[c]->voidLhsViolated(nr->rhs());
    if (infeas != ABA_INFEASCON::Feasible) {
      infeasCons_.push(new ABA_INFEASCON(master_, newCons[c], infeas));
      master_->err() << "ABA_LPSUB::addCons(): infeasible constraint added.";
      master_->err() << "All variables with nonzero coefficients are  ";
      master_->err() << "eliminated and constraint is violated." << endl;
      master_->err() << "Sorry, resolution not implemented yet." << endl;
      exit(Fatal);
    }
  }

@ The function |addVars()|.
      \BeginArg
      \docarg{|vars|}{The new variables which are added to the linear
                      program.}
      \docarg{|fsVarstat|}{The status of fixing/setting of the new variables.}
      \docarg{|lb|}{The lower bounds of the new variables.}
      \docarg{|ub|}{The upper bounds of the new variables.}

@(lpsub.cc@>=
  void ABA_LPSUB::addVars(ABA_BUFFER<ABA_VARIABLE*> &vars, 
                          ABA_BUFFER<ABA_FSVARSTAT*> &fsVarStat,
                          ABA_BUFFER<double> &lb, 
                          ABA_BUFFER<double> &ub)
  {
    @<|ABA_LPSUB::addVars()|: local variables@>;
    @<divide the added variables in eliminable and non-eliminable ones@>;
    @<remove the fixed and set added variables@>;
    @<generate the column of the added variable and add them to the LP@>;
    @<modify the right hand side if fixed or set variables are added@>;
    @<|ABA_LPSUB::addVars()|: clean up@>;
  }
  
@ @<|ABA_LPSUB::addVars()|: local variables@>=
  ABA_BUFFER<int> delVar(master_, vars.number());  // the eliminated variables
  ABA_ARRAY<double> rhsDelta(master_, sub_->nCon(), 0.0);  // the correction of the rhs
  double vValue;
  double coeff;
  bool modifyRhs = false;  // if |true| the modification of rhs required
  int oldNCol = trueNCol();
  int n = trueNCol();
  ABA_VARIABLE *v;
  
@ @<divide the added variables in eliminable and non-eliminable ones@>=
  int nVariables = vars.number();

  for (int i = 0; i < nVariables; i++) {
    v = vars[i];
    if(fsVarStat[i]->fixedOrSet()) {
      if (eliminable(i)) {
        @<the new variable is eliminated@>;
      }
      else {
        @<the new variable is fixed in the LP@>;
      }
    }
    else {
      @<the new variable will be added to the LP explicitly@>;
    }
  }

@ @<the new variable is eliminated@>=
  delVar.push(i);

  vValue = elimVal(fsVarStat[i], lb[i], ub[i]);

  valueAdd_ += v->obj() * vValue;
  orig2lp_[nOrigVar_++] = -1;

  const int nCon = sub_->nCon();

 for (int c = 0; c < nCon; c++) {
   coeff = sub_->constraint(c)->coeff(v);
   if (fabs(coeff) > master_->eps()) {
     rhsDelta[c] += vValue * coeff;
     modifyRhs    = true;
   }
 }
 
@ @<the new variable is fixed in the LP@>=
  orig2lp_[nOrigVar_++] = n;
  lp2orig_[n] = oldNCol + i;
  ++n;
  lb[i] = ub[i] = elimVal(fsVarStat[i], lb[i], ub[i]);

@ @<the new variable will be added to the LP explicitly@>=
  orig2lp_[nOrigVar_++] = n;
  lp2orig_[n] = oldNCol + i;
  ++n;

@ @<remove the fixed and set added variables@>=
  if (delVar.number()) {
    vars.leftShift(delVar);
    fsVarStat.leftShift(delVar);
    lb.leftShift(delVar);
    ub.leftShift(delVar);
  }
    
@ @<generate the column of the added variable and add them to the LP@>=
  ABA_BUFFER<ABA_COLUMN*> newCols(master_, vars.number());  
                                    // new columns added to the constraint matrix
  ABA_COLUMN colBuf(master_, nRow());  // buffer for generated columns
  ABA_COLUMN *col;

  nVariables = vars.number();
#ifdef ABACUS_NO_FOR_SCOPE
  for(i = 0; i < nVariables; i++) {
#else
  for(int i = 0; i < nVariables; i++) {
#endif
    vars[i]->genColumn(sub_->actCon(), colBuf);
    col = new ABA_COLUMN(master_, colBuf.nnz());
    col->copy(colBuf);
    col->obj(colBuf.obj());
    col->uBound(colBuf.uBound());
    col->lBound(colBuf.lBound());
    newCols.push(col);
    colBuf.clear();
  }

  ABA_LP::addCols(newCols);

@ @<|ABA_LPSUB::addVars()|: clean up@>=
#ifdef ABACUS_NO_FOR_SCOPE
  for(i = 0; i < nVariables; i++) delete newCols[i];
#else
  for(int i = 0; i < nVariables; i++) delete newCols[i];
#endif

@ @<modify the right hand side if fixed or set variables are added@>=
  if (modifyRhs) {
    const int nCon = sub_->nCon();

    ABA_ARRAY<double> newRhs(master_, nCon);
    for(int c = 0; c < nCon; c++)
      newRhs[c] = rhs(c) - rhsDelta[c];

    changeRhs(newRhs);
  }
      
@ The function |changeLBound()| sets the lower bound of variable |i|
  to |newLb|. It is not allowed to change the lower bound of an
  eliminated variable. This will cause a run-time error.

@(lpsub.cc@>=
   void ABA_LPSUB::changeLBound(int i, double newLb)
   {
    int lpVar = orig2lp_[i];

    if (lpVar == -1) {
       master_->err() << "ABA_LPSUB::changeLBound(" << i << ',' << newLb << ") ";
       master_->err() << "variable " << i << " is eliminated, ";
       master_->err() << "cannot change bounds!" << endl;
       exit (Fatal);
     }
     else ABA_LP::changeLBound(lpVar, newLb);
   }

@ The function |changeUBound()| sets the upper bound of variable |i|
  to |newUb|. It is not allowed to change the upper bound of
  an eliminated variable. This will cause a run-time error.

@(lpsub.cc@>=
   void ABA_LPSUB::changeUBound(int i, double newUb)
   {
     int lpVar = orig2lp_[i];

     if (lpVar == -1) {
       master_->err() << "ABA_LPSUB::changeUBound(" << i << ',' << newUb << ") ";
       master_->err() << "variable " << i << " is eliminated, ";
       master_->err() << "cannot change bounds!" << endl;
       exit (Fatal);
     }
     else ABA_LP::changeUBound(lpVar, newUb);
   }

@q m f trueNCol @>
@q m e t @>
@ The function |trueNCol()|. 

@q m b r @>
  \BeginReturn
  The number of columns which are
  passed to the LP-solver, i.e., the number of active variables
  of the subproblem minus the number of eliminated variables.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_LPSUB::trueNCol() const
@q m e f @>

@q m f trueNnz @>
@q m e t @>
@ The function |trueNnz()|.

@q m b r @>
  \BeginReturn
  The number of nonzeros which
  are currently present in the constraint matrix of the LP-solver.
  \EndReturn
@q m e r @>

@q m b f @>    
  int ABA_LPSUB::trueNnz() const
@q m e f @>

@q m v lBound @>
@q m e t @>
@ The function |lBound()|.

@q m b r @>
  \BeginReturn
  The lower bound of variable |i|. If a variable is eliminated, we
  return the value the eliminated variable is fixed or set to.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a variable.}
  \EndArg
@q m e a @>

  We have to redefine the function |lBound(i)| since variables may have
  been eliminated.

@(lpsub.cc@>=
@q m b f @>
  double ABA_LPSUB::lBound(int i) const
@q m e f @>
  {
    int lpVar = orig2lp_[i];

    if (lpVar != -1) return ABA_LP::lBound(lpVar);
    else             return elimVal(i);
  }

@q m v uBound @>
@q m e t @>
@ The function |uBound()|.

@q m b r @>
  \BeginReturn
  The upper bound of variable |i|. If a variable is eliminated, we
  return the value the eliminated variable is fixed or set to.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of a variable.}
  \EndArg
@q m e a @>

  We have to redefine the function |uBound(i)| since variables may have
  been eliminated.

@(lpsub.cc@>=
@q m b f @>
  double ABA_LPSUB::uBound(int i) const
@q m e f @>
  {
    int lpVar = orig2lp_[i];

    if (lpVar != -1) return ABA_LP::uBound(lpVar);
    else             return elimVal(i);
  }

@q m v value @>
@q m e t @>
@ The function |value()|.

  Since variables might be eliminated we have to add to the solution value
  of the LP-solver the objective function part of the eliminated
  variables, to get the right value of |value()|.

@q m b r @>
  \BeginReturn
  The objective function value of the linear program.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_LPSUB::value() const
@q m e f @>

@q m v xVal @>
@q m e t @>
@ The function |xVal()|.

@q m b r @>
  \BeginReturn
  The $x$-value of variable |i| after the solution of the linear program.
  \EndReturn
@q m e r @>

  We have to redefine the function |xVal(i)| since variables may have
  been eliminated.

@(lpsub.cc@>=
@q m b f @>
  double ABA_LPSUB::xVal(int i)
@q m e f @>
  {
    int lpVar = orig2lp_[i];

    if (lpVar != -1) return ABA_LP::xVal(lpVar);
    else             return elimVal(i);
  }

@q m v barXVal @>
@q m e t @>
@ The function |barXVal()|.

@q m b r @>
  \BeginReturn
  The $x$-value of variable |i| after the solution of the linear
  program before crossing over to a basic solution.
  \EndReturn
@q m e r @>

  We have to redefine the function |barXVal(i)| since variables may have
  been eliminated.

@(lpsub.cc@>=
@q m b f @>
  double ABA_LPSUB::barXVal(int i)
@q m e f @>
  {
    int lpVar = orig2lp_[i];

    if (lpVar != -1) return ABA_LP::barXVal(lpVar);
    else             return elimVal(i);
  }

@q m v reco @>
@q m e t @>
@ The function |reco()|. 

@q m b r @>
  \BeginReturn
  The reduced cost of variable |i|.
  We define the reduced costs of eliminated variables as 0.
  \EndReturn
@q m e r @>

@(lpsub.cc@>=
@q m b f @>
  double ABA_LPSUB::reco(int i)
@q m e f @>
  {
    int lpVar = orig2lp_[i];

    if (lpVar != -1) return ABA_LP::reco(lpVar);
    else             return 0.0;
  }

@q m v lpVarStat @>
@q m e t @>
@ The function |lpVarStat()|.

@q m b r @>
  \BeginReturn
  The status of the variable
  in the linear program. If the variable |i| is eliminated,
  then |ABA_LPVARSTAT::Eliminated| is returned.
  \EndReturn
@q m e r @>

@(lpsub.cc@>=
@q m b f @>
  ABA_LPVARSTAT::STATUS ABA_LPSUB::lpVarStat(int i)
@q m e f @>
  {
    int lpVar = orig2lp_[i];

    if (lpVar != -1) return ABA_LP::lpVarStat(lpVar);
    else             return ABA_LPVARSTAT::Eliminated;
  }
  
@q m v getInfeas @>
@ The function |getInfeas()| is called if the last linear program
  has been solved with the dual simplex method and is infeasible.
  In this case it computes the infeasible basic variable or constraint
  and the corresponding row of the basis inverse.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If no error occurs,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|infeasCon|}{If nonnegative, this is the number of the
                       infeasible slack variable.}
  \docarg{|infeasVar|}{If nonnegative, this is the number of the
                       infeasible structural variable. Note, either
                       |infeasCon| or |infeasVar| is nonnegative.}
  \docarg{|bInvRow|}{An array containing the corresponding row
                     of the basis inverse.}
  \EndArg
@q m e a @>

@(lpsub.cc@>=
@q m b f @>
  int ABA_LPSUB::getInfeas(int &infeasCon, int &infeasVar, double *bInvRow)
@q m e f @>
  {
    int status = ABA_LP::getInfeas(infeasCon, infeasVar, bInvRow);

    if (status) {
      master_->out() << "ABA_LPSUB::getInfeas(): ABA_LP::getInfeas() failed" << endl;
      exit(Fatal);
    }

    if (infeasVar >= 0) infeasVar = lp2orig_[infeasVar];

    return 0;
  }

@q m v infeasible @>
@q m e t @>
@ The function |infeasible()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the |LP| turned out to be
                  infeasible either if the base class |LP| detected an infeasibility
                  during the solution of the linear program
                  or infeasible constraints have been memorized during the
                  construction of the LP or during the addition of constraints,
                }
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(lpsub.cc@>=
@q m b f @>
  bool ABA_LPSUB::infeasible() const
@q m e f @>
  {
    if (ABA_LP::infeasible() || infeasCons_.number()) return true;
    else                                         return false;
  }

@q m f infeasCon @>
@q m e t @>
@ The function |infeasCon()|.

@q m b r @>
  \BeginReturn
  A pointer to the buffer holding the infeasible constraints.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_BUFFER<ABA_INFEASCON*> *ABA_LPSUB::infeasCon()
@q m e f @>

@q m v loadBasis @>
@ The function |loadBasis()| loads a new basis for the linear program.
  The function redefines a virtual function of the base class |LP|. 
  Eliminated variables have to be considered when the basis is loaded.
@q m e t @>

@q m b a @>
     \BeginArg
     \docarg{|lpVarStat|}{An array storing the status of the columns.}
     \docarg{|slackStat|}{An array storing the status of the slack variables.}
     \EndArg
@q m e a @>

@(lpsub.cc@>=
@q m b f @>
@q m b f @>
   void ABA_LPSUB::loadBasis(ABA_ARRAY<ABA_LPVARSTAT::STATUS> &lpVarStat,
                             ABA_ARRAY<ABA_SLACKSTAT::STATUS> &slackStat)
@q m e f @>
   {
     ABA_ARRAY<ABA_LPVARSTAT::STATUS> colStat(master_, trueNCol());
     int n = 0;

     const int nVar = sub_->nVar();
     
     for (int i = 0; i < nVar; i++) 
       if (!eliminated(i))
         colStat[n++] = lpVarStat[i];

     ABA_LP::loadBasis(colStat, slackStat);
   }

@ The function |varRealloc()| sets the maximal number of variables
  to |newSize|.

@(lpsub.cc@>=
  void ABA_LPSUB::varRealloc(int newSize)
  {
    ABA_LP::colRealloc(newSize);

    orig2lp_.realloc(newSize);
    lp2orig_.realloc(newSize);
  }

@ The function |conRealloc()| sets the maximal number of constraints
  to |newSize|.

@(lpsub.cc@>=
  void ABA_LPSUB::conRealloc(int newSize)
  {
    ABA_LP::rowRealloc(newSize);
    infeasCons_.realloc(newSize);
  }

