@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>


@q $Id: master.w,v 4.49 2003/08/05 13:55:23 elf Exp $ @>

@q m i ABA_MASTER @>

@ The member functions are defined in the file {\tt master.cc}.

@(master.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "master.h"

#include "sub.h"
#include "lpsub.h"
#include "lpvarstat.h"
#include "variable.h"
#include "history.h"
#include "opensub.h"
#include "fixcand.h"
#include "setbranchrule.h"
#include "standardpool.h"
#else
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/lpsub.h"
#include "abacus/lpvarstat.h"
#include "abacus/variable.h"
#include "abacus/history.h"
#include "abacus/opensub.h"
#include "abacus/fixcand.h"
#include "abacus/setbranchrule.h"
#include "abacus/standardpool.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/parmaster.h"
#include "abacus/valbranchrule.h"
#include "abacus/boundbranchrule.h"
#include "abacus/conbranchrule.h"
#include "abacus/numcon.h"
#include "abacus/rowcon.h"
#include "abacus/srowcon.h"
#include "abacus/numvar.h"
#include "abacus/colvar.h"
// test
#include "abacus/subserver.h"
#include "abacus/notifyserver.h"
#include "abacus/notification.h"
#include "abacus/broadcast.h"
#endif

#include <string.h>
#include <stdio.h>

#ifdef ABACUS_SYS_POWERPLUS
extern "C" {
#include "/home/user/christof/abacus/include/math.h"
}
#else
#include <math.h>
#endif

#ifdef ABACUS_COMPILER_VISUAL_CPP
#include <process.h>
#define getpid _getpid
#else
extern "C" {
#include <unistd.h>
}
#endif

const char* ABA_MASTER::STATUS_[]={"Optimal", "Error", "Unprocessed", "Processing",
                   "Guaranteed", "MaxLevel", "MaxCpuTime",
                   "MaxCowTime", "ExceptionFathom"};
const char* ABA_MASTER::OUTLEVEL_[] = {"Silent","Statistics","Subproblem",
                         "LinearProgram","Full"};
const char * ABA_MASTER::ENUMSTRAT_[]={"BestFirst","BreadthFirst",
                                      "DepthFirst","DiveAndBest"};
const char * ABA_MASTER::BRANCHINGSTRAT_[]={"CloseHalf","CloseHalfExpensive"};
const char* ABA_MASTER::PRIMALBOUNDMODE_[]={"None","Optimum","OptimumOne"};
const char* ABA_MASTER::SKIPPINGMODE_[]={"SkipByNode","SkipByLevel"};
const char* ABA_MASTER::CONELIMMODE_[]={"None","NonBinding","Basic"};
const char* ABA_MASTER::VARELIMMODE_[]={"None","ReducedCost"};
const char* ABA_MASTER::VBCMODE_[]={"None","File","Pipe"};
const char* ABA_MASTER::LPSOLVER_[]={"Cplex","Soplex","Xpress"};

@q m f Constructor @>
@q m e t @>
@ The constructor.
@q m b a @>
  \BeginArg
       \docarg{|problemName|}{The name of the problem being solved.
                              Must not be a 0-pointer.}
       \docarg{|cutting|}{If |true|, then cutting planes can be
                          generated if the function |ABA_SUB::separate()|
                          is redefined.}
       \docarg{|pricing|}{If |true|, then inactive variables are priced in,
                          if the function |ABA_SUB::pricing()| is redefined.}
       \docarg{|optSense|}{The sense of the optimization. The default
                           value is |ABA_OPTSENSE::Unknown|. If the sense
                           is unknown when this constructor is called,
                           e.g., if it is
                           read from a file in the constructor of the
                           derived class, 
                           then it must be initialized in the constructor of
                           the derived class.}
       \docarg{|eps|}{The zero-tolerance used within all member functions of
                   objects which have a pointer to this master
                   (default value |1.0e-4|).}
       \docarg{|machineEps|}{The machine dependent zero tolerance
                           (default value |1.0e-7|).}
       \docarg{|infinity|}{All values greater than |infinity| are
                        regarded as ``infinite big'', all values less than
                        |-infinity| are regarded as ``infinite small''
                        (default value |1.0e30|).}
      \docarg{|readParamFromFile|}{If true, then the parameter file 
           .abacus is read, otherwise the parameters are initialized
             with default values (default |true|).}
                                       
  \EndArg
@q m e a @>
  The members |primalBound_| and |dualBound_| stay uninitialized
  since this can only be done when the sense of optimization
  (minimization or maximization) is known. The initialization
  is performed automatically in the function |optimize()|.
  
@(master.cc@>=
@q m b f @>
  ABA_MASTER::ABA_MASTER(const char *problemName, bool cutting, bool pricing,
                         ABA_OPTSENSE::SENSE optSense,
                         double eps, double machineEps, double infinity,
                         bool readParamFromFile) 
@q m e f @>
    : @/
    ABA_GLOBAL(eps, machineEps, infinity), @/          
    problemName_(this, problemName), @/
    readParamFromFile_(readParamFromFile), @/
    optSense_(optSense), @/
    root_(0), @/
    rRoot_(0), @/
    openSub_(0), @/
    history_(0),@/
    enumerationStrategy_(BestFirst), @/
    branchingStrategy_(CloseHalfExpensive), @/
    nBranchingVariableCandidates_(1), @/
    defaultLpSolver_(Cplex), @/
    lpMasterCplex_(0), @/
    lpMasterSoplex_(0), @/
    lpMasterXpress_(0), @/
    conPool_(0), @/
    cutPool_(0), @/
    varPool_(0), @/
    fixCand_(0), @/
    cutting_(cutting), @/
    pricing_(pricing), @/
    nSubSelected_(0), @/
    VbcLog_(NoVbc), @/
    treeStream_(0), @/
    requiredGuarantee_(0.0), @/
    maxLevel_(INT_MAX), @/
    maxCpuTime_(this, "999999:59:59"), @/
    maxCowTime_(this, "999999:59:59"), @/
    objInteger_(false), @/
    tailOffNLp_(0), @/
    tailOffPercent_(0.000001), @/
    dbThreshold_(0), @/
    minDormantRounds_(1), @/
    outLevel_(Full), @/
    logLevel_(Silent), @/
    pbMode_(NoPrimalBound), @/
    pricingFreq_(0), @/
    skipFactor_(1), @/
    skippingMode_(SkipByNode), @/
    fixSetByRedCost_(true), @/
    printLP_(false), @/
    maxConAdd_(100), @/
    maxConBuffered_(100), @/
    maxVarAdd_(100), @/
    maxVarBuffered_(100), @/
    maxIterations_(INT_MAX), @/
    eliminateFixedSet_(false), @/
    newRootReOptimize_(false), @/
    optimumFileName_(this), @/
    showAverageCutDistance_(false), @/
    conElimMode_(NoConElim), @/
    varElimMode_(NoVarElim), @/
    conElimEps_(0.001), @/
    varElimEps_(0.001), @/
    conElimAge_(1), @/
    varElimAge_(1), @/
    status_(Unprocessed), @/
    totalCowTime_(this), @/
    totalTime_(this), @/
    lpTime_(this), @/
    lpSolverTime_(this), @/
    separationTime_(this), @/
    improveTime_(this), @/
    pricingTime_(this), @/
    branchingTime_(this), @/
    nSub_(0), @/
    nLp_(0), @/
    highestLevel_(0), @/
    nFixed_(0), @/
    nAddCons_(0), @/
    nRemCons_(0), @/
    nAddVars_(0), @/
    nRemVars_(0), @/
    nNewRoot_(0)
   {
    _createLpMasters();
    @<|ABA_MASTER::ABA_MASTER()|: allocate some members@>;
#ifdef ABACUS_PARALLEL
    parmaster_ = new ABA_PARMASTER(this);
#endif
    @<initialize the parameters of the |ABA_MASTER|@>;
    @<start the timers@>;
  }

@ @<start the timers@>=
  totalCowTime_.start();
  totalTime_.start();

@ The parameters are read from the file {\tt .abacus}, which is searched
  in the directory given by the environment variable
  |ABACUS_DIR|. The function
  |_initializeParameters()| is called again in the member function
  |optimize()| such that parameters of the |ABA_MASTER| can be overwritten
  in problem specific parameter files, which can be read via the
  virtual function |initializeParameters()| called from
  |_initializeParameters()|. In the constructors of base classes
  redefined virtual functions are not recognized in \CPLUSPLUS/.
  
  If the output level is silent, we turn the output off immediately.
  
@<initialize the parameters of the |ABA_MASTER|@>=
   _initializeParameters();

  if (outLevel_ == Silent)
    out().off();
  if (logLevel_ == Silent)
    out().logOff();

@ @<|ABA_MASTER::ABA_MASTER()|: allocate some members@>=
  history_ = new ABA_HISTORY(this);
  openSub_ = new ABA_OPENSUB(this);
  fixCand_ = new ABA_FIXCAND(this);

@q m v Destructor @>
@q m e t @>  
@ The destructor.

@(master.cc@>=
@q m b f @>
  ABA_MASTER::~ABA_MASTER()
@q m e f @>
  {
#ifdef ABACUS_PARALLEL
    delete parmaster_;
#endif
    if (treeStream_ != &cout) delete treeStream_;
    delete history_;
    delete conPool_;
    delete cutPool_;
    delete varPool_;
    delete openSub_;
    delete fixCand_;
    _deleteLpMasters();
  }



#ifdef ABACUS_PARALLEL

@q m f unpackConVar @>
@q m e t @>
@ The virtual function |unpackConVar()| constructs and unpacks an object
  of some subclass of |ABA_CONVAR| from a Message. The function is
  called every time an object of some subclass of |ABA_CONSTRAINT|
  or |ABA_VARIABLE| has to be recreated at the receiving processor.

  This function has to be implemented by the user if user defined subclasses
  of |ABA_CONSTRAINT| or |ABA_VARIABLE| are used. In this case
  the |unpackConVar()| function of the derived class has to call
  the |unpackConVar()| function of the base class if it cannot construct
  an object of the requested class.   

@q m b r @>
  \BeginReturn
  A Pointer to a newly constructed object of the subclass of |ABA_CONVAR|
  specified by |classId|.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The message from which the |ABA_CONVAR| is unpacked.}
  \docarg{|classId|}
    {The class identification of the subclass of |ABA_CONVAR|
     which should be unpacked.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  ABA_CONVAR *ABA_MASTER::unpackConVar(ABA_MESSAGE &msg, int classId) const
@q m e f @>
  {
    switch (classId) {
      case NUMCON_CLASSID:
        return new @[ABA_NUMCON(this, msg)@];
      case ROWCON_CLASSID:
        return new @[ABA_ROWCON(this, msg)@];
      case SROWCON_CLASSID:
        return new @[ABA_SROWCON(this, msg)@];
      case NUMVAR_CLASSID:
        return new @[ABA_NUMVAR(this, msg)@];
      case COLVAR_CLASSID:
        return new @[ABA_COLVAR(this, msg)@];
    }
    err() << "ABA_MASTER::unpackConVar() : cannot unpack subclass of"
          << " ABA_CONVAR with classId = " << classId << "." << endl;
    exit(Fatal);
    return 0;           // to suppress compiler warning
  }


@q m f unpackSub @>
@q m e t @>
@ The pure virtual function |unpackSub()| constructs and unpacks an object
  of the user defined subclass of |ABA_SUB| from a Message.

  This function has to be implemented by the user.

@q m b r @>
  \BeginReturn
  A Pointer to a newly constructed object of the user defined subclass of |ABA_SUB|.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The message from which the subclass of |ABA_SUB| is unpacked.}
  \EndArg
@q m e a @>

@q m b f @>
ABA_SUB* ABA_MASTER::unpackSub(ABA_MESSAGE &msg)
@q m e f @>



@q m f unpackBranchrule @>
@q m e t @>
@ The virtual function |unpackBranchrule()| construicts and unpacks an object
  of some subclass of |ABA_BRANCHRULE| from a Message. It has to be redefined
  in a derived class if user defined subclasses of |ABA_BRANCHRULE| are used.
  The |unpackBranchrule()| function of the derived class has to call
  the |unpackBranchrule()| function of the base class if it cannot construct
  an object of the requested subclass of |ABA_BRANCHRULE|. 

@q m b r @>
  \BeginReturn
  A Pointer to a newly constructed object of the subclass of |ABA_BRANCHRULE|
  specified by |classId|.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The message from which the |ABA_BRANCHRULE| is unpacked.}
  \docarg{|classId|}
    {The class identification of the subclass of |ABA_BRANCHRULE|
     which should be unpacked.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  ABA_BRANCHRULE *ABA_MASTER::unpackBranchrule(ABA_MESSAGE &msg, int classId) const
@q m e f @>
  {
    switch (classId) {
      case SETBRANCHRULE_CLASSID:
        return new @[ABA_SETBRANCHRULE(this, msg)@];
      case VALBRANCHRULE_CLASSID:
        return new @[ABA_VALBRANCHRULE(this, msg)@];
      case CONBRANCHRULE_CLASSID:
        return new @[ABA_CONBRANCHRULE(this, msg)@];
      case BOUNDBRANCHRULE_CLASSID:
        return new @[ABA_BOUNDBRANCHRULE(this, msg)@];
    }
    err() << "ABA_MASTER::unpackBranchrule() : cannot unpack subclass of"
          << " ABA_BRANCHRULE with classId = " << classId << "." << endl;
    exit(Fatal);
    return 0;           // to suppress compiler warning
  }

#endif


@q m f optimize @>
@ The function |optimize()| performs the optimization by \bab.
@q m e t @>
@q m b r @> 
  \BeginReturn
  The status of the optimization.
  \EndReturn
@q m e r @>

@(master.cc@>=
@q m b f @>
  ABA_MASTER::STATUS ABA_MASTER::optimize()
@q m e f @>
  {
    @<startup the optimization@>;
    @<perform the branch and bound algorithm@>;
    @<output history and statistics@>;
    @<|ABA_MASTER::optimize()|: clean up and return@>;
  }

@ We call |_initializeParameters()| again, because the call in the
  constructor of |ABA_MASTER| will not call a possibly redefined virtual
  function in a derived class.

  The default implementation of the virtual function |initializeOptimization()|
  does nothing. A reimplementation can be used for problem specific
  initializations.

@<startup the optimization@>=
  _initializeParameters();
  @<output the ABACUS banner@>;
#ifdef ABACUS_PARALLEL
  if (parmaster_->isHostZero()) {
    @<initialize the tree-log file@>;
  }
#else
  @<initialize the tree-log file@>;
#endif
  @<initialize the primal and the dual bound@>;
  initializeOptimization();
  @<print the parameters@>;
  @<output a headline for output level |Subproblem| and |LinearProgram|@>;

@ We do not output the banner right at the beginning of the constructor,
  because when a problem specific parameter file is read
  in the function |_initializeParameters()| the output level might become
  |Silent|.
  
@<output the ABACUS banner@>=
  if (outLevel_ == Silent) out().off();
  if (logLevel_ == Silent) out().logOff();
  
  out() << endl;
  out() << "ABACUS, Version " << ABACUS_VERSION_STRING << ", " << __DATE__
        << ", " << __TIME__ << endl;
  out() << "Copyright (c) Universitaet Koeln, Germany" << endl << endl;
#ifdef ABACUS_PARALLEL
  out() << "- PARALLEL VERSION -" << endl;
#endif

  out() << endl;
  out() << "BRANCH AND CUT OPTIMIZATION" << endl << endl;

  if (outLevel_ != Full) out().off();
  if (logLevel_ != Full) out().logOff();

@ If |VbcLog_| is |File|, then we generate a log-file which can be read
  by the VBC-tool to visualize the implicit enumeration. The name
  of the log-file is composed of the problem name, the process ID, and
  the extension {\tt .tree}. The length of the array storing the name
  of the log-file is sufficient for process ids represented by integers with
  up to 64 bit (a 64 bit integer has most 19 decimal digits).
  If |VbcLog_| is |Pipe| then we write the instructions to standard out.
  
@<initialize the tree-log file@>=
  if (VbcLog_ == File) {
    char *treeStreamName = new char[strlen(problemName_.string())+40];

    sprintf(treeStreamName, "%s.%ld.tree", problemName_.string(), getpid());
    treeStream_ = new @,@[ofstream(treeStreamName)@];

    delete [] treeStreamName;
  }
  else if (VbcLog_ == Pipe)
  {
    ABA_STRING pipeName(this);
    if (getParameter("VbcPipeName", pipeName)==0)
      treeStream_ = new @,@[ofstream(pipeName.string())@];
    else
      treeStream_ = &cout;
  }

  @<write the first lines of input for the tree interface@>;

@ @<write the first lines of input for the tree interface@>=
  if (VbcLog_ != NoVbc) {
    writeTreeInterface("#TYPE: COMPLETE TREE", false);
    writeTreeInterface("#TIME: SET", false);
    writeTreeInterface("#BOUNDS: SET", false);
    writeTreeInterface("#INFORMATION: STANDARD", false);
    writeTreeInterface("#NODE_NUMBER: NONE", false);
  }

@ The initialization of the primal and dual bound cannot be performed
  in any case in the constructor, as the sense of optimization
  might be unknown, if it is read from the constructor of a derived class.
  Therefore, we initialize these bounds now. If the initialization of the
  sense of the optimization has been forgotten, then the program terminates
  with an error.

@<initialize the primal and the dual bound@>=
  @<initialize the bounds according to sense of optimization@>;
  @<depending on |PRIMALBOUNDMODE| reinitialize |primalBound|@>;

@ @<initialize the bounds according to sense of optimization@>=
  switch (optSense_.sense()) {
    case ABA_OPTSENSE::Min:
      primalBound_ =  infinity();
      dualBound_   = -infinity();
      break;
    case ABA_OPTSENSE::Max:
      primalBound_ = -infinity();
      dualBound_   =  infinity();
      break;
    case ABA_OPTSENSE::Unknown:
      err() << "ABA_MASTER::optimize(): optimization sense unknown." << endl;
      err() << "Specify optimization sense in the constructor or " << endl;
      err() << "use function initializeOptSense()." << endl;
      exit (Fatal);
  }

@ @<depending on |PRIMALBOUNDMODE| reinitialize |primalBound|@>=
  double opt;
  if (knownOptimum(opt)) {
    switch (pbMode_) {
      case Optimum: primalBound_= opt;
                               break;
      case OptimumOne:
        if (optSense_.max()) primalBound_ = opt - 1.0;
        else                 primalBound_ = opt + 1.0;
      default: break;
    }
  }

@ The parameters are only not printed if the output level or the log
  level is |Silent|.

@<print the parameters@>=
  bool turnOutOff = false;
  if (outLevel_ != Silent && !out().isOn()) {
    out().on();
    turnOutOff = true;
  }
    
  bool turnLogOff = false;
  if (logLevel_ != Silent && !out().isLogOn()) {
    out().logOn();
    turnLogOff = true;
  }

  printParameters();

  if (turnOutOff) out().off();
  if (turnLogOff) out().logOff();

@ @<output a headline for output level |Subproblem| and |LinearProgram|@>=
  if (outLevel_ == Subproblem ||
      outLevel_ == LinearProgram ||
      logLevel_ == Subproblem ||
      logLevel_ == LinearProgram   ) {

    @<turn the streams on@>;
    out() << endl;
    out() << "   #sub   #open   current   #iter         LP       dual     primal" << endl;
    @<turn the streams off@>;
  }

@ @<turn the streams on@>=
  bool turnOn = false;

  if (outLevel_ == Subproblem ||
      outLevel_ == LinearProgram  )
    out().on();
  else if (out().isOn()) {
    out().off();
    turnOn = true;
  }

  bool turnLogOn = false;

  if (logLevel_ == Subproblem ||
      logLevel_ == LinearProgram)
    out().logOn();
  else if (out().isLogOn()) {
    out().logOff();
    turnLogOn = true;
  }

@ @<turn the streams off@>=
  if (outLevel_ == Subproblem ||
      outLevel_ == LinearProgram)
    out().off();
  if (logLevel_ == Subproblem ||
      logLevel_ == LinearProgram)
    out().logOff();

  if (turnOn)
    out().on();

  if (turnLogOn)
    out().logOn();

@ @<perform the branch and bound algorithm@>=
  @<initialize the root node@>;
  @<process all subproblems@>;

@ The function |firstSub()| is a pure virtual function returning the problem
  specific subproblem derived from the class |ABA_SUB|, which is associated with
  the root node of the \bab\ tree.

@<initialize the root node@>=
  status_ = Processing;
#ifdef ABACUS_PARALLEL
  if (parmaster_->isHostZero()) {
    root_  = firstSub();
    openSub_->insert(root_);
    // treeInterfaceNewNode(root_);
  }
#else
  root_   = firstSub();
  rRoot_  = root_;

  openSub_->insert(root_);

  treeInterfaceNewNode(root_);
#endif

#ifdef ABACUS_PARALLEL
  // test, is root used later?
  root_ = rRoot_ = 0;
#endif

@ The following lines of code are the core of the \bab\ algorithm.
  The function |select()| returns subproblems which still have to be
  processed until either the set of open subproblem becomes empty or
  one of the resource limits (e.g., maximal cpu time) is exceeded.
  If the optimization of a subproblem fails we quit the optimization
  immediately..
  
@<process all subproblems@>=
  ABA_SUB *current;

  while ((current = select())) {
    ++nSubSelected_;

    if (current->optimize()) {
      status_ = Error;
      break;
    }
#ifdef ABACUS_PARALLEL
    if (current->status() == ABA_SUB::Fathomed)
      delete current;
#endif
  }

  if (status_ == Processing) status_ = Optimal;

@ The virtual dummy function |output()| can be redefined in derived classes
  for problem specific output.
  
@<output history and statistics@>=
  @<turn the output on if the output level is not |Silent|@>;
  @<output the solution history@>;
  @<output information on the tree, variables, constraints, etc.@>;
  @<output the timing statistics@>;

  output();

  @<output the value of the best solution@>;
  @<output the status of the optimization@>;

@ @<turn the output on if the output level is not |Silent|@>=
  if (outLevel_ != Silent) out().on();
  if (logLevel_ != Silent) out().logOn();

@ @<output the solution history@>=
  out() << endl;
  out() << *history_ << endl;

@ @<output information on the tree, variables, constraints, etc.@>=
  const int w = 6;

  out() << "Miscellaneous Statistics" << endl << endl;
  out() << "  Dual bound of the root node       : ";
  out() << setWidth(w) << rootDualBound_     << endl;
  out() << "  Number of subproblems             : ";
  out() << setWidth(w) << nSub_     << endl;
  out() << "  Number of solved LPs              : ";
  out() << setWidth(w) << nLp_      << endl;
  out() << "  Highest level in tree             : ";
  out() << setWidth(w) << highestLevel_   << endl;
  out() << "  Number of fixed variables         : ";
  out() << setWidth(w) << nFixed_   << endl;
  out() << endl;
  out() << "  Number of added constraints       : ";
  out() << setWidth(w) << nAddCons_ << endl;
  out() << "  Number of removed constraints     : ";
  out() << setWidth(w) << nRemCons_ << endl;
  out() << "  Number of added variables         : ";
  out() << setWidth(w) << nAddVars_ << endl;
  out() << "  Number of removed variables       : ";
  out() << setWidth(w) << nRemVars_ << endl;
  out() << endl;
  out() << "  Number of root changes            : ";
  out() << setWidth(w) << nNewRoot_ << endl;

  _outputLpStatistics();
#ifdef ABACUS_PARALLEL
  parmaster_->outputStatistics();
#endif

  out() << endl << endl;


@ The function |_outputLpStatistics()| prints the LP solver
  specific statistics.

  This function is implemented in the file |lpif.cc|.



@ The cpu time for branching may include time for linear programming.
  Therefore, we should not subract it from the total time. Otherwise,
  the LP time for LP ranking of branching rules would be subtracted twice.
@<output the timing statistics@>=
  double lpPercent;   // percentage of lp time of total cpu time
  double sepPercent;  // percentage of separation time of total cpu time
  double heuPercent;  // percentage of heuristic time of total cpu time
  double priPercent;  // percentage of pricing time of total cpu time
  double misPercent;  // percentage of miscellaneous time of total cpu time
    
  long   misTime;     // miscellaneous time in seconds
    
  double totcsecs = (double) totalTime_.centiSeconds();

  misTime =   totalTime_.centiSeconds() - lpTime_.centiSeconds()
            - separationTime_.centiSeconds() - improveTime_.centiSeconds()
            -pricingTime_.centiSeconds();

  ABA_CPUTIMER misTimer(this, misTime);  // required for nice output of the value

  if (totcsecs < eps()) totcsecs = 1.0;  // avoid zero division in following lines

  lpPercent  = lpTime_.centiSeconds()/totcsecs*100.0;
  sepPercent = separationTime_.centiSeconds()/totcsecs*100.0;
  heuPercent = improveTime_.centiSeconds()/totcsecs*100.0;
  priPercent = pricingTime_.centiSeconds()/totcsecs*100.0;
  misPercent = misTime/totcsecs*100.0;

  const int wpc = 7;

  out() << "Timing Statistics" << endl << endl;
  out() << "  Elapsed time           : " << totalCowTime_ << endl;
  out() << "  Total cpu time         : " << totalTime_   << endl;
  out() << "  LP cpu time            : " << lpTime_;
  out() << "  (" << setWidth(wpc) <<  lpPercent << "%)" << endl;
  out() << "  LP solver cpu time     : " << lpSolverTime_ << endl;
  out() << "  Separation cpu time    : " << separationTime_;
  out() << "  (" << setWidth(wpc) <<  sepPercent << "%)" << endl;
  out() << "  Heuristics cpu time    : " << improveTime_;
  out() << "  (" << setWidth(wpc) << heuPercent << "%)" << endl;
  out() << "  Pricing cpu time       : " << pricingTime_;
  out() << "  (" << setWidth(wpc) << priPercent << "%)" << endl;
  out() << "  Branching cpu time     : " << branchingTime_ << endl;
  out() << "  Miscellaneous cpu time : " << misTimer;
  out() << "  (" << setWidth(wpc) << misPercent << "%)" << endl;

@ @<output the value of the best solution@>=
  out() << endl;
  if (feasibleFound()) 
    out() << "Best solution: " << primalBound_ << endl;
  else
    out() << "No feasible solution found." << endl;

@ @<output the status of the optimization@>=
  out() << endl << "ABACUS optimization terminated with status ";

  out() << STATUS_[status_];

  out() << "." << endl;

@ Before cleaning up we call the virtual function |terminateOptimization()|.
  The default implementation of this function does nothing but it
  can be used as entrance point for problem specific actions
  (e.g., visualization of the best solution).

  By deleting the |root_| of the branch and cut tree, recursively the
  destructors of all subproblems in the \bac\ tree are called.

  After the optimization we delete the candidates for fixing such
  that the variable pools can be safely deleted, as it is an error
  to delete a still referenced slot of a pool.

@<|ABA_MASTER::optimize()|: clean up and return@>=

  terminateOptimization();

  delete root_;

  fixCand_->deleteAll();

#ifdef ABACUS_PARALLEL
  if (VbcLog_ == Pipe && parmaster_->isHostZero()) {
#else
  if (VbcLog_ == Pipe) {
#endif
    char string[256];
    sprintf(string, "#END_OF_OUTPUT");
    writeTreeInterface(string);
  }

  return status_;

@q m v initializeOptimization @>

@ The default implementation of |initializeOptimization()| does nothing. This
  virtual function can be used as an entrance point to perform
  some initializations after |optimize()| is called.

@q m e t @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::initializeOptimization()
@q m e f @>
  { }

@q m v initializePools @>

@ The virtual function |initializePools()| sets up the default pools for
  variables, constraints, and cutting planes.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|constraints|}{The constraints of the problem formulation
                         are inserted in the constraint pool. The size
                         of the constraint pool equals the number of
                         |constraints|.}
  \docarg{|variables|}{The variables of the problem formulation are
                       inserted in the variable pool.}
  \docarg{|varPoolSize|}{The size of the pool for the variables. If
                         more variables are added the variable pool
                         is automatically reallocated.}
  \docarg{|cutPoolSize|}{The size of the pool for cutting planes.}
  \docarg{|dynamicCutPool|}{If this argument is true, then the cut
                            is automatically reallocated if more
                            constraints are inserted than |cutPoolSize|.
                            Otherwise, non-active constraints are removed
                            if the pool becomes full.
                            The default value is false.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::initializePools(ABA_BUFFER<ABA_CONSTRAINT*>  &constraints,
                                   ABA_BUFFER<ABA_VARIABLE*>    &variables,
                                   int varPoolSize,
                                   int cutPoolSize,
                                   bool dynamicCutPool)
@q m e f @>
  {
    @<initialize the constraint pool of the |ABA_MASTER|@>;
    @<initialize the variable pool of the |ABA_MASTER|@>;
    @<set up the pool for dynamically generated constraints@>;
  }

@ @<initialize the constraint pool of the |ABA_MASTER|@>=
  const int nCons = constraints.number();

  conPool_ = new @,@[ABA_STANDARDPOOL<ABA_CONSTRAINT, ABA_VARIABLE>(this, nCons)@];

#ifdef ABACUS_PARALLEL
  ABA_ID id;
  ABA_IDMAP<ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> @= @> > *conPoolMap =
    conPool_->identificationMap();
  int index = conPoolMap->index();
  for (int i = 0; i < nCons; i++) {
    ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *ps = conPool_->insert(constraints[i]);
    id.initialize(i+1, -1, index);   // initialized constraints (proc=-1)
    ps->setIdentification(id);
  }
#else
  for (int i = 0; i < nCons; i++)
    conPool_->insert(constraints[i]);
#endif

@ The size of the variable pool must be at least the number of variables
  which are inserted in this function. A variable pool is always an
  automatically growing pool, as the addition of a variable must be
  always possible for the correctness of the algorithm. Therefore, the
  third argument of the constructor of |ABA_STANDARDPOOL| is |true|.
  
@<initialize the variable pool of the |ABA_MASTER|@>=
  const int nVars = variables.number();

  if (varPoolSize < nVars) varPoolSize = nVars;
  
  varPool_ = new @,@[ABA_STANDARDPOOL<ABA_VARIABLE, ABA_CONSTRAINT>(this, varPoolSize, true)@];

#ifdef ABACUS_PARALLEL
  ABA_IDMAP<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> @= @> > *varPoolMap =
    varPool_->identificationMap();
  index = varPoolMap->index();
  for (int i = 0; i < nVars; i++) {
    ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *ps = varPool_->insert(variables[i]);
    id.initialize(i+1, -1, index);   // initialized variables (proc=-1)
    ps->setIdentification(id);
  }
#else
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVars; i++)
#else
  for (int i = 0; i < nVars; i++)
#endif
    varPool_->insert(variables[i]);
#endif

@ If the third argument of the constructor of |ABA_STANDARDPOOL| is missing
  the size of the pool is fixed, of the third argument is |true| then
  the pool is automatically increased if it is full and an insertion
  is performed.
  
@<set up the pool for dynamically generated constraints@>=
  if (cutPoolSize > 0)
    cutPool_ = new ABA_STANDARDPOOL<ABA_CONSTRAINT, ABA_VARIABLE>(this, cutPoolSize,
                                               dynamicCutPool);


@q m v initializePools @>

@ The virtual function |initializePools()| is overloaded such that
  also a first set of cutting planes can be inserted into the cutting 
  plane pool.

@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|constraints|}{The constraints of the problem formulation
                         are inserted in the constraint pool. The size
                         of the constraint pool equals the number of
                         |constraints|.}
  \docarg{|cuts|}{The constraints that are inserted in the cutting
                  plane pool. The number of constraints in the buffer
                  must be less or equal than the size of the cutting
                  plane pool |cutPoolSize|.}
  \docarg{|variables|}{The variables of the problem formulation are
                       inserted in the variable pool.}
  \docarg{|varPoolSize|}{The size of the pool for the variables. If
                         more variables are added the variable pool
                         is automatically reallocated.}
  \docarg{|cutPoolSize|}{The size of the pool for cutting planes.}
  \docarg{|dynamicCutPool|}{If this argument is true, then the cut
                            is automatically reallocated if more
                            constraints are inserted than |cutPoolSize|.
                            Otherwise, non-active constraints are removed
                            if the pool becomes full.
                            The default value is false.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::initializePools(ABA_BUFFER<ABA_CONSTRAINT*> &constraints,
                                   ABA_BUFFER<ABA_CONSTRAINT*> &cuts,
                                   ABA_BUFFER<ABA_VARIABLE*> &variables,
                                   int varPoolSize,
                                   int cutPoolSize,
                                   bool dynamicCutPool)
@q m e f @>
  {
    initializePools(constraints, variables, varPoolSize, cutPoolSize,
                    dynamicCutPool);

    @<is there enough space for all the cuts in the cut pool@>;

#ifdef ABACUS_PARALLEL
    ABA_ID id;
    int index = cutPool_->identificationMap()->index();
    for (int i = 0; i < nCuts; i++) {
      ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *ps = cutPool_->insert(cuts[i]);
      id.initialize(i+1, -1, index);   // initialized cuts (proc=-1)
      ps->setIdentification(id);
    }
#else
    for (int i = 0; i < nCuts; i++)
      cutPool_->insert(cuts[i]);
#endif
  }

@ Even if the cut pool is dynamic it is an error if the cut pool is
  too small. Because before the cut pool is enlarged, inactive, i.e.\
  just inserted, constraints would be removed.

@<is there enough space for all the cuts in the cut pool@>=
  const int nCuts = cuts.number();

  if (nCuts > cutPoolSize) {
    err() << "ABA_MASTER::initializePools(): size of cutting plane ";
    err() << " too small for all initialize cuts." << endl;
    err() << "size of cut pool: " << cutPoolSize << endl;
    err() << "number of cuts  : " << nCuts << endl;
    exit(Fatal);
  }

@q m f initializeOptSense @>
@ The function |initializeOptSense()| can be used to initialize the sense
  of the optimization in derived classes, if this has not been already 
  performed when the constructor of |ABA_MASTER| has been called.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|sense|}{The sense of the optimization (|ABA_OPTSENSE::Min| or
                  |ABA_OPTSENSE::Max|).}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::initializeOptSense(ABA_OPTSENSE::SENSE sense)
@q m e f @>
  {
    optSense_.sense(sense);
  }

@q m v terminateOptimization @>

@ The default implementation of |terminateOptimization()| does nothing. This
  virtual function can be used as an entrance point 
  after the optimization process is finished.

@q m e t @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::terminateOptimization()
@q m e f @>  
  { }

@ The function |select()| returns a pointer to an open subproblem for
  further processing. If the set of open subproblems is empty or
  one of the criteria for early termination of the optimization
  (maximal cpu time, maximal elapsed time, guarantee) is
  fulfilled 0 is returned.

@(master.cc@>=
  ABA_SUB* ABA_MASTER::select()
  {
    @<check if we should terminate the optimization@>;

    return openSub_->select();
  }

@ If one of the criteria for early termination is satisfied then
  we fathom all subproblems of the tree, in order to perform a
  correct cleaning up.

  The maximal level of the enumeration tree is no termination criterion
  in this sense, because it only prevents the generation of further
  sons of subproblems having this maximal level, but does not
  stop the optimization.
  
@<check if we should terminate the optimization@>=
  if (totalTime_.exceeds(maxCpuTime())) {
    out() << "Maximal CPU time " << maxCpuTime() << " exceeded." << endl;
    out() << "Stop optimization." << endl;
    root_->fathomTheSubTree();
    status_ = MaxCpuTime;
    return 0;
  }
    
  if (totalCowTime_.exceeds(maxCowTime())) {
    out() << "Maximal elapsed time " << maxCowTime() << " exceeded." << endl;
    out() << "Stop optimization." << endl;
    root_->fathomTheSubTree();
    status_ = MaxCowTime;
    return 0;
  }

  if (guaranteed()) {
    out() << endl;
    out() << "Guarantee " << requiredGuarantee() << " % reached." << endl;
    out() << "Terminate optimization." << endl;
    status_ = Guaranteed;
    root_->fathomTheSubTree();
    return 0;
  }

@q m f enumerationStrategy @>
@q m e t @>
@ The function |enumerationStrategy()|.

@q m b r @>
  \BeginReturn
  The enumeration strategy.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::ENUMSTRAT ABA_MASTER::enumerationStrategy() const
@q m e f @>

@q m f enumerationStrategy @>
@ This version of  the function |enumerationStrategy()| changes the
  enumeration strategy.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|strat|}{The new enumeration strategy.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::enumerationStrategy(ENUMSTRAT strat)
@q m e f @>

@q m v enumerationStrategy @>

@ The virtual function |enumerationStrategy()| analyzes the enumeration
   strategy set in the parameter file {\tt .abacus}
  and calls the corresponding comparison function for the subproblems
  |s1| and |s2|. This function should be redefined for application
  specific enumeration strategies.
@q m e t @>
@q m b r @>
  \BeginReturn
  If |s1| has higher priority than |s2| it returns 1,
  if |s2| has higher priority it returns $-1$, and if both subproblems
  have equal priority it returns 0.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|s1|}{A pointer to subproblem.}
  \docarg{|s2|}{A pointer to subproblem.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  int ABA_MASTER::enumerationStrategy(const ABA_SUB *s1, const ABA_SUB
  *s2)
@q m e f @>
  {
    switch (enumerationStrategy_) {
      case BestFirst:    return bestFirstSearch(s1, s2);
      case BreadthFirst: return breadthFirstSearch(s1, s2);
      case DepthFirst:   return depthFirstSearch(s1, s2);
      case DiveAndBest:  return diveAndBestFirstSearch(s1, s2);
      default: err() << "ABA_MASTER::enumerationStrategy(): Unknown enumeration strategy" << endl;
               exit(Fatal);
               return 0;
    }
  }

@q m f bestFirstSearch @>
@ The function |bestFirstSearch()| implements the best first search
  enumeration. If the bounds of both subproblems are equal, then
  the subproblems are compared with the function |equalSubCompare()|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|-1|}{If subproblem |s1| has a worse dual bound than |s2|,
                i.e., if it has a smaller dual bound for minimization or
                a larger dual bound for maximization problems.}
  \docarg{|1|}{If subproblem |s2| has a worse dual bound than |s1|.}
  \docarg{|0|}{If both subproblems have the same priority in the
               enumeration strategy.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|s1|}{A subproblem.}
  \docarg{|s2|}{A subproblem.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  int ABA_MASTER::bestFirstSearch(const ABA_SUB *s1, const ABA_SUB *s2) const
@q m e f @>
  {
    double dual1 = s1->dualBound();
    double dual2 = s2->dualBound();

    if (optSense()->max()) {
      if (dual1 > dual2) return  1;
      if (dual1 < dual2) return -1;
      return equalSubCompare(s1, s2);
    }
    else {
      if (dual1 > dual2) return -1;
      if (dual1 < dual2) return  1;
      return equalSubCompare(s1, s2);
    }
  }

@q m v equalSubCompare @>

@ The virtual function |equalSubCompare()| is called from the function
  |bestFirstSearch()| and from the function |depthFirstSearch()|
  if the subproblems |s1| and |s2| have the same priority.
  If both subproblems were generated by setting a binary variable, then
  that subproblem has higher priority of which the branching variable is
  set to upper bound.

  This function can be redefined to resolve equal subproblems according
  to problem specific criteria.
@q m e t @>
  As the root node is compared with itself and has no branching rule,
  we have to insert the first line of this function.
@q m b a @>
  \BeginArg
  \docarg{|s1|}{A subproblem.}
  \docarg{|s2|}{A subproblem.}
  \EndArg
@q m e a @>
@q m b r @>
  \BeginReturn
  \docarg{|0|}{If both subproblems were not generated by setting a
               variable, or the branching variable of both subproblems
               is set to the same bound.}
  \docarg{|1|}{If the branching variable of the first subproblem
             ist set to the upper bound.}
  \docarg{|-1|}{If the branching variable of the second subproblem
             ist set to the upper bound.}
  \EndReturn
@q m e r @>

@(master.cc@>=
@q m b f @>
  int ABA_MASTER::equalSubCompare(const ABA_SUB *s1, const ABA_SUB *s2) const
@q m e f @>
  {
    if (!s1->branchRule() || !s2->branchRule()) return 0;
    
    if (s1->branchRule()->branchOnSetVar()
        && s2->branchRule()->branchOnSetVar()) {
      if (((ABA_SETBRANCHRULE*) s1->branchRule())->setToUpperBound()) {
        if (((ABA_SETBRANCHRULE*) s2->branchRule())->setToUpperBound())
          return 0;
        else 
          return 1;
      }
      else if (((ABA_SETBRANCHRULE*) s2->branchRule())->setToUpperBound())
        return -1;
      return 0;
    }
    else return 0;
  }
  
@q m f depthFirstSearch @>
@ The function |depthFirstSearch()| implements the depth first search
  enumeration strategy, i.e., the subproblem with maximum |level|
  is selected. If the level of both subproblems are equal, then
  the subproblems are compared with the function |equalSubCompare()|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|-1|}{If subproblem |s1| has higher priority,}
  \docarg{|0|}{if both subproblems have equal priority,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|s1|}{The first subproblem.}
  \docarg{|s2|}{The second subproblem.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  int ABA_MASTER::depthFirstSearch(const ABA_SUB* s1, const ABA_SUB* s2) const
@q m e f @>
  {
    if(s1->level() > s2->level()) return  1;
    if(s1->level() < s2->level()) return -1;
    return equalSubCompare(s1, s2);
  }

@q m f breadthFirstSearch @>
@ The function |breadthFirstSearch()| implements the breadth first search
  enumeration strategy, i.e., the subproblem with minimum |level|
  is selected. If both subproblems have the same |level|, the smaller
  one is the one which has been generated earlier, i.e., the one with
  the smaller |id|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|-1|}{If subproblem |s1| has higher priority,}
  \docarg{|0|}{if both subproblems have equal priority,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|s1|}{The first subproblem.}
  \docarg{|s2|}{The second subproblem.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  int ABA_MASTER::breadthFirstSearch(const ABA_SUB* s1, const ABA_SUB* s2) const
@q m e f @>
  {
    if (s1->level() > s2->level()) return -1;
    if (s1->level() < s2->level()) return  1;

    if (s1->id() < s2->id()) return 1;
    return -1;
  }

@q m f diveAndBestFirstSearch @>
@ The function |diveAndBestFirstSearch()| performs depth-first search until a
  feasible solution is found, then the search process is continued
  with best-first search.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|-1|}{If subproblem |s1| has higher priority,}
  \docarg{|0|}{if both subproblems have equal priority,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|s1|}{The first subproblem.}
  \docarg{|s2|}{The second subproblem.}
  \EndArg
@q m e a @>


@(master.cc@>=
@q m b f @>
  int ABA_MASTER::diveAndBestFirstSearch(const ABA_SUB *s1, const ABA_SUB* s2) const
@q m e f @>
  {
    if (feasibleFound()) return bestFirstSearch(s1, s2);
    else                       return depthFirstSearch(s1, s2);
  }

@ In order to embed both minimization and maximization problems in this
  system we work internally with primal bounds, i.e., a value which
  is worse than the best known solution (often a value of a feasible
  solution), and dual bounds, i.e., a bound
  which is better than the best known solution. Primal and dual bounds
  are then interpreted as lower or upper bounds according to the
  sense of the optimization.


@q m f lowerBound @>
@q m e t @>
@ The function |lowerBound()|.

@q m b r @>
  \BeginReturn
  The value of the global lower bound.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_MASTER::lowerBound() const
@q m e f @>

@q m f upperBound @>
@q m e t @>
@ The function |upperBound()|.

@q m b r @>
  \BeginReturn
   The value of the global upper bound.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_MASTER::upperBound() const
@q m e f @>

@q m f primalBound @>
@q m e t @>
@ The function |primalBound()|.

@q m b r @>
  \BeginReturn
  The value of the primal bound,
  i.e., the |lowerBound()| for a maximization problem and the
  |upperBound()| for a minimization problem, respectively.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_MASTER::primalBound() const
@q m e f @>

@q m f primalBound @>
@ This version of the function |primalBound()| sets the primal bound to |x|
  and makes a new entry in the solution history. It is an error if the
  primal bound gets worse.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|x|}{The new value of the primal bound.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
#ifdef ABACUS_PARALLEL
  void ABA_MASTER::primalBound(double x, bool broadcast)
#else
  void ABA_MASTER::primalBound(double x)
#endif
@q m e f @>
  {
    if (optSense()->max()) {
      if (x < primalBound_) {
        err() << "Error: ABA_MASTER::primalBound(): got worse" << endl;
        err() << "old bound: " << primalBound_ << " new bound: " << x << endl;
        exit (Fatal);
      }
    }
    else
      if (x > primalBound_) {
        err() << "Error: ABA_MASTER::primalBound(): got worse" << endl;
        err() << "old bound: " << primalBound_ << " new bound: " << x << endl;
        exit (Fatal);
      }

    @<make sure that this is an integer value for |objInteger_|@>;
      
#ifdef ABACUS_PARALLEL
    if (broadcast && betterPrimal(x) )
      parmaster_->newPrimalBound(x);
#endif
    primalBound_ = x;
    @<update the primal bound in the Tree Interface@>;
    history_->update();
  }
  
@ After the test for integrality, we round the value to the next
  integer. The reason is that a new primal bound coming obtained
  by the solution of the relaxation in a subproblem might have some
  garbage in it (e.g., |10.000000000034| or |9.999999999987|). 
  With this garbage the 
  fathoming of node might fail in the subproblem optimization, although
  it is already correct.

@<make sure that this is an integer value for |objInteger_|@>=
  if (objInteger_) {
    if (!isInteger(x, eps())) {
      err() << "ABA_MASTER::primalBound(): value ";
      err() << setPrecision(30) << x << " is not integer, ";
      err() << "but feasible solutions with integer objective ";
      err() << "function values are expected." << endl;
      exit(Fatal);
    }

    x = floor(x + eps());
  }
    
@ @<update the primal bound in the Tree Interface@>=
  if (optSense()->max()) treeInterfaceLowerBound(x);
  else                   treeInterfaceUpperBound(x);

@q m f dualBound @>
@q m e t @>
@ The function |dualBound()|.

@q m b r @>
  \BeginReturn
  The value of the dual bound,
  i.e., the |upperBound()| for a maximization problem and the
  |lowerBound()| for a minimization problem, respectively.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_MASTER::dualBound() const
@q m e f @>

@q m f dualBound @>
@ This version of the function |dualBound()| sets the dual bound to |x| and
  makes a new entry in the solution history. It is an error if the
  dual bound gets worse.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|x|}{The new value of the dual bound.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::dualBound(double x)
@q m e f @>
  {

    if (optSense()->max()) {
      if (x > dualBound_) {
        err() << "Error: ABA_MASTER::dualBound(): got worse" << endl;
        err() << "old bound: " << dualBound_ << " new bound: " << x << endl;
        exit (Fatal);
      }
    }
    else
      if (x < dualBound_) {
        err() << "Error: ABA_MASTER::dualBound(): got worse" << endl;
        err() << "old bound: " << dualBound_ << " new bound: " << x << endl;
        exit (Fatal);
      }
      
    dualBound_ = x;

    @<update the dual bound in the Tree Interface@>;

    history_->update();
  }

@ @<update the dual bound in the Tree Interface@>=
  if (optSense()->max()) treeInterfaceUpperBound(x);
  else                   treeInterfaceLowerBound(x);

@q m f betterDual @>
@q m e t @>
@ The function |betterDual()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |x| is better than the best known dual bound.
}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|x|}{The value being compared with the best know dual bound.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::betterDual(double x) const
@q m e f @>
  {
    if (optSense_.max()) {
#ifdef ABACUS_NO_BOOL
      if (x < dualBound_) return true;
      else                return false;
#else
      return x < dualBound_;
#endif
    }
    else {
#ifdef ABACUS_NO_BOOL
      if (x > dualBound_) return true;
      else                return false;
#else
      return x > dualBound_;
#endif
    }
  }

@q m f primalViolated @>
@ The function |primalViolated()| can be used to compare a value with
  the one of the best known primal bound.
@q m e t @>

  If the objective function values of all feasible solutions are
  integer, then we do not have to be so carefully.
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |x| is not better than the best known primal bound,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|x|}{The value being compared with the primal bound.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::primalViolated(double x) const
@q m e f @>
  {
    if (optSense_.max()) {
      if (objInteger_) {
#ifdef ABACUS_NO_BOOL 
        if (x <= primalBound_) return true;
        else                   return false;
#else
        return x <= primalBound_;
#endif
      }
      else {
#ifdef ABACUS_NO_BOOL
        if (x + eps() <= primalBound_) return true;
        else                           return false;
#else
        return x + eps() <= primalBound_;
#endif
      }
    }
    else {
      if (objInteger_) {
#ifdef ABACUS_NO_BOOL
        if (x >= primalBound_) return true;
        else                   return false;
#else
        return x >= primalBound_;
#endif
      }
      else {
#ifdef ABACUS_NO_BOOL
        if (x - eps() >= primalBound_) return true;
        else                           return false;
#else
        return x - eps() >= primalBound_;
#endif
      }
    }
  }

@q m f betterPrimal @>
@ The function |betterPrimal()| can be used to check if a value is
  better than the best know primal bound.
@q m e t @>
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |x| is better than the best known primal bound,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|x|}{The value compared with the primal bound.}
  \EndArg
@q m e a @>
@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::betterPrimal(double x) const
@q m e f @>
  {
    if (optSense_.max()) {
#ifdef ABACUS_NO_BOOL
      if (x > primalBound_) return true;
      else                  return false;
#else
      return x > primalBound_;
#endif
    }
    else {
#ifdef ABACUS_NO_BOOL
      if (x < primalBound_) return true;
      else                  return false;
#else
      return x < primalBound_;
#endif
    }
  }

@q m f feasibleFound @>
@q m e t @>
@ The function |feasibleFound()|.

  We use this function ,e.g., to adapt the enumeration strategy
  in the |DiveAndBest|-Strategy.

  This function is only correct
  if any primal bound better than plus/minus infinity corresponds
  to a feasible solution.
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If a feasible solution of the optimization problem has
                  been found.}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::feasibleFound() const
@q m e f @>
  {
    if (optSense_.max()) {
#ifdef ABACUS_NO_BOOL
      if (primalBound_ > -infinity()) return true;
      else                            return false;
#else
      return primalBound_ > -infinity();
#endif
    }
    else {
#ifdef ABACUS_NO_BOOL
      if (primalBound_ <  infinity()) return true;
      else                            return false;
#else
      return primalBound_ <  infinity();
#endif
    }
  }

@q m f root @>
@ The function |root()| can be used to access the root node of the \bab\ tree.
@q m e t @>
@q m b r @>
  \BeginReturn
   A pointer to the root node of the enumeration tree.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_SUB* ABA_MASTER::root()
@q m e f @>

@q m f rRoot @>
@q m e t @>
@ The function |rRoot()|.

@q m b r @>
  \BeginReturn
  A pointer to the root of the remaining \bab\ tree,
  i.e., the subproblem which is an ancestor of all open
  subproblems and has highest level in the tree.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_SUB* ABA_MASTER::rRoot()
@q m e f @>

@ The function |rRoot()| sets the root of the remaining \bac\ tree
  to |newRoot|. If |reoptimize| is |true| a reoptimization of the
  subproblem |*newRoot| is performed.
  This is controlled via a function argument since it might not be
  desirable when we find a new |rRoot_| during the fathoming
  of a complete subtree |ABA_SUB::FathomTheSubtree()|.

@(master.cc@>=
  void ABA_MASTER::rRoot(ABA_SUB *newRoot, bool /* reoptimize */)
  {
#ifdef ABACUS_PARALLEL
    out() << "WARNING: ABA_MASTER::rRoot() not supported in parallel version (ignored)." << endl;
#else
    if (rRoot_ == newRoot) return;
    
    rRoot_ = newRoot;

    out(1) << "subproblem " << newRoot->id() << " is now root of remaining tree" << endl;

    if ((newRoot->status() == ABA_SUB::Processed ||
         newRoot->status() == ABA_SUB::Dormant     ) && newRootReOptimize_)
      newRoot->reoptimize();

    ++nNewRoot_;
#endif
  }

@q m f guaranteed @>
@ The function |guaranteed()| can  be used to check
  if the guarantee requirements are fulfilled, i.e., the difference
  between upper bound and the lower bound in respect to the lowerBound
  is less than 
  this guarantee value in percent.
  If the lower bound is zero, but the upper bound is nonzero,
  we cannot give any guarantee.

  {\bf Warning:} A guarantee for a solution can only be given if the
  pricing problem is solved exactly or no column generation is performed
  at all.
@q m e t @>
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the guarantee requirements are fulfilled,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::guaranteed()
@q m e f @>
  {
    if (fabs(lowerBound()) < machineEps() &&
        fabs(upperBound()) > machineEps())
      return false;
    
    if (guarantee() + machineEps() < requiredGuarantee_) 
      return true;
    else
      return false;
  }

@q m f guarantee @>
@ The function |guarantee()| can be used to access the guarantee which can
  be given for the best known feasible solution. It is an error
  to call this function if the lower bound is zero, but the upper bound
  is nonzero.
@q m e t @>
@q m b r @>
  \BeginReturn
  The guarantee for best known feasible solution in percent.
  \EndReturn
@q m e r @>

@(master.cc@>=
@q m b f @>
  double ABA_MASTER::guarantee()
@q m e f @>
  {
    double lb = lowerBound();

    if (fabs(lb) < machineEps()) {
      if (fabs(upperBound()) < machineEps())
        return 0.0;
      else {    
        err() << "ABA_MASTER::guarantee(): cannot compute guarantee with lower bound 0" << endl;
        exit (Fatal);
      }
    }
    return fabs((upperBound() - lb)/lb * 100.0); 
  }
  
@q m f printGuarantee @>
@ The function |printGuarantee()| writes the guarantee nicely
  formated on the output stream associated with this object.
  If no bounds are available, or the lower bound is zero, but the
  upper bound is nonzero, then we cannot give any guarantee.
@q m e t @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::printGuarantee()
@q m e f @>
  {
    double lb = lowerBound();
    double ub = upperBound();

    if (lb == -infinity() || ub == infinity() ||
        ((fabs(lb) < machineEps()) && (fabs(ub) > machineEps())))
      out() << "---";
    else
      out() << guarantee() << '%';
  }

@q m f check @>
@ The function |check()| can be used to control the correctness of the
  optimization if the value of the optimum solution has been loaded.
  This is done, if a file storing the optimum value is specified with
  the parameter {\tt OptimumFileName} in the configuration file
  {\tt .abacus}.
@q m e t @>
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the optimum solution of the problem is known and
                  equals the primal bound,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::check()
@q m e f @>
  {
    double optVal;

    if (!knownOptimum(optVal)) return false;

    if (optVal - eps() < primalBound() && primalBound() < optVal + eps())
      return true;
    else
      return false;
  }

@q m f knownOptimum @>
@ The function |knownOptimum()| opens the file 
  specified with the parameter {\tt OptimumFileName} in the configuration
  file {\tt .abacus} and tries to find a line
  with the name of the problem instance (as specified in the
  constructor of |ABA_MASTER|) as first string.
@q m e t @>
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If a line with |problemName_| has been found,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|optVal|}{If the return value is |true|, then |optVal| holds the 
                    optimum value found in
                    the line with the name of the problem instance
                    as first string. Otherwise, |optVal| is undefined.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::knownOptimum(double &optVal)
@q m e f @>
  {
    @<open the file storing the optimum solutions@>;
    @<go through the lines of the file until the problem is found@>;
  }

@ @<open the file storing the optimum solutions@>=
  ifstream optimumFile(optimumFileName_.string(), ios::in );

  if (!optimumFile) return false;

@ As the problem name might be preceded by a path we check if the
  |name| read from the file, matches the last letters of the problem name.
  
@<go through the lines of the file until the problem is found@>=
  char name[256];
  double value;

  while (!optimumFile.eof()) {
    optimumFile >> name >> value;

    if (problemName_.ending(name)) {
      optVal = value;
      return true;
    }
    optimumFile >> ws;
  }
  
  return false;

@q m v output @>
@ The function |output()| does nothing but can be redefined
  in derived classes for output before the timing statistics.
@q m e t @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::output()
@q m e f @>
  { }

@ Just for fun.

@(master.cc@>=
  void ABA_MASTER::theFuture()
  {
    out() << "making love" << endl;
    out() << "riding my bike" << endl;
    out() << "climbing a mountain" << endl;
    out() << "using my skies" << endl;
    out() << "drinking a beer" << endl << endl;
    out() << "  ... and forgetting about ABACUS" << endl;
  }

@ If you still have not stopped studying this class, then its time now.
  The rest are only member functions return values of or pointers to
  data member, and deal with the management of the various parameters.

@ The function |writeTreeInterface()| writes the string |info| to the
  stream associated with the Tree Interface. A \$ is preceded if
  the output is written to standard out for further pipelining.
  If |time| is true a time string is written in front of the
  information. The default value of |time| is |true|.

@(master.cc@>=
  void ABA_MASTER::writeTreeInterface(const char *info, bool time) const
  {
    if (VbcLog_ == NoVbc) return;

#ifdef ABACUS_PARALLEL
    if (!parmaster_->isHostZero()) {
      int len = strlen(info);
      ABA_NOTIFICATION msg(this, 0);
      msg.pack(ABA_NOTIFYSERVER::TreeInterfaceTag);
      msg.pack(len);
      msg.pack(info, len);
      msg.send();
      return;
    }
    treeInterfaceMutex_.acquire();
#endif
    if (VbcLog_ == Pipe) *treeStream_ << '$';
    if (VbcLog_ == File && time) *treeStream_ << totalTime_ << ' ';
    *treeStream_ << info << endl;
#ifdef ABACUS_PARALLEL
    treeInterfaceMutex_.release();
#endif
  }

@ The function |treeInterfaceNewNode()| adds the subproblem |sub| to the
  stream storing information for graphical output of the enumeration
  tree if this logging is turned on.

@(master.cc@>=
  void ABA_MASTER::treeInterfaceNewNode(ABA_SUB *sub) const
  {
    if (VbcLog_ != NoVbc) {
      char string[256];
      int fatherId;

#ifdef ABACUS_PARALLEL
      fatherId = sub->fatherId();
#else
      if (sub == root_) fatherId = 0;
      else              fatherId = sub->father()->id();
#endif
      sprintf(string, "N %d %d 5", fatherId, sub->id());
      writeTreeInterface(string);
    }
  }

#ifdef ABACUS_PARALLEL
  void ABA_MASTER::treeInterfaceNewNode(int fatherId, int id) const
  {
    if (VbcLog_ != NoVbc) {
      char string[256];
      sprintf(string, "N %d %d 5", fatherId, id);
      writeTreeInterface(string);
    }
  }
#endif

@ The function |treeInterfacePaintNode()| assigns the |color| to the
  subproblem |sub| in the Tree Interface.

@(master.cc@>=
  void ABA_MASTER::treeInterfacePaintNode(int id, int color) const
  {
    if (VbcLog_ == NoVbc) return;
    
    char string[256];
    sprintf(string, "P %d %d", id, color);
    writeTreeInterface(string);
  }

@ The function |treeInterfaceLowerBound()| passes the new lower bound |lb|
  to the Tree Interface.

@(master.cc@>=
  void ABA_MASTER::treeInterfaceLowerBound(double lb) const
  {
    if (VbcLog_ == NoVbc) return;
    
    char string[256];
    sprintf(string, "L %f", lb);
    writeTreeInterface(string);
  }    

@ The function |treeInterfaceUpperBound()| passes the new upper bound |ub|
  to the Tree Interface.

@(master.cc@>=
  void ABA_MASTER::treeInterfaceUpperBound(double ub) const
  {
    if (VbcLog_ == NoVbc) return;
    
    char string[256];
    sprintf(string, "U %f", ub);
    writeTreeInterface(string);
  }

@ The function |treeInterfaceNodeBounds()| updates the node information
  in the node with number |id| by writing the lower bound |lb|
  and the upper bound |ub| to the node.

@(master.cc@>=
  void ABA_MASTER::treeInterfaceNodeBounds(int id, double lb, double ub)
  {
    if (VbcLog_ == NoVbc) return;

    char string[256];

    if (isInfinity(fabs(lb))) {
      if (isInfinity(fabs(ub)))
        sprintf(string, "I %d \\iLower Bound: ---\\nUpper Bound:  ---\\i",
                         id);
      else
        sprintf(string, "I %d \\iLower Bound: ---\\nUpper Bound:  %3.2f\\i",
                         id, ub);
    }
    else {
      if (isInfinity(fabs(ub)))
        sprintf(string, "I %d \\iLower Bound: %3.2f\\nUpper Bound:  ---\\i",
                         id, lb);
      else
        sprintf(string, "I %d \\iLower Bound: %3.2f\\nUpper Bound:  %3.2f\\i",
                         id, lb, ub);
    }

    writeTreeInterface(string);
  }


@q m f problemName @>
@q m e t @>
@ The function |problemName()|.

@q m b r @>
  \BeginReturn
  A pointer to the name of the instance being optimized (as specified in
  the constructor of this class).
  \EndReturn
@q m e r @>

@(master.cc@>=
@q m b f @>
  const ABA_STRING *ABA_MASTER::problemName() const
@q m e f @>
  {
    return &problemName_;
  }

@q m f optSense @>
@q m e t @>
@ The function |optSense()|.

@q m b r @>
  \BeginReturn
  A pointer to the object holding the optimization sense of the problem.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_OPTSENSE *ABA_MASTER::optSense()
@q m e f @>

@q m f history @>
@q m e t @>
@ The function |history()|.

@q m b r @>
  \BeginReturn
  A pointer to the object storing the solution history of this branch and
  cut problem.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_HISTORY *ABA_MASTER::history()
@q m e f @>

@q m f openSub @>
@q m e t @>
@ The function |openSub()|.

@q m b r @>
  \BeginReturn
  A pointer to the set of open subproblems.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_OPENSUB *ABA_MASTER::openSub()
@q m e f @>

@ The function |fixCand()| returns a pointer to the object storing
  the variables which are candidates for being fixed.

  ABA_FIXCAND *ABA_MASTER::fixCand()

@q m f conPool @>
@q m e t @>
@ The function |conPool()|.

@q m b r @>
  \BeginReturn
  A pointer to the default pool storing the constraints of the problem
  formulation.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_STANDARDPOOL<ABA_CONSTRAINT, ABA_VARIABLE> *ABA_MASTER::conPool()
@q m e f @>

@q m f cutPool @>
@q m e t @>
@ The function |cutPool()|.

@q m b r @>
  \BeginReturn
   A pointer to the default pool for the generated cutting planes.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_STANDARDPOOL<ABA_CONSTRAINT, ABA_VARIABLE> *ABA_MASTER::cutPool()
@q m e f @>

@q m f varPool @>@
@q m e t @>
@ The function |varPool()|.

@q m b r @>
  \BeginReturn
  A pointer to the default pool storing the variables.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_STANDARDPOOL<ABA_VARIABLE, ABA_CONSTRAINT> *ABA_MASTER::varPool()
@q m e f @>

@q m f cutting @>
@q m e t @>
@ The function |cutting()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |cutting| has been set to |true| in the call of the
                  constructor of the class |ABA_MASTER|, i.e., if cutting
                  planes should be generated in the subproblem optimization.}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_MASTER::cutting() const
@q m e f @>

@q m f pricing @>
@q m e t @>
@ The function |pricing()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |pricing| has been set to true in the call of the
                  constructor of the class |ABA_MASTER|, i.e., if a columns
                  should be generated in the subproblem optimization.}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_MASTER::pricing() const
@q m e f @>

@q m f totalCowTime @>
@q m e t @>
@ The function |totalCowTime()|.

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring the total wall clock time.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_COWTIMER *ABA_MASTER::totalCowTime()
@q m e f @>
  
@q m f totalTime @>
@q m e t @>
@ The function |totalTime()|.

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring the total cpu time for the
  optimization.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CPUTIMER *ABA_MASTER::totalTime()
@q m e f @>

@q m f lpTime @>
@q m e t @>
@ The function |lpTime()|. 

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring the cpu time spent in members of
  the LP-interface.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CPUTIMER *ABA_MASTER::lpTime()
@q m e f @>

@q m f lpSolverTime @>
@q m e t @>
@ The function |lpSolverTime()|.

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring  the cpu time required by the LP solver.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CPUTIMER *ABA_MASTER::lpSolverTime()
@q m e f @>

@q m f separationTime @>
@q m e t @>
@ The function |separationTime()|.

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring the cpu time spent in the
  separation of cutting planes.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CPUTIMER *ABA_MASTER::separationTime()
@q m e f @>

@q m f improveTime @>
@q m e t @>
@ The function |improveTime()|.

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring
  the cpu time spent in the heuristics for the computation of feasible
  solutions.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CPUTIMER *ABA_MASTER::improveTime()
@q m e f @>

@q m f pricingTime @>
@q m e t @>
@ The function |pricingTime()|. 

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring the cpu time spent in pricing.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CPUTIMER *ABA_MASTER::pricingTime()
@q m e f @>

@q m f branchingTime @>
@q m e t @>
@ The function |branchingTime()|. 

@q m b r @>
  \BeginReturn
  A pointer to the timer measuring the cpu time spent in finding and
  selecting the branching rules.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CPUTIMER *ABA_MASTER::branchingTime()
@q m e f @>

@ The function |newSub()| registers a new subproblem which is on
  level |level| in enumeration tree. It is called each time a new
  subproblem is generated.

@(master.cc@>=
 void ABA_MASTER::newSub(int level)
 {
   ++nSub_;
   if(level > highestLevel_) highestLevel_ = level;
 }

@ The function |countLp()| increments the counter for linear programs
  and should be called in each optimization call of the LP-relaxation.

  void ABA_MASTER::countLp()

@ The function |newFixed()| increments the counter of the number
  of fixed variables by |n|.

  void ABA_MASTER::newFixed(int n)

@ The function |addCons()| increments the counter for the total number
  of added constraints by |n|.

  void ABA_MASTER::addCons(int n)

@ The function |removeCons()| increments the counter for the total number
  of removed constraints by |n|.

  void ABA_MASTER::removeCons(int n)

@ The function |addVars()| increments the counter for the total number
  of added variables by |n|.

  void ABA_MASTER::addVars(int n)

@ The function |removeVars()| increments the counter for the total number
  of removed variables by |n|.

  void ABA_MASTER::removeVars(int n)

@q m f nSub @>
@q m e t @>
@ The function |nSub()|.

@q m b r @>
 \BeginReturn
 The number of generated subproblems.
 \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::nSub() const
@q m e f @>

@q m f nLp @>
@q m e t @>
@ The function |nLp()|.

@q m b r @>
  \BeginReturn
  The number of optimized linear programs (only LP-relaxations).
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::nLp() const
@q m e f @>

@q m f highestLevel @>
@q m e t @>
@ The function |highestLevel()|.

@q m b r @>
  \BeginReturn
  The highest level in the tree which has been reached during the implicit
  enumeration.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::highestLevel () const
@q m e f @>

@q m f nNewRoot @>
@q m e t @>
@ The function |nNewRoot()|.

@q m b r @>
  \BeginReturn
  The number of root changes of the remaining \bac\ tree.
  \EndReturn
@q m e r @>

@q m b f @>  
  int ABA_MASTER::nNewRoot() const
@q m e f @>

@q m f nSubSelected @>
@q m e t @>
@ The function |nSubSelected()|.

@q m b r @>
  \BeginReturn
  The number of subproblems which have already
  been selected from the set of open subproblems.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::nSubSelected() const
@q m e f @>
  
@ The rest of this class only deals with reading of parameter files,
  printing the complete list of parameters, and
  returning and setting the values of the parameters. All in all,
  not very exciting!

@ The function |_initializeParameters()| reads the parameter-file
  {\tt .abacus}, which is searched in the directory given by the
  environment variable |ABACUS_DIR|, and calls the  virtual function |initializeParameters()|
  which can initialize parameters of derived classes and overwrite
  parameters of this class. All parameters are first inserted together
  with their values in a parameter table in the function |readParameters()|.
  If the virtual dummy function |initializeParameters()| is redefined
  in a derived class and also reads a parameter file with the function
  |readParameters()|, then already inserted parameters can be overwritten.

  After all parameters are input we extract with the function 
  |assignParameter()|
  all parameters. Problem specific parameters should be extracted in
  a redefined version of |initializeParameters()|.
  extracted from this table 

@(master.cc@>=
  void ABA_MASTER::_initializeParameters()
  {
    @<|ABA_MASTER::_initializeParameters()|: local variables@>;

    if(readParamFromFile_) {
      @<set up the name of the configuration file@>;  
      readParameters(configFileName);
      @<close configuration file@>;
    }else {
      @<set default values for abacus parameters@>;
    };

    initializeParameters();

#ifdef ABACUS_PARALLEL
    parmaster_->initializeParameters();
#endif
    @<get the enumeration strategy@>;
    @<get the branching strategy@>;
    @<get the number of tested candidates for branching variables@>;
    @<get the solution guarantee@>;
    @<get the maximal level in the enumeration tree@>;
    @<get the maximal cpu time@>;
    @<get the maximal wall clock time@>;
    @<get the integrality condition on the objective function@>;
    @<get the number of linear programs for the tailing off analysis@>;
    @<get the minimal improvement for the tailing off analysis@>;
    @<get the threshold for delayed branching@>;
    @<get the minimal number of rounds a subproblem should stay dormant@>;
    @<get the output and log level@>;
    @<get the primal bound initialization mode@>;
    @<get the pricing frequency@>;
    @<get the skip factor@>;
    @<get the skipping mode@>;
    @<is fixing/setting by reduced costs turned on?@>;
    @<should the LP be output every iteration?@>;
    @<get the maximal number of added constraints@>;
    @<get the size of the buffer for constraints@>;
    @<get the maximal number of added variables@>;
    @<get the size of the buffer for variables@>;
    @<get the maximal number of iterations in the cutting plane phase@>;
    @<should fixed or set variables be eliminated from the LP?@>;
    @<should a new root be reoptimized?@>;
    @<get the name of the file storing the optimal solutions@>;
    @<should the average distance of the cuts per iteration be output?@>;
    @<get the constraint elimination mode@>;
    @<get the variable elimination mode@>;
    @<get the tolerance for variable elimination@>;
    @<get the tolerance for constraint elimination@>;
    @<get the age for constraint elimination@>;
    @<get the age for constraint elimination@>;
    @<should a log-file of the enumeration tree be generated?@>;
    @<get the default LP-solver@>;
    _initializeLpParameters();
  }

@ The function |_initializeLpParameters()| initializes the LP solver
  specific Parameters. It is called after the parameter-file
  {\tt .abacus} was read and all general parameters were processed.

  This function is implemented in the file |lpif.cc|.

@ @<|ABA_MASTER::_initializeParameters()|: local variables@>=
  ABA_STRING stringVal(this);  // auxiliary variable for reading strings

@ For UNIX systems the location of the configuration file {\tt
  .abacus} is given by the environment variable |ABACUS_DIR|. For the
  Visual \CPLUSPLUS/ compiler the configuration file is expected to be
  be located in the directory the program is called from.

@<set up the name of the configuration file@>=
  char *abacusDir = getenv("ABACUS_DIR");
  if (!abacusDir) {
    err() << "environment variable ABACUS_DIR not found" << endl;
    exit(Fatal);
  }

  char *configFileName = new char[strlen(abacusDir) + strlen("/.abacus") + 1];

#ifdef ABACUS_COMPILER_VISUAL_CPP
  sprintf(configFileName, "%s\\.abacus", abacusDir);
#else
  sprintf(configFileName, "%s/.abacus", abacusDir);
#endif

@ @<close configuration file@>=
  delete [] configFileName;


@ @<set default values for abacus parameters@>=
  insertParameter("EnumerationStrategy","BestFirst");
  insertParameter("BranchingStrategy","CloseHalfExpensive");
  insertParameter("NBranchingVariableCandidates","1");
  insertParameter("Guarantee","0.0");
  insertParameter("MaxLevel","999999");
  insertParameter("MaxCpuTime","99999:59:59");
  insertParameter("MaxCowTime","99999:59:59");
  insertParameter("ObjInteger","false");
  insertParameter("TailOffNLps","0");
  insertParameter("TailOffPercent","0.0001");
  insertParameter("DelayedBranchingThreshold","0");
  insertParameter("MinDormantRounds","1");
  insertParameter("OutputLevel","Full");
  insertParameter("LogLevel","Silent");
  insertParameter("PrimalBoundInitMode","None");
  insertParameter("PricingFrequency","0");
  insertParameter("SkipFactor","1");
  insertParameter("SkippingMode","SkipByNode");
  insertParameter("FixSetByRedCost","true");
  insertParameter("PrintLP","false");
  insertParameter("MaxConAdd","100");
  insertParameter("MaxConBuffered","100");
  insertParameter("MaxVarAdd","500");
  insertParameter("MaxVarBuffered","500");
  insertParameter("MaxIterations","-1");
  insertParameter("EliminateFixedSet","false");
  insertParameter("NewRootReOptimize","false");
  insertParameter("ShowAverageCutDistance","false");
  insertParameter("ConstraintEliminationMode","Basic");
  insertParameter("ConElimEps","0.001");
  insertParameter("ConElimAge","1");
  insertParameter("VariableEliminationMode","ReducedCost");
  insertParameter("VarElimEps","0.001");
  insertParameter("VarElimAge","1");
  insertParameter("VbcLog","None");
  insertParameter("DefaultLpSolver","Cplex");


  _setDefaultLpParameters();
#ifdef ABACUS_PARALLEL
  parmaster_->setDefaultParameters();
#endif

@ The function |_setDefaultLpParameters()| initializes the LP solver
  specific default Parameters if they are not read from the parameter-file
  {\tt .abacus}.

  This function is implemented in the file |lpif.cc|.


@ @<get the enumeration strategy@>=

  enumerationStrategy_= (ENUMSTRAT)
  findParameter("EnumerationStrategy",4,ENUMSTRAT_);

@ @<get the branching strategy@>=
  
  branchingStrategy_=(BRANCHINGSTRAT)
  findParameter("BranchingStrategy",2,BRANCHINGSTRAT_);

@ @<get the number of tested candidates for branching variables@>=
  assignParameter(nBranchingVariableCandidates_,
                  "NBranchingVariableCandidates",0, INT_MAX);

@ @<get the solution guarantee@>=
  assignParameter(requiredGuarantee_,"Guarantee",0.0,infinity());

@ @<get the maximal level in the enumeration tree@>=
  assignParameter(maxLevel_,"MaxLevel",1,INT_MAX);

@ @<get the maximal cpu time@>=
  assignParameter(stringVal,"MaxCpuTime",0);
  maxCpuTime(stringVal);

@ @<get the maximal wall clock time@>=
  assignParameter(stringVal,"MaxCowTime",0);
  maxCowTime(stringVal);

@ @<get the integrality condition on the objective function@>=
  assignParameter(objInteger_,"ObjInteger" );

@ @<get the number of linear programs for the tailing off analysis@>=
  assignParameter(tailOffNLp_,"TailOffNLps",INT_MIN,INT_MAX );

@ @<get the minimal improvement for the tailing off analysis@>=
  assignParameter(tailOffPercent_, "TailOffPercent", 0.0, infinity());

@ @<get the threshold for delayed branching@>=
  assignParameter(dbThreshold_, "DelayedBranchingThreshold",0,INT_MAX);

@ @<get the minimal number of rounds a subproblem should stay dormant@>=
  assignParameter(minDormantRounds_, "MinDormantRounds",1,INT_MAX );

@ @<get the output and log level@>=
  
  if (outLevel_ != Silent) {
    outLevel_=(OUTLEVEL)
      findParameter("OutputLevel",5,OUTLEVEL_);
  }
  logLevel_=(OUTLEVEL)findParameter("LogLevel",5,OUTLEVEL_);
  @<initialize the log-files@>;

@ In order to avoid memory leaks we check if the log-files have not been
  initialized somewhere else.
  
@<initialize the log-files@>=

  if (logLevel_ != Silent) {
    char *problemName = problemName_.string();
    char *logFileName = new char[strlen(problemName)+20];

    if (!out().isLogOn()) {
#ifdef ABACUS_PARALLEL
      sprintf(logFileName, "%s.log.%d", problemName, parmaster_->hostId());
#else
      strcpy(logFileName, problemName);
      strcat(logFileName, ".log");
#endif
      out().logOn(logFileName);
    }

    if (!err().isLogOn()) {
#ifdef ABACUS_PARALLEL
      sprintf(logFileName, "%s.error.log.%d", problemName, parmaster_->hostId());
#else
      strcpy(logFileName, problemName);
      strcat(logFileName, ".error.log");
#endif

      err().logOn(logFileName);
    }
    
    delete [] logFileName;
  }
       
@ @<get the primal bound initialization mode@>=

  pbMode_=(PRIMALBOUNDMODE)
  findParameter("PrimalBoundInitMode",3,PRIMALBOUNDMODE_);

@ @<get the pricing frequency@>=
  assignParameter(pricingFreq_,"PricingFrequency",0,INT_MAX);

@ @<get the skip factor@>=
  assignParameter(skipFactor_,"SkipFactor",0,INT_MAX);

@ @<get the skipping mode@>=

  skippingMode_=(SKIPPINGMODE)
  findParameter("SkippingMode",2,SKIPPINGMODE_ );

@ @<is fixing/setting by reduced costs turned on?@>=
  assignParameter(fixSetByRedCost_,"FixSetByRedCost");

@ @<should the LP be output every iteration?@>=
  assignParameter(printLP_, "PrintLP");
  
@ @<get the maximal number of added constraints@>=
  assignParameter(maxConAdd_,"MaxConAdd",0,INT_MAX );
  
@ @<get the size of the buffer for constraints@>=
  assignParameter(maxConBuffered_,"MaxConBuffered",0,INT_MAX);

@ @<get the maximal number of added variables@>=
  assignParameter(maxVarAdd_,"MaxVarAdd",0,INT_MAX);
  
@ @<get the size of the buffer for variables@>=
  assignParameter(maxVarBuffered_,"MaxVarBuffered",0,INT_MAX);

@ @<get the maximal number of iterations in the cutting plane phase@>=
  assignParameter(maxIterations_,"MaxIterations",-1,INT_MAX);

@ @<should fixed or set variables be eliminated from the LP?@>=
  assignParameter(eliminateFixedSet_,"EliminateFixedSet");

@ @<should a new root be reoptimized?@>=
  assignParameter(newRootReOptimize_,"NewRootReOptimize");

@ The name of the file storing the optimal solutions is only an optional
  parameter which can be missing.
  
@<get the name of the file storing the optimal solutions@>=
  getParameter("OptimumFileName",optimumFileName_);

@ @<should the average distance of the cuts per iteration be output?@>=
  assignParameter(showAverageCutDistance_,"ShowAverageCutDistance");

@ @<get the constraint elimination mode@>=
  conElimMode_=(CONELIMMODE)
  findParameter("ConstraintEliminationMode",3,CONELIMMODE_);

@ @<get the variable elimination mode@>=
  varElimMode_=(VARELIMMODE)findParameter("VariableEliminationMode",2,
                                          VARELIMMODE_);

@ @<get the tolerance for variable elimination@>=
  assignParameter(varElimEps_,"VarElimEps",0.0,infinity() );

@ @<get the tolerance for constraint elimination@>=
  assignParameter(conElimEps_,"ConElimEps",0.0,infinity() );

@ @<get the age for constraint elimination@>=
  assignParameter(conElimAge_,"ConElimAge",1,INT_MAX);

@ @<get the age for variable elimination@>=
  assignParameter(varElimAge_,"VarElimAge",1,INT_MAX);

@ @<should a log-file of the enumeration tree be generated?@>=
  VbcLog_=(VBCMODE)findParameter("VbcLog",3,VBCMODE_);


@ @<get the default LP-solver@>=
  defaultLpSolver_=(LPSOLVER)findParameter("DefaultLpSolver",
                                             3,LPSOLVER_);

@q m v initializeParameters @>
@ The virtual function |initializeParameters()| is only a dummy. This
  function can be used to initialize parameters of derived classes
  and to overwrite parameters read from the file {\tt .abacus} by the
  function |\_initializeParameters()|.
@q m e t @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::initializeParameters()
@q m e f @>
  { }


@q m f printParameters @>
@ The function |printParameters()| writes all parameters of the class
  |ABA_MASTER| together
  with their values to the global output stream.
@q m e t @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::printParameters()
@q m e f @>
  {
    out() << "Branch and Cut Parameters:" << endl << endl;

    out() << "  Enumeration strategy                   : ";
    out() << ENUMSTRAT_[enumerationStrategy_];
    out() << endl;

    out() << "  Branching Strategy                     : ";
    out() << BRANCHINGSTRAT_[branchingStrategy_];
    out() << endl;
    out() << "  Tested candidates for branching var.   : ";
       out() << nBranchingVariableCandidates_ << endl;
    out() << "  Guarantee                              : ";
       out() << requiredGuarantee_ << " %" << endl;
    out() << "  Maximal enumeration level              : ";
       out() << maxLevel_ << endl;
    out() << "  CPU time limit                         : ";
       out() << maxCpuTime_ << endl;
    out() << "  Wall-clock time limit                  : ";
       out() << maxCowTime_ << endl;
    out() << "  Objective function values integer      : ";
       out() << onOff(objInteger_) << endl;
    out() << "  Tailing Off Parameters" << endl;
    out() << "                    Number of LPs        : ";
      out() << tailOffNLp_ << endl;
    out() << "                    Minimal improvement  :    ";
      out() << tailOffPercent_ << '%' << endl;
    out() << "  Delayed branching threshold            : ";
      out() << dbThreshold_ << endl;
    out() << "  Maximal number of dormant rounds       : ";
      out() << minDormantRounds_ << endl;
      
    out() << "  Output Level                           : ";
    out() << OUTLEVEL_[outLevel_];
    out() << endl;
    
    out() << "  Log Level                              : ";
    out() << OUTLEVEL_[logLevel_];
    out() << endl;

    out() << "  Primal Bound Initialization            : ";
    out() << PRIMALBOUNDMODE_[pbMode_];
    out() << endl;
    out() << "  Frequency of additional pricing        : ";
    out() << pricingFreq_ << " LPs" << endl;
    out() << "  Cutting skip factor                    : ";
    out() << skipFactor_ << endl;
    out() << "  Skipping mode                          : ";
    if (skippingMode_ == SkipByNode) out() << "by node" << endl;
    else                        out() << "by tree" << endl;

    out() << "  Fix/set by reduced costs               : ";
    out() << onOff(fixSetByRedCost_) << endl;
    out() << "  Output of the linear program           : ";
    out() << onOff(printLP_) << endl;
    out() << "  Maximal number of added constraints    : ";
    out() << maxConAdd_ << endl;
    out() << "  Maximal number of buffered constraints : ";
    out() << maxConBuffered_ << endl;
    out() << "  Maximal number of added variables      : ";
    out() << maxVarAdd_ << endl;
    out() << "  Maximal number of buffered variables   : ";
    out() << maxVarBuffered_ << endl;
    out() << "  Maximal number of iterations per" << endl;
    out() << "                     cutting plane phase : ";
    out() << maxIterations_ << endl;
    out() << "  Elimination of fixed and set variables : ";
    out() << onOff(eliminateFixedSet_) << endl;
    out() << "  Reoptimization after a root change     : ";
    out() << onOff(newRootReOptimize_) << endl;
    out() << "  File storing optimum solutions         : ";
    out() << optimumFileName_ << endl;
    out() << "  Show average distance of added cuts    : ";
    out() << onOff(showAverageCutDistance_) << endl;
    out() << "  Elimination of constraints             : ";
    out() << CONELIMMODE_[conElimMode_] << endl;
    out() << "  Elimination of variables               : ";
    out() << VARELIMMODE_[varElimMode_] << endl;
    out() << "  Tolerance for constraint elimination   : ";
    out() << conElimEps_ << endl;
    out() << "  Tolerance for variable elimination     : ";
    out() << varElimEps_ << endl;
    out() << "  Age for constraint elimination         : ";
    out() << conElimAge_ << endl;
    out() << "  Age for variable elimination           : ";
    out() << varElimAge_ << endl;
    out() << "  Default LP-solver                      : ";
    out() << LPSOLVER_[defaultLpSolver_] << endl;
     _printLpParameters();
#ifdef ABACUS_PARALLEL
    parmaster_->printParameters();
#endif
  }


@ The function |_printLpParameters()| prints the LP solver
  specific parameters.

  This function is implemented in the file |lpif.cc|.



@q m f fixSetByRedCost @>
@q m e t @>
@ The function |fixSetByRedCost()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{Then variables are fixed and set by reduced cost criteria.}
  \docarg{|false|}{Then no variables are fixed or set by reduced cost criteria.}
  \EndReturn
@q m e r @>

@q m b f  @>
  bool ABA_MASTER::fixSetByRedCost() const
@q m e f @>

@q m f fixSetByRedCost @>
@ The function |fixSetByReCost()| turns fixing and setting variables by
  reduced cost on or off.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|on|}{If |true|, then variable fixing and setting by reduced
                cost is turned on. Otherwise it is turned of.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::fixSetByRedCost(bool on)
@q m e f @>

@q m f printLP @>
@q m e t @>
@ The function |printLP()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{Then the linear program is output every iteration of the
                  subproblem optimization.}
  \docarg{|false|}{The linear program is not output.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_MASTER::printLP() const
@q m e f @>

@q m f printLP @>
@ The function |printLP()| turns the output of the linear program in 
  every iteration on or off.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|on|}{If |true|, then the linear program is output,
                otherwise it is not output.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::printLP(bool on)
@q m e f @>

@q m f maxConAdd @>
@q m e t @>
@ The function |maxConAdd()|.

@q m b r @>
  \BeginReturn
  The maximal number of constraints
  which should be added in every iteration of the cutting plane algorithm.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::maxConAdd() const
@q m e f @>

@q m f maxConAdd @>
@ The function |maxConAdd()| sets the maximal number of constraints
  that are added in an iteration of the cutting plane algorithm.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|max|}{The maximal number of constraints.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::maxConAdd(int max)
@q m e f @>

@q m f maxConBuffered @>
@q m e t @>
@ The function |maxConBuffered()|.

@q m b r @>
  \BeginReturn
  The size of the buffer for
  generated constraints in the cutting plane algorithm.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::maxConBuffered() const
@q m e f @>

@q m f maxConBuffered @>
@ The function |maxConBuffered()| changes the maximal number of
  constraints that are buffered in an iteration of the cutting plane
  algorithm.

  Note, this function changes only the default value for subproblems
  that are activated after its call. 
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|max|}{The new maximal number of buffered constraints.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::maxConBuffered(int max) 
@q m e f @>

@q m f maxVarAdd @>
@q m e t @>
@ The function |maxVarAdd()|.

@q m b r @>
  \BeginReturn
  The maximal number of variables which
  should be added in the column generation algorithm.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::maxVarAdd() const
@q m e f @>

@q m f maxVarAdd @>
@ The function |maxVarAdd()| changes the maximal number of variables that
  are added in an iteration of the subproblem optimization.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|max|}{The new maximal number of added variables.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::maxVarAdd(int max)
@q m e f @>

@q m f maxVarBuffered @>
@q m e t @>
@ The function |maxVarBuffered()|.

@q m b r @>
  \BeginReturn
  The size of the buffer for the
  variables generated in the column generation algorithm.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::maxVarBuffered() const
@q m e f @>

@q m f maxVarBuffered @>
@ The function |maxVarBuffered()| changes the maximal number of
  variables that are buffered in an iteration of the subproblem optimization.

  Note, this function changes only the default value for subproblems
  that are activated after its call. 
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|max|}{The new maximal number of buffered variables.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::maxVarBuffered(int max) 
@q m e f @>

@q m f maxIterations @>
@q m e t @>
@ The function |maxIterations()|.

@q m b r @>
  \BeginReturn
  The maximal number of iterations per subproblem optimization
  (|-1| means no iteration limit).
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::maxIterations() const
@q m e f @>
  
@q m f maxIterations @>
@ The function |maxIterations()| changes the default value for the
  maximal number of iterations of the optimization of a subproblem.

  Note, this function changes only this value for subproblems that 
  are constructed after this function call. For already constructed
  objects the value can be changed with the function 
  |ABA_SUB::maxIterations()|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|max|}{The new maximal number of iterations of the subproblem
                 optimization (|-1| means no limit).}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::maxIterations(int max)
@q m e f @>

@q m f optimumFileName @>
@q m e t @>
@ The function |optimumFileName()|.

@q m b r @>
  \BeginReturn
   The name of the file that stores the optimum solutions.
  \EndReturn
@q m e r @>

@q m b f @>
  const ABA_STRING &ABA_MASTER::optimumFileName() const
@q m e f @>

@q m f optimumFileName @>
@ The function |optimumFileName()| changes the name of the file in which
  the value of the optimum solution is searched.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|name|}{The new name of the file.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::optimumFileName(const char *name)
@q m e f @>
  

@q m f status @>
@q m e t @>

@q m b r @>
  \BeginReturn
  The status of the |ABA_MASTER|.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::STATUS ABA_MASTER::status() const
@q m e f @>

@ This version of the function |status()| sets the status of the |ABA_MASTER|.

  void ABA_MASTER::status(STATUS stat)

@q m f eliminateFixedSet @>
@q m e t @>
@ If the function |eliminateFixedSet()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{Then we try to eliminate fixed and set variables from the
                  linear program.}
  \docarg{|false|}{Fixed or set variables are not eliminated.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_MASTER::eliminateFixedSet() const
@q m e f @>

@q m f eliminateFixedSet @>
@ This version of the function |eliminateFixedSet()| can be used
  to turn the elimination of fixed and set variables on or off.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|turnOn|}{The elimination is turned on if |turnOn| is |true|,
                    otherwise it is turned off.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::eliminateFixedSet(bool turnOn)
@q m e f @>

@q m f newRootReOptimize @>
@q m e t @>
@ The function |newRootReOptimize()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{Then a new root of the remaining \bab\ tree is reoptimized
                   such that the associated
                   reduced costs can be used for the fixing of variables.}
  \docarg{|false|}{A new root is not reoptimized.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_MASTER::newRootReOptimize() const
@q m e f @>

@q m f newRootReOptimize @>
@ The function |newRootReOptimize()| turns the reoptimization of new
  root nodes of the remaining branch and bound tree on or off.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|on|}{If |true|, new root nodes are reoptimized.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::newRootReOptimize(bool on)
@q m e f @>

@q m f showAverageCutDistance @>
@q m e t @>
@ The function |showAverageCutDistance()|.

@q m b a @>
  \BeginReturn
  \docarg{|true|}{Then the average distance of the fractional solution
                  from all added cutting planes is output every iteration
                  of the subproblem optimization.}
  \docarg{|false|}{The average cut distance is not output.}
  \EndReturn
@q m e a @>

@q m b f @>
  bool ABA_MASTER::showAverageCutDistance() const
@q m e f @>

@q m f showAverageCutDistance @>
@ The function |showAverageCutDistance()| turns the output of the average
  distance of the added cuts from the fractional solution on or off.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|on|}{If |true| the output is turned on, otherwise it is
                turned off.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::showAverageCutDistance(bool on)
@q m e f @>

@q m f vbcLog @>
@q m e t @>
@ The function |vbcLog()|.

@q m b r @>
  \BeginReturn
  The mode of output for the Vbc-Tool.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::VBCMODE ABA_MASTER::vbcLog() const
@q m e f @>

@q m f vbcLog @>
@ The function |vbcLog()| changes the mode of output for the Vbc-Tool.
  This function should only be called before the optimization is
  started with the function |ABA_MASTER::optimize()|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|mode|}{The new mode.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::vbcLog(VBCMODE mode)
@q m e f @>

@q m f conElimMode @>
@q m e t @>
@ The function |conElimMode()|.

@q m b r @>
  \BeginReturn
  The mode for the elimination of constraints.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::CONELIMMODE ABA_MASTER::conElimMode() const
@q m e f @>

@q m f conElimMode @>
@ The function |conElimMode()| changes the constraint elimination
  mode.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|mode|}{The new constraint elimination mode.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::conElimMode(CONELIMMODE mode)
@q m e f @>

@q m f varElimMode @>
@q m e t @>
@ The function |varElimMode()|.

@q m b r @>
  \BeginReturn
  The mode for the elimination of variables.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::VARELIMMODE ABA_MASTER::varElimMode() const
@q m e f @>

@q m f varElimMode @>
@ The function |varElimMode()| changes the variable elimination
  mode.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|mode|}{The new variable elimination mode.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::varElimMode(VARELIMMODE mode)
@q m e f @>

@q m f conElimEps @>
@q m e t @>
@ The function |conElimEps()|.

@q m b r @>
  \BeginReturn
  The zero tolerance for the elimination
  of constraints by the slack criterion.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_MASTER::conElimEps() const
@q m e f @>

@q m f conElimEps @>
@ The function |conElimEps()| chances the tolerance for the
  elimination of constraints by the slack criterion.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|eps|}{The new tolerance.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::conElimEps(double eps)
@q m e f @>


@q m f varElimEps @>
@q m e t @>
@ The function |varElimEps()|.

@q m b r @>
  \BeginReturn
  The zero tolerance for the elimination of variables by the reduced cost
  criterion.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_MASTER::varElimEps() const
@q m e f @>

@q m f varElimEps @>
@ The function |varElimEps()| changes the tolerance for the
  elimination of variables by the reduced cost criterion.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|eps|}{The new tolerance.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::varElimEps(double eps)
@q m e f @>

@q m f varElimAge @>
@q m e t @>
@ The function |varElimAge()|.

@q m b r @>
  \BeginReturn
  The age for the elimination of variables by the reduced cost
  criterion.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::varElimAge() const
@q m e f @>

@q m f varElimAge @>
@ The function |varElimAge()| changes the age for the
  elimination of variables by the reduced cost criterion.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|age|}{The new age.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::varElimAge(int age)
@q m e f @>

@q m f conElimAge @>
@q m e t @>
@ The function |conElimAge()|.

@q m b r @>
  \BeginReturn
  The age for the elimination of constraints.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::conElimAge() const
@q m e f @>

@q m f conElimAge @>
@ The function |conElimAge()| changes the age for the
  elimination of constraints.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|age|}{The new age.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::conElimAge(int age)
@q m e f @>

@q m f branchingStrategy @>
@q m e t @>
@ The function |branchingStrategy()|.

@q m b r @>
  \BeginReturn
  The branching strategy.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::BRANCHINGSTRAT ABA_MASTER::branchingStrategy() const
@q m e f @>

@q m f branchingStrategy @>
@ This version of the function |branchingStrategy()| changes the
  branching strategy.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|strat|}{The new branching strategy.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::branchingStrategy(BRANCHINGSTRAT strat)
@q m e f @>

@q m f defaultLpSolver @>
@q m e t @>
@ The function |defaultLpSolver()|.

@q m b r @>
  \BeginReturn
  The default LP-Solver.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::LPSOLVER ABA_MASTER::defaultLpSolver() const
@q m e f @>

@q m f defaultLpSolver @>
@ This version of the function |defaultLpSolver()| changes the
  default LP-solver.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|lpSolver|}{The new solver.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::defaultLpSolver(LPSOLVER lpSolver)
@q m e f @>

@q m f nBranchingVariableCandidates @>
@q m e t @>
@ The function |nBranchingVariableCandidates()|.

@q m b r @>
  \BeginReturn
  The number of variables that should be tested for the selection of the
  branching variable.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::nBranchingVariableCandidates() const
@q m e f @>

@q m f nBranchingVariableCandidates @>
@ This version of the function |nbranchingVariableCandidates()| sets
  the number of tested branching variable candidates.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|n|}{The new value of the number of tested variables for 
               becoming branching variable.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::nBranchingVariableCandidates(int n)
@q m e f @>
  {
    if (n < 1) {
      err() << "ABA_MASTER::nBranchingVariableCandidates() invalid argument" << endl;
      err() << "correct value: positive integer number" << endl;
      exit(Fatal);
    } 
    nBranchingVariableCandidates_ = n;
  }

@q m f requiredGuarantee @>
@q m e t @>
@ The function |requiredGuarantee()|

@q m b r @>
  The guarantee specification for the optimization.
@q m e r @>

@q m b f @>
  double ABA_MASTER::requiredGuarantee() const
@q m e f @>

@q m f requiredGuarantee @>
@ This version of the function |requiredGuarantee()| changes the guarantee
  specification.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|g|}{The new guarantee specification (in percent). 
               This must be a nonnative
               value. Note, if the guarantee specification is changed
               after a single node of the enumeration tree has been 
               fathomed, then the overall guarantee might differ from
               the new value.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::requiredGuarantee(double g)
@q m e f @>
  {
    if (g < 0.0) {
      err() << "ABA_MASTER::guarantee(" << g << "): ";
      err() << "choose nonnegative value." << endl;
      exit (Fatal);
    }

    requiredGuarantee_ = g;
  }

@q m f maxLevel @>
@q m e t @>
@ The function |maxLevel()|.

@q m b r @>
  \BeginReturn
  The maximal depth up to
  which the enumeration should be performed.
  By default the maximal enumeration depth is |INT\_MAX|. 
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::maxLevel() const
@q m e f @>

@q m f maxLevel @>
@ This version of the function |maxLevel()| changes the maximal enumeration
  depth. If it is set to 1
  the \bac\ algorithm becomes a pure cutting plane algorithm.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|max|}{The new value of the maximal enumeration level.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::maxLevel(int max)
@q m e f @>
  {
    if (max < 1) {
      err() << "ABA_MASTER::maxLevel(" << max << "): ";
      err() << "only positive integers are valid" << endl;
      exit(Fatal);
    }
    maxLevel_ = max;
  }

@q m f maxCpuTime @>
@q m e t @>
@ The function maxCpuTime().

@q m b r @>
  \BeginReturn
  The maximal cpu time which can be used by the optimization.
  \EndReturn
@q m e r @>

@q m b f @>
  const ABA_STRING& ABA_MASTER::maxCpuTime() const
@q m e f @>

@q m f maxCpuTime @>
@ The function |maxCpuTime()| sets the maximal usable cpu time for
  the optimization.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|t|}{The new value of the maximal cpu time in the form
   {\tt "hh:mm:ss"}.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::maxCpuTime(const ABA_STRING &t)
@q m e f @>

@q m f maxCowTime @>
@q m e t @>
@ The function |maxCowTime()|.

@q m b r @>
  \BeginReturn
  The maximal wall-clock time for the optimization.
  \EndReturn
@q m e r @>

@q m b f @>
  const ABA_STRING& ABA_MASTER::maxCowTime() const
@q m e f @>

@q m f maxCowTime @>
@ This version of the  function |maxCowTime()| set the maximal wall-clock
   time for the optimization.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|t|}{The new value of the maximal wall-clock time
               in the form {\tt hh:mm:ss}.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::maxCowTime(const ABA_STRING &t)
@q m e f @>

@q m f objInteger @>
@q m e t @>
@ If the function |objInteger()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{Then we assume that all feasible solutions have integral
                  objective function values,}
  \docarg{|false|}{otherwise.}
   \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_MASTER::objInteger() const
@q m e f @>

@q m f objInteger @>
@ This version of function |objInteger()| sets the assumption
  that the objective function values of all feasible solutions are
  integer.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|b|}{The new value of the assumption.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::objInteger(bool b)
@q m e f @>

@q m f tailOffNLp @>
@q m e t @>
@ The function |tailOffNLp()|.

@q m b r @>
  \BeginReturn
  The number of linear programs considered in the tailing off analysis.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::tailOffNLp() const
@q m e f @>

@q m f tailOffNLp  @>
@ The function |tailOffNLp()| sets the number of linear programs
  considered in the tailing off analysis. This new value is only
  relevant for subproblems activated {\bf after} the change of this value.
@q m e t @>

@q m b a @>=
  \BeginArg
  \docarg{|n|}{The new number of LPs for the tailing off analysis.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::tailOffNLp(int n)
@q m e f @>

@q m f tailOffPercent @>
@q m e t @>
@ The function |tailOffPercent()|.

@q m b r @>
  \BeginReturn
  The minimal change of the dual bound for the tailing off analysis in percent.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_MASTER::tailOffPercent() const
@q m e f @>

@q m f tailOffPercent @>
@ This version of the function |tailOffPercent()| sets the minimal change
  of the dual bound for the tailing off analysis. This change is only
  relevant for subproblems activated {\bf after} calling this function.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|p|}{The new value for the tailing off analysis.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::tailOffPercent(double p) 
@q m e f @>
  {
    if (p < 0.0) {
      err() << "ABA_MASTER::tailing_off(p): choose nonnegative value" << endl;
      exit(Fatal);
    }
    tailOffPercent_ = p;
  }

@q m f outLevel @>
@q m e t @>
@ The function |outLevel()|.

@q m b r @>
  \BeginReturn
  The output mode.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::OUTLEVEL ABA_MASTER::outLevel() const
@q m e f @>

@q m f outLevel @>
@ The version of the function |outLevel()| sets the output mode.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|mode|}{The new value of the output mode.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::outLevel(OUTLEVEL mode)
@q m e f @>

@q m f logLevel @>
@q m e t @>
@ The function |logLevel()|.

@q m b r @>
  \BeginReturn
  The output mode for the log-file.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::OUTLEVEL ABA_MASTER::logLevel() const
@q m e f @>

@q m f logLevel @>
@ This version of the function |logLevel()| sets the output mode for the
  log-file.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|mode|}{The new value of the output mode.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::logLevel(OUTLEVEL mode)
@q m e f @>

@q m f delayedBranching @>
@q m e t @>
@ The function |delayedBranching()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the number of optimizations |nOpt| of a subproblem
                  exceeds the delayed branching threshold,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|nOpt|}{The number of optimizations of a subproblem.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  bool ABA_MASTER::delayedBranching(int nOpt) const
@q m e f @>
  {
    if (nOpt >= dbThreshold_ + 1) return false;
    else                          return true;
  }

@q m f dbThreshold @>
@ The function |dbThreshold()| sets the number of
  optimizations of a subproblem until sons are created in
  |ABA_SUB::branching()|.
  If this value is 0, then a branching step is performed at the
  end of the subproblem optimization as usually if the subproblem
  can be fathomed. Otherwise, if this value is strictly positive,
  the subproblem is put back for a later optimization. This can be
  advantageous if in the meantime good cutting planes or primal bounds
  can be generated. The number of times the subproblem is put back
  without branching is indicated by this value.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|threshold|}{The new value of the delayed branching threshold.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::dbThreshold(int threshold)
@q m e f @>

@q m f dbThreshold @>
@q m e t @>
@ The function |dbThreshold()|.

@q m b r @>
  \BeginReturn
  The number of optimizations
  of a subproblem until sons are created. For further detatails we
  refer to |dbThreshold(int)|.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::dbThreshold() const
@q m e f @>

  
@q m f minDormantRounds @>
@q m e t @>
@ The function |minDormantRounds()| 

@q m b r @>
  \BeginReturn
  The maximal number of rounds, i.e., number of subproblem optimizations,
  a subproblem is dormant, i.e., it is not selected from the set
  of open subproblem if its status is |Dormant|, if possible.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::minDormantRounds() const
@q m e f @>

@q m f minDormantRounds @>
@ The function |minDormantRounds()| sets the number of rounds a subproblem
  should stay dormant.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|nRounds|}{The new minimal number of dormant rounds.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::minDormantRounds(int nRounds)
@q m e f @>

@q m f pbMode @>
@q m e t @>
@ The function |pbMode()|.

@q m b r @>
  \BeginReturn
  The mode of the primal bound initialization.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::PRIMALBOUNDMODE ABA_MASTER::pbMode() const
@q m e f @>

@q m f pbMode @>
@ This version of the function |pbMode()| sets the mode of the primal
  bound initialization.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|mode|}{The new mode of the primal bound initialization.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::pbMode(PRIMALBOUNDMODE mode)
@q m e f @>

@q m f pricingFreq @>
@q m e t @>
@ The function |pricingFreq()|.

@q m b a @>
  \BeginReturn
  The number of linear programs
  being solved between two additional pricing steps.
  If no additional pricing steps should be
  executed this parameter has to be set to 0.
  The default value of the pricing frequency is 0. This parameter
  does not influence the execution of pricing steps which are
  required for the correctness of the algorithm. 
  \EndReturn
@q m e a @>

@q m b f @>
  int ABA_MASTER::pricingFreq() const
@q m e f @>

@q m f pricingFreq @>
@ This version of the function |pricingFreq()| sets the number of linear
  programs
  being solved between two additional pricing steps.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|f|}{The pricing frequency.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::pricingFreq(int f)
@q m e f @>
  {
    if (f < 0) {
      err() << "ABA_MASTER::pricingFreq(): nonnegative frequency expected" << endl;
      exit(Fatal);
    }
    pricingFreq_ = f;
  }

@q m f skipFactor @>
@q m e t @>
@ The functions |skipFactor()|.

@q m b r @>
  \BeginReturn
  The frequency of subproblems in which constraints or variables should be
  generated.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_MASTER::skipFactor() const
@q m e f @>

@q m f skipFactor @>
@ This version of the function |skipFactor()| sets the frequency for
  constraint and variable generation.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|f|}{The new value of the frequency.}
  \EndArg
@q m e a @>

@(master.cc@>=
@q m b f @>
  void ABA_MASTER::skipFactor(int f)
@q m e f @>
  {
    if (f < 0) {
      err() << "ABA_MASTER::skipFactor(): nonnegative value expected" << endl;
      exit(Fatal);
    }
    skipFactor_ = f;
  }

@q m f skippingMode @>
@q m e t @>
@ If the function |skippingMode()|.

@q m b r @>
  \BeginReturn
  The skipping strategy.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER::SKIPPINGMODE ABA_MASTER::skippingMode() const
@q m e f @>

@q m f skippingMode @>
@ This version of the function |skippingMode()| sets the skipping strategy.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|mode|}{The new skipping strategy.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_MASTER::skippingMode(SKIPPINGMODE mode)
@q m e f @>

@ The function |rootDualBound()| updates the final dual bound of the root
  node. This function should be only called at the end of the root node
  optimization.

@(master.cc@>=
  void ABA_MASTER::rootDualBound(double x) 
  {
    rootDualBound_ = x;
  }



