@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: numcon.w,v 4.12 2003/08/05 13:55:24 elf Exp $ @>

@q m i ABA_NUMCON @>

@ All member functions are defined in the file {\tt numcon.cc}.

@(numcon.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "numcon.h"
#include "master.h"
#include "colvar.h"
#else
#include "abacus/numcon.h"
#include "abacus/master.h"
#include "abacus/colvar.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
  \BeginArg
  \docarg{|master|} {A pointer to the corresponding master of the optimization.}
  \docarg{|sub|}{A pointer to the subproblem associated with the
                   constraint. This can be also the 0-pointer.}
  \docarg{|sense|} {The sense of the constraint.}
  \docarg{|dynamic|} {If this argument is |true|, then the constraint
                          can be removed from the active constraint set
                          during the cutting plane phase of the subproblem
                          optimization.}
  \docarg{|local|}{If this argument is |true|, then the constraint
                       is considered to be only locally valid. As a local
                       constraint is associated with a subproblem, |sub|
                       must not be 0 if |local| is |true|.
      }
  \docarg{|liftable|}{If this argument is |true|, then a lifting
                          procedure must be available, i.e., that the
                          coefficients of variables which have not been
                          active at generation time of the constraint
                          can be computed.}
  \docarg{|number|}{The identification number of the constraint.}
  \docarg{|rhs|}{The right hand side of the constraint.}
  \EndArg
@q m e a @>

@(numcon.cc@>=
@q m b f @>
  ABA_NUMCON::ABA_NUMCON(ABA_MASTER *master, 
                         const ABA_SUB *sub, 
                         ABA_CSENSE::SENSE sense, 
                         bool dynamic,
                         bool local, 
                         bool liftable,
                         int number, 
                         double rhs) 
@q m e f @>
  : @/
   ABA_CONSTRAINT(master, sub,  sense, rhs, dynamic, local, liftable), @/
   number_(number)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(numcon.cc@>=
@q m b f @>
  ABA_NUMCON::~ABA_NUMCON()
@q m e f @>
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the constraint from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the constraint is initialized.}
  \EndArg
@q m e a @>

@(numcon.cc@>=
@q m b f @>
  ABA_NUMCON::ABA_NUMCON(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    ABA_CONSTRAINT(master, msg)
  {
    msg.unpack(number_);
  }

@q m f pack @>
@ The function |pack()| packs the data of the constraint in an
  |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the constraint is packed.}
  \EndArg
@q m e a @>

@(numcon.cc@>=
@q m b f @>
  void ABA_NUMCON::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_CONSTRAINT::pack(msg);
    msg.pack(number_);
  }

@q m f classId @>
@ The function |classId()| returns an integer which identifies the class.

@q m b r @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e r @>

@(numcon.cc@>=
@q m b f @>
  int ABA_NUMCON::classId() const
@q m e f @>
  {
    return NUMCON_CLASSID;
  }

#endif


@q m f Output Operator @>
@ The output operator writes the identification number and the right
  hand side to an output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The variable being output.}
  \EndArg
@q m e a @>

@(numcon.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_NUMCON &rhs)
@q m e f @>
  {
    return out << "number = " << rhs.number_ << "  rhs = " << rhs.rhs_ << endl;
  }

@q m v coeff @>
@q m e t @>
@ The function |coeff()|.

@q m b r @>
  \BeginReturn
  The coefficient of the variable |v|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|v|}{The variable of which the coefficient is determined.
               It must point to an object of the class |ABA_COLVAR|.}
  \EndArg
@q m e a @>

@(numcon.cc@>=
@q m b f @>
  double ABA_NUMCON::coeff(ABA_VARIABLE *v)
@q m e f @>
  {
    ABA_COLVAR *colvar = (ABA_COLVAR *) v;

    return colvar->coeff(number_);
  }

@q m v print @>
@ The function |print()| writes the row format of the constraint on 
  an output stream.
  It redefines the virtual function |print()| of the base class
  |ABA_CONVAR|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \EndArg
@q m e a @>

@(numcon.cc@>=
@q m b f @>
  void ABA_NUMCON::print(ostream &out)
@q m e f @>
  {
    out << *this;
  }

@q m f number @>
@q m e t @>
@ The function |number()|.

@q m b r @>
  \BeginReturn
  Returns the identification number of the constraint.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_NUMCON::number() const
@q m e f @>
