@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: opensub.w,v 4.11 2003/08/05 13:55:25 elf Exp $ @>

@q m i ABA_OPENSUB @>

@ The member functions are defined in the file {\tt opensub.cc}.

@(opensub.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "opensub.h"
#include "sub.h"
#include "master.h"
#include "optsense.h"
#else
#include "abacus/opensub.h"
#include "abacus/sub.h"
#include "abacus/master.h"
#include "abacus/optsense.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/parmaster.h"
#include "abacus/notifyserver.h"
#include "abacus/notification.h"
#endif

@q m f Constructor @>

@ The constructor does not initialize the member |dualBound_| since
  this can only be done if we know the sense of the objective function
  which is normally unknown when the constructor of the class |ABA_MASTER|
  is called which again calls this constructor.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|master|}{A pointer to the corresponding master of the optimization.}
  \EndArg
@q m e a @>

@(opensub.cc@>=
@q m b f @>
  ABA_OPENSUB::ABA_OPENSUB(ABA_MASTER *master)
@q m e f @>
  : @/
  master_(master), @/
  list_(master), @/
  n_(0)
#ifdef ABACUS_PARALLEL
  , @/
  cond_(mutex_), @/
  terminate_(false), @/
  hasTerminated_(false)
#endif
 {  }

@ The function |insert()| adds a subproblem to the set of
  open subproblems.

  \BeginArg
  \docarg{|sub|}{The subproblem that is inserted.}
  \EndArg

@(opensub.cc@>=
  void ABA_OPENSUB::insert(ABA_SUB *sub)
  {
#ifdef ABACUS_PARALLEL
    mutex_.acquire();
#endif
    @<update the dual bound of all subproblems in the |list_|@>;
    list_.append(sub);
    ++n_;
#ifdef ABACUS_PARALLEL
    cond_.signal();
    mutex_.release();
    master_->parmaster()->newOpenSubCount(n_, dualBound_);
#endif
  }

@ @<update the dual bound of all subproblems in the |list_|@>=
  if (empty())
    dualBound_ = sub->dualBound();
  else {
    if (master_->optSense()->max()) {
      if (sub->dualBound() > dualBound_)
        dualBound_ = sub->dualBound();
    }
    else
      if (sub->dualBound() < dualBound_)
        dualBound_ = sub->dualBound();
  }
  

@ @(opensub.cc@>=
#ifdef ABACUS_PARALLEL

@ The function |terminate()| terminates the function |select()|.

@(opensub.cc@>=
  void ABA_OPENSUB::terminate()
  {
    terminate_ = true;
    while (!hasTerminated_)
      cond_.signal();
  }
 
@q m f getSubproblemWithBound @>
@q m e t @>
@ The function |getSubproblemWithBound()| extracts the best subproblem
  of the list of open subproblems if its  bound is
  at least as good as the given value.
@q m e t @>

@q m b a @> 
  \BeginArg
  \docarg{bound}{The requested dual bound}
  \EndArg
@q m e a @>

@q m b r @> 
  \BeginReturn
  A pointer to such a subproblem or 0.
  \EndReturn
@q m e r @>

@(opensub.cc@>=
@q m b f @>
  ABA_SUB* ABA_OPENSUB::getSubproblemWithBound(double bound)
@q m e f @>
{
  mutex_.acquire();

  ABA_DLISTITEM<ABA_SUB*> *item;
  ABA_SUB *s;

  forAllDListElem(list_, item, s) {
    if (s->status() == ABA_SUB::Dormant) {
      s->newDormantRound();
      if (s->nDormantRounds() < master_->minDormantRounds())
        continue;
    }

    // is the bound of s good enough?
    if (master_->optSense()->max()) {
      if (s->dualBound() >= bound)
        break;
    }
    else {
      if (s->dualBound() <= bound)
        break;
    }
  }

  if (item==0) {
    mutex_.release();
    return 0;
  }

  // subproblem found
  assert(s == item->elem());
  list_.remove(item);

  updateDualBound();
  mutex_.release();

  master_->parmaster()->newOpenSubCount(n_, dualBound_);
  return s;
}

#endif


@ The function |select()| selects a subproblem according to the
  strategy in |master| and removes it from the list
  of open subproblems. 

  The function |select()| scans the list of open subproblems, and selects
  the subproblem with highest priority from the set of open subproblems.
  Dormant subproblems are ignored if possible.

  \BeginReturn
  The selected subproblem.
  If the set of open subproblems is empty, 0 is returned.
  \EndReturn

@(opensub.cc@>=
  ABA_SUB* ABA_OPENSUB::select()
  {
#ifdef ABACUS_PARALLEL
    mutex_.acquire();

    while (!terminate_) {

      // perform load balancing if needed
      mutex_.release();
      for (int i=0; i<3; i++)
        if (master_->parmaster()->balance())
          break;
      mutex_.acquire();

      // select the next subproblem to optimize
      ABA_DLISTITEM<ABA_SUB*> *minItem = list_.first();
      ABA_DLISTITEM<ABA_SUB*> *item;
      ABA_SUB *s;

      forAllDListElem(list_, item, s) {
        if (s->status() == ABA_SUB::Dormant) {
          s->newDormantRound();
          if (s->nDormantRounds() < master_->minDormantRounds())
            continue;
        }
        if (master_->enumerationStrategy(s, minItem->elem()) > 0)
          minItem = item;
      }

      if (minItem != 0) {                 // subproblem found
        ABA_SUB* min = minItem->elem();
        list_.remove(minItem);

        updateDualBound();

        mutex_.release();

        master_->parmaster()->newOpenSubCount(n_, dualBound_);
        return min;
      }

      // host zero should poll for termination
      if (master_->parmaster()->isHostZero()) {
        ABA_NOTIFICATION msg(master_, 0);
        msg.pack(ABA_NOTIFYSERVER::TriggerTerminationCheckTag);
        msg.send();
        ACE_Time_Value abstime(0,400000);
        abstime += ACE_OS::gettimeofday();
        master_->parmaster()->startIdleTime();
        cond_.wait(&abstime);                // wait at most 0.4 s
        master_->parmaster()->stopIdleTime();
      }
      else {               // the other hosts should wait until signaled
        master_->parmaster()->startIdleTime();
        cond_.wait();
        master_->parmaster()->stopIdleTime();
      }
    }

    mutex_.release();

    hasTerminated_ = true;     // terminate |ABA_OPENSUB::terminate()|

    return 0;

#else
    ABA_DLISTITEM<ABA_SUB*> *minItem = list_.first();
    ABA_DLISTITEM<ABA_SUB*> *item;
    ABA_SUB *s;

    forAllDListElem(list_, item, s) {
      if (s->status() == ABA_SUB::Dormant) {
        s->newDormantRound();
        if (s->nDormantRounds() < master_->minDormantRounds())
          continue;
      }
      if (master_->enumerationStrategy(s, minItem->elem()) > 0)
        minItem = item;
    }

    if (minItem == 0) return 0;

    ABA_SUB* min = minItem->elem();
    list_.remove(minItem);

    --n_;
    
    updateDualBound();

    return min;
#endif
  }

@ The function |remove()| removes  subproblem from the set of open subproblems.

  \BeginArg
  \docarg{|sub|}{The subproblem that is removed.}
  \EndArg

@(opensub.cc@>=
  void ABA_OPENSUB::remove(ABA_SUB* sub)
  {
#ifdef ABACUS_PARALLEL
    mutex_.acquire();
#endif
    if (n_) {
      list_.remove(sub);
      --n_;
      updateDualBound();
    }
#ifdef ABACUS_PARALLEL
    mutex_.release();
    master_->parmaster()->newOpenSubCount(n_, dualBound_);
#endif
  }

@ The function |prune()| removes all elements from the set of opens
  subproblems.

@(opensub.cc@>=
  void ABA_OPENSUB::prune()
  {
#ifdef ABACUS_PARALLEL
    mutex_.acquire();
#endif
    while (!list_.removeHead());
    n_ = 0;
#ifdef ABACUS_PARALLEL
    mutex_.release();
    master_->parmaster()->newOpenSubCount(n_, dualBound_);
#endif
  }

@q m f number @>
@q m e t @>
@ The function |number()|.

@q m b r @>
  \BeginReturn
  The current number of open subproblems contained in this set.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_OPENSUB::number() const
@q m e f @>
  
@q m f empty @>
@q m e t @>
@ The function |empty()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If there is no subproblem in the set of open subproblems,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_OPENSUB::empty() const
@q m e f @>

@q m f dualBound @>
@q m e t @>
@ The function |dualBound()|. 

@q m b r @>
  \BeginReturn
  The value of the dual bound of all subproblems in the list.
  \EndReturn
@q m e r @>

@(opensub.cc@>=
@q m b f @>
  double ABA_OPENSUB::dualBound() const
@q m e f @>
  {
#ifdef ABACUS_PARALLEL
    mutex_.acquire();
#endif
    double ret;
    if (empty()) {
      if (master_->optSense()->max()) ret = -master_->infinity();
      else                            ret = master_->infinity();
    }
    else
      ret = dualBound_;
#ifdef ABACUS_PARALLEL
    mutex_.release();
#endif
    return ret;
  }

@ The function |updateDualBound()| updates the member |dualBound_|
  according to the dual bounds of the subproblems contained in this set.

@(opensub.cc@>=
  void ABA_OPENSUB::updateDualBound()
  {

    ABA_DLISTITEM<ABA_SUB*> *item;  // to scan the list of subproblems
    ABA_SUB              *s;

#ifdef ABACUS_PARALLEL
    n_ = 0;
#endif
    if (master_->optSense()->max()) {
      dualBound_ = -master_->infinity();
      
      forAllDListElem(list_, item, s) {
#ifdef ABACUS_PARALLEL
        n_++;
#endif
        if (s->dualBound() > dualBound_)
          dualBound_ = s->dualBound();
      }
    }
    else {
      dualBound_ = master_->infinity();
      
      forAllDListElem(list_, item, s) {
#ifdef ABACUS_PARALLEL
        n_++;
#endif
        if (s->dualBound() < dualBound_)
          dualBound_ = s->dualBound();
      }
    }
  }

