@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: ostream.w,v 4.17 2003/08/05 13:55:25 elf Exp $ @>

@q m i ABA_OSTREAM @>

@ The member functions are defined in the file {\tt ostream.cc}.

@(ostream.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "ostream.h"
#include "string.h"
#include "history.h"
#include "timer.h"
#include "lpvarstat.h"
#include "row.h"
#include "constraint.h"
#include "lp.h"
#else
#include "abacus/ostream.h"
#include "abacus/string.h"
#include "abacus/history.h"
#include "abacus/timer.h"
#include "abacus/lpvarstat.h"
#include "abacus/row.h"
#include "abacus/constraint.h"
#include "abacus/lp.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/id.h"
#endif

@q m f Constructor @>
@ The constructor turns the output on and associates it with
  a ``real'' stream.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The ``real'' stream (usually |cout| or |cerr|.)}
  \docarg{|logStreamName|}{If |logStreamName| is not 0, 
                           then the output also directed to a log-file
                           with this name. 
                           The default value of |logStreamName| is 0.}
  \EndArg
@q m e a @>

@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM::ABA_OSTREAM(ostream &out, const char *logStreamName) 
@q m e f @>
  : @/
    ostream(out.rdbuf()),
    out_(*this), @/
    on_(true), @/
    log_(0)
#ifdef ABACUS_PARALLEL
    , @/
    prompt_(0), @/
    firstPosition_(true)
#endif
  {
    if (logStreamName == 0) logOn_ = false;
    else {
      logOn_ = true;
      log_ = new @,@[ ofstream(logStreamName) @];

      if (!(*log_)) {
        cerr << "ABA_OSTREAM::ABA_OSTREAM(): opening log-file ";
        cerr << *logStreamName << " failed." << endl;
        exit(Fatal);
      }
    }
  }

@q m f Destructor @>
@q m e t @>
@ The destructor.

@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM::~ABA_OSTREAM()
@q m e f @>
  {
    delete log_;
#ifdef ABACUS_PARALLEL
    delete [] prompt_;
#endif
  }
  
@q m f Output Operator @>
@ We reimplement the output operator |<<| for all fundamental types, for
  |const char *|, and for some other classes listed below.
  If the output is turned on the
  operator of the base class |ostream| is called. If also the
  output to the logfile is turned on, we write the same message also
  to the log-file.  
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|o|}{The item being output.}
  \EndArg
@q m e a @>



@(ostream.cc@>=
@q m b f @>
 ABA_OSTREAM& ABA_OSTREAM::operator<<(char o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@ @<body of the output operator@>=
#ifdef ABACUS_PARALLEL
    if(on_) {
      if (firstPosition_) out_ << prompt_;
      out_ << o;
    }
    if(logOn_) {
      if (firstPosition_) *log_ << prompt_;
      *log_ << o;
    }
    firstPosition_ = false;
#else
    if(on_) out_ << o;
    if(logOn_) *log_ << o;
#endif
    return *this;

@q m f Output Operator @>
@q m e t @>

@ @(ostream.cc@>=    
@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(unsigned char o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(signed char o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(short o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(unsigned short o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(int o) 
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(unsigned int o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(long o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(unsigned long o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(float o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(double o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const char *o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const ABA_STRING &o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const ABA_TIMER &o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const ABA_HISTORY &o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const ABA_LP &o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const ABA_LPVARSTAT &o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const ABA_CSENSE &o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

#ifdef ABACUS_PARALLEL

@q m f Output Operator @>
@q m e t @>

@q m b f @>
  ABA_OSTREAM& ABA_OSTREAM::operator<<(const ABA_ID &o)
@q m e f @>
  {
    @<body of the output operator@>;
  }

#endif

@ A manipulator is a function having as argument a reference to an
  |ABA_OSTREAM| and returning an |ABA_OSTREAM|. Manipulators are used that
  we can call, e.g., the function |endl(o)| by just writing its
  name omitting brackets and the function argument.

    \BeginReturn
    A reference to the output stream.
    \EndReturn

    \BeginArg
    \docarg{|m|}{An output stream manipulator.}
    \EndArg


@(ostream.cc@>=
  ABA_OSTREAM& ABA_OSTREAM::operator<<(ABA_OSTREAM_MANIP m)
  {
    return (*m)(*this);
  }

@q m f off @>
@ The function |off()| turns the output off.
@q m e t @>

@(ostream.cc@>=
@q m b f @>
  void ABA_OSTREAM::off()
@q m e f @>
  {
    on_ = false;
  }

@q m f on @>
@ The function |on()| turns the output on.
@q m e t @>

@(ostream.cc@>=
@q m b f @>
  void ABA_OSTREAM::on()
@q m e f @>
  {
    on_ = true;
  }

@q m f logOn @>
@ The function |logOn()| turns the output to the logfile on.
@q m e t @>

@(ostream.cc@>=
@q m b f @>
  void ABA_OSTREAM::logOn()
@q m e f @>
  {
    if(log_ == 0) {
      cerr << "ABA_OSTREAM::logOn: logfile not initialized, cannot be turned on." << endl;
      exit(Fatal);
    }
    logOn_ = true;
  }

@q m f logOn @>
@ This version of |logOn()| turns the output to the logfile on and
  sets the log-file to |logStreamName|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|logStreamName|}{The name of the log-file.}
  \EndArg
@q m e a @>

@(ostream.cc@>=    
@q m b f @>
  void ABA_OSTREAM::logOn(const char *logStreamName)
@q m e f @>
  {
    delete log_;
    
    log_ = new @,@[ ofstream(logStreamName) @];

    if (!(*log_)) {
      cerr << "ABA_OSTREAM::ABA_OSTREAM(): opening log-file ";
      cerr << *logStreamName << " failed." << endl;
      exit(Fatal);
    }

    logOn();
  }

@q m f logOff @>
@ The function |logOff()| turns the output to the logfile off.
@q m e t @>

@(ostream.cc@>=
@q m b f @>
  void ABA_OSTREAM::logOff()
@q m e f @>
  {
    logOn_ = false;
  }

@q m f log @>
@q m e t @>
@ The function |log()|.

@q m b r @>
  \BeginReturn
  A pointer to the stream associated with the log-file.
  \EndReturn
@q m e r @>

@(ostream.cc@>=
@q m b f @>
  ofstream* ABA_OSTREAM::log() const
@q m e f @>
  {
    return log_;
  }

@q m f setFormatFlag @>
@ The function |setFormatFlag()| can be used to set the format flags
  of the output stream and the log file similar to the function 
  |ios::set()| of the iostream library. For a documentation of all
  possible flags we refer to the documentation of the GNU \CPLUSPLUS/
  iostream Library.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|flag|}{The flag being set.}
  \EndArg
@q m e a @>

@(ostream.cc@>=
@q m b f @>
#if defined ABACUS_COMPILER_VISUAL_CPP || defined ABACUS_COMPILER_SGI || \
    defined ABACUS_COMPILER_SUN
  void ABA_OSTREAM::setFormatFlag(long flag)
#else
  void ABA_OSTREAM::setFormatFlag(fmtflags flag)
#endif
@q m e f @>
  {
    out_.setf(flag);
    if (log_) log_->setf(flag);
  }

@q m f isOn @>
@q m e t @>
@ The function |isOn()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the output is turned on,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(ostream.cc@>=
@q m b f @>
  bool ABA_OSTREAM::isOn() const
@q m e f @>
  {
    return on_;
  }

@q m f isLogOn @>
@q m e t @>
@ The function |isLogOn()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the output to the logfile is turned on,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(ostream.cc@>=
@q m b f @>
  bool ABA_OSTREAM::isLogOn() const
@q m e f @>
  {
    return logOn_;
  }

#ifdef ABACUS_PARALLEL

@ The function |setPrompt()| sets a string which is output as a prefix
  at the beginning of each line. 

@q m b a @>
  \BeginArg
  \docarg{|prompt|}{The prompt.}
  \EndArg
@q m e a @>

@(ostream.cc@>=
@q m b f @>
  void ABA_OSTREAM::setPrompt(const char *prompt)
@q m e f @>
  {
    delete [] prompt_;
    prompt_ = 0;
    if (prompt) {
      prompt_ = new @[char[::strlen(prompt)+1]@];
      ::strcpy(prompt_, prompt);
    }
  }

#endif

@q m f flush @>
@ The function |flush()| flushes the output and the log stream buffers
  of the stream |o|. This function can be called via the
  manipulator |o << flush;|.
@q m e t @>

@q m b r @> 
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|o|}{An output stream.}
  \EndArg
@q m e a @>
  
@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM& flush(ABA_OSTREAM &o)
@q m e f @>
  {
    if (o.on_) o.out_ << flush;
    if(o.logOn_) *(o.log_) << flush;
    return o;
  }

@q m f endl @>
@ The function |endl()| writes an end of line to the output and
  log-file of the stream |o| and flushes both stream buffers.
  This function can be called via the manipulator |o << endl;|.
@q m e t @>
  
@q m b r @> 
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|o|}{An output stream.}
  \EndArg
@q m e a @>
  
@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM& endl(ABA_OSTREAM &o)
@q m e f @>
  {
    o << '\n';
    if (o.on_) o.out_ << flush;
    if(o.logOn_) *(o.log_) << flush;
#ifdef ABACUS_PARALLEL
    o.firstPosition_ = true;
#endif
    return o;
  }

@q m f \_setWidth @>
@ The function |\_setWidth()| sets the width of the field for the next
  output operation on the log and the output stream.

  In most cases the manipulator |setWith| is more convenient to use.
@q m e t @>
  
@q m b r @> 
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|o|}{An output stream.}
  \docarg{|w|}{The width of the field.}
  \EndArg
@q m e a @>
  
@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM& _setWidth(ABA_OSTREAM &o, int w)
@q m e f @>
  {
    if (o.on_) o.out_ << setw(w);
    if(o.logOn_) *(o.log_) << setw(w);
    return o;
  }

@q m f \_setPrecision @>
@ The function |\_setPrecision()| sets the precision for the output stream.

  In most cases the manipulator |setPrecision| is more convenient to use.
@q m e t @>
  
@q m b r @> 
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|o|}{An output stream.}
  \docarg{|p|}{The precision.}
  \EndArg
@q m e a @>
  
@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM& _setPrecision(ABA_OSTREAM &o, int p)
@q m e f @>
  {
    if (o.on_) o.out_ << setprecision(p);
    if(o.logOn_) *(o.log_) << setprecision(p);
    return o;
  }

@ The Constructor for a manipulator of the class |ABA_OSTREAM| with an
  integer argument.
  
  \BeginArg
  \docarg{|f|}{A pointer to the manipulating function.}
  \docarg{|i|}{The argument of the manipulating function.}
  \EndArg

@(ostream.cc@>=
  ABA_OSTREAM_MANIP_INT::ABA_OSTREAM_MANIP_INT(ABA_OSTREAM &(*f)(ABA_OSTREAM &o, int  i), 
                                       int i) : @/
    i_(i), @/
    f_(f)
  {}

@ The output operator of the maninpulator class calls the manipulating
  function.

  \BeginReturn
   A pointer to the ouput stream.
  \EndReturn

  \BeginArg
  \docarg{|o|}{An output stream}
  \docarg{|m|}{A manipulator object.}
  \EndArg

@(ostream.cc@>=
  ABA_OSTREAM &operator<<(ABA_OSTREAM &o, const ABA_OSTREAM_MANIP_INT &m)
  {
    return (*(m.f_))(o, m.i_);
  }

@q m f setWidth @>
@ The function |setWidth| can be used for output streams of the class 
  |ABA_OSTREAM| as the function |setw| for the class |ostream|, e.g.:

  |master\_->out() << setw(10) << x << endl;|
@q m e t @>

@q m b r @>
  \BeginReturn
  A manipulator object with the function |\_setWidth()|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|p|}{The width of the output field.}
  \EndArg
@q m e a @>

@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM_MANIP_INT setWidth(int p)
@q m e f @>
  {
    return ABA_OSTREAM_MANIP_INT(&_setWidth, p);
  }

@q m f setPrecision @>
@ The function |setWidth| can be used for output streams of the class 
  |ABA_OSTREAM| in the same way as the function |setprecision| 
  for the class |ostream|, e.g.:

  |master\_->out() << setprecision(10) << x << endl;|
@q m e t @>

@q m b r @>
  \BeginReturn
  A manipulator object with the function |\_setPrecision()|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|p|}{The precision for the output stream.}
  \EndArg
@q m e a @>

@(ostream.cc@>=
@q m b f @>
  ABA_OSTREAM_MANIP_INT setPrecision(int p)
@q m e f @>
  {
    return ABA_OSTREAM_MANIP_INT(&_setPrecision, p);
  }
