@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: row.w,v 4.13 2003/08/05 13:55:26 elf Exp $ @>

@q m i ABA_ROW @>

@ The member functions of |ABA_ROW| are implemented in the
  file {\tt row.cc}.

@(row.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "row.h"
#include "global.h"
#else
#include "abacus/row.h"
#include "abacus/global.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@q m e t @>
@ A constructor.

@q m b a @>
  \BeginArg
  \docarg{|glob|} {A pointer to the corresponding global object.}
  \docarg{|nnz|} {The number of nonzero elements of the row.}
  \docarg{|s|} {The array storing the nonzero elements.}
  \docarg{|c|} {The array storing the nonzero coefficients of the
                    elements of |s|.}
  \docarg{|sense|}{The sense of the row.}
  \docarg{|r|} {The right hand side of the row.}
  \EndArg
@q m e a @>

@(row.cc@>=
@q m b f @>   
  ABA_ROW::ABA_ROW(ABA_GLOBAL *glob, 
                   int nnz,
                   const ABA_ARRAY<int> &s, 
                   const ABA_ARRAY<double> &c, 
                   const ABA_CSENSE sense, 
                   double r)
@q m e f @>
  : @/
    ABA_SPARVEC(glob, nnz, s, c), @/
    sense_(sense), @/
   rhs_(r)
  {  }

@q m f Constructor @>
@ This is an equivalent constructor using |ABA_CSENSE::SENSE| instead
  of an object of the class |SENSE| to initialize the sense of the
  constraint.
@q m e t @>

@(row.cc@>=
@q m b f @>
  ABA_ROW::ABA_ROW(ABA_GLOBAL *glob, 
                   int nnz,
                   const ABA_ARRAY<int> &s, 
                   const ABA_ARRAY<double> &c, 
                   const ABA_CSENSE::SENSE sense, 
                   double r)
@q m e f @>
  : @/
    ABA_SPARVEC(glob, nnz, s, c), @/
    sense_(glob, sense), @/
    rhs_(r)
  {  }

@q m f Constructor @>
@ This is also an equivalent constructor except that |s| and |c|
  are C-style arrays.
@q m e t @>

@(row.cc@>=
@q m b f @>
  ABA_ROW::ABA_ROW(ABA_GLOBAL *glob, int nnz,
                   int *s, double *c,
                   ABA_CSENSE::SENSE sense, double r) 
@q m e f @>
  : @/
    ABA_SPARVEC(glob, nnz, s, c), @/
    sense_(glob, sense), @/
    rhs_(r)
  { }
                
@q m f Constructor @>      
@ A constructor without initialization of the nonzeros of the row.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \docarg{|size|}{The maximal numbers of nonzeros.}
  \EndArg
@q m e a @>

@(row.cc@>=
@q m b f @>
  ABA_ROW::ABA_ROW(ABA_GLOBAL *glob, int size) 
@q m e f @>
  : @/
    ABA_SPARVEC(glob, size), @/
    sense_(glob)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(row.cc@>=
@q m b f @>
  ABA_ROW::~ABA_ROW()
@q m e f @>
  {}
    

#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>
@ The message constructor creates the row from an |ABA_MESSAGE|.

@q m b a @>
    \BeginArg
    \docarg
      {|glob|}{A pointer to the corresponding global object.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(row.cc@>=
@q m b f @>
  ABA_ROW::ABA_ROW(const ABA_GLOBAL *glob, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    ABA_SPARVEC(glob, msg), @/
    sense_(glob, msg)
  {
    msg.unpack(rhs_);
  }


@q m f pack @>
@ The function |pack()| packs the data of the row in an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the row is packed.}
  \EndArg
@q m e a @>

@(row.cc@>=
@q m b f @>
  void ABA_ROW::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_SPARVEC::pack(msg);
    sense_.pack(msg);
    msg.pack(rhs_);
  }

#endif


@q m f Output Operator @>
@ The output operator writes the row on an output stream in format
  like {\tt -2.5 x1 + 3 x3 <= 7}. Only variables with nonzero coefficients
  are output.
  The output operator does neither output a |'+'| before the first
  coefficient of a row, if it is positive, nor outputs coefficients
  with absolute value 1.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The row being output.}
  \EndArg
@q m e a @>

@(row.cc@>=
@q m b f @>
  ostream &operator<<(ostream& out, const ABA_ROW &rhs)
@q m e f @>
  {
    int    s;
    double c;
    char   sign;
    double eps = rhs.glob_->machineEps();
    const int rhsNnz = rhs.nnz();

    for (int i = 0; i < rhsNnz; i++) {
      s = rhs.support(i);
      c = rhs.coeff(i);

      if (c < 0.0) {
        sign = '-';
        c = -c;
      }
      else sign = '+';

      if (i > 0 || sign == '-')  // do not print first |'+'| of row
        out << sign << ' ';
      if (c < 1.0 - eps || 1.0 + eps < c)  // do not print coefficient 1
        out << c << ' ';
      out << 'x' << s << ' ';   

      if (i && !(i % 10)) out << endl;
    }

    return out << rhs.sense_ << ' ' << rhs.rhs();
  }

@q m f rhs @>
@q m e t @>
@ The function |rhs()|.

@q m b r @>
  \BeginReturn
  The right hand side stored in the row format.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_ROW::rhs() const
@q m e f @>

@q m f rhs @>
@ This version of |rhs()| sets the right hand side of the row.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|r|}{The new value of the right hand side.}
  \EndArg
@q m e a @>

@q m b f @>
  void  ABA_ROW::rhs(double r) 
@q m e f @>

@q m f sense @>
@q m e t @>
@ The function |sense()|.

@q m b r @>
  \BeginReturn
  A pointer to the sense of the row.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_CSENSE *ABA_ROW::sense()
@q m e f @>

@q m f sense @>
@ This version of |sense()| sets the sense of the row.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|s|}{The new sense of the row.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_ROW::sense(ABA_CSENSE &s)
@q m e f @>

@q m f sense @>
@ And another version of |sense()| to set the sense of the row.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|s|}{The new sense of the row.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_ROW::sense(ABA_CSENSE::SENSE s)
@q m e f @>

@q m f copy @>
@ The function |copy()| behaves like an assignment operator,
  however, the maximal number of the elements of this row
  only has to be at least the number of nonzeros of |row|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|row|}{The row that is copied.}
  \EndArg
@q m e a @>

@(row.cc@>=
@q m b f @>
  void ABA_ROW::copy(const ABA_ROW &row)
@q m e f @>
  {
    sense_ = row.sense_;
    rhs_   = row.rhs_;

    ABA_SPARVEC::copy(row);
  }

@q m f delInd @>
@ The function |delInd()| removes the indices listed in |buf|
  from the support of the row and subtracts |rhsDelta| from
  its right hand side.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|buf|}{The components being removed from the row.}
  \docarg{|rhsDelta|}{The correction of the right hand side of the row.}
  \EndArg
@q m e a @>

@(row.cc@>=
@q m b f @>
  void ABA_ROW::delInd(ABA_BUFFER<int> &buf, double rhsDelta)
@q m e f @>
  {
    leftShift(buf);
    rhs_ -= rhsDelta;
  }

