@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: rowcon.w,v 4.10 2003/08/05 13:55:26 elf Exp $ @>

@q m i ABA_ROWCON @>

@ All member functions are defined in the file {\tt rowcon.cc}.

@(rowcon.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "rowcon.h"
#include "master.h"
#include "numvar.h"
#else
#include "abacus/rowcon.h"
#include "abacus/master.h"
#include "abacus/numvar.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
  \BeginArg
  \docarg{|master|}{A pointer to the corresponding master of the optimization.}
  \docarg{|sub|}{A pointer to the subproblem associated with the
                   constraint. This can also be the 0-pointer.}
  \docarg{|sense|}{The sense of the constraint.}
  \docarg{|nnz|}{The number of nonzero elements of the constraint.}
  \docarg{|support|}{The array storing the variables with nonzero
                       coefficients.}
  \docarg{|coeff|}{The nonzero coefficients of the variables stored
                     in |support|.}
  \docarg{|rhs|}{The right hand side of the constraint.}
  \docarg{|dynamic|}{If this argument is |true|, then the constraint
                     can be removed from the active constraint set
                     during the cutting plane phase of the subproblem
                     optimization.}
  \docarg{|local|}{If this argument is |true|, then the constraint
                   is considered to be only locally valid. As a
                   locally valid constraint is associated with
                   a subproblem, |sub| must not be 0 if |local| is |true|.
      }
  \docarg{|liftable|}{If this argument is |true|, then a lifting
                      procedure must be available, i.e., that the
                      coefficients of variables which have not been
                      active at generation time of the constraint
                      can be computed.}
  \EndArg
@q m e a @>

@(rowcon.cc@>=
@q m b f @>
  ABA_ROWCON::ABA_ROWCON(ABA_MASTER *master, 
                         const ABA_SUB *sub, 
                         ABA_CSENSE::SENSE sense, 
                         int nnz, 
                         const ABA_ARRAY<int> &support, 
                         const ABA_ARRAY<double> &coeff,
                         double rhs, 
                         bool dynamic, 
                         bool local,
                         bool liftable) 
@q m e f @>
  : @/
    ABA_CONSTRAINT(master, sub, sense, rhs, dynamic, local, liftable), @/
    row_(master, nnz, support, coeff, sense, rhs)
   { }

@q m f Constructor @>
@ This constructor is equivalent to the previous constructor
  except that it uses C-style arrays for |support| and |coeff|.
@q m e t @>

@(rowcon.cc@>=
@q m b f @>
  ABA_ROWCON::ABA_ROWCON(ABA_MASTER *master, 
                         const ABA_SUB *sub, 
                         ABA_CSENSE::SENSE sense, 
                         int nnz,
                         int *support, 
                         double *coeff,
                         double rhs, 
                         bool dynamic, 
                         bool local,
                         bool liftable) 
@q m e f @>
  : @/
    ABA_CONSTRAINT(master, sub, sense, rhs, dynamic, local, liftable), @/
    row_(master, nnz, support, coeff, sense, rhs)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(rowcon.cc@>=
@q m b f @>
  ABA_ROWCON::~ABA_ROWCON()
@q m e f @>
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the constraint from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the constraint is initialized.}
  \EndArg
@q m e a @>

@(rowcon.cc@>=
@q m b f @>
  ABA_ROWCON::ABA_ROWCON(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    ABA_CONSTRAINT(master, msg), @/
    row_(master, msg)
  {  }

@q m f pack @>
@ The function |pack()| packs the data of the constraint in an
  |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the constraint is packed.}
  \EndArg
@q m e a @>

@(rowcon.cc@>=
@q m b f @>
  void ABA_ROWCON::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_CONSTRAINT::pack(msg);
    row_.pack(msg);
  }

@q m f classId @>
@ The function |classId()| returns an integer which identifies the class.

@q m b r @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e r @>

@(rowcon.cc@>=
@q m b f @>
  int ABA_ROWCON::classId() const
@q m e f @>
  {
    return ROWCON_CLASSID;
  }

#endif


@q m v coeff @>
@ The function |coeff()| computes the coefficient of a variable which
  must be of type |ABA_NUMVAR|.
  It redefines the virtual function |coeff()| of the base class |ABA_CONSTRAINT|.

  {\bf Warning:} The worst case complexity of the call of this function is
  the number of nonzero elements of the constraint.
@q m e t @>

@q m b r @>
  \BeginReturn
  The coefficient of the variable |v|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|v|}{The variable of which the coefficient is determined.}
  \EndArg
@q m e a @>

@(rowcon.cc@>=
@q m b f @>
  double ABA_ROWCON::coeff(ABA_VARIABLE *v)
@q m e f @>
  {
    ABA_NUMVAR *numVar = (ABA_NUMVAR *) v;
    return row_.origCoeff(numVar->number());
  }

@q m v print @>
@ The function |print()| writes the row format of the constraint on an
  output stream.
  It redefines the virtual function |print()| of the base class
  |ABA_CONVAR|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \EndArg
@q m e a @>

@(rowcon.cc@>=
@q m b f @>
  void ABA_ROWCON::print(ostream &out)
@q m e f @>
  {
    out << row_;
  }

@q m f row @>
@q m e t @>
@ The function |row()|.

@q m b r @>
  \BeginReturn
  A pointer to the object of the class |ABA_ROW| representing the constraint.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_ROW *ABA_ROWCON::row()
@q m e f @>

