@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: set.w,v 4.8 2003/08/05 13:55:26 elf Exp $ @>

@q m i ABA_SET @>

@ The member functions are implemented in the file {\tt set.cc}.

@(set.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "set.h"
#else
#include "abacus/set.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
      \BeginArg
      \docarg{|glob|}{A pointer to the corresponding global object.}
      \docarg{|size|}{Only integers between 0 and |size-1| can be inserted
                     in the set.}
      \EndArg
@q m e a @>

@(set.cc@>=
@q m b f @>
  ABA_SET::ABA_SET(ABA_GLOBAL *glob, int size)
@q m e f @>
  : @/
    glob_(glob), @/
    parent_(glob, size)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.
 
@(set.cc@>=
@q m b f @>
  ABA_SET::~ABA_SET()
@q m e f @>
  {
  }

@q m f makeSet @>
@ The function |makeSet()| creates a set storing only one element
  and adds it to the collection of sets.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{The single element of the new set.}
  \EndArg
@q m e a @>

@(set.cc@>=
@q m b f @>
  void ABA_SET::makeSet(int x)
@q m e f @>
  {
    parent_[x] = x;
  }

@q m f unionSets @>
@ The function |unionSets()|
  unites the two sets which contain |x| and
  |y|, respectively. This operation may only be performed if
  both |x| and |y| have earlier been added to the collection of sets
  by the function |makeSet()|. 

  We do not use the heuristic  attaching the smaller subtree
  to the bigger one, since we want to guarantee that the representative
  of |x| is always the representative of the two united sets. 
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If both sets have been disjoint before the function call,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{An element of the first set of the union operation.}
  \docarg{|y|}{An element in the second set of the union operation.}
  \EndArg
@q m e a @>

@(set.cc@>=
@q m b f @>
  bool ABA_SET::unionSets(int x, int y)
@q m e f @>
  {
    int xParent = findSet(x);
    int yParent = findSet(y);

    if (xParent == yParent) return false;
    
    parent_[yParent] = xParent;

    return true;
  }

@q m f findSet @>
@ The function |findSet()| finds the representative of the
  set containing |x|.
  This operation may be only performed if
  |x| has been earlier added to the collection of sets
  by the function |makeSet()|.
@q m e t @>

  A path compression is performed, i.e., all nodes of the
  tree on the path from |x| to the root are directly attached to the
  root of the tree.

@q m b r @>
  \BeginReturn
  The representative of the set containing |x|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{An element of the searched set.}
  \EndArg
@q m e a @>

@(set.cc@>=
@q m b f @>
  int ABA_SET::findSet(int x)
@q m e f @>
  {
    if (x != parent_[x])
      parent_[x] = findSet(parent_[x]);
    return parent_[x];
  }
