@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: setbranchrule.w,v 4.15 2003/08/05 13:55:26 elf Exp $ @>

@q m i ABA_SETBRANCHRULE @>

@ All member functions are defined in the file {\tt br|_|set.cc}.

@(setbranchrule.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "setbranchrule.h"
#include "master.h"
#include "sub.h"
#include "lpsub.h"

#else
#include "abacus/setbranchrule.h"
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/lpsub.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@q m e t @>

@ The constructor.
@q m b a @>
   \BeginArg
   \docarg{|master|}{A pointer to the corresponding master of the optimization.}
   \docarg{|variable|}{The branching variable.}
   \docarg{|status|}{The status the variable is set to
                   (|SetToLowerBound| or |SetToUpperBound|).}
   \EndArg
@q m e a @>

@(setbranchrule.cc@>=
@q m b f @>
  ABA_SETBRANCHRULE::ABA_SETBRANCHRULE(ABA_MASTER *master, 
                                       int variable, 
                                       ABA_FSVARSTAT::STATUS status)
@q m e f @>
  : @/
    ABA_BRANCHRULE(master), @/
    variable_(variable), @/
    status_(status)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(setbranchrule.cc@>=
@q m b f @>
  ABA_SETBRANCHRULE::~ABA_SETBRANCHRULE()
@q m e f @>
  { }

#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the |ABA_SETBRANCHRULE| from an |ABA_MESSAGE|.

@q m e t @>

@q m b a @>
    \BeginArg
    \docarg
      {|master|}{A pointer to the corresponding master of the optimization.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(setbranchrule.cc@>=
@q m b f @>
  ABA_SETBRANCHRULE::ABA_SETBRANCHRULE(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    ABA_BRANCHRULE(master, msg)
  {
    msg.unpack(variable_);
    status_ = (ABA_FSVARSTAT::STATUS) msg.unpackInt();
    msg.unpack(oldLpBound_);
  }

@q m f pack @>
@ The virtual function |pack()| packs the data of the |ABA_SETBRANCHRULE|
  in an |ABA_MESSAGE| object. The |pack()| function of the base class
  has to be called before the local data is packed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the |ABA_SETBRANCHRULE| is packed.}
  \EndArg
@q m e a @>

@(setbranchrule.cc@>=
@q m b f @>
  void ABA_SETBRANCHRULE::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_BRANCHRULE::pack(msg);
    msg.pack(variable_);
    msg.pack((int)status_);
    msg.pack(oldLpBound_);
  }

@q m f classId @>
@ The abstract virtual function |classId()| of |ABA_BRANCHRULE| has to be
  implemented in each derived subclass. It has to return an integer
  which identifies the class.

@q m b r @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e r @>

@(setbranchrule.cc@>=
@q m b f @>
  int ABA_SETBRANCHRULE::classId() const
@q m e f @>
  {
    return SETBRANCHRULE_CLASSID;
  }

#endif


@q m f Output Operator @>
@ The output operator writes the number of the branching variable and
  its status on an output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The branching rule being output.}
  \EndArg
@q m e a @>

@(setbranchrule.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_SETBRANCHRULE &rhs)
@q m e f @>
  {
    return out << "x" << rhs.variable_ << " = " << (int) rhs.status_;
  }

@q m v extract @>
@ The function |extract()| modifies a subproblem by setting the branching
  variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the subproblem can be modified according to the
               branching rule.}
  \docarg{|1|}{If a contradiction occurs.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The subproblem being modified.}
  \EndArg
@q m e a @>

@(setbranchrule.cc@>=
@q m b f @>
  int ABA_SETBRANCHRULE::extract(ABA_SUB *sub)
@q m e f @>
  {
    if (sub->fsVarStat(variable_)->contradiction(status_))
      return 1;
    
    sub->fsVarStat(variable_)->status(status_);
    return 0;
  }

@q m v extract @>
@ The function |extract()| is overloaded to modify directly the linear
  programming relaxation. This required to evaluate the quality of
  a branching rule with linear programming methods. The changes
  have to be undone with the function |unextract()| before the next linear
  program is solved.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|lp|}{A pointer to the linear programming relaxation of a 
                subproblem.}
  \EndArg
@q m e a @>

@(setbranchrule.cc@>=
@q m b f @>
  void ABA_SETBRANCHRULE::extract(ABA_LPSUB *lp)
@q m e f @>
  {
    if (status_ == ABA_FSVARSTAT::SetToLowerBound) {
      oldLpBound_ = lp->uBound(variable_);
      lp->changeUBound(variable_, lp->lBound(variable_));
    }
    else {
      oldLpBound_ = lp->lBound(variable_);
      lp->changeLBound(variable_, lp->uBound(variable_));
    }
  }


@ The function |unExtract()|.
@q m v unExtract @>
@q m e t @>

@(setbranchrule.cc@>=
@q m b f @>
  void ABA_SETBRANCHRULE::unExtract(ABA_LPSUB *lp)
@q m e f @>
  {
    if (status_ == ABA_FSVARSTAT::SetToLowerBound)
      lp->changeUBound(variable_, oldLpBound_);
    else
      lp->changeLBound(variable_, oldLpBound_);
  }

@q m v branchOnSetVar @>
@ The function |branchOnSetVar()| redefines the virtual function
  of the base class |ABA_BRANCHRULE| as this branching rule is setting
  a binary variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  Always |true|.
  \EndReturn
@q m e r @>

@(setbranchrule.cc@>=
@q m b f @>
  bool ABA_SETBRANCHRULE::branchOnSetVar()
@q m e f @>
  {
    return true;
  }

@q m f setToUpperBound @>
@q m e t @>
@ The function |setToUpperBound()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the branching variable is set to the upper bound,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(setbranchrule.cc@>=
@q m b f @>
  bool ABA_SETBRANCHRULE::setToUpperBound() const
@q m e f @>
  {
    if (status_ == ABA_FSVARSTAT::SetToUpperBound) return true;
    else                                       return false;
  }


@q m f variable @>
@q m e t @>
@ The function |variable()|.

@q m b r @>
  \BeginReturn
  The number of the branching variable.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SETBRANCHRULE::variable() const
@q m e f @>


