@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: slackstat.w,v 4.9 2003/08/05 13:55:26 elf Exp $ @>

@q m i ABA_SLACKSTAT @>

@ All member functions are defined in the file {\tt slackstat.cc}.

@(slackstat.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "slackstat.h"
#include "global.h"
#else
#include "abacus/slackstat.h"
#include "abacus/global.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@ This constructor initializes the status as |Unknown|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_SLACKSTAT::ABA_SLACKSTAT(const ABA_GLOBAL *glob) 
@q m e f @>
  
@q m f Constructor @>
@ A constructor with initialization.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \docarg{|status|}{The slack variable receives the status |status|.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_SLACKSTAT::ABA_SLACKSTAT(const ABA_GLOBAL *glob, STATUS status) 
@q m e f @>


@ @(slackstat.cc@>=
#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the status from a message.
@q m e t @>

@q m b a @>
    \BeginArg
    \docarg{|glob|}{A pointer to a global object.}
    \docarg{|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(slackstat.cc@>=
@q m b f @>
  ABA_SLACKSTAT::ABA_SLACKSTAT(const ABA_GLOBAL *glob, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    glob_(glob)
  {
    status_ = (STATUS) msg.unpackInt();
  }


@q m f pack @>
@ The function |pack()| packs the data of the status in a message.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the status is packed.}
  \EndArg
@q m e a @>

@(slackstat.cc@>=
@q m b f @>
  void ABA_SLACKSTAT::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    msg.pack((int)status_);
  }

#endif


@q m f Output Operator @>
@ The output operator writes the status to an output stream in the
  format {\tt Basic}, {\tt NonBasicZero}, {\tt Non\-Basic\-Non\-Zero}, or
  {\tt Unknown}.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The status being output.}
  \EndArg
@q m e a @>

@(slackstat.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_SLACKSTAT &rhs)
@q m e f @>
  {
    switch (rhs.status_) {
      case ABA_SLACKSTAT::Basic:           out << "Basic";
                                       break;
      case ABA_SLACKSTAT::NonBasicZero:    out << "NonBasic Zero";
                                       break;
      case ABA_SLACKSTAT::NonBasicNonZero: out << "NonBasic NonZero";
                                       break;
      case ABA_SLACKSTAT::Unknown:         out << "Unknown";
                                       break;
    }
    return out;
  }

@q m f status @>
@q m e t @>
@ The function |status()|.

@q m b r @>
  \BeginReturn
  The status of the slack variable.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_SLACKSTAT::STATUS ABA_SLACKSTAT::status() const
@q m e f @>

@q m f status @>
@ This version of the function |status()| sets the status of the slack 
  variable.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|stat|}{The new status of the slack variable.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_SLACKSTAT::status(STATUS stat)
@q m e f @>

@q m f status @>
@ This version of the function |status()| sets the status to the one
  of |*stat|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|stat|}{The status of the slack variable is set to |*stat|.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_SLACKSTAT::status(const ABA_SLACKSTAT *stat)
@q m e f @>
  
