@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: soplexif.w,v 4.21 2003/08/05 13:55:26 elf Exp $ @>

@q m i ABA_SOPLEXIF @>

@ The member functions of the class |ABA_SOPLEXIF| are implemented in the
  file |soplexif.cc|.

@(soplexif.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "lpmastersoplex.h"
#include "soplexif.h"
#include "column.h"
#include "row.h"
#else
#include "abacus/master.h"
#include "abacus/lpmastersoplex.h"
#include "abacus/soplexif.h"
#include "abacus/column.h"
#include "abacus/row.h"
#endif

#include "soplex.hh"
#include "spxsteeppr.hh"
#include "spxfastrt.hh"
#include "slufactor.hh"

#include <limits.h>

@<initialize some Soplex templates@>;
@<the implementation of the members of |ABA_SOPLEXIF|@>;


@ Since we turned off implicit template instantiation we have to
  enforce it also explicitely for soplex templates.
 
@<initialize some Soplex templates@>=
#include "idlist.hh"
#include "dataset.hh"
#include "dataarray.hh"
#include "islist.hh"

#ifndef ABACUS_COMPILER_VISUAL_CPP
  template class IdList<SVSet_DLPSV>;
  template class DataSet<SVSet_DLPSV>;
  template class DataArray<int>;
  template class DataArray<SVector_Element>;
  template class IsList<SVSet_DLPSV>;
#endif

@q m f Constructor @>
@ This constructor does not initialize the problem data of the linear
     program. It must be loaded later with the function |initialize()|.
@q m e t @>

@q m b a @>
     \BeginArg
     \docarg{|master|}{A pointer to the corresponding master of the
                     optimization.}
      \EndArg
@q m e a @>

@<the implementation of the members of |ABA_SOPLEXIF|@>=
@q m b f @>
  ABA_SOPLEXIF::ABA_SOPLEXIF(ABA_MASTER *master) 
@q m e f @>
  : @/
    ABA_LP (master), @/
    soplex_(0), @/
    pricer_(0), @/
    ratioTester_(0), @/
    linSolver_(0), @/
    orig2col_(0), @/
    col2orig_(0), @/
    orig2row_(0), @/
    sense_(0), @/
    primal_(0), @/
    slacks_(0), @/
    dual_(0), @/
    reduCosts_(0), @/
    cStat_(0), @/
    rStat_(0), @/
    maxRow_(0), @/
    maxCol_(0)
  {
    lpMasterSoplex_ = master->lpMasterSoplex();
    setUp();
  }

@ The function |setUp()| performs common initializations of all
  constructors.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::setUp()
  {
    lpSolverTime_.start();
    @<initialize pricer, ratio tester, and equation solver@>;
    @<set up an empty soplex object@>;
    lpSolverTime_.stop();
  }

@ @<initialize pricer, ratio tester, and equation solver@>=
  pricer_      = new SPxSteepPR();
  ratioTester_ = new SPxFastRT();
  linSolver_   = new SLUFactor();

@ Before the first linear programming can be solved we have to load
  a pricer, a ratio tester, and a solver for linear equation systems.

@<set up an empty soplex object@>=
  soplex_ = new SoPlex();

  soplex_->load(pricer_);
  soplex_->load(ratioTester_);
  soplex_->load(linSolver_); 
  if (lpMasterSoplex_->soplexRowRep())
    soplex_->setRep(SoPlex::ROW);
  else
    soplex_->setRep(SoPlex::COLUMN);

@ Soplex treats all values with absolute value greater than the
  |SPxLP::inifinity| as infinity. If the value for infinity
  within this
  framework is smaller than |SPxLP::infinity| we stop with an error message
  since unbounded variables might be treated differently.
  
@q m f Constructor @>
@ A constructor with initialization.
@q m e t @>

@q m b a @>
      \BeginArg
      \docarg{|master|}{A pointer to the corresponding master of the optimization.}
      \docarg{|sense|}{The sense of the objective function.}
      \docarg{|nCol|}{The number of columns (variables).}
      \docarg{|maxCol|}{The maximal number of columns.}
      \docarg{|nRow|}{The number of rows.}
      \docarg{|maxRow|}{The maximal number of rows.}
      \docarg{|obj|}{An array with the objective function coefficients.}
      \docarg{|lb|}{An array with the lower bounds of the columns.}
      \docarg{|ub|}{An array with the upper bounds of the columns.}
      \docarg{|rows|}{An array storing the rows of the problem.}
      \EndArg
@q m e a @>

@<the implementation of the members of |ABA_SOPLEXIF|@>=
@q m b f @>
  ABA_SOPLEXIF::ABA_SOPLEXIF(ABA_MASTER *master, 
                             ABA_OPTSENSE sense, 
                             int nRow, 
                             int maxRow, 
                             int nCol, 
                             int maxCol,
                             ABA_ARRAY<double> &obj, 
                             ABA_ARRAY<double> &lb, 
                             ABA_ARRAY<double> &ub,
                             ABA_ARRAY<ABA_ROW*> &rows) 
@q m e f @>
: @/
    ABA_LP (master), @/
    soplex_(0), @/
    pricer_(0), @/
    ratioTester_(0), @/
    linSolver_(0), @/
    orig2col_(0), @/
    col2orig_(0), @/
    orig2row_(0), @/
    sense_(0), @/
    primal_(0), @/
    slacks_(0), @/
    dual_(0), @/
    reduCosts_(0), @/
    cStat_(0), @/
    rStat_(0), @/
    maxRow_(maxRow), @/
    maxCol_(maxCol)
  {
    lpMasterSoplex_ = master->lpMasterSoplex();
    setUp();
    _initialize(sense, nRow, maxRow, nCol, maxCol, obj, lb, ub, rows);
  }


@q m v Destructor @>
@q m e t @>
@ The destructor.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
@q m b f @>
  ABA_SOPLEXIF::~ABA_SOPLEXIF()
@q m e f @>
  {
    lpSolverTime_.start();

    delete soplex_;
    delete pricer_;
    delete ratioTester_;
    delete linSolver_;

    lpSolverTime_.stop();

    delete orig2col_;
    delete col2orig_;
    delete orig2row_;
    delete sense_;
    delete primal_;
    delete slacks_;
    delete dual_;
    delete reduCosts_;
    delete [] cStat_;
    delete [] rStat_;
  }

@q m f Output Operator @>
@ The output operator writes the linear program to an output stream
  in the format of the LP-solver Soplex.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The linear program being output.}
  \EndArg
@q m e a @>

@<the implementation of the members of |ABA_SOPLEXIF|@>=
@q m b f @>
  ostream &operator<<(ostream& out, const ABA_SOPLEXIF& rhs)
@q m e f @>
  {
    return out << *(rhs.soplex_);
  }

@ The function |_initialize()| implements the corresponding pure virtual
  function of the base class |LP| and loads the linear program
  defined by the following arguments to Soplex.
      \BeginArg
      \docarg{|sense|}{The sense of the objective function.}
      \docarg{|nCol|}{The number of columns (variables).}
      \docarg{|maxCol|}{The maximal number of columns.}
      \docarg{|nRow|}{The number of rows.}
      \docarg{|maxRow|}{The maximal number of rows.}
      \docarg{|obj|}{An array with the objective function coefficients.}
      \docarg{|lb|}{An array with the lower bounds of the columns.}
      \docarg{|ub|}{An array with the upper bounds of the columns.}
      \docarg{|rows|}{An array storing the rows of the problem.}
      \EndArg

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_initialize(ABA_OPTSENSE sense, int nRow, int maxRow,
                          int nCol, int maxCol,
                          ABA_ARRAY<double> &obj, ABA_ARRAY<double> &lBound,
                          ABA_ARRAY<double> &uBound, ABA_ARRAY<ABA_ROW*> &rows)
  {
    @<initialize the optimization sense@>;
    @<initialize the columns@>;
    @<initialize the rows@>;
    @<allocate the result members@>;
  }


@ @<initialize the optimization sense@>=
  soplex_->changeSense(optSense2soplex(sense));

@ The variable |column| is only a dummy.
@<initialize the columns@>=
  maxCol_ = maxCol;

  LPColSet colSet (nCol);
  DSVector column(0);

  double l;
  double u;

  for(int i = 0; i < nCol; i++) {
    l = lBound[i];
    u = uBound[i];

    if (master_->isInfinity(l))           l = LPSolver::infinity;
    else if (master_->isMinusInfinity(l)) l = -LPSolver::infinity;

    if (master_->isInfinity(u))           u = LPSolver::infinity;
    else if (master_->isMinusInfinity(u)) u = -LPSolver::infinity;

    colSet.add(obj[i], l, column, u);
  }
  
  lpSolverTime_.start();
  soplex_->addCols(colSet);
  lpSolverTime_.stop();

  orig2col_ = new ABA_ARRAY<int>(master_, maxCol);
  col2orig_ = new ABA_ARRAY<int>(master_, maxCol);
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nCol; i++) {
#else
  for (int i = 0; i < nCol; i++) {
#endif
    (*orig2col_)[i] = i;
    (*col2orig_)[i] = i;
  }

@ @<initialize the rows@>=
  @<count the number of nonzeros in the |rows|@>;
  maxRow_ = maxRow;

  LPRowSet rowSet(nRow, nnz);
  DSVector vector(nCol);

  sense_ = new ABA_ARRAY<ABA_CSENSE::SENSE>(master_, maxRow);

  for(int c = 0; c < nRow; c++) {
    @<initialize a single row@>;
  }
 
  lpSolverTime_.start();
  soplex_->addRows(rowSet);
  lpSolverTime_.stop();

  orig2row_ = new ABA_ARRAY<int>(master_, maxRow);

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRow; i++)
#else
  for (int i = 0; i < nRow; i++)
#endif
    (*orig2row_)[i] = i;

@ @<count the number of nonzeros in the |rows|@>=
  int nnz = 0;

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRow; i++)
#else
  for (int i = 0; i < nRow; i++)
#endif
    nnz += rows[i]->nnz();

@ Soplex does not work with LP rows in our format ($\le$, $=$, $\ge$)
  but requires range constraints of the form $l \le a^Tx \le r$.
  While there is a constructor of the class |LPRow| supporting also
  our format, we do not use it because we have to access the right
  hand side of the constraints later. Therefore, we must be sure in
  which way a constraint of our format is converted to a range
  constraint.

@<initialize a single row@>=
  vector.clear();

  const int rNnz = rows[c]->nnz();

  for (int i = 0; i < rNnz; i++)
    vector.add(rows[c]->support(i), rows[c]->coeff(i));    

  LPRow *soplexRow;
  double r = rows[c]->rhs();
  if (master_->isInfinity(r))           r =  LPSolver::infinity;
  else if (master_->isMinusInfinity(r)) r = -LPSolver::infinity;

  (*sense_)[c] = rows[c]->sense()->sense();

  switch (rows[c]->sense()->sense()) {
  case ABA_CSENSE::Less:
      soplexRow = new LPRow(-LPRow::infinity, vector, r);
      break;
  case ABA_CSENSE::Equal:
      soplexRow = new LPRow(r, vector, r);
      break;
  case ABA_CSENSE::Greater:
      soplexRow = new LPRow(r, vector, LPRow::infinity);
      break;
  }
  rowSet.add(*soplexRow);
  delete soplexRow;

@ @<allocate the result members@>=
  primal_ = new DVector(maxCol);
  slacks_ = new DVector(maxRow);
  dual_   = new DVector(maxRow);
  reduCosts_ = new DVector(maxCol);

  cStat_ = new signed char[maxCol];
  rStat_ = new signed char[maxRow];


@ The function |_loadBasis()| loads a basis for Soplex.
  \BeginArg
  \docarg{|lpVarStat|}{An array storing the status of the columns.}
  \docarg{|lpVarStat|}{An array storing the status of the slack variables.}
  \EndArg

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_loadBasis(ABA_ARRAY<ABA_LPVARSTAT::STATUS> &lpVarStat,
                           ABA_ARRAY<ABA_SLACKSTAT::STATUS> &slackStat)
  {
    return;
    for (int i = 0; i < nCol(); i++)
      cStat_[i] = lpVarStat2soplex(lpVarStat[i]);

#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < nRow(); i++)
#else
    for (int i = 0; i < nRow(); i++)
#endif
      rStat_[i] = slackStat2soplex(slackStat[i]);

    lpSolverTime_.start();
    soplex_->setBasis(rStat_, cStat_);
    lpSolverTime_.stop();
  }

@ The function |_getInfeas()| is currently not provided by Soplex.

  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_getInfeas(int &infeasRow, int &infeasCol, double *bInvRow)
  {
    master_->err() << "ABA_SOPLEXIF::_getInfeas(): not available" << endl;
    exit(Fatal);
    return 0;  // to suppress compiler warning
  }

@ The function |_remRows()| removes the rows listed in |ind|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_remRows(ABA_BUFFER<int> &ind)
  {
    @<map the removed rows to Soplex indices@>;
    @<remove the rows in Soplex@>;
    @<update the mapping of the rows@>;
  }
  
@ Soplex removes rows that have a negative value in |perm|.
@<map the removed rows to Soplex indices@>=
  int nRow = _nRow();
  int *perm = new int[nRow];
  for (int i = 0; i < nRow; i++)
    perm[i] = 0;

  const int nInd = ind.number();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nInd; i++)
#else
  for (int i = 0; i < nInd; i++)
#endif
    perm[ind[i]] = -1;

@ @<remove the rows in Soplex@>=
  lpSolverTime_.start();
  soplex_->removeRows(perm);
  lpSolverTime_.stop();

@ @<update the mapping of the rows@>=
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRow; i++)
#else
  for (int i = 0; i < nRow; i++)
#endif
    if (perm[i] >= 0)
      (*orig2row_)[i] = perm[i];  

  orig2row_->leftShift(ind);
  sense_->leftShift(ind);

  delete [] perm;

  
@ The function |_addRows()| adds the |rows| to the linear program.
  It implements the pure virtual function of the base class |LP|.

  If more rows are added than the current maximal number of rows we
  enlarge the maximal number of rows to the new number of rows plus
  a ten percent reserve.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_addRows(ABA_BUFFER<ABA_ROW*> &rows)
  {
    @<has Soplex enough memory for adding rows@>;
    @<count number of nonzeros in new rows@>;
    @<fill up the data structure for adding rows in SOPLEX@>;
    @<add the new rows to Soplex@>;
    @<update the row ids@>;
  }
  
@ @<has Soplex enough memory for adding rows@>=
  if (_nRow() + rows.number() >= maxRow_)
    _rowRealloc((int) ((_nRow() + rows.number())*1.1 + 1));

@ @<count number of nonzeros in new rows@>=
    const int nRows = rows.number();
    int nnz = 0;
    for(int r = 0; r < nRows; r++) nnz += rows[r]->nnz();



@ @<fill up the data structure for adding rows in SOPLEX@>=
  int oldNRow = nRow();

  LPRowSet rowSet(nRows, nnz);
  DSVector vector(nCol());

  for(int c = 0; c < nRows; c++) {
    vector.clear();

    const int rNnz = rows[c]->nnz();

    for (int i = 0; i < rNnz; i++) {
      const int orig = (*orig2col_)[i];
      vector.add(rows[c]->support(orig), 
                 rows[c]->coeff(orig));    
    }

    LPRow *soplexRow;
    
    double r = rows[c]->rhs();
    if (master_->isInfinity(r))           r = LPSolver::infinity;
    else if (master_->isMinusInfinity(r)) r = -LPSolver::infinity;

    (*sense_)[c] = rows[c]->sense()->sense();

    switch (rows[c]->sense()->sense()) {
    case ABA_CSENSE::Less:
        soplexRow = new LPRow(-LPRow::infinity, vector, r);
        break;
    case ABA_CSENSE::Equal:
        soplexRow = new LPRow(r, vector, r);
        break;
    case ABA_CSENSE::Greater:
        soplexRow = new LPRow(r, vector, LPRow::infinity);
        break;
    }

    rowSet.add(*soplexRow);

    delete soplexRow;
  }
 
@ @<add the new rows to Soplex@>=
  lpSolverTime_.start();
  soplex_->addRows(rowSet);
  lpSolverTime_.stop();

@ @<update the row ids@>=
  for (int i = 0; i < nRows; i++) {
    (*orig2row_)[oldNRow + i] = oldNRow + i;
    (*sense_)[oldNRow + i]    = rows[i]->sense()->sense();
  }

@ The function |_remCols()| removes the columns listed in |vars|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_remCols(ABA_BUFFER<int> &vars)
  {
    @<map the removed colums to Soplex indices@>;
    @<remove the columns in Soplex@>;
    @<update the mapping of the columns@>;
    @<delete allocated memory (|ABA_SOPLEXIF::_remCols()|)@>;
  }
  
@ Soplex removes variables that have a negative value in |perm|.

@<map the removed colums to Soplex indices@>=
  int nCol = _nCol();
  int *perm = new int[nCol];
  for (int i = 0; i < nCol; i++)
    perm[i] = 0;

  const int nVars = vars.number();

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVars; i++)
#else
  for (int i = 0; i < nVars; i++)
#endif
    perm[vars[i]] = -1;

@ @<remove the columns in Soplex@>=
  lpSolverTime_.start();
  soplex_->removeCols(perm);
  lpSolverTime_.stop();

@ @<update the mapping of the columns@>=
  int p;
  int newNCol = 0;
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nCol; i++) {
#else
  for (int i = 0; i < nCol; i++) {
#endif
    p = perm[i];
    if (p >= 0) {
      (*orig2col_)[newNCol] = p;
      (*col2orig_)[p]       = newNCol;
      newNCol++;
    }
  }
  
@ @<delete allocated memory (|ABA_SOPLEXIF::_remCols()|)@>=
  delete [] perm;

@ The function |_addCols()| adds the columns |newCols| to the linear program.
  It implements the pure virtual function of the base class |LP|.

  If more columns are added than the current maximal number of columns we
  enlarge the maximal number of columns to the new number of columns plus
  a ten percent reserve.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_addCols(ABA_BUFFER<ABA_COLUMN*> &newCols)
  {
    @<has Soplex enough memory for adding columns@>;
    @<count number of nonzeros in new cols@>;
    @<fill up the data structure for adding cols in SOPLEX@>;
    @<add the new cols to Soplex;@>;
    @<update the col ids@>;
  }
    
@ @<has Soplex enough memory for adding columns@>=
  if (_nCol() + newCols.number() >= maxCol_)
    _colRealloc((int) ((_nCol() + newCols.number())*1.1 + 1));

@ @<count number of nonzeros in new cols@>=
    int nnz = 0;
    const int nNewCols = newCols.number();
    for(int r = 0; r < nNewCols; r++) nnz += newCols[r]->nnz();

@ @<fill up the data structure for adding cols in SOPLEX@>=
  int oldNCol = nCol();

  LPColSet colSet(nNewCols, nnz);
  DSVector vector(nRow());
  double l;
  double u;

  for(int c = 0; c < nNewCols; c++) {
    vector.clear();
    const int cNnz = newCols[c]->nnz();
    for (int i = 0; i < cNnz; i++) {
      const int orig = (*orig2row_)[i];
      vector.add(newCols[c]->support(orig), 
                 newCols[c]->coeff(orig));
    }

    l = newCols[c]->lBound();
    if (master_->isInfinity(l))           l = LPSolver::infinity;
    else if (master_->isMinusInfinity(l)) l = -LPSolver::infinity;

    u = newCols[c]->uBound();
    if (master_->isInfinity(u))           u = LPSolver::infinity;
    else if (master_->isMinusInfinity(u)) u = -LPSolver::infinity;

    LPCol soplexCol(newCols[c]->obj(), vector, u, l);

    colSet.add(soplexCol);
  }
 
@ @<add the new cols to Soplex;@>=

  lpSolverTime_.start();
  soplex_->addCols(colSet);
  lpSolverTime_.stop();

@ @<update the col ids@>=
  int newNCol = oldNCol + nNewCols;

  for (int i = oldNCol; i < newNCol; i++) {
    (*orig2col_)[i] = i;
    (*col2orig_)[i] = i;
  }
  

@ The function |changeRhs()| sets the right hand side of the linear
  program to |newRhs|. This array must be have at least length of the
  number of rows.
  This function implements the pure virtual function of the base class
  |LP|.

  Since Soplex works with range constraints we have to modify both the
  left hand side and the right hand side.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_changeRhs(ABA_ARRAY<double> &newRhs)
  {
    int nRow = _nRow();
    DVector soplexRhs(nRow);
    DVector soplexLhs(nRow);

    int index;
    double r;

    for (int i = 0; i < nRow; i++) {
      index = (*orig2row_)[i];
      r = newRhs[i];

      if (master_->isInfinity(r))           r = LPSolver::infinity;
      else if (master_->isMinusInfinity(r)) r = -LPSolver::infinity;
      
      switch ((*sense_)[i]) {
      case ABA_CSENSE::Less:
        soplexLhs[index] = LPRow::infinity;
        soplexRhs[index] = r;
        break;
      case ABA_CSENSE::Equal:
        soplexLhs[index] = r;
        soplexRhs[index] = r;
        break;
      case ABA_CSENSE::Greater:
        soplexLhs[index] = r;
        soplexRhs[index] = LPRow::infinity;
        break;
      }
    }
  
    lpSolverTime_.start();
    soplex_->changeRange(soplexLhs, soplexRhs);
    lpSolverTime_.stop();
  }
      
@ The function |_changeLBound()| sets the lower bound of column |i| to
  |newLb|.
  It implements the pure virtual function of the base class |LP|.
                      
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_changeLBound(int i, double newLb)
  {
    if (master_->isInfinity(newLb))           newLb = LPSolver::infinity;
    else if (master_->isMinusInfinity(newLb)) newLb = -LPSolver::infinity;

    lpSolverTime_.start();
    soplex_->changeLower((*orig2col_)[i], newLb);
    lpSolverTime_.stop();
  }

@ The function |_changeUBound()| sets the upper bound of column |i| to
  |newLb|.
  It implements the pure virtual function of the base class |LP|.
                      
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_changeUBound(int i, double newUb)
  {
    if (master_->isInfinity(newUb))           newUb = LPSolver::infinity;
    else if (master_->isMinusInfinity(newUb)) newUb = -LPSolver::infinity;

    lpSolverTime_.start();
    soplex_->changeUpper((*orig2col_)[i], newUb);
    lpSolverTime_.stop();
  }

@ The function |_pivotSlackVariableIn()| pivots the slack variables
  stored in the buffer |rows| into the basis. This function defines
  the pure virtual function of the base class |LP|.

  \BeginReturn
  \docarg{|0|}{All variables could be pivoted in,}
  \docarg{|1|}{otherwise.}
  \EndReturn

  \BeginArg
  \docarg{|rows|}{The numbers of the slack variables that should be
                  pivoted in.}
  \EndArg

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_pivotSlackVariableIn(ABA_BUFFER<int> &rows)
  {
    master_->err() << "WARNING: ABA_SOPLEXIF::_pivotSlackVariableIn(): function ";
    master_->err() << " not available in Soplex" << endl;
    return 1;
  }
  
@ The function |_primalSimplex()| calls the primal simplex method.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_LP::OPTSTAT ABA_SOPLEXIF::_primalSimplex()
  {
    @<apply the primal simplex method of Soplex@>;
    @<get the solution of the primal simplex method of Soplex@>;
  }

@ @<apply the primal simplex method of Soplex@>=
  lpSolverTime_.start();
  if (soplex_->rep() == SoPlex::ROW)
    soplex_->setType(SoPlex::LEAVE);
  else
    soplex_->setType(SoPlex::ENTER);
  LPSolver::Status status = soplex_->solve();
  lpSolverTime_.stop();

@ @<get the solution of the primal simplex method of Soplex@>=
  getSol();

  switch (status) {
    case LPSolver::SOLVED:     return Optimal;
    case LPSolver::INFEASIBLE: return Infeasible;
    case LPSolver::UNBOUNDED:  return Unbounded;
    case LPSolver::PRIMAL:     return Feasible;
    default: master_->err() << "ABA_SOPLEXIF::_primalSimplex(): status is ";
#ifdef ABACUS_COMPILER_VISUAL_CPP
             master_->err() << (int) status << endl;
#else
             master_->err() << status << endl;
#endif
             return Error;
  }

@ The function |_dualSimplex()| calls the dual simplex method.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_LP::OPTSTAT ABA_SOPLEXIF::_dualSimplex()
  {
    @<apply the dual simplex method of Soplex@>;
    @<get the solution of the dual simplex method of Soplex@>;
  }
  
@ @<apply the dual simplex method of Soplex@>=
  lpSolverTime_.start();
  if (soplex_->rep() == SoPlex::COLUMN)
    soplex_->setType(SoPlex::LEAVE);
  else
    soplex_->setType(SoPlex::ENTER);
  LPSolver::Status status = soplex_->solve();
  lpSolverTime_.stop();

@ The statuses |PRIMAL| and |DUAL| of Soplex mean that the solution of
  Soplex is primal or dual feasible, respectively, but not yet
  optimal. In these two cases we also return the status Optimal,
  because this case usually occurs if we limited the number of
  iterations of the simplex algorithm. 

  We should think about introducing equivalent statuses to |PRIMAL|
  and |DUAL| also in \ABACUS.

@<get the solution of the dual simplex method of Soplex@>=
    getSol();

    switch (status) {
      case LPSolver::SOLVED:     return Optimal;
      case LPSolver::INFEASIBLE: return Infeasible;
      case LPSolver::UNBOUNDED:  return Unbounded;
      case LPSolver::PRIMAL:     return Optimal;
      case LPSolver::DUAL:       return Optimal;
      default: master_->err() << "ABA_SOPLEXIF::_dualSimplex(): status is ";
#ifdef ABACUS_COMPILER_VISUAL_CPP
               master_->err() << (int) status << endl;
#else
               master_->err() << status << endl;
#endif
               return Error;
    }

@ The function |_barrier()| calls the barrier method.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_LP::OPTSTAT ABA_SOPLEXIF::_barrier(bool /*doCrossover*/)
  {
    master_->err() << "WARNING: Soplex does NOT provide a barrier method.";
    master_->err() << endl;
    return _primalSimplex();
  }

@q m f soplex @>
@ The function |soplex()| should be used carefully
  because it breaks the class principle. 
  In particular, it should not be used to to modify the internal data
  structures of Soplex.
@q m e t @>
 However, we require this
 direct access for the separators in the mixed integer optimizer.

@q m b r @>
  \BeginReturn
  A pointer to the internal problem
  representation of Soplex. 
  \EndReturn
@q m e r @>

@<the implementation of the members of |ABA_SOPLEXIF|@>=
@q m b f @>
   SoPlex *ABA_SOPLEXIF::soplex()
@q m e f @>
   {
     return soplex_;
   }

@ The function |_sense()| returns the sense of the optimization.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_OPTSENSE ABA_SOPLEXIF::_sense() const
  {
    ABA_OPTSENSE s;

    if (soplex_->sense() == LPSolver::MINIMIZE)
      s.sense(ABA_OPTSENSE::Min);
    else
      s.sense(ABA_OPTSENSE::Max);

    return s;
  }

@ The function |_sense()| changes the optimization sense.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_sense(const ABA_OPTSENSE &newSense)
  {
    soplex_->changeSense(optSense2soplex(newSense));
  }


@ The function |_nRow()| returns the number of rows of the linear program.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_nRow() const
  {
    return soplex_->nofRows();
  }

@ The function |_maxRow()| is required although Soplex internally
  works without any row limit because some data members of our
  interface have a fixed size.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_maxRow() const
  {
     return maxRow_;
  }

@ The function |_nCol()| returns the number of columns of the linear
  program.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_nCol() const
  {
    return soplex_->nofCols();
  }

@ The function |_maxCol()| is required although Soplex internally
  works without any row limit because some data members of our
  interface have a fixed size.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_maxCol() const
  {
     return maxCol_;
  }




@ The function |_obj()| returns the objective function coefficient of
  column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_obj(int i) const
  {
     return soplex_->obj((*orig2col_)[i]);
  }

@ The function |_lBound()| returns the lower bound of column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_lBound(int i) const
  {
    return soplex_->lower((*orig2col_)[i]);
  }

@ The function |_uBound()| returns the upper bound of column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_uBound(int i) const
  {
     return soplex_->upper((*orig2col_)[i]);
  }

@ The function |_rhs()| returns the right hand side of row |i|.
  It implements the pure virtual function of the base class |LP|.

  Soplex uses range constraints. Therefore our right hand side of
  $\ge$ constraints is the left hand side of the internal
  representation of Soplex.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_rhs(int i) const
  {
    if ((*sense_)[i] == ABA_CSENSE::Greater) 
      return soplex_->lhs((*orig2row_)[i]);
    else
      return soplex_->rhs((*orig2row_)[i]);
  }

@ The function |_row()| stores a copy of row |i| in |r|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_row(int i, ABA_ROW &r) const
  {
    @<get row |i| from Soplex@>;
    @<insert the data into |r|@>;
  }

@ @<get row |i| from Soplex@>=
  LPRow soplexRow(_nCol());

  soplex_->getRow((*orig2row_)[i], soplexRow);

@ @<insert the data into |r|@>=
  SVector *vec = &(soplexRow.rowVector());

  r.clear();

  const int s = vec->size();
    
  for (int j = 0; j < s; j++)
    r.insert((*col2orig_)[vec->index(j)], vec->value(j));

  r.sense(soplex2csense(soplexRow.type()));  
  r.rhs(soplexRow.value());

@ The function |_nnz()| returns the number of nonzero elements in the
  constraint matrix (not including the right hand side).
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_nnz() const
  {
    return soplex_->nofNZEs();
  }

@ The function |_value()| returns the optimum value of the linear program.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_value() const
  {
    return value_;
  }

@ The function |_xVal()| returns the value of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_xVal(int i)
  {
    return (*primal_)[(*orig2col_)[i]];
  }

@ The function |_barXVal()| returns the value of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_barXVal(int i)
  {
    master_->err() << "SOPLEX does not provide a barrier method." << endl;
    exit(Fatal);
    return 0.0; // to suppress compiler warning
  }

@ The function |_reco()| returns the reduced cost of the column |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_reco(int i)
  {
    return (*reduCosts_)[(*orig2col_)[i]];
  }

@ The function |_yVal()| returns the value of the dual column of the
  row |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_yVal(int i)
  {
    return (*dual_)[(*orig2row_)[i]];
  }

@ The function |_slack()| returns the value of the slack column of the
  row  |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  double ABA_SOPLEXIF::_slack(int i)
  {

    int index = (*orig2row_)[i];
    double r;

    switch ((*sense_)[i]) {
    case ABA_CSENSE::Less:
      r = soplex_->rhs(index) - (*slacks_)[index];
      break;
    case ABA_CSENSE::Equal:
      r =  0.0;
      break;
    case ABA_CSENSE::Greater:
      r = soplex_->lhs(index) -  (*slacks_)[index];
      break;
    }
    return r;
  }

@ The function |_lpVarStat()| returns the status of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_LPVARSTAT::STATUS ABA_SOPLEXIF::_lpVarStat(int i)
  {
    return soplex2lpVarStat((LPSolver::VarStatus) cStat_[(*orig2col_)[i]]);
  }

@ The function |_slackStat()| returns the status of the slack column |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_SLACKSTAT::STATUS ABA_SOPLEXIF::_slackStat(int i)
  {
    return soplex2slackStat((LPSolver::VarStatus) rStat_[(*orig2row_)[i]]);
  }

@ The function |csense2soplex()| returns the |sense| of a row
  in the internal representation of Soplex associated with |sense|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  LPRow::Type ABA_SOPLEXIF::csense2soplex(ABA_CSENSE *sense) const
  {
    switch(sense->sense()) {
    case ABA_CSENSE::Less:    return LPRow::LESS_EQUAL;
    case ABA_CSENSE::Equal:   return LPRow::EQUAL;
    case ABA_CSENSE::Greater: return LPRow::GREATER_EQUAL;
    default: master_->err() << "ABA_SOPLEXIF::csense2soplex(" << *sense << "):";
        master_->err() << "unknown sense" << endl;
        exit(Fatal);
        return LPRow::LESS_EQUAL;  // to suppress compiler warnings
    }
  }

@ The function |soplex2csense()| converts the Soplex representation of the
  row sense to the \ABACUS\ representation.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_CSENSE::SENSE ABA_SOPLEXIF::soplex2csense(LPRow::Type sense) const
  {
    switch(sense) {
      case LPRow::LESS_EQUAL:    return ABA_CSENSE::Less;
      case LPRow::EQUAL:         return ABA_CSENSE::Equal;
      case LPRow::GREATER_EQUAL: return ABA_CSENSE::Greater;
#ifdef ABACUS_COMPILER_VISUAL_CPP
      default: master_->err() << "ABA_SOPLEXIF::soplex2csense(" << (int) sense << ")" << endl;
#else
      default: master_->err() << "ABA_SOPLEXIF::soplex2csense(" << sense << ")" << endl;
#endif
               master_->err() << "unknown sense" << endl;
               exit(Fatal);
               return ABA_CSENSE::Less; // to suppress compiler warning
    }
  }

@ The function |getSol()| extracts the solution, i.e., the value,
  the status, the values of the variables, slack variables, and dual
  variables, the reduced costs, and the statuses of the variables
  and slack variables form the internal Soplex data structure.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::getSol()
  {
    @<get the solution of the LP@>;
    @<get the basis status@>;
  }

@ Since Soplex makes an error (violates an array bound) when the
  result vectors |primal_|, |slacks_|, |dual_|, or |reduCosts_| are
  longer than the number of rows or number of columns of the linear
  program, respectively, we have to use temporary arrays to extract
  the results from Soplex.

@<get the solution of the LP@>=
  int soplexNCols = soplex_->nofCols();
  int soplexNRows = soplex_->nofRows();

  DVector tmpPrimal(soplexNCols);
  DVector tmpSlacks(soplexNRows);
  DVector tmpDual(soplexNRows);
  DVector tmpReduCosts(soplexNCols);

  lpSolverTime_.start();

  LPSolver::Status status = soplex_->getResult(&value_, &tmpPrimal, &tmpSlacks,
                                                &tmpDual, &tmpReduCosts);

  lpSolverTime_.stop();

  for (int i = 0; i < soplexNCols; i++) {
    (*primal_)[i] = tmpPrimal[i];
    (*reduCosts_)[i] = tmpReduCosts[i];
  }

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < soplexNRows; i++) {
#else
  for (int i = 0; i < soplexNRows; i++) {
#endif
    (*dual_)[i] = tmpDual[i];
    (*slacks_)[i] = tmpSlacks[i];
  }

  xValStatus_ = recoStatus_= yValStatus_ = slackStatus_ = Available;

@ @<get the basis status@>=
  lpSolverTime_.start();
  status =  soplex_->getBasis(rStat_, cStat_);
  lpSolverTime_.stop();

  if (status != LPSolver::SOLVED)
    basisStatus_ = Missing;
  else
    basisStatus_ = Available;

@ The function |optSense2soplex()| returns the |sense| of the objective
  function in the internal representation of Soplex associated with
  the linear program.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  LPSolver::Sense ABA_SOPLEXIF::optSense2soplex(ABA_OPTSENSE sense) const
  {
    if(sense.min()) return LPSolver::MINIMIZE;
    else            return LPSolver::MAXIMIZE;
  }
  

@ The function |lpVarStat2soplex()| returns the variable status
  of Soplex associated with |stat|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  signed char ABA_SOPLEXIF::lpVarStat2soplex(ABA_LPVARSTAT::STATUS stat) const
  {
    LPSolver::VarStatus soplexStat;

    switch (stat) {
      case ABA_LPVARSTAT::AtLowerBound: 
          soplexStat = LPSolver::ON_LOWER;
          break;
      case ABA_LPVARSTAT::Basic:        
          soplexStat = LPSolver::BASIC;
          break;
      case ABA_LPVARSTAT::AtUpperBound: 
          soplexStat = LPSolver::ON_UPPER;
          break;
      case ABA_LPVARSTAT::Unknown:      
          soplexStat = LPSolver::ON_LOWER;
          break;
      default: 
#ifdef ABACUS_COMPILER_VISUAL_CPP
          master_->err() << "ABA_SOPLEXIF::lpVarStat2soplex(" << (int) stat << "): ";
#else
          master_->err() << "ABA_SOPLEXIF::lpVarStat2soplex(" << stat << "): ";
#endif
          master_->err() << "unknown status" << endl;
          exit(Fatal);
    }
    return (signed char) soplexStat;
  }
  
@ The function |soplex2lpVarStat()| returns the variable status
  of ABACUS associated with the internal Soplex variable status |stat|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_LPVARSTAT::STATUS ABA_SOPLEXIF::soplex2lpVarStat(signed char stat) const
  {
    LPSolver::VarStatus soplexStatus = (LPSolver::VarStatus) stat;

    switch (soplexStatus) {
    case LPSolver::ON_LOWER: return ABA_LPVARSTAT::AtLowerBound;
    case LPSolver::BASIC:    return ABA_LPVARSTAT::Basic;
    case LPSolver::FIXED:    return ABA_LPVARSTAT::AtUpperBound;
    case LPSolver::ZERO:     return ABA_LPVARSTAT::AtUpperBound;
    case LPSolver::ON_UPPER: return ABA_LPVARSTAT::AtUpperBound;
    default: 
#ifdef ABACUS_COMPILER_VISUAL_CPP
        master_->err() << "ABA_SOPLEXIF::soplex2lpVarStat(" << (int) soplexStatus << "): ";
#else
        master_->err() << "ABA_SOPLEXIF::soplex2lpVarStat(" << soplexStatus << "): ";
#endif
        master_->err() << "unknown status" << endl;
        exit(Fatal);
        return ABA_LPVARSTAT::AtLowerBound; // to suppress compiler warning
    }
  }

@ The function |slackStat2Soplex()| returns the slack variable status of
  Soplex associated with |stat|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  signed char ABA_SOPLEXIF::slackStat2soplex(ABA_SLACKSTAT::STATUS stat) const
  {
    LPSolver::VarStatus soplexStatus;

    switch(stat) {
    case ABA_SLACKSTAT::NonBasicZero:    
        soplexStatus = LPSolver::ZERO;
        break;
    case ABA_SLACKSTAT::Basic:
        soplexStatus = LPSolver::BASIC;
        break;
    case ABA_SLACKSTAT::Unknown:
        soplexStatus = LPSolver::ZERO;
        break;
    default:
#ifdef ABACUS_COMPILER_VISUAL_CPP
        master_->err() << "ABA_SOPLEXIF::slackStat2soplex(" << (int) stat << "): ";
#else
        master_->err() << "ABA_SOPLEXIF::slackStat2soplex(" << stat << "): ";
#endif
        master_->err() << " corresponding SOPLEX status unknown" << endl;
        exit (Fatal);
    }
    return (signed char) soplexStatus;
  }

@ The function |soplex2slackStat()| returns the slack variable status
  associated with the internal Soplex slack variable status |stat|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  ABA_SLACKSTAT::STATUS ABA_SOPLEXIF::soplex2slackStat(signed char stat) const
  {
    LPSolver::VarStatus soplexStatus = (LPSolver::VarStatus) stat;

    switch (soplexStatus) {
    case LPSolver::ZERO:     return ABA_SLACKSTAT::NonBasicZero;
    case LPSolver::FIXED:    return ABA_SLACKSTAT::NonBasicNonZero;
    case LPSolver::ON_LOWER: return ABA_SLACKSTAT::NonBasicZero;
    case LPSolver::ON_UPPER: return ABA_SLACKSTAT::NonBasicZero;
    case LPSolver::BASIC:    return ABA_SLACKSTAT::Basic;
#ifdef ABACUS_COMPILER_VISUAL_CPP
    default: master_->err() << "ABA_SOPLEXIF::soplex2slackStat(" << (int) soplexStatus << "): ";
#else
    default: master_->err() << "ABA_SOPLEXIF::soplex2slackStat(" << soplexStatus << "): ";
#endif
        master_->err() << "unknown status" << endl;
        exit(Fatal);
        return ABA_SLACKSTAT::NonBasicZero; // to suppress compiler warning
    }
  }

@ The function |_rowRealloc()| reallocates the internal memory such that
  |newSize| rows can be stored.
  It implements the corresponding pure virtual function of the base
  class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_rowRealloc(int newSize)
  {
    @<reallocate |rStat_|@>;
    orig2row_->realloc(newSize);
    sense_->realloc(newSize);
    dual_->reDim(newSize);
    slacks_->reDim(newSize);
    maxRow_ = newSize;
  }

@ @<reallocate |rStat_|@>=
  signed char *newRStat = new signed char[newSize];

  int length = newSize < maxRow_ ? newSize : maxRow_;

  for (int i = 0; i < length; i++)
    newRStat[i] = rStat_[i];

  delete [] rStat_;
  rStat_ = newRStat;
    

@ The function |_colRealloc()| reallocates the internal memory such that
  |newSize| columns can be stored.
  It implements the corresponding pure virtual function of the base
  class |LP|.

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  void ABA_SOPLEXIF::_colRealloc(int newSize)
  {
    @<reallocate |cStat_|@>;
    orig2col_->realloc(newSize);
    col2orig_->realloc(newSize);
    primal_->reDim(newSize);
    reduCosts_->reDim(newSize);
    maxCol_ = newSize;
  }

@ @<reallocate |cStat_|@>=
  signed char *newCStat = new signed char[newSize];

  int length = newSize < maxCol_ ? newSize : maxCol_;

  for (int i = 0; i < length; i++)
    newCStat[i] = cStat_[i];

  delete [] cStat_;
  cStat_ = newCStat;
      

@ The function |_setSimplexIterationLimit()| changes the iteration
  limit of the Simplex algorithm.
  This function defines a pure virtual function of the base class
  |LP|.

  It seems that there are some problems if |SoPlex::setTermination()|
  is used. If the iteration limit is set and a linear program could
  not be solved to optimality, then later reoptimizations of the
  problem are not performed correctly.

  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be set,}
  \docarg{|1|}{otherwise.}
  \EndReturn

  \BeginArg
  \docarg{|limit|}{The new value of the iteration limit.}
   \EndArg

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_setSimplexIterationLimit(int limit)
  {
    return 1;

    soplex_->setTermination(LPSolver::infinity, -1, limit);

    return 0;
  }

@ The function |getSimplexIterationLimit()| defines a pure virtual
 function of the base class |LP|.

  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be get,}
  \docarg{|1|}{otherwise.} 
  \EndReturn

  \BeginArg
  \docarg{|limit|}{Stores the iteration limit if the  return value is 0.}
  \EndArg

@<the implementation of the members of |ABA_SOPLEXIF|@>=
  int ABA_SOPLEXIF::_getSimplexIterationLimit(int &limit)
  {
    double value;
    double time;

    soplex_->getTermination(&value, &time, &limit);

    return 0;
  }

