@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: sparvec.w,v 4.17 2003/08/05 13:55:27 elf Exp $ @>

@q m i ABA_SPARVEC @>

@ The  member functions and friend functions
  are defined in the file {\tt sparvec.cc}.
  
@(sparvec.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "sparvec.h"
#else
#include "abacus/sparvec.h"
#endif

#include <math.h>

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @> 

@ The constructor for an empty sparse vector.  
@q m e t @>
@q m b a @>
  \BeginArg
 \docarg{|glob|}{A pointer to the corresponding global object.}  
 \docarg{|size|}{The maximal number of
                 nonzeros of the sparse vector (without reallocation).}
  \docarg{|reallocFac|} {The reallocation factor (in percent of the
    original size), which is used in a default reallocation if a variable
    is inserted when the sparse vector is already full. Its default value
    is 10.  }  
  \EndArg
@q m e a @>
If no memory for |support_| and |coeff_| is
allocated then an automatic allocation will be performed when the
function |insert()| is called the first time.

@(sparvec.cc@>=
@q m b f @>
  ABA_SPARVEC::ABA_SPARVEC(ABA_GLOBAL *glob, 
                           int size, 
                           double reallocFac) 
@q m e f @>
    : @/
    glob_(glob), @/
    size_(size), @/
    nnz_(0), @/
    reallocFac_(reallocFac)

    {
      if (size) {
        support_ = new int[size];
        coeff_   = new double[size];
      }
      else {
        support_ = 0;
        coeff_   = 0;
      }
    }

@q m f Constructor @>

@ A constructor with initialization of the support and coefficients
  of the sparse vector. The minimum value of |size| and |s.size|
  is the number of nonzeros of the sparse vector.
@q m e t @>
@q m b a @>
      \BeginArg
      \docarg{|glob|}{A pointer to the corresponding global object.}
      \docarg{|size|}{The maximal number of nonzeros (without reallocation).}
      \docarg{|s|}{An array storing the support of the sparse vector, i.e.,
                  the elements for which a (normally nonzero) coefficient
                  is given in |c|.}
     \docarg{|c|}{An array storing the coefficients of the support elements
                  given in |s|. This array must have at least
                  the length of the minimum of |size| and |s.size()|.}
      \docarg{|reallocFac|}{The reallocation factor (in percent of the original
                    size), which is used in a default reallocation if
                    a variable is inserted when the sparse vector is already
                    full. Its default value is 10.
      }
      \EndArg
@q m e a @>
  If |size| is 0, then also no elements are copied in the |for|-loop
  since |nnz_| will be also 0.

@(sparvec.cc@>=
@q m b f @>
  ABA_SPARVEC::ABA_SPARVEC(ABA_GLOBAL *glob, 
                           int size, 
                           const ABA_ARRAY<int> &s,
                           const ABA_ARRAY<double> &c, 
                           double reallocFac) 
@q m e f @>
    : @/
    glob_(glob), @/
    size_(size), @/
    reallocFac_(reallocFac)

  {
    if (size ) {
      support_ = new int[size];
      coeff_   = new double[size];
    }
    else {
      support_ = 0;
      coeff_   = 0;
    }
      
    nnz_ = size < s.size() ? size : s.size();
    
    for(int i = 0; i < nnz_; i++) {
      support_[i] = s[i];
      coeff_[i]   = c[i];
    }

  }

@q m f Constructor @>

@ This constructor is equivalent to the previous one except that it is
  using C-style arrays for the initialization of the sparse vector.

@q m e t @>

@(sparvec.cc@>=
@q m b f @>
  ABA_SPARVEC::ABA_SPARVEC(ABA_GLOBAL *glob, 
                           int nnz, 
                           int *s, 
                           double *c, 
                           double reallocFac)
@q m e f @>
   : @/
    glob_(glob), @/
    size_(nnz), @/
    nnz_(nnz), @/
    reallocFac_(reallocFac)

  {
    if (nnz) {
      support_ = new int[nnz];
      coeff_   = new double[nnz];
    }
    else {
      support_ = 0;
      coeff_   = 0;
    }
      
    for(int i = 0; i < nnz; i++) {
      support_[i] = s[i];
      coeff_[i]   = c[i];
    }

  }

@q m f Copy Constructor @>
@q m e t @>
@ The copy constructor.

@q m b a @>
  \BeginArg
  \docarg{|rhs|}{The sparse vector that is copied.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  ABA_SPARVEC::ABA_SPARVEC(const ABA_SPARVEC& rhs) 
@q m e f @>
    : @/
    glob_(rhs.glob_), @/
    size_(rhs.size_), @/
    nnz_(rhs.nnz_), @/
    reallocFac_(rhs.reallocFac_)

  {
    if (size_) {
      support_ = new int[size_];
      coeff_   = new double[size_];
      
      for(int i = 0; i < nnz_; i++) {
        support_[i] = rhs.support_[i];
        coeff_[i]   = rhs.coeff_[i];
      }
    }
    else {
      support_ = 0;
      coeff_   = 0;
    }
  }

@q m f Destructor @>
@q m e t @>
@ The destructor.

@(sparvec.cc@>=
@q m b f @>
  ABA_SPARVEC::~ABA_SPARVEC()
@q m e f @>
  {
    delete [] support_;
    delete [] coeff_;
  }
  

#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>
@ The message constructor creates the sparse vector from an |ABA_MESSAGE|.

@q m b a @>
    \BeginArg
    \docarg
      {|glob|}{A pointer to the corresponding global object.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  ABA_SPARVEC::ABA_SPARVEC(const ABA_GLOBAL *glob, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    glob_(glob)
  {
    msg.unpack(size_);
    msg.unpack(nnz_);
    msg.unpack(reallocFac_);

    if (size_) {
      support_ = new int[size_];
      coeff_   = new double[size_];

      msg.unpack(support_, nnz_);
      msg.unpack(coeff_, nnz_);
    }
    else {
      support_ = 0;
      coeff_   = 0;
    }
  }


@q m f pack @>
@ The function |pack()| packs the data of the sparse vector
  in an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the sparse vector
                 is packed.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  void ABA_SPARVEC::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    msg.pack(size_);
    msg.pack(nnz_);
    msg.pack(reallocFac_);

    if (size_) {
      msg.pack(support_, nnz_);
      msg.pack(coeff_, nnz_);
    }
  }

#endif


@q m f Assignment Operator @>

@ The assignment operator requires that the left hand and the
  right hand side have the same length (otherwise use the function
  |copy()|). 
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the left hand side.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|rhs|}{The right hand side of the assignment.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  const ABA_SPARVEC& ABA_SPARVEC::operator=(const ABA_SPARVEC& rhs)
@q m e f @>
  {
    if (this == &rhs)
      return *this;
      
    if(size_ != rhs.size_) {
      glob_->err() << "ABA_SPARVEC::operator= : length of operands are different ";
      glob_->err() << "(" << size_ << " != " << rhs.size_ << ")." << endl;
      exit(Fatal);
    }

    for(int i = 0; i < rhs.nnz_; i++) {
      support_[i] = rhs.support_[i];
      coeff_[i]   = rhs.coeff_[i];
    }

    nnz_ = rhs.nnz_;
    glob_ = rhs.glob_;
    return *this;
  }

@q m f Output Operator @>
@ The output operator writes the elements of the support  and
   their coefficients line by line on an output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The sparse vector being output.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
ostream& operator<<(ostream &out, const ABA_SPARVEC &rhs)
@q m e f @>
{
  for (int i = 0; i < rhs.nnz_; i++)
    out << rhs.support_[i] << " " << rhs.coeff_[i] << endl;
  return out;
}

@q m f support @>
@q m e t @>
@ The function |support()|.

  A range check is performed if the function is compiled with
  |-DABACUSSAFE|.

@q m b r @>
 \BeginReturn
 The support of the |i|-th nonzero element.
 \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the nonzero element.}
  \EndArg
@q m e a @>  

@q m b f @>    
  int ABA_SPARVEC::support(int i) const
@q m e f @>

@q m f coeff @>
@ The function |coeff()|.
@q m e t @>

  A range check is performed if the function is compiled with
  |-DABACUSSAFE|.

@q m b r @>
  \BeginReturn
  The coefficient of the |i|-th nonzero element.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the nonzero element.}
  \EndArg
@q m e a @>  

@q m b f @>
  double ABA_SPARVEC::coeff(int i) const
@q m e f @>

@q m f origCoeff @>
@ The function |origCoeff()| 
@q m e t @>

@q m b r @>
  \BeginReturn
  The coefficient having support |i|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the original coefficient.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  double ABA_SPARVEC::origCoeff(int i) const
@q m e f @>
  {
#ifdef TEST_SPARVEC_ORDER
    { int k;
    for (k = 1; k < nnz_; k++)
      if (support_[k-1]>support_[k])
        break;
    if (k==nnz_)
      cout << "ABA_SPARVEC::origCoeff vector is in order.\n";
    else
      cout << "ABA_SPARVEC::origCoeff vector is not in order.\n";
    }
#endif
    for (int k = 0; k < nnz_; k++) 
      if (support_[k] == i) return coeff_[k];
    return 0.0;
  }

@q m f insert @>
@ The function |insert()| adds a new support/coefficient pair
  to the vector. If necessary a reallocation
  of the member data is performed automatically.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|s|}{The new support.}
  \docarg{|c|}{The new coefficient.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_SPARVEC::insert(int s, double c)
@q m e f @>

@q m f leftShift @>
@ The function |leftShift()| deletes the
  elements listed in a buffer from the sparse vector.
  The numbers of indices in this buffer must be upward sorted.
  The elements before the first element in the buffer are unchanged.
  Then the elements
  which are not deleted are shifted left in the arrays.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|del|}{The numbers of the elements removed from the sparse vector.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  void ABA_SPARVEC::leftShift(ABA_BUFFER<int> &del)
@q m e f @>
  {
    const int nDel = del.number();

    if (nDel == 0) return;
    
    int i,j;
    int current = del[0];

    @<shift all elements left of the last deleted element to the left side@>;
    @<shift all elements right of the last deleted element to the left side@>;

    nnz_ -= nDel;
  }

@ All elements in the arrays between the removed elements |del[i]| and
  del[i+1] are shifted left in the inner loop.
  
@<shift all elements left of the last deleted element to the left side@>=
  for (i = 0; i < nDel - 1; i++) {

#ifdef ABACUSSAFE
    if(del[i] < 0 || del[i] >= nnz_) {
      glob_->err() << "ABA_BUFFER:leftShift(): shift index " << i << " not valid." << endl;
      exit(Fatal);
    }
#endif      

    const int last = del[i+1];

    for(j = del[i]+1; j < last; j++) {
      support_[current] = support_[j];
      coeff_[current]   = coeff_[j];
      ++current;
    }
  }
    

@ @<shift all elements right of the last deleted element to the left side@>=
#ifdef ABACUSSAFE
  if(del[nDel -1] < 0 || del[nDel - 1] >= nnz_) {
    glob_->err() << "ABA_BUFFER:leftShift(): shift index " << i << " not valid." << endl;
    exit(Fatal);
  }
#endif      

  for (j = del[nDel - 1] + 1; j < nnz_; j++) {
    support_[current] = support_[j];
    coeff_[current]   = coeff_[j];
    ++current;
  }

@q m f copy @>
@ The function |copy()| is very similar to the assignment operator,
  yet the size of the two vectors need not be equal and only
  the support, the coefficients, and the number of nonzeros is copied.
  A reallocation is performed if required.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|vec|}{The sparse vector that is copied.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  void ABA_SPARVEC::copy(const ABA_SPARVEC &vec)
@q m e f @>
  {
    if(size_ < vec.nnz())
      realloc(vec.nnz());

    nnz_ = vec.nnz();

    for(int i = 0; i < nnz_; i++) {
      support_[i] = vec.support(i);
      coeff_[i]   = vec.coeff(i);
    }
  }

@q m f clear @>
@ The function |clear()| removes all nonzeros from the sparse vector.
@q m e t @>

@q m b f @>
  void ABA_SPARVEC::clear()
@q m e f @>

@q m f rename @>
@ The function |rename()| replaces the index
  of the support by  new names.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newName|}{The new names (support) of the elements of the sparse
                     vector. The array |newName| must have at least a length
                     equal to the
                     maximal element in the support of the sparse vector.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  void ABA_SPARVEC::rename(ABA_ARRAY<int> &newName)
@q m e f @>
  {
    for(int i = 0; i < nnz_; i++)
      support_[i] = newName[support_[i]];
  }

@q m f size @>
@q m e t @>
@ The function |size()|.

@q m b r @>
  \BeginReturn
  The maximal length of the sparse vector.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SPARVEC::size() const
@q m e f @>

@q m f nnz @>
@q m e t @>
@ The function |nnz()|.

@q m b r @>
  \BeginReturn
  The number of nonzero elements.
  This is not necessarily the correct number of nonzeros, yet the
  number of coefficient/support pairs, which are stored.
  Some of these pairs may have a zero coefficient.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SPARVEC::nnz() const
@q m e f @>

@q m f norm @>
@q m e t @>
@ The function |norm()|.

@q m b r @>
  \BeginReturn
  The Euclidean norm of the sparse vector.
  \EndReturn
@q m e r @>

@(sparvec.cc@>=
@q m b f @>
  double ABA_SPARVEC::norm()
@q m e f @>
  {
    double v = 0.0;

    for (int i = 0; i < nnz_; i++)
      v += coeff_[i]*coeff_[i];

    return sqrt(v);
  }

@q m f realloc @>
@ The function |realloc()| increases the size of the sparse vector
  by |reallocFac_| percent of the original size. 
  This function is
  called if an automatic reallocation takes place.
@q m e t @>

@(sparvec.cc@>=
@q m b f @>
  void ABA_SPARVEC::realloc()
@q m e f @>
  {
    realloc((int) ((1.0 + reallocFac_/100.0) * size_) + 1);
  }

@q m f realloc @>
@ This other version of |realloc()|  reallocates the sparse vector to
  a given length. It is an error to decrease
  size below the current number of nonzeros.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newSize|}{The new maximal number of nonzeroes that can be
                     stored in the sparse vector.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  void ABA_SPARVEC::realloc(int newSize)
@q m e f @>
  {
    if (newSize < nnz_) {
      glob_->err() << "ABA_SPARVEC::realloc(" << newSize << "):" << endl;
      glob_->err() << "  length of vector becomes less than number of nonzeros ";
      glob_->err() << nnz_ << "." << endl;
      exit (Fatal);
    }
    
    int    *newSupport = new int[newSize];
    double *newCoeff    = new double[newSize];

    for (int i = 0; i < nnz_; i++) {
      newSupport[i] = support_[i];
      newCoeff[i]   = coeff_[i];
    }

    delete [] support_;
    delete [] coeff_;

    support_ = newSupport;
    coeff_   = newCoeff;
    size_    = newSize;
  }

@q m f rangeCheck @>
@ This function
  terminates the program with an error message if |i| is negative
  or greater or equal than the number of nonzero elements.
@q m e t @>

  If the class |ABA_SPARVEC| is compiled with the flag |-DABACUSSAFE|,
  then before each access operation on element |i| of the sparse
  vector the function |rangeCheck()| is called.

@q m b a @>
  \BeginArg
  \docarg{|i|}{An integer that should be checked if it is in the range of
               the sparse vector.}
  \EndArg
@q m e a @>

@(sparvec.cc@>=
@q m b f @>
  void ABA_SPARVEC::rangeCheck(int i) const
@q m e f @>
  {
    if (i < 0 || i >= nnz_) {
      glob_->err() << "ABA_SPARVEC::rangeCheck(): index " << i;
      glob_->err() << " out of ranges of nonzeros [0," << nnz_-1 << "]." << endl;
      exit(Fatal);
    }
  }

