@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: srowcon.w,v 4.10 2003/08/05 13:55:27 elf Exp $ @>

@q m i ABA_SROWCON @>

@ All member functions are defined in the file {\tt srowcon.cc}.

@(srowcon.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "srowcon.h"
#include "master.h"
#include "numvar.h"
#include "active.h"
#else
#include "abacus/srowcon.h"
#include "abacus/master.h"
#include "abacus/numvar.h"
#include "abacus/active.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.
@q m b a @>
       \BeginArg
       \docarg{|master|}{A pointer to the corresponding master of the optimization.}
       \docarg{|sub|}{A pointer to the subproblem associated with the
                     constraint. This can be also the 0-pointer.}
       \docarg{|sense|}{The sense of the constraint.}
       \docarg{|nnz|}{The number of nonzero elements of the constraint.}
       \docarg{|support|}{The array storing the variables with nonzero
                          coefficients.}
       \docarg{|coeff|}{The nonzero coefficients of the variables stored
                        in |support|.}
       \docarg{|rhs|}{The right hand side of the constraint.}
       \docarg{|dynamic|}{If this argument is |true|, then the constraint
                           can be removed from the active constraint set
                           during the cutting plane phase of the subproblem
                           optimization.}
      \docarg{|local|}{If this argument is |true|, then the constraint
                       is considered to be only locally valid. As a
                       locally valid constraint is associated with
                       a subproblem, |sub| must not be 0 if |local| is |true|.
      }
      \docarg{|liftable|}{If this argument is |true|, then a lifting
                          procedure must be available, i.e., that the
                          coefficients of variables which have not been
                          active at generation time of the constraint
                          can be computed.}
      \EndArg
@q m e a @>

@(srowcon.cc@>=
@q m b f @>
  ABA_SROWCON::ABA_SROWCON(ABA_MASTER *master, 
                           const ABA_SUB *sub, 
                           ABA_CSENSE::SENSE sense, 
                           int nnz, 
                           const ABA_ARRAY<int> &support, 
                           const ABA_ARRAY<double> &coeff,
                           double rhs, 
                           bool dynamic, 
                           bool local,
                           bool liftable) 
@q m e f @>
  : @/
    ABA_ROWCON(master, sub, sense, nnz, support, coeff, rhs, dynamic, local,
           liftable)
  { }


@q m f Constructor @>
@ This constructor is equivalent to the previous constructor
  except that it uses C-style arrays for |support| and |coeff|.
@q m e t @>

@(srowcon.cc@>=
@q m b f @>
  ABA_SROWCON::ABA_SROWCON(ABA_MASTER *master, 
                           const ABA_SUB *sub, 
                           ABA_CSENSE::SENSE sense,
                           int nnz, 
                           int *support, 
                           double *coeff,
                           double rhs, 
                           bool dynamic, 
                           bool local,
                           bool liftable) 
@q m e f @>
  : @/
    ABA_ROWCON(master, sub, sense, nnz, support, coeff, rhs, dynamic, local,
           liftable)
  { }

@q m v Destructor @>
@q m e t @>
@ The destructor.

@(srowcon.cc@>=
@q m b f @>
  ABA_SROWCON::~ABA_SROWCON()
@q m e f @>
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the constraint from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the constraint is initialized.}
  \EndArg
@q m e a @>

@(srowcon.cc@>=
@q m b f @>
  ABA_SROWCON::ABA_SROWCON(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    ABA_ROWCON(master, msg)
  {  }

@q m f pack @>
@ The function |pack()| packs the data of the constraint in an
  |ABA_MESSAGE| object.

  This function needs not to be implemented here because it just calls the
  |pack()| function of the base class. It is provided for the convenience
  of the user only.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the constraint is packed.}
  \EndArg
@q m e a @>

@(srowcon.cc@>=
@q m b f @>
  void ABA_SROWCON::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_ROWCON::pack(msg);
  }

@q m f classId @>
@ The function |classId()| returns an integer which identifies the class.

@q m b r @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e r @>

@(srowcon.cc@>=
@q m b f @>
  int ABA_SROWCON::classId() const
@q m e f @>
  {
    return SROWCON_CLASSID;
  }

#endif


@q m v genRow @>
@ The function |genRow()| generates the row format of the constraint
  associated with the variable set |var|.
  This function redefines a virtual function of the base class
  |ABA_ROWCON|.
@q m e t @>

@q m b r @>
  \BeginReturn
  It returns the number of nonzero elements in the row format.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|var|}{The variable set for which the row format is generated
                 is only a dummy since the the variable set is assumed
                 to be fixed for this constraint class.}
  \docarg{|row|}{Holds the row format of the constraint after the execution
                   of this function.}
  \EndArg
@q m e a @>

@(srowcon.cc@>=
@q m b f @>
  int ABA_SROWCON::genRow(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *var, 
                          ABA_ROW &row)
@q m e f @>
  {
    row.copy(row_);

    return row_.nnz();
  }

@q m v slack @>
@ The function |slack()| computes the slack of a vector  associated
  with the variable set |variables|. This function redefines a virtual
  function of the base class |ABA_ROWCON|.
@q m e t @>

@q m b r @>
  \BeginReturn
  The slack of the vector |x|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|variable|}{The variable set for which the row format is generated
                 is only a dummy since the the variable set is assumed
                 to be fixed for this constraint class.}

  \docarg{|x|}{An array of length equal to the number of variables.}
  \EndArg
@q m e a @>

@(srowcon.cc@>=
@q m b f @>
  double ABA_SROWCON::slack(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *variables, 
                            double *x)
@q m e f @>
  {

    int nnz = row_.nnz();

    double lhs = 0.0;

    for (int i = 0; i < nnz; i++)
      lhs += x[row_.support(i)] * row_.coeff(i);

    return rhs() - lhs;
  }

