@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: string.w,v 4.9 2003/08/05 13:55:28 elf Exp $ @>

@q m i ABA_STRING @>

@ The member functions are implemented in the file {\tt string.cc}.
  We also include the file |<string.h>| that we can use the string
  manipulation functions from the C-library.

@(string.cc@>=
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef ABACUS_OLD_INCLUDE
#include "string.h"
#include "global.h"
#else
#include "abacus/string.h"
#include "abacus/global.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \docarg{|cString|}{The initializing string, by default the empty string.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  ABA_STRING::ABA_STRING(ABA_GLOBAL *glob, const char *cString) 
@q m e f @>
  : @/
    glob_(glob)
  {
    string_ = new char[strlen(cString) + 1];
    ::strcpy(string_, cString);
  }

@q m f Constructor @>
@ A constructor building a string from a string  and an integer.
  This constructor is especially useful for building variable
  or constraint names like {\tt con18}.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|glob|}{A pointer to the corresponding global object.}
  \docarg{|cString|}{The initializing string.}
  \docarg{|index|}{The integer value appending to the |cString| (must
                   be less than {\tt MAX\_INT}).}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  ABA_STRING::ABA_STRING(ABA_GLOBAL *glob, const char *cString, int index) 
@q m e f @>
  : @/
    glob_(glob)
  {
    string_ = new char[strlen(cString) + 11];
    sprintf(string_, "%s%d", cString, index);
  }

@q m f Copy Constructor @>
@q m e t @>  
@ The copy constructor.

@q m b a @>
  \BeginArg
  \docarg{|rhs|}{The string that is copied.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  ABA_STRING::ABA_STRING(const ABA_STRING &rhs)
@q m e f @>
  : @/
    glob_(rhs.glob_)
  {
    string_ = new char[rhs.size() + 1];
    ::strcpy(string_, rhs.string_);
  }

@q m f Destructor @>
@q m e t @>
@ The destructor.

@(string.cc@>=
@q m b f @>
  ABA_STRING::~ABA_STRING()
@q m e f @>
  {
    delete [] string_;
  }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the |ABA_STRING| from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the string is initialized.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  ABA_STRING::ABA_STRING(const ABA_GLOBAL *glob, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    glob_(glob), @/
    string_(0)
  {
    unpack(msg);
  }

@q m f unpack @>
@ The function |unpack()| unpacks the string from an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the string is unpacked.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  void ABA_STRING::unpack(ABA_MESSAGE &msg)
@q m e f @>
  {
    int size;
    msg.unpack(size);
    delete [] string_;
    string_ = new char[size + 1];
    msg.unpack(string_, size);
    string_[size] = '\0';
  }

@q m f pack @>
@ The function |pack()| packs the string in an |ABA_MESSAGE| object.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the string is packed.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  void ABA_STRING::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    int size = ::strlen(string_);
    msg.pack(size);
    msg.pack(string_, size);
  }

#endif


@q m f Assignment Operator @>
@ The assignment operator makes a copy of the right hand side and reallocates
  memory if required.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the object.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|rhs|}{The right hand side of the assignment.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  const ABA_STRING& ABA_STRING::operator=(const ABA_STRING &rhs)
@q m e f @>
  {
    if (&rhs == this) return *this;

    if(size() != rhs.size()) {
      delete [] string_;
      string_ = new char[rhs.size() + 1];
    }

    ::strcpy(string_, rhs.string_);

    return *this;
  }

@q m f Assignment Operator @>
@ The assignment operator is overloaded for character strings.
@q m e t @>

@(string.cc@>=
@q m b f @>
  const ABA_STRING& ABA_STRING::operator=(const char *rhs)
@q m e f @>
  {
    if(size() != ((int) strlen(rhs))) {
      delete [] string_;
      string_ = new char[strlen(rhs) + 1];
    }

    ::strcpy(string_, rhs);

    return *this;
  }

@q m f Comparison Operator @>
@q m e t @>
@ The comparison operator.

  Note: the C-library function |strcmp()| returns 0 if both strings equal.

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If both strings are not equal,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|lhs|}{The left hand side of the comparison.}
  \docarg{|rhs|}{The right hand side of the comparison.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  int operator==(const ABA_STRING &lhs, const ABA_STRING &rhs)
@q m e f @>
  {
    return strcmp(lhs.string_, rhs.string_) ? 0 : 1;
  }

@q m f Comparison Operator @>
@ The comparison operator is overloaded for character strings on
  the right hand side.
@q m e t @>

@(string.cc@>=
@q m b f @>
  int operator==(const ABA_STRING &lhs, const char *rhs)
@q m e f @>
  {
    return strcmp(lhs.string_, rhs) ? 0 : 1;
  }

@q m f Not-Equal Operator @>
@q m e t @>
@ The not-equal operator.

  Note: the C-library function |strcmp()| returns 0 if both strings equal.

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If both strings are equal,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|lhs|}{The left hand side of the comparison.}
  \docarg{|rhs|}{The right hand side of the comparison.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  int operator!=(const ABA_STRING &lhs, const ABA_STRING &rhs)
@q m e f @>
  {
    return strcmp(lhs.string_, rhs.string_) ? 1 : 0;
  }

@q m f Not-Equal Operator @>
@ The not-equal operator is overloaded for character strings on
  the right hand side.
@q m e t @>

@(string.cc@>=
@q m b f @>
  int operator!=(const ABA_STRING &lhs, const char *rhs)
@q m e f @>
  {
    return strcmp(lhs.string_, rhs) ? 1 : 0;
  }

@q m f Output Operator @>
@q m e t @>
@ The output operator.

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The string being output.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  ostream& operator<<(ostream &out, const ABA_STRING &rhs)
@q m e f @>
  {
     return out << rhs.string_;
  }


@q m f Subscript Operator @>
@ With the subscript operator a single character of the string can
  be accessed or modified.  
@q m e t @>
  If the class is compiled with the preprocessor flag |-DABACUSSAFE|,
  then a range check is performed.

@q m b r @>
  \BeginReturn
  A reference to the |i|-th character of
  the string. 
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the character that should be accessed or
               modified. The first character has number 0.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  char& ABA_STRING::operator[](int i)
@q m e f @>
  {
#ifdef ABACUSSAFE
    rangeCheck(i);
#endif
    return string_[i];
  }

@q m f Subscript Operator @>
@ The subscript operator is overloaded for constant use.
@q m e t @>

@(string.cc@>=
@q m b f @>
  const char& ABA_STRING::operator[](int i) const
@q m e f @>
  {
#ifdef ABACUSSAFE
    rangeCheck(i);
#endif
    return string_[i];
  }
  
@q m f size @>
@q m e t @>
@ The function |size()|.

@q m b r @>
  \BeginReturn
  The length of the string, not including the {\tt '$\backslash$0'} 
  terminating the string.
  \EndReturn
@q m e r @>

@(string.cc@>=
@q m b f @>
  int ABA_STRING::size() const
@q m e f @>
  {
    return ::strlen(string_);
  }

@q m f ascii2int @>
@ The function |ascii2int()| is very similar to the function |atoi()|
  from |<string.h>|. It converts the substring starting
  at component |i| and ending in the first following component
  with {\tt '$\backslash$0'} to an integer. 
  |ascii2int(0)| converts the complete string.
@q m e t @>

@q m b r @>
  \BeginReturn
  The string converted to an integer value.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the character at which the conversion should
               start. The default value of |i| is 0.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  int ABA_STRING::ascii2int(int i) const
@q m e f @>
  {
    return atoi(string_ + i);
  }

@q m f ascii2double @>
@ The function |ascii2double()| emulates the function |atof()| of the
  standard C library and converts the string to a floating point number.
@q m e t @>

@q m b r @>
  \BeginReturn
  The string converted to a floating point number.
  \EndReturn
@q m e r @>

@(string.cc@>=
@q m b f @>
  double ABA_STRING::ascii2double() const
@q m e f @>
  {
    return atof(string_);
  }

@q m f ascii2unsignedint @>
@ The function |ascii2unsignedint()| converts the string to an
  |unsigned int| value.
@q m e t @>

@q m b r @>
  \BeginReturn
  The string converted to an unsigned integer.
  \EndReturn
@q m e r @>

@(string.cc@>=
@q m b f @>
  unsigned int ABA_STRING::ascii2unsignedint() const
@q m e f @>
  {
    return (unsigned int) strtoul(string_, NULL, 10);
  }

@q m f ascii2bool @>
@ The function |ascii2bool()| converts the string to a boolean value.
  This is only possible for the strings |"true"| and |"false"|.
@q m e t @>

@q m b r @>
  \BeginReturn
  The string converted to |true| or |false|.
  \EndReturn
@q m e r @>

@(string.cc@>=
@q m b f @>
  bool ABA_STRING::ascii2bool() const
@q m e f @>
  {
    if (!strcmp(string_, "true")) return true;
    if (!strcmp(string_, "false")) return false;
    else {
      glob_->err() << "ABA_STRING::ascii2bool(): string neither true nor false, ";
      glob_->err() << "but " << string_ << "." << endl;
      exit(Fatal);
      return false;  // to suppress compiler warnings
    }
  }

@q m f ending @>
@q m e t @>
@ The function |ending()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the string ends with the string |end|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|end|}{The string with which the ending of the string is compared.}
  \EndArg
@q m e a @>

@(string.cc@>=
@q m b f @>
  bool ABA_STRING::ending(const char *end) const
@q m e f @>
  {
    int l1 = ::strlen(string_);
    int l2 = ::strlen(end);

    if (l1 < l2) return false;

    if (!strcmp(string_+l1-l2, end)) return true;
    else                             return false;
    
  }

@q m f string @>
@q m e t @>
@ The function |string()|.

@q m b r @> 
  \BeginReturn
  The |char*| representing the string to make it accessible for 
  C-functions.
  \EndReturn
@q m e r @>

@(string.cc@>=
@q m b f @>
  char *ABA_STRING::string() const
@q m e f @>
  {
    return string_;
  }

@ The function |rangeCheck()| terminates the program with an
  error message if |i| is not the position of a character of
  the string. The |'\0'| at the end of the string is not a valid
  character in this sense.
  
@(string.cc@>=
  void ABA_STRING::rangeCheck(int i) const
  {
    if (i < 0 || i >= size()) {
      glob_->err() << "ABA_STRING::operator[" << i << "]: not in valid range ";
      glob_->err() << "[0," << size() - 1 << "]" << endl;
      exit (Fatal);
    }
  }
  
