@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: tailoff.w,v 4.10 2003/08/05 13:55:30 elf Exp $ @>

@q m i ABA_TAILOFF @>

@ All member functions are defined in the file {\tt tailoff.cc}.

@(tailoff.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "tailoff.h"
#include "master.h"
#else
#include "abacus/tailoff.h"
#include "abacus/master.h"
#endif

#include <math.h>

@q m f Constructor @>
@ The constructor takes the length of the tailing off history
  from |ABA_MASTER::tailOffNLp()|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|master|}{A pointer to the corresponding master of the optimization.}
  \EndArg
@q m e a @>

@(tailoff.cc@>=
@q m b f @>
  ABA_TAILOFF::ABA_TAILOFF(ABA_MASTER *master) 
@q m e f @>
  : @/
    master_(master) @/
  {
    if (master->tailOffNLp() > 0)
      lpHistory_ = new @,@[ ABA_RING<double>(master, master->tailOffNLp()) @];
    else
      lpHistory_ = 0;
  }

@q m f Destructor @>
@q m e t @>
@ The destructor.

@(tailoff.cc@>=
@q m b f @>
  ABA_TAILOFF::~ABA_TAILOFF()
@q m e f @>
  {
    delete lpHistory_;
  }

@q m f Output Operator @>
@ The output operator writes the memorized LP-values on an output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The tailing-off manager  being output.}
  \EndArg
@q m e a @>

@(tailoff.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_TAILOFF &rhs)
@q m e f @>
  {
    out << "LP-history:" << endl;
    if (rhs.lpHistory_) 
      out << *(rhs.lpHistory_);
    else
      out << "no LP-history available" << endl;

    return out;
  }

@ A new LP-solution value can be stored by calling the function
  |update()|. This update should be performed after every solution
  of an LP in the cutting plane generation phase of the subproblem
  optimization process.

  \BeginArg
  \docarg{|value|}{The LP-solution value.}
  \EndArg

@(tailoff.cc@>=
  void ABA_TAILOFF::update(double value)
  {
    if (lpHistory_) lpHistory_->insert(value);
  }

@ The function |reset()| clears the solution history. This function
  should be called if variables are added, because
  normally the solution value of the LP-relaxation gets worse
  after the addition of variables. Such a
  change could falsely indicate a tailing-off effect if the
  history of LP-values is not reset.

@(tailoff.cc@>=
  void ABA_TAILOFF::reset()
  {
    if (lpHistory_) lpHistory_->clear();
  }

@q m f tailOff @>
@ The function |tailOff()|  checks if there is a tailing-off effect.
  We assume a tailing-off effect if during the last |ABA_MASTER::tailOffNLps()|
  iterations of the cutting plane algorithms
  the dual bound changed at most |ABA_MASTER::tailOffPercent()| percent.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If a tailing off effect is observed,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(tailoff.cc@>=
@q m b f @>
  bool ABA_TAILOFF::tailOff() const
@q m e f @>
  {
    if (!lpHistory_) return false;
    
    if (!lpHistory_->filled()) return false;  // not enough iterations

    if (fabs((lpHistory_->oldest() - lpHistory_->newest())*100.0
               /lpHistory_->oldest())
             < master_->tailOffPercent()) return true;
    else return false;
  }


@q m f diff @>
@ The function |diff()| can be used to retrieve the difference between
  the last and a previous LP-solution in percent.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the difference could be computed, i.e., the old 
               LP-value |nLPs| before the last one is store in the
               history,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|nLps|}{The number of LPs before the last solved linear program
                  with which the last solved LP-value should be compared.}
  \docarg{|d|}{Contains the absolute difference bewteen the value of the 
               last solved
               linear program and the value of the linear program solved
               |nLPs| before in percent relative to the older value.}
  \EndArg
@q m e a @>

@(tailoff.cc@>=
@q m b f @>
  int ABA_TAILOFF::diff(int nLps, double &d) const
@q m e f @>
  {
    double oldVal;
    if (lpHistory_->previous(nLps, oldVal))
      return 1;

    double lastVal = lpHistory_->newest();

    d = fabs((lastVal - oldVal)*100.0/oldVal);

    return 0;
  }

