@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: timer.w,v 4.8 2003/08/05 13:55:31 elf Exp $ @>

@q m i ABA_TIMER @>

@ The member and friend functions are implemented in the file {\tt timer.cc}.

@(timer.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "timer.h"
#include "string.h"
#include "global.h"
#else
#include "abacus/timer.h"
#include "abacus/string.h"
#include "abacus/global.h"
#endif

@ The constructor for a timer with a pointer to the global object |glob|.
  After the application of the constructor the timer is not running,
  i.e., to measure time it has to be started explicitly.
  
@(timer.cc@>=
  ABA_TIMER::ABA_TIMER(ABA_GLOBAL *glob) : @/
    glob_(glob), @/
    totalTime_(0), @/
    running_(false)
    { }

@ This constructor initializes the total time of the timer
  with |centiSeconds| and
  the pointer to the corresponding global object |glob|.
  The timer is not running, too.

@(timer.cc@>=
  ABA_TIMER::ABA_TIMER(ABA_GLOBAL *glob, long centiSeconds) : @/
    glob_(glob), @/
    totalTime_(centiSeconds), @/
    running_(false)
    { }
 
@ The destructor.

@(timer.cc@>=
  ABA_TIMER::~ABA_TIMER()
  { }

@q m f Output Operator @>  
@ The output operator writes the time in the format
  {\tt hours:minutes:seconds.seconds/100} on an output stream.
@q m e t @>

  After the time has been divided in hours, minutes and seconds
  we have to take care that an additional leading zero is output
  if minutes or seconds have a value less than ten.

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The timer being output.}
  \EndArg
@q m e a @>

@(timer.cc@>=
@q m b f @>
  ostream& operator<<(ostream& out, const ABA_TIMER& rhs)
@q m e f @>
  {
    long centiSeconds = rhs.centiSeconds();

    long sec  = centiSeconds/100;
    long mSec = centiSeconds - 100*sec;
    long rSec = sec%60;
    long min  = sec/60;
    long rMin = min%60;
    
    out << min/60 << ":";
    if(rMin < 10) out << '0';
    out << rMin << ':';
    if(rSec < 10) out << '0';
    out << rSec << '.';
    if (mSec < 10) out << '0';
    out << mSec;
    return out;
  }

@q m f start @>
@ The timer is started with the function |start()|. For safety
  starting a running timer is an error.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|reset|}{If this flag is set to true, the timer is reset before
                  it is started (default=false)}
  \EndArg
@q m e a @>

@(timer.cc@>=
@q m b f @>
  void ABA_TIMER::start(bool reset)
@q m e f @>
  {
    if (reset)
      totalTime_ = 0;
    else
      if (running_) {
        glob_->err() << "ABA_TIMER::start(): you cannot start a running timer." << endl;
        exit(Fatal);
      }
    running_   = true;
    startTime_ = theTime();
  }

@q m f stop @>
@ The function |stop()| stops the timer and adds the difference between
  the current time and the starting time to the total time.
  Stopping a non-running timer is an error.
@q m e t @>

@(timer.cc@>=
@q m b f @>
  void ABA_TIMER::stop()
@q m e f @>
  {
    if(!running_) {
      glob_->err() << "ABA_TIMER::stop : you cannot stop a non-running timer." << endl;
      exit(Fatal);
    }
    
    totalTime_ += theTime() - startTime_;
    running_   = false;

  }
@q m f reset @>
@ The function |reset()| stops the timer and sets the |totalTime| to 0.
@q m e t @>

@(timer.cc@>=
@q m b f @>
  void ABA_TIMER::reset()
@q m e f @>
  {
    running_   = false;
    totalTime_ = 0;
  }

@q m f running @>
@q m e t @>

@ The function |running()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the timer is running,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(timer.cc@>=
@q m b f @>
  bool ABA_TIMER::running() const
@q m e f @>
  {
    return running_;
  }

@q m f centiSeconds @>
@q m e t @>
@ The function |centiSeconds()|.

@q m b r @>
  \BeginReturn
  The currently
  spent time in  $1\over100$-seconds. It is
  not necessary to stop the timer to get the correct time.
  \EndReturn
@q m e r @>

@(timer.cc@>=
@q m b f @>
  long ABA_TIMER::centiSeconds() const
@q m e f @>
  {
    long cs;
    
    if(running_) cs = totalTime_ + theTime() - startTime_;
    else         cs = totalTime_;

    return cs;
  }

@q m f seconds @>
@q m e t @>
@ The function |seconds()|.

@q m b r @>
  \BeginReturn
  The currently spent time in seconds. It is
  not necessary to stop the timer to get the correct time.
  The result is rounded down to the next integer value.
  \EndReturn
@q m e r @>

@(timer.cc@>=
@q m b f @>
  long ABA_TIMER::seconds() const
@q m e f @>
  {
    return centiSeconds()/100;
  }

@q m f minutes @>
@q m e t @>
@ The function |minutes()|.

@q m b r @>
  \BeginReturn
  The currently spent time in minutes. It is
  not necessary to stop the timer to get the correct time.
  The result is rounded down to the next integer value.
  \EndReturn
@q m e r @>

@(timer.cc@>=
@q m b f @>
  long ABA_TIMER::minutes() const
@q m e f @>
  {
    return centiSeconds()/6000;
  }

@q m f hours @>
@q m e t @>
@ The function |hours()|.

@q m b r @>
  \BeginReturn
  The currently spent time in hours. It is
  not necessary to stop the timer to get the correct time.
  The result is rounded down to the next integer value.
  \EndReturn
@q m e r @>

@(timer.cc@>=
@q m b f @>
  long ABA_TIMER::hours() const
@q m e f @>
  {
    return seconds()/3600;
  }
  
@q m f exceeds @>
@q m e t @>
@ The function |exceeds()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the currently spent time exceeds |maxTime|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|maxTime|}{A string of the form
    |[[h:]m:]s|, where |h| are the hours, |m| the minutes, and |s| the
    seconds. Hours and minutes are optional. |h| can be an arbitrary
    nonnegative integer, |s| and |m| have to be integers in $\{0,\dots,59\}$.
    If |m| or |s| are less than 10, then a leading 0 is allowed
    (e.g. 3:05:09).
  }
  \EndArg
@q m e a @>

@(timer.cc@>=
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

@q m b f @>
  bool ABA_TIMER::exceeds(const ABA_STRING &maxTime) const
@q m e f @> 
  {
    long maxSecs;  // |maxTime| in seconds
    
    @<convert |maxTime| in seconds@>;
    if (seconds() >= maxSecs) return true;
    else                      return false;
  }

@ @<convert |maxTime| in seconds@>=
  long s  = 0;   // seconds in |maxTime|
  long m  = 0;   // minutes in |maxTime|
  long h  = 0;   // hours in |maxTime|
  ABA_STRING buf = maxTime;  // copy of |maxTime| which is modified during the conversion
  
  @<begin at the end of |maxTime| and read the seconds@>;
  @<proceed with the minutes@>;
  @<proceed with the hours@>;
  
  maxSecs = s + 60*m + 3600*h;
  
@ We start at the end of |buf| and go left until a |':'| is found
  or we have reached the first character of the string. The string
  on the right side of the current position are the seconds.
  
@<begin at the end of |maxTime| and read the seconds@>=
  int pos = buf.size() - 1;
  while (pos >= 0 && buf[pos] != ':') pos--;
  s = buf.ascii2int(pos + 1);

@ If we have not reached the beginning of |buf| we replace the |':'|
  by |'\0'| to mark the new end of the string. We go left
  until we reach the beginning of the string or the next |':'|.
  The string on the right side of the current position are the
  minutes.
  
@<proceed with the minutes@>=
  if(pos >= 0) {
    buf[pos] = '\0';
    pos--;
    while (pos >= 0 && buf[pos] != ':') pos--;
    m = buf.ascii2int(pos + 1);

  }

@ If we still have not reached the beginning of the string, we again replace
  the |':'| by a |'\0'| to mark the new end of the string. If there
  are still characters on the left side from the current position,
  then this string is the hours.
  
@<proceed with the hours@>=
  if (pos >= 0) {
    buf[pos] = '\0';
    if (--pos >= 0) h = buf.ascii2int(0);
  }
             
@q m f add @>
@q m e t @>
@ The function |addCentiSeconds()|.

@q m b a @>
  \BeginArg
  \docarg{|centiSeconds|}{The number of centiseconds to be added.}
  \EndArg
@q m e a @>

@(timer.cc@>=
@q m b f @>
  void ABA_TIMER::addCentiSeconds(long centiSeconds)
@q m e f @>
  {
#ifdef ABACUS_PARALLEL
    mutex_.acquire();
#endif
    totalTime_ += centiSeconds;
#ifdef ABACUS_PARALLEL
    mutex_.release();
#endif
  }

@  \BeginPureVirtual

@q m f theTime @>
  The function |theTime()| is required for measuring the time difference
  between the time of the call and some base point (e.g., the program
  start). It is a pure virtual function because in derived classes
  different implementation for elapsed time and CPU time are required.
@q m e t @>

@q m b r @>
  \BeginReturn
     The time since some base point (e.g., the program start for the
     cpu time) in $1\over100$ seconds.
  \EndReturn
@q m e r @>

|
@q m b f @>
   virtual long theTime() const = 0
@q m e f @>
|

