@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: valbranchrule.w,v 4.12 2003/08/05 13:55:31 elf Exp $ @>

@q m i ABA_VALBRANCHRULE @>

@ All member functions are defined in the file {\tt br|_|value.cc}.

@(valbranchrule.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "valbranchrule.h"
#include "master.h"
#include "fsvarstat.h"
#include "sub.h"
#include "lpsub.h"
#else
#include "abacus/valbranchrule.h"
#include "abacus/master.h"
#include "abacus/fsvarstat.h"
#include "abacus/sub.h"
#include "abacus/lpsub.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/message.h"
#endif

@q m f Constructor @>
@q m e t @>
@ The constructor.

@q m b a @>
   \BeginArg
   \docarg{|master|}{The corresponding master of the optimization.}
   \docarg{|variable|}{The branching variable.}
   \docarg{|value|}{The value the branching variable is set to.}
   \EndArg
@q m e a @>

@(valbranchrule.cc@>=
@q m b f @>
  ABA_VALBRANCHRULE::ABA_VALBRANCHRULE(ABA_MASTER *master, int variable, double value)
@q m e f @>
  : @/
    ABA_BRANCHRULE(master), @/
    variable_(variable), @/
    value_(value)
  { }

@q m v Destructor @>
@q m e t @>

@ The destructor.

@(valbranchrule.cc@>=
@q m b f @>
  ABA_VALBRANCHRULE::~ABA_VALBRANCHRULE()
@q m e f @>
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@q m e t @>

@ The message constructor creates the |ABA_VALBRANCHRULE| from an |ABA_MESSAGE|.

@q m e t @>

@q m b a @>
    \BeginArg
    \docarg
      {|master|}{A pointer to the corresponding master of the optimization.}
    \docarg
      {|msg|}{The message from which the object is initialized.}
    \EndArg
@q m e a @>

@(valbranchrule.cc@>=
@q m b f @>
  ABA_VALBRANCHRULE::ABA_VALBRANCHRULE(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
    : @/
    ABA_BRANCHRULE(master, msg)
  {
    msg.unpack(variable_);
    msg.unpack(value_);
    msg.unpack(oldLpLBound_);
    msg.unpack(oldLpUBound_);
  }

@q m f pack @>
@ The virtual function |pack()| packs the data of the |ABA_VALBRANCHRULE|
  in an |ABA_MESSAGE| object. The |pack()| function of the base class
  has to be called before the local data is packed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the |ABA_VALBRANCHRULE| is packed.}
  \EndArg
@q m e a @>

@(valbranchrule.cc@>=
@q m b f @>
  void ABA_VALBRANCHRULE::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_BRANCHRULE::pack(msg);
    msg.pack(variable_);
    msg.pack(value_);
    msg.pack(oldLpLBound_);
    msg.pack(oldLpUBound_);
  }

@q m f classId @>
@ The abstract virtual function |classId()| of |ABA_BRANCHRULE| has to be
  implemented in each derived subclass. It has to return an integer
  which identifies the class.

@q m b r @>
  \BeginReturn
  The class identification.
  \EndReturn
@q m e r @>

@(valbranchrule.cc@>=
@q m b f @>
  int ABA_VALBRANCHRULE::classId() const
@q m e f @>
  {
    return VALBRANCHRULE_CLASSID;
  }

#endif


@q m f Output Operator @>
@ The output operator writes the branching variable together with its
  value to an output stream.
@q m e t @>

@q m b r @>
  \BeginReturn
  A reference to the output stream.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|rhs|}{The branching rule being output.}
  \EndArg
@q m e a @>

@(valbranchrule.cc@>=
@q m b f @>
  ostream &operator<<(ostream &out, const ABA_VALBRANCHRULE &rhs)
@q m e f @>
  {
    return out << "x" << rhs.variable_ << " = " << rhs.value_;
  }

@q m v extract @>
@ The function |extract()| modifies a subproblem by setting the branching
  variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the subproblem can be modified according to the
               branching rule.}
  \docarg{|1|}{If a contradiction occurs.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The subproblem being modified.}
  \EndArg
@q m e a @>

@(valbranchrule.cc@>=
@q m b f @>
  int ABA_VALBRANCHRULE::extract(ABA_SUB *sub)
@q m e f @>
  {
    if (sub->fsVarStat(variable_)->contradiction(ABA_FSVARSTAT::Set, value_))
      return 1;
    
    sub->fsVarStat(variable_)->status(ABA_FSVARSTAT::Set, value_);
    return 0;
  }

@q m v extract @>
@ The function |extract()| is overloaded to modify directly the linear
  programming relaxation. This required to evaluate the quality of
  a branching rule.
@q m e t @>

@(valbranchrule.cc@>=
@q m b f @>
  void ABA_VALBRANCHRULE::extract(ABA_LPSUB *lp)
@q m e f @>
  {
     oldLpLBound_ = lp->lBound(variable_);
     oldLpUBound_ = lp->uBound(variable_);

     lp->changeLBound(variable_, value_);
     lp->changeUBound(variable_, value_);
  }


@q m f unExtract @>
@q m e t @>
@ The function |unExtract()|.

@(valbranchrule.cc@>=
@q m b f @>
  void ABA_VALBRANCHRULE::unExtract(ABA_LPSUB *lp)
@q m e f @>
  {
    lp->changeLBound(variable_, oldLpLBound_);
    lp->changeUBound(variable_, oldLpUBound_);
  }


@q m f variable @>
@q m e t @>
@ The function |variable()|.

@q m b r @>
  \BeginReturn
  The number of the branching variable.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_VALBRANCHRULE::variable() const
@q m e f @>

@q m f value @>
@q m e t @>
@ The function |value()|.

@q m b r @>
  \BeginReturn
  The value of the branching variable.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_VALBRANCHRULE::value() const
@q m e f @>

