@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: variable.w,v 4.16 2003/08/05 13:55:31 elf Exp $ @>

@q m i ABA_VARIABLE @>

@ All member functions are defined in the file {\tt variable.cc}.

@(variable.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "variable.h"
#include "master.h"
#include "sub.h"
#include "constraint.h"
#include "vartype.h"
#include "column.h"
#include "active.h"
#else
#include "abacus/variable.h"
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/constraint.h"
#include "abacus/vartype.h"
#include "abacus/column.h"
#include "abacus/active.h"
#endif

#ifdef ABACUS_SYS_POWERPLUS
extern "C" {
#include "/home/user/christof/abacus/include/math.h"
}
#else
#include <math.h>
#endif

@q m f Constructor @>
@q m e t @>

@ The constructor.
@q m b a @>
     \BeginArg
     \docarg
      {|master|}
      {A pointer to the corresponding master of the optimization.}
     \docarg
       {|sub|}
       {A pointer to the subproblem associated with the variable. This
        can also be the 0-pointer.
       }
     \docarg
      {|dynamic|}
      {If this argument is |true|, then the variable can also be
       removed again from the set of active variables after it is
       added once.}
     \docarg
       {|local|}
       {If this argument is |true|, then the variable is only locally
        valid, otherwise it is globally valid. As a locally valid variable is
        always associated with a subproblem, the argument |sub| must
        not be 0 if |local| is |true|.
       }
     \docarg
       {|obj|}{The objective function coefficient.}
     \docarg
       {|lBound|}
       {The lower bound of the variable.}
     \docarg
       {|uBound|}
       {The upper bound of the variable.}
     \docarg
       {|type|}
       {The type of the variable.}
  \EndArg
@q m e a @>  

@(variable.cc@>=
@q m b f @>
  ABA_VARIABLE::ABA_VARIABLE(ABA_MASTER *master, 
                             const ABA_SUB *sub, 
                             bool dynamic, 
                             bool local,
                             double obj, 
                             double lBound,
                             double uBound,
                             ABA_VARTYPE::TYPE type) 
@q m e f @>
  : @/
    ABA_CONVAR(master, sub, dynamic, local), @/
    fsVarStat_(master), @/
    obj_(obj), @/
    lBound_(lBound), @/
    uBound_(uBound), @/
    type_(type)
  { }

@q m v Destructor @>
@q m e t @>

@ The destructor.

@(variable.cc@>=
@q m b f @>
  ABA_VARIABLE::~ABA_VARIABLE()
@q m e f @>
  { }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the variable from an |ABA_MESSAGE|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the variable is initialized.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  ABA_VARIABLE::ABA_VARIABLE(const ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    ABA_CONVAR(master, msg), @/
    fsVarStat_(master, msg), @/
    type_(msg)
  {
    msg.unpack(obj_);
    msg.unpack(lBound_);
    msg.unpack(uBound_);
  }

@q m f pack @>
@ The function |pack()| packs the data of the variable
  in an |ABA_MESSAGE| object. This virtual function has to be redefined
  in a derived class if additional data should be packed. In this
  case the |pack()| function of the base class must be called
  by the |pack()| function of the derived class.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the variable is packed.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  void ABA_VARIABLE::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    ABA_CONVAR::pack(msg);

    fsVarStat_.pack(msg);
    type_.pack(msg);
    msg.pack(obj_);
    msg.pack(lBound_);
    msg.pack(uBound_);
  }

#endif


@q m f varType @>
@q m e t @>
@ The function |varType()|.

@q m b r @>
  \BeginReturn
  The type of the variable.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_VARTYPE::TYPE ABA_VARIABLE::varType() const
@q m e f @>
  
@q m f discrete @>
@q m e t @>
@ The function |discrete()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the type of the variable is |Integer| or |Binary|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_VARIABLE::discrete()
@q m e f @>

@q m f binary @>
@q m e t @>
@ The function |binary()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the type of the variable is |Binary|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_VARIABLE::binary()
@q m e f @>

@q m f integer @>
@q m e t @>
@ The function |integer()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the type of the variable is |Integer|,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_VARIABLE::integer()
@q m e f @>


@q m v obj @>
@q m e t @>
@ The virtual function |obj()|.

@q m b r @>
  \BeginReturn  
   The objective function coefficient.
  \EndReturn
@q m e r @>

@(variable.cc@>=
@q m b f @>
  double ABA_VARIABLE::obj()
@q m e f @>
  {
    return obj_;
  }
  
@q m f lBound @>
@q m e t @>
@ The function |lBound()|.

@q m b r @>
  \BeginReturn
  The lower bound of the variable.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_VARIABLE::lBound() const
@q m e f @>

@q m f lBound @>
@ This version of the function |lBound()| sets the lower bound of the
  variable.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newBound|}{The new value of the lower bound.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_VARIABLE::lBound(double newBound)
@q m e f @>

@q m f uBound @>
@q m e t @>
@ The function |uBound()|.

@q m b r @>
  \BeginReturn
  The upper bound of the variable.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_VARIABLE::uBound() const
@q m e f @>

@q m f uBound @>
@ This version of the function |uBound()| sets the upper bound of the variable.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newBound|}{The new value of the upper bound.}
  \EndArg
@q m e a @>
  
@q m b f @>
  void ABA_VARIABLE::uBound(double newBound)
@q m e f @>

@q m f fsVarStat @>
@q m e t @>
@ The function |fsVarStat()|.

@q m b r @>
  \BeginReturn
  A pointer to the global status of fixing and setting of the variable.
  Note, this is the global status of fixing/setting that might differ
  from the local status of fixing/setting a variable returned by
  the function |ABA_SUB::fsVarStat()|.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_FSVARSTAT *ABA_VARIABLE::fsVarStat()
@q m e f @>

@q m v valid @>
@q m e t @>
@ The function |valid()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the variable is globally
                  valid, or the subproblem |sub| is an ancestor in the
                  enumeration
                  tree of the subproblem associated with the variable,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{The subproblem for which validity of the variable is
                 checked.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  bool ABA_VARIABLE::valid(ABA_SUB *sub)
@q m e f @>
  {
#ifdef ABACUSSAFE
    if (local_ && sub == 0) {
      master_->err() << "ABA_VARIABLE::valid(): cannot evaluate for locally ";
      master_->err() << "valid variable with 0-pointer to subproblem.";
      exit(Fatal);
    }
#endif

    if (!local_ || sub->ancestor(sub_)) return true;
    else                                return false;
  }

@q m v coeff @>
@ The function |coeff()| computes the coefficient of the variable in the
  constraint |con|. Per default the coefficient of a variable is
  computed indirectly via the coefficient of a constraint. Problem
  specific redefinitions might be required.
@q m e t @>

@q m b r @>
  \BeginReturn
  The coefficient of the variable in the constraint |con|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|con|}{The constraint of which the coefficient should be computed.}
  \EndArg
@q m e a @>
  
@(variable.cc@>=
@q m b f @>
  double ABA_VARIABLE::coeff(ABA_CONSTRAINT *con)
@q m e f @>
  {
    return con->coeff(this);
  }

@q m v genColumn @>
@ The function |genColumn()| computes the column |col| of the variable
  associated with the active constraints |*actCon|.
  Note, the upper and lower bound of the column are initialized
  with the global upper and lower bound of the variable.
  Therefore, an adaption with the local bounds might be required.
@q m e t @>

@q m b r @>
  \BeginReturn
  The number of nonzero entries in |col|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|actCon|}{The constraints for which the column of the variable
                    should be computed.}
  \docarg{|col|}{Stores the column when the function terminates.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  int ABA_VARIABLE::genColumn(ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE> *actCon, 
                              ABA_COLUMN &col)
@q m e f @>
  {
    double eps      = master_->machineEps();
    double minusEps = -eps;
    double co;
    int    n        = actCon->number();

    _expand();
    
    for (int i = 0; i < n; i++) {
      co = (*actCon)[i]->coeff(this);
      if (co > eps || co < minusEps) col.insert(i,co);
    }

    col.obj(obj());
    col.lBound(lBound());
    col.uBound(uBound());

    _compress();
    
    return col.nnz();
      
  }

@q m v violated @>
@ The function |violated()| checks, if a variable does not price
  out correctly, i.e., if the reduced cost |rc| is positive for
  a maximization problem and negative for a minimization problem,
  respectively.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the variable does not price out correctly.}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|rc|}{The reduced cost of the variable.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  bool ABA_VARIABLE::violated(double rc) const
@q m e f @>
  {
    if (master_->optSense()->max()) {
#ifdef ABACUS_NO_BOOL
      if (rc > master_->eps()) return true;
      else                     return false;
#else
      return rc > master_->eps();
#endif
    }
    else {
#ifdef ABACUS_NO_BOOL
      if (rc < -master_->eps()) return true;
      else                      return false;
#else
      return rc < -master_->eps();
#endif
    }
  }

@q m v violated @>
@ This version of the function |violated()| checks if the variable
  does not price out correctly, i.e., if the reduced cost
  of the variable associated with the constraint set |constraints|
  and the dual variables |y| are positive for a maximization problem
  and negative for a minimization problem, respectively. 
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the variable does not price out correctly.}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|constraints|}{The constraints associated with the dual
                         variables |y|.}
  \docarg{|y|}{The dual variables of the constraint.}
  \docarg{|r|}{If |r| is
               not the 0-pointer, it will store the reduced cost after the
               function call. Per default |r| is 0.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  bool ABA_VARIABLE::violated(ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE> *constraints,
                              double *y, 
                              double *r)
@q m e f @>
  {
    double rc = redCost(constraints, y);
    
    if (r) *r = rc;
    
    return violated(rc);
  }

@q m v redCost @>
@ The function |redCost()|computes the reduced cost of the variable
  corresponding the constraint set |actCon| and the dual variables |y|.
@q m e t @>

  Given the dual variables $y$, then the reduced cost of a variable
  with objective function coefficient $c_e$, column $a_{.e}$ are defined
  as $c_e - \transpose{y} a_{.e}$.

@q m b r @>
  \BeginReturn
  The reduced cost of the variable.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|actCon|}{The constraints associated with the dual
                         variables |y|.}
  \docarg{|y|}{The dual variables of the constraint.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  double ABA_VARIABLE::redCost(ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE> *actCon, 
                               double *y)
@q m e f @>
  {
    double c;
    double eps = master_->machineEps();
    double minusEps = -eps;
    double rc = obj();
    int    n  = actCon->number();

    _expand();

    for (int i = 0; i < n; i++) {
      c = (*actCon)[i]->coeff(this);
      if (c > eps || c < minusEps) 
        rc -= y[i] * c;
    }

    _compress();
    
    return rc;
  }

@q m v useful @>
@ An (inactive) discrete variable is considered as |useful()| if its
  activation might not produce only solutions worse than the
  best known feasible solution. This is the same criterion for
  fixing inactive variables by reduced cost criteria.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the variable is considered as useful,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
     \BeginArg
     \docarg{|actCon|}{The active constraints.}
     \docarg{|y|}{The dual variables of these constraints.}
     \docarg{|lpVal|}{The value of the linear program.}
     \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  bool ABA_VARIABLE::useful(ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE> *actCon, 
                            double *y, 
                            double lpVal)
@q m e f @>
  {
    if (!discrete()) return true;
    
    double rc = redCost(actCon, y);

    if (master_->optSense()->max()) {
      if (lpVal + rc > master_->primalBound()) return true;
      else                                 return false;
    }
    else {
      if (lpVal + rc < master_->primalBound()) return true;
      else                                 return false;
    }
  }

@q m f printCol @>
@ The function |printCol()| writes the column of the variable
  corresponding to the |constraints| on the stream |out|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|out|}{The output stream.}
  \docarg{|constraints|}{The constraints for which the column should be
                         written.}
  \EndArg
@q m e a @>

@(variable.cc@>=
@q m b f @>
  void ABA_VARIABLE::printCol(ostream &out,
                              ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE> *constraints)
@q m e f @>
  {
    ABA_COLUMN col(master_, constraints->number());

    genColumn(constraints, col);

    out << col;
  }

