@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: xpressif.w,v 4.27 2003/08/05 13:55:32 elf Exp $ @>

@q m i ABA_XPRESSIF @>

@ The member functions of the class |ABA_XPRESSIF| are implemented in the
  file |xpressif.cc|.

@(xpressif.cc@>=
#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "lpmasterxpress.h"
#include "xpressif.h"
#include "column.h"
#include "row.h"
#else
#include "abacus/master.h"
#include "abacus/lpmasterxpress.h"
#include "abacus/xpressif.h"
#include "abacus/column.h"
#include "abacus/row.h"
#endif

#include <string.h>

#include "xpresso.h"


#ifdef ABACUSSAFE
#include <assert.h>
#endif

@<the implementation of the members of |ABA_XPRESSIF|@>;


@ Helper routines for debugging XPRESS. When the macro |XPRESS_TRACE| is
  set the XPRESS library calls are traced and written to the file
  |xpress.trace.c|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
#ifdef XPRESS_TRACE
#include <fstream.h>

static ofstream trace("xpress.trace.c");

static void decl(char* name, int* a, int count)
{
    trace << "    int " << name << "[" << count << "] = {" << a[0];
    for (int i=1; i<count; i++)
        trace << "," << a[i];
    trace << "};\n";
}

static void decl(char* name, double* a, int count)
{
    trace << "    double " << name << "[" << count << "] = {" << a[0];
    for (int i=1; i<count; i++)
        trace << "," << a[i];
    trace << "};\n";
}

static void decl(char* name, char* a, int count)
{
    trace << "    char " << name << "[" << count << "] = {'" << a[0];
    for (int i=1; i<count; i++)
        trace << "','" << a[i];
    trace << "'};\n";
}
#endif


@ The initialization of the static members.

@<the implementation of the members of |ABA_XPRESSIF|@>=

#ifdef ABACUSSAFE
static int nRow_;               // debugging stuff
static int nCol_;
#endif

// The mappings of the row and column statuses have to be checked.


// ABA_CSENSE: enum SENSE {Less, Equal, Greater};
char ABA_XPRESSIF::csense2xpress[3] = {'L', 'E', 'G'};

// ABA_SLACKSTAT: enum STATUS {Basic, NonBasicZero, NonBasicNonZero,
//                              Unknown};
// Unknown -> Basic !?
//
int ABA_XPRESSIF::slackStat2xpress[4] = {1, 0, 2, 0};

// ABA_LPVARSTAT: enum STATUS {AtLowerBound, Basic, AtUpperBound,
//                             NonBasicFree, Eliminated, Unknown};
// Unknown -> AtLowerBound !?
//
int ABA_XPRESSIF::lpVarStat2xpress[6] = {0, 1, 2, 0, -1, 0};

//rstatus  An array containing the basis status of the slack, surplus or
//         artificial variable associated with each row.
//         The status must be one of:
//         0  slack, surplus or artificial is non-basic at lower bound;
//         1  slack, surplus or artificial is basic;
//         2  slack or surplus is non-basic at upper bound.
//
//cstatus  An array containing the basis status of the columns in the 
//         constraint matrix. The status must be one of:
//         0  variable is non-basic at lower bound or superbasic at zero if 
//            the variable has no lower bound;
//         1  variable is basic;
//         2  variable is at upper bound.

ABA_SLACKSTAT::STATUS ABA_XPRESSIF::xpress2slackStat[3] = {
  ABA_SLACKSTAT::NonBasicZero,
  ABA_SLACKSTAT::Basic,
  ABA_SLACKSTAT::NonBasicNonZero
};
ABA_LPVARSTAT::STATUS ABA_XPRESSIF::xpress2lpVarStat[3] = {
  ABA_LPVARSTAT::AtLowerBound,
  ABA_LPVARSTAT::Basic,
  ABA_LPVARSTAT::AtUpperBound
};


@q m f Constructor @>
@ The constructor does not initialize the problem data of the linear
     program. It must be loaded later with the function |initialize()|.
@q m e t @>

@q m b a @>
     \BeginArg
     \docarg{|master|}{A pointer to the corresponding master of the
                     optimization.}
     \EndArg
@q m e a @>

@<the implementation of the members of |ABA_XPRESSIF|@>=
@q m b f @>
  ABA_XPRESSIF::ABA_XPRESSIF(ABA_MASTER *master) 
@q m e f @>
  : @/
    ABA_LP (master), @/
    x_(0), @/
    slack_(0), @/
    dual_(0), @/
    dj_(0), @/
    rstatus_(0), @/
    cstatus_(0) @/
  {
    lpMasterXpress_ = master->lpMasterXpress();
    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  initlz(NULL, 0);\n";
#endif
    if (::initlz(NULL, 0))
      printErrnoAndExit("ABA_XPRESSIF::ABA_XPRESSIF(): "
                        "Initialization of the XPRESS optimizer failed." );

    if (!lpMasterXpress_->xpressOutput_)
      seticv(N_PRTMSG, 0 );
    if (!lpMasterXpress_->xpressPresolve_)
      seticv(N_IFPRES, 0 );
    geticv(N_IFPRES, &ifpres);

    lpSolverTime_.stop();
  }


@q m v Destructor @>
@q m e t @>
@ The destructor.

@<the implementation of the members of |ABA_XPRESSIF|@>=
@q m b f @>
  ABA_XPRESSIF::~ABA_XPRESSIF()
@q m e f @>
  {
    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  freexo();\n";
#endif
    ::freexo();
    lpSolverTime_.stop();

    delete [] x_;
    delete [] slack_;
    delete [] dual_;
    delete [] dj_;
    delete [] rstatus_;
    delete [] cstatus_;
  }

@q m f print @>
@ The function |print()| writes the program loaded by XPRESS to
  the standard output. This function is especially useful for debugging,
  but breaks the stream concept of \ABACUS.
@q m e t @>

  The constant |L_tmpnam| und the function |tmpnam()| are declared
  in |<stdio.h>| and can be used to generate unique names. The XPRESS
  function |output()| writes the problem to file which is then
  written to the output.

@<the implementation of the members of |ABA_XPRESSIF|@>=
@q m b f @>
  void ABA_XPRESSIF::print()
@q m e f @>
  {
    char s[L_tmpnam];

    tmpnam(s);

    ::output(s);
    char *systemCall = new char[L_tmpnam+10];

    strcpy(systemCall, "cat ");
    strcat(systemCall, s);
    system(systemCall);

    strcpy(systemCall, "rm -f ");
    strcat(systemCall, s);
    system(systemCall);

    delete [] systemCall;
  }

@ The function |printErrnoAndExit()| outputs an error message an 
  terminates the program.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::printErrnoAndExit(const char *msg) const
  {
    static char *error_message[] = {
      "Invalid row type",                 // 401
      "Invalid row index",                // 402
      "Invalid global entity type",       // 403
      "Invalid set type",                 // 404
      "Invalid column index",             // 405
      "Invalid row range",                // 406
      "Invalid column range",             // 407
      "Row or column name too long",      // 408
      "Invalid branching directive",      // 409
      "Invalid row basis status",         // 410
      "Invalid column basis status",      // 411
      "Invalid parameter index",          // 412
      "Not enough spare rows",            // 413
      "Not enough spare columns",         // 414
      "Not enough spare matrix elements", // 415
      "Invalid bound type",               // 416
      "417", "418", "419", "420", "421",
      "Solution is not available",        // 422
      "Duplicate rows or columns"         // 423
    };

    master_->err() << msg << endl;

    int err;
    getipv(N_ERRNO, &err);
    if (err>=401 && err<=423)
      master_->err() << error_message[err-401];
    else
      master_->err() << "errno = " << err;

    master_->err() << "." << endl;
    exit(Fatal);
  }


@ The function |_initialize()| implements the corresponding pure virtual
  function of the base class |LP| and loads the linear program
  defined by the following arguments to the XPRESS data structures.
      \BeginArg
      \docarg{|sense|}{The sense of the objective function.}
      \docarg{|nCol|}{The number of columns (variables).}
      \docarg{|maxCol|}{The maximal number of columns.}
      \docarg{|nRow|}{The number of rows.}
      \docarg{|maxRow|}{The maximal number of rows.}
      \docarg{|obj|}{An array with the objective function coefficients.}
      \docarg{|lb|}{An array with the lower bounds of the columns.}
      \docarg{|ub|}{An array with the upper bounds of the columns.}
      \docarg{|rows|}{An array storing the rows of the problem.}
      \EndArg

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_initialize(ABA_OPTSENSE sense, int nRow, int maxRow,
                          int nCol, int maxCol,
                          ABA_ARRAY<double> &objFkt, ABA_ARRAY<double> &lBound,
                          ABA_ARRAY<double> &uBound, ABA_ARRAY<ABA_ROW*> &rows)
  {
    optsense_ = sense;

#ifdef ABACUSSAFE
    nRow_=nRow;
    nCol_=nCol;
#endif

    char   *qrtype = new char[nRow];
    double *rhs    = new double[nRow];
    double *obj    = new double[nCol];
    double *bdl    = new double[nCol];
    double *bdu    = new double[nCol];

    for(int i = 0; i < nRow; i++) {
      ABA_ROW *r = rows[i];
      qrtype[i] = csense2xpress[r->sense()->sense()];
      rhs[i] = r->rhs();
    }

#ifdef ABACUS_NO_FOR_SCOPE
    for(i = 0; i < nCol; i++) {
#else
    for(int i = 0; i < nCol; i++) {
#endif
      obj[i] = objFkt[i];
      bdl[i] = lBound[i];
      bdu[i] = uBound[i];
    }

    // transform the row oriented form of the matrix in a column oriented form
    int *colSize = new int[nCol];

#ifdef ABACUS_NO_FOR_SCOPE
    for (i=0; i<nCol; i++)
#else
    for (int i=0; i<nCol; i++)
#endif
      colSize[i] = 0;
    for(int r=0; r<nRow; r++) {  
      ABA_ROW *row = rows[r];
      const int rowNnz = row->nnz();
#ifdef ABACUS_NO_FOR_SCOPE
      for(i=0; i<rowNnz; i++)
#else
      for(int i=0; i<rowNnz; i++)
#endif
        colSize[row->support(i)]++;
    }

    int *mstart = new int[nCol];   // start of columns
    int *nextpos = new int[nCol];  // next free position in column

    int nnz = 0;
#ifdef ABACUS_NO_FOR_SCOPE
    for (i=0; i<nCol; i++) {
#else
    for (int i=0; i<nCol; i++) {
#endif
      mstart[i] = nextpos[i] = nnz;
      nnz += colSize[i];
    }
    
    int *mrwind = new int[nnz];
    double *dmatval = new double[nnz];

    // fill in the columns of the matrix
#ifdef ABACUS_NO_FOR_SCOPE
    for (r=0; r<nRow; r++) {
#else
    for (int r=0; r<nRow; r++) {
#endif
      ABA_ROW *row = rows[r];
      const int rowNnz = row->nnz();
#ifdef ABACUS_NO_FOR_SCOPE
      for(i=0; i<rowNnz; i++) {
#else
      for(int i=0; i<rowNnz; i++) {
#endif
        int c = row->support(i);
        mrwind[nextpos[c]] = r;
        dmatval[nextpos[c]] = row->coeff(i);
        nextpos[c]++;
      }
    }
#ifdef ABACUSSAFE
#ifdef ABACUS_NO_FOR_SCOPE
    for (i=1; i<nCol; i++)
#else
    for (int i=1; i<nCol; i++)
#endif
      assert(nextpos[i-1]==mstart[i]);
    assert(nextpos[nCol-1]==nnz);
#endif
    delete[] nextpos;

    // reserve 10% extra nnz space
#ifdef XPRESS_TRACE
    trace << "  seticv(N_NRXTRA, " << maxRow-nRow+1 << ");\n";
    trace << "  seticv(N_NCXTRA, " << maxCol-nCol << ");\n";
    trace << "  seticv(N_NMXTRA, " << nnz/10 << ");\n";
#endif
    seticv(N_NRXTRA, maxRow-nRow + 1);
    seticv(N_NCXTRA, maxCol-nCol);
    seticv(N_NMXTRA, nnz/10);
    // TODO:
    // N_M_S, N_NEXTRA, N_IFSCAL, N_IFNRW
    // N_ZTOLDA: zero tolerance on matrix elements

    // load the problem into the XPRESS data structures
    lpSolverTime_.start();
    if (nRow == 0)
      seticv(N_M_S, 100000);
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("qrtype", qrtype, nRow);
    decl("rhs", rhs, nRow);
    decl("obj", obj, nCol);
    decl("bdl", bdl, nCol);
    decl("bdu", bdu, nCol);
    decl("colSize", colSize, nCol);
    decl("mstart", mstart, nCol);
    decl("mrwind", mrwind, nnz);
    decl("dmatval", dmatval, nnz);
    trace << "    loadprob(\"ABACUS\", " << nCol << ", " << nRow
          << ", qrtype, rhs, NULL, obj, mstart, colSize, mrwind"
          << ", dmatval, bdl, bdu);\n  }" << endl;
#endif
    if ( loadprob(master_->problemName()->string(), nCol, nRow, qrtype,
                  rhs, NULL, obj, mstart, colSize, mrwind, dmatval, bdl, bdu) )
      printErrnoAndExit("ABA_XPRESSIF::_initialize(): loadprob() failed.");
    lpSolverTime_.stop();

    // free memory
    delete[] qrtype;
    delete[] rhs;
    delete[] obj;
    delete[] bdl;
    delete[] bdu;
    delete[] colSize;
    delete[] mstart;
    delete[] mrwind;
    delete[] dmatval;
  }


@ The function |_loadBasis()| loads a basis for the XPRESS optimizer.
  \BeginArg
  \docarg{|lpVarStat|}{An array storing the status of the columns.}
  \docarg{|lpVarStat|}{An array storing the status of the slack variables.}
  \EndArg

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_loadBasis(ABA_ARRAY<ABA_LPVARSTAT::STATUS> &lpVarStat,
                           ABA_ARRAY<ABA_SLACKSTAT::STATUS> &slackStat)
  {
    int nRow, nCol;
    getipv(N_NROW, &nRow);
    getipv(N_NCOL, &nCol);
    if (nRow>slackStat.size() || nCol>lpVarStat.size()) {
      master_->err() << "ABA_XPRESSIF::_loadBasis(): Number of rows and/or "
                     << "columns in XPRESS and ABACUS differs!" << endl;
      master_->err() << "XPRESS: rows=" << nRow << " columns=" << nCol << endl;
      master_->err() << "ABACUS: rows=" << slackStat.size()
                     << " columns=" << lpVarStat.size() << endl;
      exit(Fatal);
    }

    int *rstatus = new int[nRow];
    int *cstatus = new int[nCol];

    for (int i = 0; i < nRow; i++) {
      rstatus[i] = slackStat2xpress[slackStat[i]];
      if (rstatus[i]<0) {
        master_->err() << "ABA_XPRESSIF::slackStat2xpress(" << slackStat[i] << "): ";
        master_->err() << "corresponding XPRESS status unknown" << endl;
        exit (Fatal);
      }
    }
#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < nCol; i++) {
#else
    for (int i = 0; i < nCol; i++) {
#endif
      cstatus[i] = lpVarStat2xpress[lpVarStat[i]];
      if (cstatus[i]<0) {
        master_->err() << "ABA_XPRESSIF::lpVarStat2xpress(" << lpVarStat[i] << "): ";
        master_->err() << "corresponding XPRESS status unknown" << endl;
        exit (Fatal);
      }
    }

    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("rstatus", rstatus, nRow);
    decl("cstatus", cstatus, nCol);
    trace << "    loadbasis(rstatus, cstatus);\n  }\n";
#endif
    if (::loadbasis(rstatus, cstatus) ) {
      master_->err() << "WARNING: ABA_XPRESSIF::loadbasis() : call of ";
      master_->err() << "loadbasis() failed." << endl;
    }
    lpSolverTime_.stop();

    delete [] rstatus;
    delete [] cstatus;
  }

@ The function |reinitialize()| reallocates the internal XPRESS
  data structures.
    \BeginArg
     \docarg{|marSizeNew|}{The new maximal number of rows.}
     \docarg{|macSizeNew|}{The new maximal number of columns.}
     \docarg{|matSizeNew|}{The new maximal number of nonzero elements of
                          the constraint matrix (not including the right hand
                          side and the bounds of the columns).}

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::reinitialize(int marSizeNew, int macSizeNew, int matSizeNew)
  {
    master_->out() << "XPRESS: reinitializing ..." << endl;
    lpMasterXpress_->countXpressReinitialization();

    int nRow, nCol, nnz, sizeRow, sizeCol, sizeNnz;
    getipv(N_NROW, &nRow);
    getipv(N_NCOL, &nCol);
    getipv(N_NELEM, &nnz);
    getipv(N_NRSPAR, &sizeRow); sizeRow+=nRow;
    getipv(N_NCSPAR, &sizeCol); sizeCol+=nCol;
    getipv(N_NMSPAR, &sizeNnz); sizeNnz+=nnz;

    if (marSizeNew<sizeRow) marSizeNew = sizeRow;
    if (macSizeNew<sizeCol) macSizeNew = sizeCol;
    if (matSizeNew<sizeNnz) matSizeNew = sizeNnz;

    if (marSizeNew==sizeRow && macSizeNew==sizeCol && matSizeNew==sizeNnz) {
      master_->out() << "        (no reallocation needed)" << endl;
      return;
    }

#ifdef ABACUSSAFE
    assert(nCol>0);
#endif

    // read the LP
    char   *qrtype  = new char[nRow];
    double *rhs     = new double[nRow];
    double *obj     = new double[nCol];
    double *bdl     = new double[nCol];
    double *bdu     = new double[nCol];
    int    *mstart  = new int[nCol+1];
    int    *mrwind  = new int[nnz+1];
    double *dmatval = new double[nnz+1];

    int err = 0;
    lpSolverTime_.start();
    if (nRow>0) {
      err |= ::getrowtype(qrtype, 0, nRow-1);
      err |= ::getrhs(rhs, 0, nRow-1);
    }
    err |= ::getobj(obj, 0, nCol-1);
    err |= ::getbdl(bdl, 0, nCol-1);
    err |= ::getbdu(bdu, 0, nCol-1);
    int nels;
    err |= ::getcols(mstart, mrwind, dmatval, nnz+1, &nels, 0, nCol-1);
    lpSolverTime_.stop();
    if (err)
      printErrnoAndExit("ABA_XPRESSIF::_reinitialize(): failed "
                        "when reading the matrix.");
#ifdef ABACUSSAFE
    assert(nels==nnz);
#endif
    // read the basis if not available
    if (basisStatus_ == Missing) {
      delete [] rstatus_;
      delete [] cstatus_;
      rstatus_ = new int[nRow];
      cstatus_ = new int[nCol];

      // read the solution from memory rather than from the solution file
      int ifmem;
      geticv(N_IFMEM, &ifmem);
      seticv(N_IFMEM, ifmem|8);

      lpSolverTime_.start();
      if (::getbasis(rstatus_, cstatus_))
        basisStatus_ = Missing;
      else
        basisStatus_ = Available;
#ifdef ABACUSSAFE
      for (int i=0; i<nRow; i++)
        if (!(rstatus_[i]>=0 && rstatus_[i]<=2)) {
          master_->err() << "DEBUG: ABA_XPRESSIF::reinitialize() : error in "
                            "getbasis(rstatus_, cstatus_)" << endl;
          master_->err() << "DEBUG: rstatus_[" << i << "]=" << rstatus_[i] << endl;
          basisStatus_ = Missing;
          break;
        }
#ifdef ABACUS_NO_FOR_SCOPE
      for (i=0; i<nCol; i++)
#else
      for (int i=0; i<nCol; i++)
#endif
        if (!(cstatus_[i]>=0 && cstatus_[i]<=2)) {
          master_->err() << "DEBUG: ABA_XPRESSIF::reinitialize() : error in "
                            "getbasis(rstatus_, cstatus_)" << endl;
          master_->err() << "DEBUG: cstatus_[" << i << "]=" << cstatus_[i] << endl;
          basisStatus_ = Missing;
          break;
        }
#ifdef XPRESS_TRACE
      if (basisStatus_ == Missing) {
        trace << "/* getbasis() -> rstatus:";
        for (int i=0; i<nRow; i++)
          trace << " " << rstatus_[i];
        trace << "   getbasis() -> cstatus:";
        for (int i=0; i<nCol; i++)
          trace << " " << cstatus_[i];
        trace << "\n*/\n";
        printErrnoAndExit("ABA_XPRESSIF::_reinitialize(): failed.");
      }
#endif
#endif
      lpSolverTime_.stop();
    }

    // reserve additional space
#ifdef XPRESS_TRACE
    trace << "  seticv(N_NRXTRA, " << marSizeNew-nRow+1 << ");\n";
    trace << "  seticv(N_NCXTRA, " << macSizeNew-nCol << ");\n";
    trace << "  seticv(N_NMXTRA, " << matSizeNew-nnz << ");\n";
#endif
    seticv(N_NRXTRA, marSizeNew-nRow + 1);
    seticv(N_NCXTRA, macSizeNew-nCol);
    seticv(N_NMXTRA, matSizeNew-nnz);
    // TODO:
    // N_M_S, N_NEXTRA, N_IFSCAL, N_IFNRW, N_ZTOLDA

    // reload the problem into the XPRESS data structures
    lpSolverTime_.start();
    if (nRow == 0)
      seticv(N_M_S, 100000);
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("qrtype", qrtype, nRow);
    decl("rhs", rhs, nRow);
    decl("obj", obj, nCol);
    decl("bdl", bdl, nCol);
    decl("bdu", bdu, nCol);
    decl("mstart", mstart, nCol+1);
    decl("mrwind", mrwind, nnz);
    decl("dmatval", dmatval, nnz);
    trace << "    loadprob(\"ABACUS\", " << nCol << ", " << nRow
          << ", qrtype, rhs, NULL, obj, mstart, NULL, mrwind"
          << ", dmatval, bdl, bdu);\n  }\n";
#endif
    if ( loadprob(master_->problemName()->string(), nCol, nRow, qrtype, rhs,
                  NULL, obj, mstart, NULL, mrwind, dmatval, bdl, bdu) )
      printErrnoAndExit("ABA_XPRESSIF::_reinitialize(): loadprob() failed.");
    lpSolverTime_.stop();

    // restore the basis if available
    if (basisStatus_ == Available) {
      lpSolverTime_.start();
#ifdef XPRESS_TRACE
      trace << "  {\n";
      decl("rstatus_", rstatus_, nRow);
      decl("cstatus_", cstatus_, nCol);
      trace << "    loadbasis(rstatus_, cstatus_);\n  }\n";
#endif
      if (::loadbasis(rstatus_, cstatus_) ) {
        master_->err() << "WARNING: ABA_XPRESSIF::reinitialize() : call of ";
        master_->err() << "loadbasis() failed." << endl;
      }
      lpSolverTime_.stop();
    }

    // free memory
    delete[] qrtype;
    delete[] rhs;
    delete[] obj;
    delete[] bdl;
    delete[] bdu;
    delete[] mstart;
    delete[] mrwind;
    delete[] dmatval;
  }


@ The function |_getInfeas()| be can be called if the last linear program
  has been solved with the dual simplex method and is infeasible.
  In this case it computes the infeasible basic variable or constraint
  and the corresponding row |bInvRow| of the basis inverse.
  Either |infeasRow| or |infeasCol| is nonnegative. Then this number
  refers to an infeasible variable or slack variable, respectively.
  The function returns 0 if it is successful, 1 otherwise.

  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_getInfeas(int &infeasRow, int &infeasCol, double *bInvRow)
  {
#if 0
    infeasRow = -1;
    infeasCol = -1;

    int junb;
    if (getunb(&junb))
      printErrnoAndExit("ABA_XPRESSIF::_getInfeas(): getunb() failed.");
    int nRow;
    getipv(N_NROW, &nRow);
    if (junb<nRow)
      infeasRow = junb;
    else
      infeasCol = junb - nRow;

    master_->err() << "WARNING: ABA_XPRESSIF::_getInfeas() does not return values " << endl;
    master_->err() << "in parameter bInvRow, since they are not provided by XRESS!" << endl;

    return 0;
#endif

    master_->err() << "ABA_XPRESSIF::_getInfeas(): not available" << endl;
    return 1;
  }


@ The function |_remRows()| removes the rows listed in |ind|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_remRows(ABA_BUFFER<int> &ind)
  {
    const int n = ind.number();
    int *index = new int[n];

    for (int i = 0; i < n; i++)
      index[i] = ind[i];

    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("index", index, n);
    trace << "    delrows(" << n << ", index);\n  }\n";
#endif
    if (::delrows(n, index) )
      printErrnoAndExit("ABA_XPRESSIF::_remRows(): delrows() failed.");
    lpSolverTime_.stop();

#ifdef ABACUSSAFE
    nRow_-=n;
#endif

    delete [] index;
  }
  
  
@ The function |_addRows()| adds the |rows| to the linear program.
  It implements the pure virtual function of the base class |LP|.

  If more rows are added than the current maximal number of rows we
  enlarge the maximal number of rows to the new number of rows plus
  a ten percent reserve.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_addRows(ABA_BUFFER<ABA_ROW*> &rows)
  {
    int newRow = rows.number();
    int newNnz = 0;
    for(int i = 0; i < newRow; i++)
      newNnz += rows[i]->nnz();

    int freeRow, freeNnz;
    getipv(N_NRSPAR, &freeRow);
    getipv(N_NMSPAR, &freeNnz);

    if (newRow > freeRow || newNnz > freeNnz) {
      int nRow, nCol, freeCol, nnz;
      getipv(N_NROW, &nRow);
      getipv(N_NCOL, &nCol);
      getipv(N_NCSPAR, &freeCol);
      getipv(N_NELEM, &nnz);
      reinitialize(nRow+newRow, nCol+freeCol, nnz+newNnz);
    }

    // at this point we assume that enough space is available

    char   *qrtype  = new char[newRow];
    double *rhs     = new double[newRow];
    int    *mstart  = new int[newRow];      // start of new rows
    int    *mclind  = new int[newNnz];
    double *dmatval = new double[newNnz];

    // fill in the new rows in the arrays
    int ind = 0;
    for(int r = 0; r < newRow; r++) {
      ABA_ROW *row = rows[r];
      qrtype[r] = csense2xpress[row->sense()->sense()];
      rhs[r] = row->rhs();
      const int rowNnz = row->nnz();
      mstart[r] = ind;
      for(int i=0; i<rowNnz; i++) {
        mclind[ind] = row->support(i);
        dmatval[ind] = row->coeff(i);
        ind++;
      }
    }

    // add the rows to the XPRESS data structures
    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("qrtype", qrtype, newRow);
    decl("rhs", rhs, newRow);
    decl("mstart", mstart, newRow);
    decl("mclind", mclind, newNnz);
    decl("dmatval", dmatval, newNnz);
    trace << "    addrows(" << newRow << ", " << newNnz
          << ", qrtype, rhs, NULL, mstart, mclind, dmatval);\n  }\n";
#endif
    if (::addrows(newRow, newNnz, qrtype, rhs, NULL,
                 mstart, mclind, dmatval) )
      printErrnoAndExit("ABA_XPRESSIF::_addRows(): addrows() failed.");
    lpSolverTime_.stop();

#ifdef ABACUSSAFE
    nRow_+=newRow;
#endif

    // free memory
    delete[] qrtype;
    delete[] rhs;
    delete[] mstart;
    delete[] mclind;
    delete[] dmatval;
  }
  

@ The function |_remCols()| removes the columns listed in |vars|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_remCols(ABA_BUFFER<int> &vars)
  {
    const int n = vars.number();
    int *index = new int[n];

    for (int i = 0; i < n; i++)
      index[i] = vars[i];

    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("index", index, n);
    trace << "    delcols(" << n << ", index);\n  }\n";
#endif
    if (::delcols(n, index) )
      printErrnoAndExit("ABA_XPRESSIF::_remCols(): delcols() failed.");
    lpSolverTime_.stop();

#ifdef ABACUSSAFE
    nCol_-=n;
#endif

    delete [] index;
  }


@ The function |_addCols()| adds the columns |cols| to the linear program.
  It implements the pure virtual function of the base class |LP|.

  If more columns are added than the current maximal number of columns we
  enlarge the maximal number of columns to the new number of columns plus
  a ten percent reserve.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_addCols(ABA_BUFFER<ABA_COLUMN*> &cols)
  {
    int newCol = cols.number();
    int newNnz = 0;
    for(int i = 0; i < newCol; i++)
      newNnz += cols[i]->nnz();

    int freeCol, freeNnz;
    getipv(N_NCSPAR, &freeCol);
    getipv(N_NMSPAR, &freeNnz);

    if (newCol > freeCol || newNnz > freeNnz) {
      int nRow, freeRow, nCol, nnz;
      getipv(N_NROW, &nRow);
      getipv(N_NRSPAR, &freeRow);
      getipv(N_NCOL, &nCol);
      getipv(N_NELEM, &nnz);
      reinitialize(nRow+freeRow, nCol+newCol, nnz+newNnz);
    }

    // at this point we assume that enough space is available

    double *obj     = new double[newCol];
    double *bdl     = new double[newCol];
    double *bdu     = new double[newCol];
    int    *mstart  = new int[newCol];      // start of new columns
    int    *mrwind  = new int[newNnz];
    double *dmatval = new double[newNnz];

    // fill in the new columns in the arrays
    int ind = 0;
    for (int c=0; c<newCol; c++) {
      ABA_COLUMN *col = cols[c];
      obj[c] = col->obj();
      bdl[c] = col->lBound();
      bdu[c] = col->uBound();
      const int colNnz = col->nnz();
      mstart[c] = ind;
      for(int i=0; i<colNnz; i++) {
        mrwind[ind] = col->support(i);
        dmatval[ind] = col->coeff(i);
        ind++;
      }
    }

    // add the columns to the XPRESS data structures
    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("obj", obj, newCol);
    decl("bdl", bdl, newCol);
    decl("bdu", bdu, newCol);
    decl("mstart", mstart, newCol);
    decl("mrwind", mrwind, newNnz);
    decl("dmatval", dmatval, newNnz);
    trace << "    addcols(" << newCol << ", " << newNnz
          << ", obj, mstart, mrwind, dmatval, bdl, bdu);\n  }\n";
#endif
    if (::addcols(newCol, newNnz, obj, mstart, mrwind, dmatval, bdl, bdu) )
      printErrnoAndExit("ABA_XPRESSIF::_addCols(): allcols() failed.");
    lpSolverTime_.stop();

#ifdef ABACUSSAFE
    nCol_+=newCol;
#endif

    // free memory
    delete[] obj;
    delete[] bdl;
    delete[] bdu;
    delete[] mstart;
    delete[] mrwind;
    delete[] dmatval;
  }



@ The function |changeRhs()| sets the right hand side of the linear
  program to |newRhs|. This array must be have at least length of the
  number of rows.
  This function implements the pure virtual function of the base class
  |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_changeRhs(ABA_ARRAY<double> &newRhs)
  {
    int nRow;
    getipv(N_NROW, &nRow);

    int *mindex = new int[nRow];
    double *rhs = new double[nRow];

    for (int i = 0; i < nRow; i++) {
      mindex[i] = i;
      rhs[i] = newRhs[i];
    }

    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("mindex", mindex, nRow);
    decl("rhs", rhs, nRow);
    trace << "    chgrhs(" << nRow << ", mindex, rhs);\n  }\n";
#endif
    if (::chgrhs(nRow, mindex, rhs) )
      printErrnoAndExit("ABA_XPRESSIF::_changeRhs(): chgrhs() failed.");
    lpSolverTime_.stop();

    delete [] mindex;
    delete [] rhs;
  }
      
@ The function |_changeLBound()| sets the lower bound of column |i| to
  |newLb|.
  It implements the pure virtual function of the base class |LP|.
                      
@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_changeLBound(int i, double newLb)
  {
    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("index", &i, 1);
    decl("bound", &newLb, 1);
    trace << "    chgbds(1, index, \"L\", bound);\n  }\n";
#endif
    if (::chgbds(1, &i, "L", &newLb) )
      printErrnoAndExit("ABA_XPRESSIF::_changeLBound(): chgbds() failed.");
    lpSolverTime_.stop();
  }

@ The function |_changeUBound()| sets the upper bound of column |i| to
  |newLb|.
  It implements the pure virtual function of the base class |LP|.
                      
@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_changeUBound(int i, double newUb)
  {
    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    trace << "  {\n";
    decl("index", &i, 1);
    decl("bound", &newUb, 1);
    trace << "    chgbds(1, index, \"U\", bound);\n  }\n";
#endif
    if (::chgbds(1, &i, "U", &newUb) )
      printErrnoAndExit("ABA_XPRESSIF::_changeUBound(): chgbds() failed.");
    lpSolverTime_.stop();
  }

@ The function |_pivotSlackVariableIn()| pivots the slack variables
  stored in the buffer |rows| into the basis. This function defines
  the pure virtual function of the base class |LP|.

  \BeginReturn
  \docarg{|0|}{All variables could be pivoted in,}
  \docarg{|1|}{otherwise.}
  \EndArg

  \BeginArg
  \docarg{|rows|}{The numbers of the slack variables that should be
                  pivoted in.}
  \EndArg

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_pivotSlackVariableIn(ABA_BUFFER<int> &rows)
  {
    int count = 0;
    const int rowsNumber = rows.number();

    for (int i = 0; i < rowsNumber; i++) {
      int colout;   // Column to leave basis
      int nout;     // Number of cols found to leave
      if (getpivots(rows[i], &colout, NULL, NULL, &nout, 1))
        printErrnoAndExit("ABA_XPRESSIF:_pivotSlackVariableIn(): "
                          "getpivots() faild");
      if (nout==1) {
        if (pivot(rows[i], colout))
          printErrnoAndExit("ABA_XPRESSIF:_pivotSlackVariableIn(): "
                            "pivot() faild");
        count++;
      }
    }
    return count;
  }


@ The private function |_optimize()| optimizes the LP
  by using the primal or dual simplex method or
  the barrier method.

  \BeginArg
  \docarg{|method|}{" ", "d", "b" for primal, dual, barrier method.}
  \EndArg

@<the implementation of the members of |ABA_XPRESSIF|@>=
  ABA_LP::OPTSTAT ABA_XPRESSIF::_optimize(const char *method)
  {
    int ifimem;
    geticv(N_IFMEM, &ifimem);
    seticv(N_IFMEM, ifimem|12);    // do not write the solution to a file

    // A bunch of control variables may be set here
    seticv(N_IPHSF, lpMasterXpress_->xpressPricing_);

    int nRow;
    getipv(N_NROW, &nRow);
    if (nRow == 0) {               // add a dummy row before optimizing LP
      double zero = 0;
      if ( ::addrows(1, 0, "G", &zero, NULL, &nRow, NULL, NULL) )
        printErrnoAndExit("ABA_XPRESSIF::_optimize(): can't add dummy row.");
    }

    lpSolverTime_.start();
    switch (optsense_.sense()) {
      case ABA_OPTSENSE::Min:
#ifdef XPRESS_TRACE
        trace << "  minim(\"" << method << "\");\n";
#endif
        if (::minim(method))
          printErrnoAndExit("ABA_XPRESSIF::_optimize(): minim() failed.");
        break;
      case ABA_OPTSENSE::Max:
#ifdef XPRESS_TRACE
        trace << "  maxim(\"" << method << "\");\n";
#endif
        if (::maxim(method))
          printErrnoAndExit("ABA_XPRESSIF::_optimize(): maxim() failed.");
        break;
      case ABA_OPTSENSE::Unknown: {
        master_->err() << "ABA_XPRESSIF::_optimize(): "
                          "optimization sense unknown." << endl;
        exit(Fatal);
      }
    }
    lpSolverTime_.stop();

    getSol();

    int status;
    getipv(N_STATUS, &status);
    //  1       optimal
    //  2       infeasible
    //  3       objective worse than cutoff
    //  4       unfinished
    //  5       unbounded
    //  6       cutoff in dual

    if (nRow == 0)
      if ( ::delrows(1, &nRow) )
        printErrnoAndExit("ABA_XPRESSIF::_optimize(): can't delete dummy row.");

    switch (status) {
      case 1: return Optimal;
      case 2: return Infeasible;
      case 5: return Unbounded;
    }
    master_->err() << "ABA_XPRESSIF::_optimize(): XPRESS status is ";
    master_->err() << status << endl;
    return Error;
  }


@ The function |_primalSimplex()| calls the primal simplex method.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  ABA_LP::OPTSTAT ABA_XPRESSIF::_primalSimplex()
  {
    ABA_LP::OPTSTAT optstat;

    optstat = _optimize(" ");

    if (yValStatus_==Missing && (optstat==Infeasible || optstat==Unbounded)) {
      int pres;
      geticv(N_IFPRES, &pres);
      if (pres!=0) {
        master_->out() << "XRESS: No dual solution available. Trying again"
                          " with presolver turned off." << endl;
        seticv(N_IFPRES, 0);
        optstat = _optimize(" ");

        seticv(N_IFPRES, pres);
      }
    }

    return optstat;
  }

@ The function |_dualSimplex()| calls the dual simplex method.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  ABA_LP::OPTSTAT ABA_XPRESSIF::_dualSimplex()
  {
    ABA_LP::OPTSTAT optstat;

    optstat = _optimize("d");

    if (yValStatus_==Missing && (optstat==Infeasible || optstat==Unbounded)) {
      int pres;
      geticv(N_IFPRES, &pres);
      if (pres!=0) {
        master_->out() << "XRESS: No dual solution available. Trying again"
                          " with presolver turned off." << endl;
        seticv(N_IFPRES, 0);
        optstat = _optimize("d");

        seticv(N_IFPRES, pres);
      }
    }

    return optstat;
  }
  


@ The function |_barrier()| calls the barrier method.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  ABA_LP::OPTSTAT ABA_XPRESSIF::_barrier(bool doCrossover)
  {
    ABA_LP::OPTSTAT optstat;

    if(!doCrossover) {
        master_->err() << "Sorry, XPRESS implements no barrier without crossover." << endl; 
        master_->err() << "Doing barrier with crossover instead." <<       endl; 
    }

    optstat = _optimize("b");

    if (yValStatus_==Missing && (optstat==Infeasible || optstat==Unbounded)) {
      int pres;
      geticv(N_IFPRES, &pres);
      if (ifpres!=0) {
        master_->out() << "XRESS: No dual solution available. Trying again"
                          " with presolver turned off." << endl;
        seticv(N_IFPRES, 0);
        optstat = _optimize("b");

        seticv(N_IFPRES, pres);
      }
    }

    return optstat;
  }


@ The function |_sense()| returns the sense of the optimization.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  ABA_OPTSENSE ABA_XPRESSIF::_sense() const
  {
    return optsense_;
  }

@ The function |_sense()| changes the optimization sense.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_sense(const ABA_OPTSENSE &newSense)
  {
    optsense_ = newSense;
  }


@ The function |_nRow()| returns the number of rows of the linear program.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_nRow() const
  {
    int nRow;
    getipv(N_NROW, &nRow);

#ifdef ABACUSSAFE
    if (nRow!=nRow_) {
      master_->err() << "ABA_XPRESSIF::_nRow()(): Number of rows "
                     << "in XPRESS and ABACUS differs!" << endl;
      master_->err() << "XPRESS=" << nRow << ", ABACUS=" << nRow_ << endl;
      exit(Fatal);
    }
#endif

    return nRow;
  }

@ The function |_maxRow()| is required although Xpress internally
  works without any row limit because some data members of our
  interface have a fixed size.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_maxRow() const
  {
    int nRow, freeRow;
    getipv(N_NROW, &nRow);
    getipv(N_NRSPAR, &freeRow);
    return nRow+freeRow;
  }

@ The function |_nCol()| returns the number of columns of the linear
  program.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_nCol() const
  {
    int nCol;
    getipv(N_NCOL, &nCol);

#ifdef ABACUSSAFE
    if (nCol!=nCol_) {
      master_->err() << "ABA_XPRESSIF::_nCol()(): Number of columns "
                     << "in XPRESS and ABACUS differs!" << endl;
      master_->err() << "XPRESS=" << nCol << ", ABACUS=" << nCol_ << endl;
      exit(Fatal);
    }
#endif

    return nCol;
  }

@ The function |_maxCol()| is required although Soplex internally
  works without any row limit because some data members of our
  interface have a fixed size.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_maxCol() const
  {
    int nCol, freeCol;
    getipv(N_NCOL, &nCol);
    getipv(N_NCSPAR, &freeCol);
    return nCol+freeCol;
  }


@ The function |_obj()| returns the objective function coefficient of
  column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_obj(int i) const
  {
    double obj;
    if (::getobj(&obj, i, i))
      printErrnoAndExit("ABA_XPRESSIF::_obj(): getobj() failed.");
    return obj;
  }

@ The function |_lBound()| returns the lower bound of column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_lBound(int i) const
  {
    double bound;
    if (::getbdl(&bound, i, i))
      printErrnoAndExit("ABA_XPRESSIF::_lBound(): getbdl() failed.");
    return bound;
  }

@ The function |_uBound()| returns the upper bound of column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_uBound(int i) const
  {
    double bound;
    if (::getbdu(&bound, i, i))
      printErrnoAndExit("ABA_XPRESSIF::_uBound(): getbdu() failed.");
    return bound;
  }

@ The function |_rhs()| returns the right hand side of row |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_rhs(int i) const
  {
    double rhs;
    if (::getrhs(&rhs, i, i))
      printErrnoAndExit("ABA_XPRESSIF::_rhs(): getrhs() failed.");
    return rhs;
  }

@ The function |_row()| stores a copy of row |i| in |r|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_row(int i, ABA_ROW &r) const
  {
    int nCol, nnz;
    getipv(N_NCOL, &nCol);

    int    mstart[2];
    int    *mclind  = new int[nCol];
    double *dmatval = new double[nCol];

    if (::getrows(mstart, mclind, dmatval, nCol, &nnz, i, i))
      printErrnoAndExit("ABA_XPRESSIF::_row(): getrows() failed.");

#ifdef ABACUSSAFE
    assert(mstart[0]==0 && mstart[1]==nnz);
#endif

    r.clear();
    
    for (int j = 0; j < nnz; j++)
      r.insert(mclind[j], dmatval[j]);

    char rowtype;
    if (::getrowtype(&rowtype, i, i))
      printErrnoAndExit("ABA_XPRESSIF::_row(): getrowtype() failed.");
    switch (rowtype) {
      case 'L': r.sense(ABA_CSENSE::Less); break;
      case 'E': r.sense(ABA_CSENSE::Equal); break;
      case 'G': r.sense(ABA_CSENSE::Greater); break;
      default:
        printErrnoAndExit("ABA_XPRESSIF::_row(): getrowtype() returns invalid type.");
    }

    double rhs;
    if (::getrhs(&rhs, i, i))
      printErrnoAndExit("ABA_XPRESSIF::_row(): getrhs() failed.");
    r.rhs(rhs);

    delete [] mclind;
    delete [] dmatval;
  }


@ The function |_nnz()| returns the number of nonzero elements in the
  constraint matrix (not including the right hand side).
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_nnz() const
  {
    int nnz;
    getipv(N_NELEM, &nnz);
    return nnz;
  }

@ The function |_value()| returns the optimum value of the linear program.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_value() const
  {
    double value;
    getdpv(N_DOBJVL, &value);
    return value;
  }

@ The function |_xVal()| returns the value of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_xVal(int i)
  {
    return x_ ? x_[i] : 0;
  }

@ The function |_barXVal()| returns the value of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_barXVal(int i)
  {
    return x_ ? x_[i] : 0;
  }

@ The function |_reco()| returns the reduced cost of the column |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_reco(int i)
  {
    return dj_ ? dj_[i] : 0;
  }

@ The function |_yVal()| returns the value of the dual column of the
  row |i|. XPRESS returns a negated dual value.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_yVal(int i)
  {
    return dual_ ? -dual_[i] : 0;
  }

@ The function |_slack()| returns the value of the slack column of the
  row  |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  double ABA_XPRESSIF::_slack(int i)
  {
    return slack_ ? slack_[i] : 0;
  }

@ The function |_lpVarStat()| returns the status of the column |i|.
  It implements the pure virtual function of the base class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  ABA_LPVARSTAT::STATUS ABA_XPRESSIF::_lpVarStat(int i)
  {
    return cstatus_ ? xpress2lpVarStat[cstatus_[i]] : ABA_LPVARSTAT::Unknown;
  }

@ The function |_slackStat()| returns the status of the slack column |i|.
  It implements the pure virtual function of the base class |LP|.
  
@<the implementation of the members of |ABA_XPRESSIF|@>=
  ABA_SLACKSTAT::STATUS ABA_XPRESSIF::_slackStat(int i)
  {
    return rstatus_ ? xpress2slackStat[rstatus_[i]] : ABA_SLACKSTAT::Unknown;
  }


@ The function |getSol()| extracts the solution, i.e., the value,
  the status, the values of the variables, slack variables, and dual
  variables, the reduced costs, and the statuses of the variables
  and slack variables form the internal Soplex data structure.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::getSol()
  {
    delete [] x_;
    delete [] slack_;
    delete [] dual_;
    delete [] dj_;
    delete [] rstatus_;
    delete [] cstatus_;

    int nRow, nCol;
    getipv(N_NROW, &nRow);
    getipv(N_NCOL, &nCol);

    x_       = new double[nCol];
    slack_   = new double[nRow];
    dual_    = new double[nRow];
    dj_      = new double[nCol];
    rstatus_ = new int[nRow];
    cstatus_ = new int[nCol];

    lpSolverTime_.start();
#ifdef XPRESS_TRACE
    for (int i=0; i<nRow; i++)
      slack_[i]=dual_[i]=rstatus_[i]=0;
    for (int i=0; i<nCol; i++)
      x_[i]=dj_[i]=cstatus_[i]=0;
#endif
    if (::solution(x_, slack_, dual_, dj_))
      xValStatus_ = recoStatus_ = yValStatus_ = slackStatus_ = Missing;
    else
      xValStatus_ = recoStatus_ = yValStatus_ = slackStatus_ = Available;
#ifdef XPRESS_TRACE
    trace << "  { /* solution(x_, slack_, dual_, dj_) */\n";
    trace << "    /* _value() = " << _value() << ", status = ";
    trace << (slackStatus_==Missing?"Missing":"Available") << " */\n";
    decl("x_", x_, nCol);
    decl("slack_", slack_, nRow);
    decl("dual_", dual_, nRow );
    decl("dj_", dj_, nCol);
    trace << "  }\n";
#endif

    if (::getbasis(rstatus_, cstatus_))
      basisStatus_ = Missing;
    else
      basisStatus_ = Available;
#ifdef XPRESS_TRACE
    trace << "  { /* getbasis(rstatus_, cstatus_) */\n";
    trace << "    /* status = ";
    trace << (basisStatus_==Missing?"Missing":"Available") << " */\n";
    decl("rstatus_", rstatus_, nRow );
    decl("cstatus_", cstatus_, nCol);
    trace << "  }\n";
#endif
    lpSolverTime_.stop();
  }


@ The function |_rowRealloc()| reallocates the internal memory such that
  |newSize| rows can be stored.
  It implements the corresponding pure virtual function of the base
  class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_rowRealloc(int newSize)
  {
    int nRow, nCol, freeCol, nnz, freeNnz;
    getipv(N_NROW, &nRow);
    getipv(N_NCOL, &nCol);
    getipv(N_NCSPAR, &freeCol);
    getipv(N_NELEM, &nnz);
    getipv(N_NMSPAR, &freeNnz);
    
    if (freeNnz<500)
        freeNnz = 500;
    if (nRow)
        freeNnz += 2*newSize*(nnz/nRow);

    reinitialize(newSize, nCol+freeCol, nnz+freeNnz);
  }


@ The function |_colRealloc()| reallocates the internal memory such that
  |newSize| columns can be stored.
  It implements the corresponding pure virtual function of the base
  class |LP|.

@<the implementation of the members of |ABA_XPRESSIF|@>=
  void ABA_XPRESSIF::_colRealloc(int newSize)
  {
    int nRow, nCol, freeRow, nnz, freeNnz;
    getipv(N_NROW, &nRow);
    getipv(N_NCOL, &nCol);
    getipv(N_NRSPAR, &freeRow);
    getipv(N_NELEM, &nnz);
    getipv(N_NMSPAR, &freeNnz);

    if (freeNnz<500)
        freeNnz = 500;
    if (nCol)
        freeNnz += 2*newSize*(nnz/nCol);

    reinitialize(nRow+freeRow, newSize, nnz+freeNnz);
  }


@ The function |_setSimplexIterationLimit()| changes the iteration
  limit of the Simplex algorithm.
  This function defines a pure virtual function of the base class
  |LP|.

  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be set,}
  \docarg{|1|}{otherwise.}
  \EndReturn

  \BeginArg
  \docarg{|limit|}{The new value of the iteration limit.}
   \EndArg

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_setSimplexIterationLimit(int limit)
  {
    if (limit==0)
      seticv(N_IFPRES, ifpres );
    else
      seticv(N_IFPRES, 0 );
      
    seticv(N_ITRLIM, limit);
    return 0;
  }

@ The function |_getSimplexIterationLimit()| defines a pure virtual
  function of the base class |LP|.

  \BeginReturn 
  \docarg{|0|}{If the iteration limit could be get,}
  \docarg{|1|}{otherwise.} 
  \EndReturn

  \BeginArg
  \docarg{|limit|}{Stores the iteration limit if the  return value is 0.}
  \EndArg

@<the implementation of the members of |ABA_XPRESSIF|@>=
  int ABA_XPRESSIF::_getSimplexIterationLimit(int &limit)
  {
    geticv(N_ITRLIM, &limit);
    return 0;
  }

