@q ABACUS - A Branch And CUt System                                          @>
@q Copyright (C) 1995 - 2003                                                 @>
@q University of Cologne, Germany                                            @>
@q                                                                           @>
@q This library is free software; you can redistribute it and/or             @>
@q modify it under the terms of the GNU Lesser General Public                @>
@q License as published by the Free Software Foundation; either              @>
@q version 2.1 of the License, or (at your option) any later version.        @>
@q                                                                           @>
@q This library is distributed in the hope that it will be useful,           @>
@q but WITHOUT ANY WARRANTY; without even the implied warranty of            @>
@q MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         @>
@q Lesser General Public License for more details.                           @>
@q                                                                           @>
@q You should have received a copy of the GNU Lesser General Public          @>
@q License along with this library; if not, write to the Free Software       @>
@q Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA @>

@q $Id: sub.w,v 4.40 2003/08/05 13:55:29 elf Exp $ @>

@q m i ABA_SUB @>

@ The member functions are implemented in the file {\tt sub.cc}.

@(sub.cc@>=
#include <iomanip>
using namespace std;

#ifdef ABACUS_OLD_INCLUDE
#include "master.h"
#include "sub.h"
#include "lpsub.h"
#include "tailoff.h"
#include "constraint.h"
#include "branchrule.h"
#include "lpvarstat.h"
#include "slackstat.h"
#include "bprioqueue.h"
#include "variable.h"
#include "column.h"
#include "poolslot.h"
#include "cutbuffer.h"
#include "opensub.h"
#include "fixcand.h"
#include "setbranchrule.h"
#include "boundbranchrule.h"
#include "active.h"
#include "standardpool.h"
#else
#include "abacus/master.h"
#include "abacus/sub.h"
#include "abacus/lpsub.h"
#include "abacus/tailoff.h"
#include "abacus/constraint.h"
#include "abacus/branchrule.h"
#include "abacus/lpvarstat.h"
#include "abacus/slackstat.h"
#include "abacus/bprioqueue.h"
#include "abacus/variable.h"
#include "abacus/column.h"
#include "abacus/poolslot.h"
#include "abacus/cutbuffer.h"
#include "abacus/opensub.h"
#include "abacus/fixcand.h"
#include "abacus/setbranchrule.h"
#include "abacus/boundbranchrule.h"
#include "abacus/active.h"
#include "abacus/standardpool.h"
#endif

#ifdef ABACUS_PARALLEL
#include "abacus/parmaster.h"
#include "abacus/idmap.h"
#include "abacus/debug.h"
#endif

@q m f Constructor @>
@ The constructor for the root node of the enumeration tree.
@q m e t @>
@q m b a @>
  \BeginArg
     \docarg{|master|}{A pointer to the corresponding master of the optimization.}
     \docarg{|conRes|}{The additional memory allocated for constraints.}
     \docarg{|varRes|}{The additional memory allocated for variables.}
     \docarg{|nnzRes|}{The additional memory allocated for nonzero
                       elements of the constraint matrix.}
     \docarg{|relativeRes|}{If this argument is |true|, then reserve space
                            for variables, constraints, and nonzeros given
                            by the previous three arguments, is given in
                            percent of the original numbers. Otherwise,
                            the numbers are interpreted as absolute values
                            (casted to integer). The default value is |true|.}
     \docarg{|constraints|}{The pool slots of the initial constraints.
                            If the value is 0, then the constraints
                            of the default constraint pool are taken.
                            The default value is 0.}
     \docarg{|variables|}{The pool slots of the initial variables.
                          If the value is 0, then the variables of the
                          default variable pool are taken.
                          The default value is 0.}
   \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  ABA_SUB::ABA_SUB(
               ABA_MASTER *master,
               double conRes, 
               double varRes, 
               double nnzRes, 
               bool relativeRes,
               ABA_BUFFER<ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *> *constraints,
               ABA_BUFFER<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *> *variables) 
@q m e f @>
    : @/
    master_(master), @/
    actCon_(0), @/
    actVar_(0), @/
    father_(0), @/
    lp_(0), @/
    fsVarStat_(0), @/
    lpVarStat_(0), @/
    lBound_(0), @/
    uBound_(0), @/
    slackStat_(0), @/
    tailOff_(0), @/
    dualBound_(master->dualBound()), @/
    nIter_(0), @/
    lastIterConAdd_(0), @/
    lastIterVarAdd_(0), @/
    branchRule_(0), @/
    allBranchOnSetVars_(true), @/
    lpMethod_(ABA_LP::BarrierAndCrossover), @/
    addVarBuffer_(0), @/
    addConBuffer_(0), @/
    removeVarBuffer_(0), @/
    removeConBuffer_(0), @/
    xVal_(0), @/
    yVal_(0), @/
    genNonLiftCons_(false), @/
    level_(1), @/
#ifdef ABACUS_PARALLEL
    fatherId_(0), @/
#else
    id_(1), @/
#endif
    status_(Unprocessed), @/
    sons_(0), @/
    maxIterations_(master->maxIterations()), @/
    nOpt_(0), @/
    relativeReserve_(relativeRes), @/
    varReserve_(varRes), @/
    conReserve_(conRes), @/
    nnzReserve_(nnzRes), @/
    activated_(false), @/
    ignoreInTailingOff_(false), @/
    localTimer_(master)
  {
#ifdef ABACUS_PARALLEL
    master_->parmaster()->incWorkCount();
    id_ = master_->parmaster()->newId(fatherId_);
#endif
    @<initialize the active constraints of the root node@>;
    @<initialize the active variables of the root node@>;
    @<register the subproblem at the master@>;
  }

@ @<initialize the active constraints of the root node@>=
    int maxCon;
    
    if (constraints) {
      @<initialize the active constraints with |constraints|@>;
    }
    else  {
      @<initialize the active constraints with the default constraint pool@>;
    }

    slackStat_ = new @,@[ABA_ARRAY<ABA_SLACKSTAT*>(master_, maxCon)@];

    const int nConstraints = nCon();

    for (int i = 0; i < nConstraints; i++)
      (*slackStat_)[i] = new ABA_SLACKSTAT(master_);

@ @<initialize the active constraints with |constraints|@>=
  if (relativeRes)
    maxCon   = (int) (constraints->number() * (1.0 + conRes/100.0));
  else
    maxCon   = constraints->number() + (int) conRes;
        
  actCon_ = new @,@[ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE>(master_, maxCon)@];
      
  actCon_->insert(*constraints);

@ @<initialize the active constraints with the default constraint pool@>=
  if (relativeRes)
    maxCon  = (int) (master_->conPool()->number() * (1.0 + conRes/100.0));
  else
    maxCon = master_->conPool()->number() + (int) conRes;
        
  actCon_ = new @,@[ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE>(master_, maxCon)@];

  const int nConPool = master_->conPool()->number();

  for (int i = 0; i < nConPool; i++)
    actCon_->insert(master_->conPool()->slot(i));

@ @<initialize the active variables of the root node@>=
  int maxVar;
  if (variables) {
    @<initialize the active variables with |variables|@>;
  }
  else  {
    @<initialize the active variables with the default variable pool@>;
  }

  @<initializes the local variables statuses and the bounds@>;

@ @<initialize the active variables with |variables|@>=
  if (relativeRes)
    maxVar  = (int) (variables->number() * (1.0 + varRes/100.0));
  else
    maxVar = variables->number() + (int) varRes;
  actVar_ = new @,@[ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT>(master_, maxVar)@];
      
  actVar_->insert(*variables);

@ @<initialize the active variables with the default variable pool@>=
  if (relativeRes)
    maxVar  = (int) (master_->varPool()->number() * (1.0 + varRes/100.0));
  else
    maxVar = master_->varPool()->number() + (int) varRes;
  actVar_ = new @,@[ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT>(master_, maxVar)@];

  const int nVarPool = master_->varPool()->number();
  for (int i = 0; i < nVarPool; i++)
    actVar_->insert(master_->varPool()->slot(i));

@ By initializing |*fsVarStat_| with the global status we
  can both handle prefixed variables and enable automatically
  that all fixed variables stay fixed in the case of a restart
  of the optimization process.

@<initializes the local variables statuses and the bounds@>=
  fsVarStat_  = new @,@[ABA_ARRAY<ABA_FSVARSTAT*>(master_, maxVar)@];
  lpVarStat_  = new @,@[ABA_ARRAY<ABA_LPVARSTAT*>(master_, maxVar)@];
  lBound_     = new @,@[ABA_ARRAY<double>(master_, maxVar)@];
  uBound_     = new @,@[ABA_ARRAY<double>(master_, maxVar)@];
                                 
  ABA_VARIABLE *v;

  const int nVariables = nVar();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVariables; i++) {
#else
  for (int i = 0; i < nVariables; i++) {
#endif
    v                 = variable(i);
    (*fsVarStat_)[i]  = new ABA_FSVARSTAT(v->fsVarStat());
    (*lpVarStat_)[i]  = new ABA_LPVARSTAT(master_);
    (*lBound_)[i]     = v->lBound();
    (*uBound_)[i]     = v->uBound();
  }

@ @<register the subproblem at the master@>=
#ifndef ABACUS_PARALLEL
  master_->newSub(level_);
  master_->treeInterfaceNodeBounds(id_, lowerBound(), upperBound());
#endif

@q m f Constructor @>
@ The constructor for non-root nodes of the enumeration tree.
@q m e t @>
@q m b a @>
  \BeginArg
      \docarg{|master|}{A pointer to the corresponding master of the optimization.}
      \docarg{|father|}{A pointer to the father in the enumeration tree.}
      \docarg{|branchRule|}{The rule defining the subspace of the
                           solution space associated with this subproblem.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  ABA_SUB::ABA_SUB(ABA_MASTER *master, ABA_SUB *father, ABA_BRANCHRULE *branchRule) 
@q m e f @>
    : @/
    master_(master), @/
    actCon_(0), @/
    actVar_(0), @/
    father_(father), @/
    lp_(0), @/
    fsVarStat_(0), @/
    lpVarStat_(0), @/
    lBound_(0), @/
    uBound_(0), @/
    slackStat_(0), @/
    tailOff_(0), @/
    dualBound_(father->dualBound_), @/
    nIter_(0), @/
    lastIterConAdd_(0), @/
    lastIterVarAdd_(0), @/
    branchRule_(branchRule), @/
    lpMethod_(ABA_LP::Dual), @/
    addVarBuffer_(0), @/
    addConBuffer_(0), @/
    removeVarBuffer_(0), @/
    removeConBuffer_(0), @/
    xVal_(0), @/
    yVal_(0), @/
    bInvRow_(0), @/
    genNonLiftCons_(false), @/
    level_(father->level() + 1), @/
#ifdef ABACUS_PARALLEL
    fatherId_(father_->id_), @/
#else
    id_(master->nSub() + 1), @/
#endif
    status_(Unprocessed), @/
    sons_(0), @/
    maxIterations_(master->maxIterations()), @/
    nOpt_(0), @/
    relativeReserve_(father_->relativeReserve_), @/
    varReserve_(father_->varReserve_), @/
    conReserve_(father_->conReserve_), @/
    nnzReserve_(father_->nnzReserve_), @/
    activated_(false), @/
    ignoreInTailingOff_(false) , @/
    localTimer_(master)
  {
#ifdef ABACUS_PARALLEL
    master_->parmaster()->incWorkCount();
    id_ = master_->parmaster()->newId(fatherId_);
#endif
    branchRule_->initialize(this);
#ifdef ABACUS_NO_BOOL
    if (father_->allBranchOnSetVars_ && branchRule_->branchOnSetVar())
      allBranchOnSetVars_ = true;
    else 
      allBranchOnSetVars_ = false;
#else
    allBranchOnSetVars_ = father_->allBranchOnSetVars_ &&
                                             branchRule_->branchOnSetVar();
#endif
#ifdef ABACUS_PARALLEL
    @<determine the initial maximal number of constraints and variables@>;
    initializeVars(initialMaxVar);
    initializeCons(initialMaxCon);
    // in the parallel version the father is no longer needed (prohibit access)
    father_ = 0;
#else
    master_->newSub(level_);
    master_->treeInterfaceNodeBounds(id_, lowerBound(), upperBound());
#endif
  }

@q m v Destructor @>
@q m e t @>
@ The destructor only deletes the sons of the node.
  The deletion of allocated memory is already performed when the node
  is fathomed. 
  We recursively call the destructors of all subproblems contained
  in the enumeration tree below this subproblem itself.

  If a subproblem has no sons and its status is either |Unprocessed|
  or |Dormant|, then it is still contained in the set of open
  subproblems, where it is removed from.
  
@(sub.cc@>=
@q m b f @>
  ABA_SUB::~ABA_SUB()
@q m e f @>
  {
    if (sons_) {
      const int nSons = sons_->number();

      for (int i = 0; i < nSons; i++) delete (*sons_)[i];
      delete sons_;
    }
    else if (status_ == Unprocessed || status_ == Dormant)
      master_->openSub()->remove(this);
#ifdef ABACUS_PARALLEL
    // we have to delete the allocated members if the subproblem is deleted
    // directly (e.g. from ABA_BALANCER)
    @<|delete| allocated memory of the fathomed subproblem@>;

    master_->parmaster()->decWorkCount();
#endif
  }


#ifdef ABACUS_PARALLEL

@q m f Constructor @>
@ The message constructor creates the subproblem from an |ABA_MESSAGE|.

  A subproblem can only be received, if its parent is already existing
  at the receiving processor or the subproblem is the root
  of the \bab\ tree. This is verified by a comparison of the ABA_ID
  of the parent subproblem. All missing active variables/constraints
  are requested and received by this function.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object from which the subproblem is initialized.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  ABA_SUB::ABA_SUB(ABA_MASTER *master, ABA_MESSAGE &msg)
@q m e f @>
  : @/
    master_(master), @/
    actCon_(0), @/
    actVar_(0), @/
    father_(0), @/
    lp_(0), @/
    fsVarStat_(0), @/
    lpVarStat_(0), @/
    lBound_(0), @/
    uBound_(0), @/
    slackStat_(0), @/
    tailOff_(0), @/
    branchRule_(0), @/
    addVarBuffer_(0), @/
    addConBuffer_(0), @/
    removeVarBuffer_(0), @/
    removeConBuffer_(0), @/
    xVal_(0), @/
    yVal_(0), @/
    bInvRow_(0), @/
    genNonLiftCons_(false), @/
    sons_(0), @/
    activated_(false), @/
    ignoreInTailingOff_(false) , @/
    localTimer_(master)
  {
    master_->parmaster()->incWorkCount();

    master_->parmaster()->incSubReceivedCount();
    if (debug(DEBUG_MESSAGE_SUB))
      master_->out() << "DEBUG_MESSAGE_SUB: receiving subproblem ..." << endl;
    @<unpack a bitvector of the valid pointers@>
    @<unpack the active constraints/variables@>
    @<unpack the local data of the subproblem@>
    if (debug(DEBUG_MESSAGE_SUB))
      master_->out() << "DEBUG_MESSAGE_SUB: subproblem received." << endl;

    if (branchRule_)
      branchRule_->initialize(this);

    //master_->newSub(level_);
    //master_->treeInterfaceNodeBounds(id_, lowerBound(), upperBound());
  }

@ The local variable |bits| is unpacked. Each bit set indicates that an
  object of the corresponding pointer has to be unpacked. Otherwise the
  corresponding pointer has to be set to 0.

@<unpack a bitvector of the valid pointers@>=
  int bits;
  msg.unpack(bits);

@ First we receive the IDs of the active constraints. Then we send the
  indexes of the constraints back which are not locally available.
  These constraints will then be sent to us.

@<unpack the active constraints/variables@>=
  if (bits&1) {         // actCon_ && actVar_
    ABA_BUFFER<ABA_ID>  *idBufferCon, *idBufferVar;
    ABA_BUFFER<int> *neededCon, *neededVar;

    actCon_ = new @[ ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE>(master_, msg,
                     &idBufferCon, &neededCon) @];
    actVar_ = new @[ ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT>(master_, msg,
                     &idBufferVar, &neededVar) @];

    if (debug(DEBUG_MESSAGE_SUB)) {
      master_->out() << "DEBUG_MESSAGE_SUB: unpacked IDs of active Constraints:"
                     << *idBufferCon << endl;
      master_->out() << "DEBUG_MESSAGE_SUB: needed active Constraints:"
                     << *neededCon << endl;
      master_->out() << "DEBUG_MESSAGE_SUB: unpacked IDs of active Variables:"
                     << *idBufferVar << endl;
      master_->out() << "DEBUG_MESSAGE_SUB: needed active Variables:"
                     << *neededVar << endl;
    }

    // msg.clear();
    neededCon->pack(msg);
    neededVar->pack(msg);
    msg.send();         // send needed ABA_IDs

    msg.receive();      // receive needed constraints/variables
    actCon_->unpackNeeded(msg, *idBufferCon, *neededCon);
    actVar_->unpackNeeded(msg, *idBufferVar, *neededVar);
  }
  else {
    master_->out() << "warning: subproblem with actVar_==0 and actCon_==0"
                      " received!" << endl;
  }
  
@ At this point the the active constraints/variables are unpacked.
  Now we unpack all of the remaining local data of the subproblem.

@<unpack the local data of the subproblem@>=
  assert((bits&2)==0);       //  father_ must be 0
  if (bits&4)
    fsVarStat_ = new @[ ABA_ARRAY<ABA_FSVARSTAT*>(master, msg) @];
  if (bits&8)
    lpVarStat_ = new @[ ABA_ARRAY<ABA_LPVARSTAT*>(master, msg) @];
  if (bits&16)
    lBound_ = new @[ ABA_ARRAY<double>(master, msg) @];
  if (bits&32)
    uBound_ = new @[ ABA_ARRAY<double>(master, msg) @];
  if (bits&64)
    slackStat_ = new @[ ABA_ARRAY<ABA_SLACKSTAT*>(master, msg) @];
  msg.unpack(dualBound_);
  msg.unpack(nIter_);
  msg.unpack(lastIterConAdd_);
  msg.unpack(lastIterVarAdd_);
  if (bits&128) {
    int classId;
    msg.unpack(classId);
    branchRule_ = master_->unpackBranchrule(msg, classId);
  }
  msg.unpack(allBranchOnSetVars_);
  lpMethod_ = (ABA_LP::METHOD) msg.unpackInt();
  msg.unpack(level_);
  msg.unpack(id_);
  msg.unpack(fatherId_);
  status_ = (STATUS) msg.unpackInt();
  msg.unpack(maxIterations_);
  msg.unpack(nOpt_);
  msg.unpack(relativeReserve_);
  msg.unpack(varReserve_);
  msg.unpack(conReserve_);
  msg.unpack(nnzReserve_);
  msg.unpack(nDormantRounds_);


@q m f pack @>
@ The function |pack()| packs the data of the subproblem
  in an |ABA_MESSAGE| object. This virtual function has to be redefined
  in a derived class if additional data should be packed. In this
  case the |pack()| function of the base class must be called
  by the |pack()| function of the derived class.
  
  Active constraints/variables which are missing at the receiving
  processor will be packed by this function too.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|msg|}{The |ABA_MESSAGE| object in which the subproblem is packed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::pack(ABA_MESSAGE &msg) const
@q m e f @>
  {
    master_->parmaster()->incSubSentCount();
    if (debug(DEBUG_MESSAGE_SUB))
      master_->out() << "DEBUG_MESSAGE_SUB: sending subproblem ..." << endl;
    @<pack a bitvector of the valid pointers@>
    @<pack the active constraints/variables@>
    @<pack the local data of the subproblem@>
    if (debug(DEBUG_MESSAGE_SUB))
      master_->out() << "DEBUG_MESSAGE_SUB: subproblem sent." << endl;
  }

@ Pack a bitvector indicating which pointers to dynamic data are not 0.

@<pack a bitvector of the valid pointers@>=
  assert((actCon_==0)==(actVar_==0));
  msg.pack( (actCon_?1:0) +
            (father_?2:0) + (fsVarStat_?4:0) + (lpVarStat_?8:0) +
            (lBound_?16:0) + (uBound_?32:0) + (slackStat_?64:0) +
            (branchRule_?128:0) );

@ We first determine which constraints/variables are not present at
  the receiving processor by sending the IDs of all active constraints.
  The receiving processor sends us the indexes of those constraints/variables
  back which are missing. Afterwards they will be packed.

@<pack the active constraints/variables@>=
  if (actCon_) {
    actCon_->pack(msg);
    actVar_->pack(msg);
    if (debug(DEBUG_MESSAGE_SUB)) {
      master_->out() << "DEBUG_MESSAGE_SUB: # active Constraints:" << actCon_->number() << endl;
      master_->out() << "DEBUG_MESSAGE_SUB: # active Variables  :" << actVar_->number() << endl;
    }
    msg.send();                 // send required IDs

    msg.receive();              // receive missing IDs
    ABA_BUFFER<int> neededCon(master_, msg);
    ABA_BUFFER<int> neededVar(master_, msg);
    // msg.clear();

    if (debug(DEBUG_MESSAGE_SUB)) {
      master_->out() << "DEBUG_MESSAGE_SUB: packing needed Constraints:" << neededCon << endl;
      master_->out() << "DEBUG_MESSAGE_SUB: packing needed Variables  :" << neededVar << endl;
    }
    actCon_->packNeeded(msg, neededCon);
    actVar_->packNeeded(msg, neededVar);
  }

@ At this point the packing of the active constraints/variables is finished.
  Now we pack all of the remaining local data of the subproblem.

@<pack the local data of the subproblem@>=
  if (father_) {
    master_->err() << "ABA_SUB::pack(): father_ has to be 0!" << endl;
    exit(Fatal);
  }
  if (fsVarStat_)
    fsVarStat_->pack(msg, actVar_->number());
  if (lpVarStat_)
    lpVarStat_->pack(msg, actVar_->number());
  if (lBound_)
    lBound_->pack(msg, actVar_->number());
  if (uBound_)
    uBound_->pack(msg, actVar_->number());
  if (slackStat_)
    slackStat_->pack(msg, actCon_->number());
  msg.pack(dualBound_);
  msg.pack(nIter_);
  msg.pack(lastIterConAdd_);
  msg.pack(lastIterVarAdd_);
  if (branchRule_) {
    msg.pack(branchRule_->classId());
    branchRule_->pack(msg);
  }
  msg.pack(allBranchOnSetVars_);
  msg.pack((int)lpMethod_);             // ABA_LP::METHOD
  msg.pack(level_);
  msg.pack(id_);
  msg.pack(fatherId_);
  msg.pack((int)status_);               // STATUS
  msg.pack(maxIterations_);
  msg.pack(nOpt_);
  msg.pack(relativeReserve_);
  msg.pack(varReserve_);
  msg.pack(conReserve_);
  msg.pack(nnzReserve_);
  msg.pack(nDormantRounds_);
  assert(!activated_);
  assert(!ignoreInTailingOff_);

@ @(sub.cc@>=
#endif


@q m v optimize @>
@ The function |optimize()| performs the optimization of the
  subproblem.
@q m e t @>

  After activating the subproblem, i.e., allocating and initializing
  memory, and initializing  the |LP|, the optimization process
  constitutes of the three phases
  |Cutting|, |Branching|, and |Fathoming|, which are alternately processed.
  The function |fathoming()| always returns |Done|. However, we think
  that the code is better readable instead of taking it out of the
  |while| loop.
  The optimization stops if 
  the |PHASE| |Done| is reached. Note, |Done| does not necessarily mean
  that the subproblem is solved to optimality!
  
  After the node is processed we deallocate memory, which is not required
  for further computations or of which the corresponding data can be
  easily reconstructed. This is performed in |_deactivate()|.

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the optimization has been performed without error,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::optimize()
@q m e f @>
  {

    PHASE phase;  // current phase of the subproblem optimization

    @<update the global dual bound@>;
    @<output a banner for the subproblem@>;

    ++nOpt_;
#ifdef ABACUS_PARALLEL
    // update idle timers
    master_->parmaster()->updateIdleTimers(master_->nSub()==0);

    // count locally optimized subproblems
    master_->newSub(level_);
#endif

    phase = _activate ();

    while (phase != Done) {
      switch (phase) {
        case Cutting:   phase = cutting();
                        break;
        case Branching: phase = branching();
                        break;
        case Fathoming: phase = fathoming ();
                        break;
        default:        master_->err() << "ABA_SUB::optimize(): unknown phase ";
                        master_->err() << (int) phase << endl;
                        master_->err() << "Further processing not possible." << endl;
                        exit(Fatal);
      }
    }
    _deactivate ();

#ifdef ABACUS_PARALLEL
    // the subproblem should be fathomed, if it has been processed
    if (status_ == Processed)
      fathoming();
#endif
    @<output some infos about the enumeration tree@>;

    return 0;
  }
  
@ @<output a banner for the subproblem@>=
  master_->out() << endl << "************************************************" << endl;
#ifdef ABACUS_PARALLEL
    master_->out() << "Subproblem ";
    master_->parmaster()->printId(id_);
    master_->out() << " on Level " << level_ << ":" << endl << endl;
#else
  master_->out() << "Subproblem " << id_ << " on Level " << level_ << ":" << endl << endl;
#endif

  if (master_->optSense()->max()) {
    master_->out(1) << "Global Lower Bound: " << lowerBound()       << endl;
    master_->out(1) << "Local  Upper Bound: " << upperBound()       << endl;
    master_->out(1) << "Global Upper Bound: " << master_->upperBound() << endl;
  }
  else {
    master_->out(1) << "Local  Lower Bound: " << lowerBound()       << endl;
    master_->out(1) << "Global Lower Bound: " << master_->lowerBound() << endl;
    master_->out(1) << "Global Upper Bound: " << upperBound()       << endl;
  }

  master_->out(1) << "Current Guarantee : ";
  master_->printGuarantee();
  master_->out() << endl << endl;

@ The global dual bound is the maximum (minimum) of the
  dual bound of the subproblem and the dual bounds of the
  subproblems which still have to be processed if this
  is a maximization (minimization) problem.
  
@<update the global dual bound@>=
#ifdef ABACUS_PARALLEL
  // propagate the dual bound of the subproblem if needed, in case it
  // has been changed directly
  dualBound(dualBound_);
#else
  double newDual = dualBound_;
    
  if (master_->optSense()->max()) {
    if (master_->openSub()->dualBound() > newDual)
      newDual = master_->openSub()->dualBound();
  }
  else
    if (master_->openSub()->dualBound() < newDual)
      newDual = master_->openSub()->dualBound();

  if (master_->betterDual(newDual)) master_->dualBound(newDual);
#endif

@ @<output some infos about the enumeration tree@>=
  if (master_->outLevel() == ABA_MASTER::Subproblem ||
      master_->outLevel() == ABA_MASTER::LinearProgram ||
      master_->logLevel() == ABA_MASTER::Subproblem ||
      master_->logLevel() == ABA_MASTER::LinearProgram   ) {

    @<|ABA_SUB::optimize()|: turn the streams on@>;
    @<output a line about the subproblem optimization@>;
    @<|ABA_SUB::optimize()|: turn the streams off@>;
  }
  else {
    master_->out() << "Enumeration Tree" << endl;
    master_->out(1) << "Number of Subproblems:   " << master_->nSub() << endl;
    master_->out(1) << "Number of Open Problems: " << master_->openSub()->number() << endl;
  }


@ If the |logLevel| is not |Subproblem| or |LinearProgram| and turned on then
  we have to turn it off, such that there is not this redundant output.
  With the variable |turnLogOn| we memorize if we have to turn the logging
  on again after the output is performed.
  
@<|ABA_SUB::optimize()|: turn the streams on@>=
  bool turnOutOn = false;

  if (master_->outLevel() == ABA_MASTER::Subproblem ||
      master_->outLevel() == ABA_MASTER::LinearProgram  )
    master_->out().on();
  else if (master_->out().isOn()) {
    master_->out().off();
    turnOutOn = true;
  }
    

  bool turnLogOn = false;

  if (master_->logLevel() == ABA_MASTER::Subproblem ||
      master_->logLevel() == ABA_MASTER::LinearProgram)
    master_->out().logOn();
  else if (master_->out().isLogOn()) {
    master_->out().logOff();
    turnLogOn = true;
  }

@ We output the total number of subproblems, the number of open subproblems,
  the number of iterations in the cutting plane phase, the dual bound,
  and the primal bound.
  
@<output a line about the subproblem optimization@>=
  master_->out() << setWidth(7) << master_->nSub() << " ";
  master_->out() << setWidth(7) << master_->openSub()->number() << "  ";
#ifdef ABACUS_PARALLEL
  master_->out() << setWidth(8);
  master_->parmaster()->printId(id_);
  master_->out() << " ";
#else
  master_->out() << setWidth(8) << id_ << " ";
#endif
  master_->out() << setWidth(7) << nIter_ << " ";
  if (infeasible())
    master_->out() << setWidth(10) << "infeas" << " ";
  else
    master_->out() << setWidth(10) << dualBound() << " ";
  master_->out() << setWidth(10) << master_->dualBound() << " ";
  if (master_->feasibleFound())
    master_->out() << setWidth(10) << master_->primalBound();
  else
    master_->out() << setWidth(10) << "---";
  master_->out() << endl;

@ @<|ABA_SUB::optimize()|: turn the streams off@>=
  if (master_->outLevel() == ABA_MASTER::Subproblem ||
      master_->outLevel() == ABA_MASTER::LinearProgram)
    master_->out().off();
  if (master_->logLevel() == ABA_MASTER::Subproblem ||
      master_->logLevel() == ABA_MASTER::LinearProgram)
    master_->out().logOff();

  if (turnLogOn)
    master_->out().logOn();

  if (turnOutOn)
    master_->out().on();

@ The function |_activate()| allocates and initializes memory of the
  subproblem at the beginning of the optimization. The function
  returns the next phase of the optimization. This is either |Cutting|
  or |Fathoming| if the subproblem immediately turns out to be infeasible.

  Since many objects of the class |ABA_SUB| can exist at the
  same time, yet in a sequential algorithm only one problem is active,
  a lot of memory can be saved if some memory is dynamically allocated
  when the subproblem becomes active and other information is
  stored in a compressed format for dormant problems.

  These allocations and decompressions are performed by the function
  |_activate()|, the respective deallocations and compression of
  data is executed by the function |_deactivate()|.

  Currently for all subproblems which have not been processed already
  (except for the root) we initialize
  the active constraints and variables with the respective data from the
  father node adapted by the branching information since so
  we can make sure that all fixed and set variables are active.
  A more flexible strategy might be desirable but also dangerous.

  The virtual function |activate()| can perform problem specific
  activations. It is
  called before variables are fixed by
  logical implications, because, e.g., for problems on graphs,
  the graph associated with the subproblem might have to be activated.

  Moreover, the function |_activate()| is redundant in the sense that
  it is called only once and could be substituted by a function. However,
  having a future generalization to non \lpbab\ in mind, we keep this
  function.

@(sub.cc@>=
  ABA_SUB::PHASE ABA_SUB::_activate()
  {
    @<activate the subproblem in the VBC-Tool@>;
    @<can the subproblem be fathomed without processing?@>;
#ifdef ABACUS_PARALLEL
    assert(actCon_ && actVar_);
    // actVar_, actCon_ etc. are already valid.
    // we only have to allocate the local members
    int initialMaxVar = actVar_->max();
    int initialMaxCon = actCon_->max();
    @<allocate local members of the subproblem@>;
    if (status_ == Unprocessed && branchRule_) {
      if (branchRule_->extract(this))
        return Fathoming;
    }
    else
      assert(branchRule_ == 0);
#else
    @<determine the initial maximal number of constraints and variables@>;
    @<allocate local members of the subproblem@>;
    @<perform activations for unprocessed non-root nodes@>;
#endif
    @<remove missing variables and constraints from the active sets@>;
    @<set the active flags of variables and constraints@>;
    @<perform problem specific activations@>;
    @<update fixed and set variables and set by logical implications@>;
    @<output the dimension of the subproblem@>;
    @<initialize the linear program of the subproblem@>;
    
    return Cutting;
  }

@ @<activate the subproblem in the VBC-Tool@>=
#ifdef ABACUS_PARALLEL
  master_->treeInterfacePaintNode(id_, 4);
#else
  master_->treeInterfacePaintNode(id_, 6);
#endif

@ If during the subproblem was waiting for further processing
  in the list of open subproblems,
  a primal bound better than its dual bound has been found, then
  we can fathom the subproblem immediately.

@<can the subproblem be fathomed without processing?@>=
  if (boundCrash()) return Fathoming;

@ We overestimate the initial number of constraints and variables in
  order to avoid too many reallocations during the optimization process.

  If the subproblem is the root node or has been processed already, then
  this overestimation is not required, because extra memory has either
  been allocated already in the constructor or is available from the
  last optimization respectively.
   
@<determine the initial maximal number of constraints and variables@>=
  int initialMaxVar;
  int initialMaxCon;

#ifdef ABACUS_PARALLEL
  if (status_ == Unprocessed && level_ > 1) {
#else
  if (status_ == Unprocessed && this != master_->root()) {
#endif
    if (relativeReserve_) {
      initialMaxVar = (int) (father_->nVar() *(1.0 + varReserve_/100.0));
      initialMaxCon = (int) (father_->nCon() *(1.0 + conReserve_/100.0));
    }
    else {
      initialMaxVar = father_->nVar() + (int) varReserve_;
      initialMaxCon = father_->nCon() + (int) conReserve_;
    }
  }
  else {
    initialMaxVar = maxVar();
    initialMaxCon = maxCon();
  }
    
@ @<allocate local members of the subproblem@>=
  tailOff_  = new ABA_TAILOFF(master_);
  addVarBuffer_    = new @,@[ABA_CUTBUFFER<ABA_VARIABLE, ABA_CONSTRAINT>(master_, 
                                                       master_->maxVarBuffered())@];
  addConBuffer_    = new @,@[ABA_CUTBUFFER<ABA_CONSTRAINT, ABA_VARIABLE>(master_,
                                                       master_->maxConBuffered())@];
  removeVarBuffer_ = new @,@[ABA_BUFFER<int>(master_, initialMaxVar)@];
  removeConBuffer_ = new @,@[ABA_BUFFER<int>(master_, initialMaxCon)@];
  xVal_            = new double[initialMaxVar];
  yVal_            = new double[initialMaxCon];

@ The initialization of constraints and variables is performed by
  virtual functions such that easily other initialization methods
  can be applied. 
  
  If there is a contradiction between a branching rule and the local
  information of the subproblem we can immediately fathom the subproblem.

@<perform activations for unprocessed non-root nodes@>=
  if (status_ == Unprocessed && this != master_->root()) {
    initializeVars(initialMaxVar);
    initializeCons(initialMaxCon);
    if (branchRule_->extract(this))
      return Fathoming;
  }
  
@ After the function |_activate()| is performed we assume during the
  complete subproblem optimization that all active
  variables and constraints are available in some pool! Therefore, we
  remove now all missing variables and constraints from their active sets.
  
@<remove missing variables and constraints from the active sets@>=
  @<remove missing variables from the active variables@>;
  @<remove missing constraints from the active variables@>;

@ It is a fatal error if a fixed or set variable is missing.

@<remove missing variables from the active variables@>=
  ABA_BUFFER<int> removeVars(master_, nVar());

  int nVariables = nVar();

  for (int i = 0; i < nVariables; i++)
    if ((*actVar_)[i] == 0) {
      removeVars.push(i);
      if ((*fsVarStat_)[i]->fixedOrSet()) {
        master_->err() << "ABA_SUB::_activate(): ";
        master_->err() << "active fixed or set variable not available in pool" << endl;
        exit (Fatal);
      }
      delete (*fsVarStat_)[i];
      delete (*lpVarStat_)[i];
    }

  if (removeVars.number()) {
    master_->out() << removeVars.number() << " variables missing for initialization" << endl;

    actVar_->remove(removeVars);
    fsVarStat_->leftShift(removeVars);
    lpVarStat_->leftShift(removeVars);
    lBound_->leftShift(removeVars);
    uBound_->leftShift(removeVars);
  }
  
@ @<remove missing constraints from the active variables@>=
  ABA_BUFFER<int> removeCons(master_, nCon());

  const int nConstraints = nCon();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nConstraints; i++)
#else
  for (int i = 0; i < nConstraints; i++)
#endif
    if ((*actCon_)[i] == 0) {
      removeCons.push(i);
      delete (*slackStat_)[i];
    }
  if (removeCons.number())
    master_->out() << removeCons.number() << " constraints missing for initialization" << endl;

  actCon_->remove(removeCons);
  slackStat_->leftShift(removeCons);
      
@ Setting |status_| to |Active| at this point is necessary, since if the
  subproblem turns out to be fathomed already during processing
  this function, then in |fathom()| the variables and constraints
  have to be deactivated.
  
@<set the active flags of variables and constraints@>=
  const int nActVar = actVar_->number();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nActVar; i++)
#else
  for (int i = 0; i < nActVar; i++)
#endif
    (*actVar_)[i]->activate();

  const int nActCon = actCon_->number();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nActCon; i++)
#else
  for (int i = 0; i < nActCon; i++)
#endif
    (*actCon_)[i]->activate();

  status_  = Active;  

@ We have to memorize if activate() has been called such that in 
  |_deactivate()| only |deactivate()| is called when |activate()| has
  been performed. This is necessary because these lines are only
  reached if the dual bound is still better than the primal bound.

@<perform problem specific activations@>=
  activate();
  activated_ = true;

@ We update global variable fixings which have been performed
  while the subproblem was sleeping. If there is a contradiction to
  set variables we can fathom the node.

  The adaption of branching variables may allow us to set further
  variables by logical implications. Again contradictions to
  already fixed variables can lead to an immediate |Fathoming|
  of the node.

@<update fixed and set variables and set by logical implications@>=
  ABA_FSVARSTAT *global;     // global status of a variable
  ABA_FSVARSTAT *local;      // local status of a variable
  double     newBound;   // the new local bound

  nVariables = nVar();
#ifdef ABACUS_NO_FOR_SCOPE    
  for (i = 0; i < nVariables; i++) {
#else
  for (int i = 0; i < nVariables; i++) {
#endif
    global = variable(i)->fsVarStat();
    local  = (*fsVarStat_)[i];
    if (global->fixed()) {
      if (global->contradiction(local)) {
        infeasibleSub();
        return Fathoming;
      }
      local->status(global);
      newBound      = fixSetNewBound(i);
      (*lBound_)[i] = newBound;
      (*uBound_)[i] = newBound;
    }
  }

  bool newValues;  // in this context only required as a dummy

  if (_setByLogImp(newValues)) {
    infeasibleSub();
    return Fathoming;
  }

@ @<output the dimension of the subproblem@>=
  if (master_->out().isOn() || master_->out().isLogOn()) {
    @<output number of active constraints and variables@>;
    @<output number of fixed and set variables@>;
  }

@ @<output number of active constraints and variables@>=
  master_->out() << endl;
  master_->out() << "Subproblem Size" << endl;
  master_->out(1) << "Number of Active Constraints : " << nCon() << endl;
  master_->out(1) << "Number of Active Variables   : " << nVar() << endl;
  
@ @<output number of fixed and set variables@>=
  int nFixed = 0;  // number of fixed variables
  int nSet   = 0;  // number of set variables

  const int nVariables = nVar();
  for (int i = 0; i < nVariables; i++)
    if ((*fsVarStat_)[i]->fixed()) ++nFixed;
    else if ((*fsVarStat_)[i]->set()) ++nSet;

  master_->out(1) << "Number of Fixed Variables    : " << nFixed << endl;
  master_->out(1) << "Number of Set Variables      : " << nSet   << endl;

@ If the |LP| turns out to be infeasible already in
  the initialization phase, we can again fathom the \bac\ node.
  
@<initialize the linear program of the subproblem@>=
  if (initializeLp()) {
    infeasibleSub();
    return Fathoming;
  }

@q m v activate @>
@ The virtual function |activate()| does nothing but can be used
  as an entrance point for problem specific activations by
v  a reimplementation in derived classes.
@q m e t @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::activate()
@q m e f @>
  { }
  
@q m v initializeVars @>
@ The function |initializeVars()| initializes the active variable set.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|maxVar|}{The maximal number of variables of the
                    subproblem.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::initializeVars(int maxVar)
@q m e f @>
  {
    actVar_ = new @,@[ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT>(master_, 
                                                   father_->actVar_, maxVar)@];

    fsVarStat_ = new @,@[ABA_ARRAY<ABA_FSVARSTAT*>(master_, maxVar)@];
    lpVarStat_ = new @,@[ABA_ARRAY<ABA_LPVARSTAT*>(master_, maxVar)@];
    lBound_    = new @,@[ABA_ARRAY<double>(master_, maxVar)@];
    uBound_    = new @,@[ABA_ARRAY<double>(master_, maxVar)@];

    const int nVariables = nVar();
                                 
    for (int i = 0; i < nVariables; i++) {
      (*lpVarStat_)[i]  = new ABA_LPVARSTAT(father_->lpVarStat(i));
      (*fsVarStat_)[i]  = new ABA_FSVARSTAT(father_->fsVarStat(i));
      (*lBound_)[i]     = father_->lBound(i);
      (*uBound_)[i]     = father_->uBound(i);
    }
  }

@q m v initializeCons @>
@ The function |initializeCons()| initializes the active constraint
  set.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|maxCon|}{The maximal number of constraints of the
                    subproblem.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::initializeCons(int maxCon)
@q m e f @>
  {
    actCon_ = new @,@[ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE>(master_, 
                                                  father_->actCon_, maxCon)@];

    slackStat_ = new @,@[ABA_ARRAY<ABA_SLACKSTAT*>(master_, maxCon)@];

    const int nConstraints = nCon();
    
    for (int i = 0; i < nConstraints; i++)
      (*slackStat_)[i] = new ABA_SLACKSTAT(*(father_->slackStat(i)));
  }
  

@ The function |_deactivate()| deallocates the memory which is not
  required after the optimization of the subproblem.
  The virtual dummy function |deactivate()| can perform problem specific
  deactivations.

  As the function |_activate()|,
  the function |_deactivate()| is redundant in the sense that
  it is called only once and could be substituted by a function. However,
  having a future generalization to non \lpbab\ in mind, we keep this
  function.

@(sub.cc@>=
  void ABA_SUB::_deactivate()
  {
    if (activated_) deactivate();
    @<deactivate the subproblem in the VBC-Tool@>;
    @<delete members redundant for inactive subproblems@>;
    @<reset the active flags of variables and constraints@>;    
    @<deactive the root node@>;
  }

@ @<deactivate the subproblem in the VBC-Tool@>=
#ifdef ABACUS_PARALLEL
  master_->treeInterfacePaintNode(id_, 7+master_->parmaster()->hostId() );
#else
  master_->treeInterfacePaintNode(id_, 1);
#endif

@ @<delete members redundant for inactive subproblems@>=
  delete tailOff_;
  tailOff_ = 0;

  localTimer_.start(true);

  delete lp_;

  master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());

  lp_ = 0;

  delete addVarBuffer_;
  addVarBuffer_ = 0;
  delete addConBuffer_;
  addConBuffer_ = 0;
  delete removeVarBuffer_;
  removeVarBuffer_ = 0;
  delete removeConBuffer_;
  removeConBuffer_ = 0;
  delete [] xVal_;
  xVal_ = 0;
  delete [] yVal_;
  yVal_ = 0;

@ If the node being deactivated has just been fathomed then
  |actVar_| and |actCon_| are 0. In this case the deactivation
  has been performed already in the function |fathom()|.
  
@<reset the active flags of variables and constraints@>=
  int i;
  
  if (actVar_) {
    const int nActVar = actVar_->number();
    for (i = 0; i < nActVar; i++)
      (*actVar_)[i]->deactivate();
  }

  if (actCon_) {
    const int nActCon = actCon_->number();
    for (i = 0; i < nActCon; i++)
      (*actCon_)[i]->deactivate();
  }
  
@ @<deactive the root node@>=
#ifdef ABACUS_PARALLEL
  if (level_ == 1)
#else
  if (this == master_->root())
#endif
    master_->rootDualBound(dualBound_);

@q m v deactivate @>
@ The virtual function |deactivate()| can be used as entrance point
  for problem specific deactivations after the subproblem optimization.
  The default version of this function does nothing. This function is
  only called if the function |activate()| for the subproblem has been
  executed.
@q m e t @>
  This function is called from |_deactivate()|.

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::deactivate()
@q m e f @>
  { }

@ The function |_setByLogImp()| tries to set variables according
  to logical implications of already set and fixed variables.
  Since logical implications are problem specific
  the virtual function |setByLogImp()| is called to find
  variables which can be set. If a variable is set to a new value,
  i.e., a value different from the one in the last solved LP,
  |newValues| is set to |true|.
  If such a setting implies a contradiction, |_setByLogImp()| returns
  1, otherwise it returns 0.

@(sub.cc@>=
  int ABA_SUB::_setByLogImp(bool &newValues)
  {
    master_->out() << "Setting Variables by Logical Implications: " << flush;
    @<call the virtual function to set variables by logical implications@>;
    @<check for contradictions and variables set to new values@>;
    @<|ABA_SUB::_setByLogImp()|: clean up and return@>;
  }

@ @<call the virtual function to set variables by logical implications@>=
  ABA_BUFFER<int>        variables(master_, nVar());
  ABA_BUFFER<ABA_FSVARSTAT*> status(master_, nVar());

  setByLogImp(variables, status);

@ @<check for contradictions and variables set to new values@>=
  int contra = 0;
  bool lNewValues;

  newValues = false;

  const int nVariables = variables.number();
    
  for (int i = 0; i < nVariables; i++) {
    contra = set(variables[i], status[i], lNewValues);
    if (contra) break;
    if (lNewValues) newValues = true;
  }

@ @<|ABA_SUB::_setByLogImp()|: clean up and return@>=
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVariables; i++)
#else
  for (int i = 0; i < nVariables; i++)
#endif
    delete status[i];

  if (contra)
    master_->out() << "contradiction found" << endl;
  else
    master_->out() << nVariables << " variables set" << endl;
    
  return contra;

@q m v setByLogImp @>
@ The default implementation of |setByLogImp()| does nothing.
  In derived classes this function can be reimplemented.
@q m e t @>
@q m b a @>
  \BeginArg
     \docarg{|variable|}{The variables which should be set have to be
                         inserted in this buffer.}
     \docarg{|status|}{The status of the set variables.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::setByLogImp(ABA_BUFFER<int> &variables, 
                            ABA_BUFFER<ABA_FSVARSTAT*> &status)
@q m e f @>
  { }

@q m v cutting @>
@ The function |cutting()| iteratively solves the LP-relaxation,
  generates constraints and/or variables.
  Also generating variables can be regarded as ``cutting'',
  namely as generating cuts for the dual problem.
@q m e t @>
  A reader even studying these lines has been very brave! Therefore,
  the first reader of these lines is invited to a beer from the
  author.

@q m b r @>
  \BeginReturn
  \docarg{|Fathoming|}{If one of the conditions for fathoming the
                       subproblem is satisfied.}
  \docarg{|Branching|}{If the subproblem should be splitted in further
                       subproblems.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
   ABA_SUB::PHASE ABA_SUB::cutting ()
@q m e f @>
   {
     @<|ABA_SUB::cutting()|: local variables@>;

     while (1) {
       @<add and remove variables and constraints@>;
       @<is this the last iteration before |Branching|?@>;
       @<solve the LP-relaxation@>;
       @<make a feasibility test@>;
       @<improve the primal solution@>;
       @<test some minor termination criteria@>;
       @<perform primal and dual separation@>;
     }   // while
   }

@ Before we are going to branch we would like to remove (e.g., non-binding)
  constraints. Such
  final modifications can be performed in the function
  |prepareBranching()|. If in this function the problem is modified,
  the variable |lastIteration| becomes |true| and now we perform the
  branching indeed. Only for convenience we modify the active constraints
  at the beginning of the cutting plane algorithm in this case.

@<|ABA_SUB::cutting()|: local variables@>=
   int status;      // return status of some called functions
   bool newValues;  // |true| if variable fix or set to new value
   bool lastIteration = false; 

   int nVarRemoved;
   int nConRemoved;
   int nVarAdded;
   int nConAdded;

@ Added/removed variables/constraints are not directly added/removed but
  stored in a buffer. Here, at the beginning of the inner loop of the
  cutting plane algorithm we update the active constraints and variables
  and the linear program. If more constraints/variables are buffered than
  actually should be added, we select the best ones if possible.
  
@<add and remove variables and constraints@>=

  @<test if feasibility of basis could be destroyed@>;


  master_->out() << endl;
  master_->out() << "Update the Problem" << endl;

  @<remove all buffered constraints@>;
  @<remove all buffered variables@>;
  @<select constraints from the buffer and add them@>;
  @<select variables from the buffer and add them@>;
  
  lpMethod_ = chooseLpMethod(nVarRemoved, nConRemoved, nVarAdded, nConAdded);

@ This test does not check all possibilities, e.g., variables
  might have been fixed, and hence, destroy the feasibility of the basis.

@<test if feasibility of basis could be destroyed@>=

  if (addVarBuffer_->number() && addConBuffer_->number()) {
    master_->out() << "ABA_SUB::cutting(): WARNING: adding variables and constraints" << endl;
    master_->out() << "                         basis might become infeasible" << endl;
  }

  if (removeVarBuffer_->number() && removeConBuffer_->number()) {
    master_->out() << "ABA_SUB::cutting(): WARNING: removing variables and constraints" << endl;
    master_->out() << "                         basis might become infeasible" << endl;
  }

@ @<remove all buffered constraints@>=  
  if (removeConBuffer_->number()) {
    nConRemoved = _removeCons(*removeConBuffer_);
    removeConBuffer_->clear();
    master_->out(1) << "removed constraints: " << nConRemoved << endl;
  }
  else
    nConRemoved = 0;

@ @<remove all buffered variables@>=
  if (removeVarBuffer_->number()) {
    nVarRemoved = _removeVars(*removeVarBuffer_);
    removeVarBuffer_->clear();
    master_->out(1) << "removed variables:   " << nVarRemoved << endl;
  }
  else
    nVarRemoved = 0;

@ The function |_selectCons()| tries to select the best constraints
  of the buffered ones if more constraints have been generated than should be
  added.
  
@<select constraints from the buffer and add them@>=
  if (addConBuffer_->number()) {
      ABA_BUFFER<ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE>*> newCons(master_,
                                                      addConBuffer_->number());
    
    _selectCons(newCons);
    nConAdded = addCons(newCons);
    lastIterConAdd_ = nIter_;
    master_->out(1) << "added constraints:   " << nConAdded << endl;
  }
  else
    nConAdded = 0;

@ The function |_selectVars()| tries to select the best variables
  of the buffered ones if more variables have been generated than should be
  added.

@<select variables from the buffer and add them@>=
      
  if (addVarBuffer_->number()) {
    @<check if there are non-liftable constraints@>;
    ABA_BUFFER<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *> newVars(master_,  
                                                      addVarBuffer_->number());

    _selectVars(newVars);
    nVarAdded = addVars(newVars);
    lastIterVarAdd_ = nIter_;
    master_->out(1) << "added variables:     " << nVarAdded << endl;
  }
  else
    nVarAdded = 0;

@ If variables are added but non-liftable constraints are present, then
  we cannot generate the columns correctly.
  
@<check if there are non-liftable constraints@>=
#ifdef ABACUSSAFE
  const int nConstraints = nCon();

  for (int i = 0; i < nConstraints; i++)
    if (!constraint(i)->liftable()) {
      master_->err() << "ABA_SUB::cutting(): adding variables, where constraint ";
      master_->err() << i << " cannot be lifted" << endl;
    }
#endif    

@ If we entered the cutting plane algorithm only to remove constraints
  before the branching is performed, we do not solve the LP-relaxation.
  
@<is this the last iteration before |Branching|?@>=
  if (lastIteration) return Branching;
  
@ If the function |solveLp()| returns 1, then the linear program is infeasible,
  also in respect to possibly inactive variables. If it returns 2, then
  the linear program is infeasible, but inactive variables have been
  generated. Therefore we iterate.

  If the function |_pricing()| returns a nonzero |status|, then the variables
  have been added. Hence, we iterate. Otherwise, the LP-solution is a dual
  bound for the subproblem and we can check the guarantee requirements.
  
@<solve the LP-relaxation@>=   
  ++nIter_;

  status = solveLp();
  if (status == 1) return Fathoming;
  if (status == 2) continue;

  @<print the LP-solution for output level |Subproblem| and |LinearProgram|@>;

  if (master_->primalViolated(dualRound(lp_->value()))) {
    status = _pricing(newValues);
    if (status)       continue;
    return Fathoming;
  }

  @<count the number of discrete variables being fractional@>;

@ For output level |Subproblem| we print a line only for the first solved
  linear program of the root node. For output level |LinearProgram| a line
  is output in each iteration.
  
@<print the LP-solution for output level |Subproblem| and |LinearProgram|@>=
  if (master_->outLevel() == ABA_MASTER::LinearProgram ||
      master_->logLevel() == ABA_MASTER::LinearProgram ||
#ifdef ABACUS_PARALLEL
      (level_ == 1 && nIter_ == 1
#else
      (this == master_->root() && nIter_ == 1
#endif
       && (master_->outLevel() == ABA_MASTER::Subproblem ||
           master_->logLevel() == ABA_MASTER::Subproblem   ))) {

    @<|ABA_SUB::cutting()|: turn the streams on@>;
    @<output a line about the linear program@>;
    @<|ABA_SUB::cutting()|: turn the streams off@>;
  }

@ @<|ABA_SUB::cutting()|: turn the streams on@>=
  bool turnOutOn = false;

  if (master_->outLevel() == ABA_MASTER::LinearProgram) 
    master_->out().on();
  else if (master_->out().isOn()) {
    master_->out().off();
    turnOutOn = true;
  }

  bool turnLogOn = false;
    
  if (master_->logLevel() == ABA_MASTER::LinearProgram) 
    master_->out().logOn();
  else if (master_->out().isLogOn()) {
    master_->out().logOff();
    turnLogOn = true;
  }
    
@ @<output a line about the linear program@>=
  master_->out() << setWidth(7) << master_->nSub() << " ";
  master_->out() << setWidth(7) << master_->openSub()->number() << "  ";
#ifdef ABACUS_PARALLEL
  master_->out() << setWidth(8);
  master_->parmaster()->printId(id_);
  master_->out() << " ";
#else
  master_->out() << setWidth(8) << id_ << " ";
#endif
  master_->out() << setWidth(7) << nIter_ << " ";
  master_->out() << setWidth(10) << lp_->value() << " ";
  master_->out() << setWidth(10) << master_->dualBound() << " ";
  if (master_->feasibleFound())
    master_->out() << setWidth(10) << master_->primalBound();
  else
    master_->out() << setWidth(10) << "---";
  master_->out() << endl;

@ @<|ABA_SUB::cutting()|: turn the streams off@>=
  if (master_->outLevel() == ABA_MASTER::LinearProgram) 
    master_->out().off();
  if (master_->logLevel() == ABA_MASTER::LinearProgram) 
    master_->out().logOff();

  if (turnLogOn)
    master_->out().logOn();

  if (turnOutOn)
    master_->out().on();

@ @<count the number of discrete variables being fractional@>=
  int nFractional = 0;
  int nDiscrete   = 0;
  double frac;

  const int nVariables = nVar();
  
  for (int i = 0; i < nVariables; i++)
    if (variable(i)->discrete()) {
      ++nDiscrete;
      frac = fracPart(xVal_[i]);
      if ((frac > master_->eps()) && (frac < 1.0 - master_->machineEps()))
        ++nFractional;
    }

  master_->out()  << endl;
  master_->out(1) << nFractional << " of " << nDiscrete << " discrete variables ";
  master_->out()  << "are fractional" << endl;

@ The function |betterPrimal()| might return |false| although we have
  a better feasible solution, because the primal bound might have been
  updated already in the function |feasible()|. This is an optional
  feature for the user of the framework in order to simplify the bookkeeping
  according to his needs. If no variables are added by the function
  |_pricing()|, then the LP solution is also dual feasible and we can
  fathom the subproblem, otherwise we continue the cutting plane algorithm.
  
@<make a feasibility test@>=
  if (feasible()) {
    master_->out() << "LP-solution is feasible" << endl;
    if (master_->betterPrimal(lp_->value()))
      master_->primalBound(lp_->value());
    status = _pricing(newValues);
    if (status)     continue;
    return Fathoming;
  }

@ Even if the function |_improve()| returns a nonzero value indicating
  that a better solution has been found, we check if it is better than
  the current primal bound, as the primal bound might have been
  already updated during the application of the primal heuristics.
  Like in the function |feasible()| this is an optional feature to simplify
  the bookkeeping of the user.

  If we have found a better solution we reset the tailing off control
  because the subproblem seems to be promising.

  It is not unusual that inactive variables are added during the
  application of primal heuristics. In this case we go immediately to
  the beginning of the cutting plane loop without separating variables
  are constraints.
  
@<improve the primal solution@>=
  double primalValue;  // value of a feasible solution found by primal heuristics
  
  status = _improve(primalValue);

  if (status && master_->betterPrimal(primalValue))
    master_->primalBound(primalValue);

  if (status) {
    tailOff_->reset();
    if (master_->primalViolated(dualRound(lp_->value()))) {
      status = _pricing(newValues);
      if (status)       continue;
      return Fathoming;
    }
  }
  
  if (addVarBuffer_->number()) continue;


@ Note, if |pausing()| returns |true|, then we enter the
  |Branching| phase but there no subproblems are generated.
  
@<test some minor termination criteria@>=
  bool terminate = false;  // becomes |true| if one of the criteria is satisfied
  bool forceFathom = false;  // becomes |true| if fathoming should be forced

  @<check if problem specific fathoming criteria is satisfied@>;
  @<check if problem specific branching criteria is satisfied@>;
  @<check if maximal CPU time is exceeded@>;
  @<check if maximal elapsed time is exceeded@>;
  @<check if there is a tailing-off effect@>;
  @<should we pause the subproblem@>;
  @<check if the iteration limit is reached@>;
  @<price out inactive variables if a termination criterion is fulfilled@>;
  
@ The default implementation of |exceptionFathom()| returns always |false|.
@<check if problem specific fathoming criteria is satisfied@>=
  if (exceptionFathom()) {
    master_->out() << "exceptionFathom(): try fathoming.";
    terminate = true;
    forceFathom = true;
    master_->status(ABA_MASTER::ExceptionFathom);
  }

@ The default implementation of |exceptionBranch()| returns always |false|.
@<check if problem specific branching criteria is satisfied@>=
  if (exceptionBranch()) {
    master_->out() << "exceptionBranch(): try branching." << endl;
    terminate = true;
  }

@ @<check if maximal CPU time is exceeded@>=
  if (!terminate &&
      master_->totalTime_.exceeds(master_->maxCpuTime())) {
    master_->out() << "Maximal CPU time ";
    master_->out() << master_->maxCpuTime() << " exceeded" << endl;
    master_->out() << "Stop subproblem optimization." << endl;
    master_->status(ABA_MASTER::MaxCpuTime);
    terminate = true;
    forceFathom = true;
  }

@ @<check if maximal elapsed time is exceeded@>=
  if (!terminate &&
      master_->totalCowTime_.exceeds(master_->maxCowTime())) {
    master_->out() << "Maximal elapsed time ";
    master_->out() << master_->maxCowTime() << " exceeded" << endl;
    master_->out() << "Stop subproblem optimization." << endl;
    master_->status(ABA_MASTER::MaxCowTime);
    terminate = true;
    forceFathom = true;
  }
  
@ @<check if there is a tailing-off effect@>=
  if (tailOff_->tailOff()) {
    master_->out() << "Try to tail off subproblem processing" << endl;
    terminate = tailingOff();
    if (!terminate) {
      master_->out() << "problem specific resolution: ";
      master_->out() << "no branching enforced" << endl;
      tailOff_->reset();
    }
  }

@ @<should we pause the subproblem@>=
  if (!terminate && pausing()) {
    master_->out() << "Try to pause subproblem" << endl;
    terminate = true;
  }

@ @<check if the iteration limit is reached@>=
  if (!terminate && (maxIterations_ > 0) && (nIter_ >= maxIterations_)) {
    master_->out() << "Iteration limit reached in subproblem: enforce branching" << endl;
    terminate = true;
  }

@ The guarantee and the time limit criteria cause a fathoming of the
  subproblem, whereas the other criteria cause a branching.
  In the function |prepareBranching()| the active constraints and
  variables can  still be modified. In this case the modifications
  takes place at the beginning of the cutting plane algorithm. But nevertheless,
  after the modifications the linear program is not solved.
  
@<price out inactive variables if a termination criterion is fulfilled@>=
  if (terminate) {
    status = _pricing(newValues);
    if (status)       continue;
    if (guaranteed() || forceFathom) return Fathoming;
    if (newValues)    continue;
    status = prepareBranching(lastIteration);
    if (status) continue;
    else        return Branching;
  }

@ @<perform primal and dual separation@>=
  @<should we skip the separation in this subproblem@>;
  if (primalSeparation()) {
    @<perform primal separation@>;
  }
  else {  // dual separation
    @<perform dual separation@>;
  }
  
@ @<should we skip the separation in this subproblem@>=
  if (master_->skippingMode() == ABA_MASTER::SkipByNode) {
    if ((master_->nSubSelected() - 1) % master_->skipFactor() != 0)
      return Branching;
  }
  else {
    if ((level_ - 1) % master_->skipFactor() != 0)
      return Branching;
  }

@ We do not check the return status for a successful separation, but check
  if new constraints have been stored in the buffer |cutBuffer_| because
  violated inequalities might have been generated already earlier.

  We eliminate constraints only if also constraints are generated, because
  we prefer to have the same constraint set if variables are generated
  by |_pricing()|. If no variables are added in |_pricing()|, then
  the function
  |prepareBranching()| performs the elimination of the constraints.
  
@<perform primal separation@>=
  status = _separate();
       
  if (addConBuffer_->number()) _conEliminate();
  else {
    status = _pricing(newValues);
    if (status) continue;
    else {
      if (newValues)    continue;
      if (guaranteed()) return Fathoming;
      status = prepareBranching(lastIteration);
      if (status) continue;
      else        return Branching;
    }
  }
    
@ Like in the previous section for the separation we check also the buffer
  for new generated variables. 
  
@<perform dual separation@>=
  status = _pricing(newValues);
       
  if (addVarBuffer_->number()) _varEliminate();
  else if (guaranteed()) return Fathoming;
  else if (newValues)    continue;
  else {
    if (_separate()) continue;
    else {
      status = prepareBranching(lastIteration);
      if (status) continue;
      else        return Branching;
    }
  }

@q m v prepareBranching @>
@ The function |prepareBranching()| is called before a branching step
  to remove constraints. 
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If constraints have been removed,}
  \docarg{|0|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|lastIteration|}{This argument is always set to |true| in the
                           function call.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::prepareBranching(bool &lastIteration)
@q m e f @>
  {
    lastIteration = true;
    int nElim = _conEliminate();

    if (nElim) {
      lpMethod_ = ABA_LP::Primal;
      return 1;
    }
    else return 0;
  }
         
@q m v solveLp @>
@ The function |solveLp()| solves the LP-relaxation of the subproblem.
@q m e t @>

  As soon as the |LP|-relaxation becomes infeasible in a static
  branch and cut algorithm the respective subproblem can be fathomed.
  Otherwise, we memorize the value of the LP-solution to control
  the tailing off effect.

  {\bf We assume that the |LP| is never primal unbounded.}

@q m b r @>
  \BeginReturn
  \docarg{|0|}{The linear program has an optimimal solution.}
  \docarg{|1|}{If the linear program is infeasible.}
  \docarg{|2|}{If the linear program is infeasible for the current
               variable set, but non-liftable constraints have to be
               removed before a pricing step can be performed.}
  \EndReturn
@q m e r @>
 
@(sub.cc@>=
@q m b f @>
  int ABA_SUB::solveLp ()
@q m e f @>
  {
    @<output some infos on the linear program@>;
    @<optimize the linear program@>;
    @<buffer the solution of the linear program@>;
    @<buffer the dual variables of the linear program@>;
    @<analyze the result of the linear program@>;
  }

@ The ``true'' number of nonzeros is the number of nonzeros not including
  the coefficients of the eliminated variables.
  
@<output some infos on the linear program@>=
  master_->out() << endl;
  master_->out() << "Solving LP " << nIter_ << endl;
  master_->out(1) << "Number of Constraints:  " <<  nCon() << endl;
  master_->out(1) << "Number of Variables  :  " <<  nVar() << "   (not eliminated ";
  master_->out()  << lp_->trueNCol() << ")" << endl;
  master_->out(1) << "True nonzeros        :  " << lp_->trueNnz() << endl;

@ @<optimize the linear program@>=
  ABA_LP::OPTSTAT status;

  master_->countLp();

  localTimer_.start(true);

  status = lp_->optimize(lpMethod_);
  master_->lpSolverTime_.addCentiSeconds( lp_->lpSolverTime_.centiSeconds() );
  lp_->lpSolverTime_.reset();

  master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());

  if (master_->printLP())
    master_->out() << *lp_;

#ifdef TTT1
  char fileName[256];
#ifdef ABACUS_PARALLEL
  sprintf(fileName, "%s.%d.%s.bas", master_->problemName()->string(), 
          master_->nLp(), master_->parmaster()->hostname().string() );
#else
  sprintf(fileName, "%s.%d.bas", master_->problemName()->string(), 
          master_->nLp());
#endif

  if(lp_->writeBasisMatrix(fileName)) {
    master_->err() << "Writing basis to file " << fileName << " failed." << endl;
  }
#endif

@ @<buffer the solution of the linear program@>=
  if (lp_->xValStatus() != ABA_LP::Available) {
    if (!lp_->infeasible()) {
      master_->err() << "ABA_SUB::solveLp(): no LP-solution available." << endl;
      exit(Fatal);
    }
  }
  else {
    const int nVariables = nVar();
    for (int i = 0; i < nVariables; i++)
      xVal_[i] = lp_->xVal(i);
  }

@ If there are constraints but the dual variables are missing we
  stop for safety.
  
@<buffer the dual variables of the linear program@>=
  if ((lp_->yValStatus() != ABA_LP::Available) && nCon()) {
    master_->err() << "ABA_SUB::solveLp(): no dual variables available." << endl;
    exit(Fatal);
  }
  else {
    const int nConstraints = nCon();
    for (int i = 0; i < nConstraints; i++)
      yVal_[i] = lp_->yVal(i);
  }

@ @<analyze the result of the linear program@>=
  if (status == ABA_LP::Optimal) {
    @<output the solution and get the basis@>;
  }
  else  if (status == ABA_LP::Infeasible) {
    @<try to add variables to make the linear program feasible@>;
  }
  else {
    @<stop, a severe error occurred during the solution of the LP@>;
  }

@ The flag |ignoreInTailingOff_| might have been set by the function
  |ingnoreInTailingOff()| such that the current LP solution is not
  considered in the tailing off analysis.

@<output the solution and get the basis@>=
  master_->out() << endl;
  master_->out(1) << "LP-solution            : " << lp_->value() << endl;
  master_->out(1) << "Best feasible solution : " << master_->primalBound()  << endl;

  if (ignoreInTailingOff_)
    ignoreInTailingOff_ = false;
  else 
    tailOff_->update(lp_->value());

  getBase();    

  return 0;

@q m v exceptionFathom @>
@ The function |exceptionFathom()| can be used to specify a problem
  specific fathoming criterium that is checked before the separation or
  pricing. The default implementation always returns |false|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the subproblem should be fathomed,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::exceptionFathom()
@q m e f @>
  {
    return false;
  }

@q m v exceptionBranch @>
@ The function |exceptionBranch()| can be used to specify a problem
  specific criteria for enforcing a branching step. This criterium
  is checked before the separation or
  pricing. The default implementation always returns |false|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the subproblem should be fathomed,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::exceptionBranch()
@q m e f @>
  {
    return false;
  }

@ The function |infeasibleSub()| sets the dual bound correctly (plus
  or minus infinity) for an infeasible subproblem.
  
@<try to add variables to make the linear program feasible@>=
  if (!master_->pricing()) {
    infeasibleSub();
    return 1;
  }
  if (!removeNonLiftableCons()) return 2;
  getBase();
  if (_makeFeasible ()) {
    infeasibleSub();
    return 1;
  }
  else return 2;

@ @<stop, a severe error occurred during the solution of the LP@>=
  master_->err() << "ABA_SUB::solveLp() return status of ABA_LP::optimize() is ";
  master_->err() << (int) status << " (do not know how to proceed)" << endl;
  exit (Fatal);
  return 0;

@q m v fixAndSetTime @>
@ The virtual function |fixAndSetTime()| controls if variables should be
  fixed or set when all variables price out corretly.
  The default implementation always returns |true|.
@q m e t @>
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If variables should be fixed and set,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::fixAndSetTime()
@q m e f @>
  {
    return true;
  }

@ The function |int _makeFeasible()| is called if the
  |LP| is infeasible and adds inactive variables, which can
  make the |LP| feasible again, to the set of active variables.
  The function returns |0| if feasibility might have been
  restored and |1| if it is guaranteed that the linear program
  is infeasible on the complete variable set.

@(sub.cc@>=
  int ABA_SUB::_makeFeasible()
  {
    if (!master_->pricing()) return 1;

    master_->out() << "ABA_SUB::_makeFeasible()" << endl;
    @<make the current basis global dual feasible@>;
    @<can we fathom the subproblem?@>;
    @<emulate an iteration of the dual simplex method@>;
  }

@ If the variables are added we return and solve the linear program again
  since these variables might restore already the feasibility and we
  can continue this function only if the basis is dual feasible.

  The second argument of |_pricing()| is |false|, because no variables
  should be fixed and set (it is useless for an infeasible subproblem).

@<make the current basis global dual feasible@>=
  bool newValues;  // only a dummy here
  
  int status = _pricing(newValues, false);
  if (status == 1)
    return 0;
  else if (status == 2) {
    master_->out() << "ABA_SUB::_makeFeasible(): pricing failed due to ";
    master_->out() << "non-liftable constraints" << endl;
    exit(Fatal);
  }
    
  
@ If the basis is dual feasible, then the associated objective function
  value is a dual bound for the optimization of the subproblem. Hence
  we can fathom the node if the primal bound is violated.

@<can we fathom the subproblem?@>=
  if (master_->primalViolated(dualRound(lp_->value()))) return 1;

@ An inactive variable is only a candidate if it is not implicitly set 0, i.e.,
  let $l$ be the objective function value of the linear program, $p$ be
  the value of the best known primal feasible solution, and $r_e$ the
  reduced cost of the variable $e$. If our problem is a maximization problem
  $e$ is only added if $l + r_e >= p$ holds, or if it is minimization problem
  $l + r_e <= p$ holds.

  Let $B$ be the basis matrix corresponding to the dual feasible LP solution,
  at which the primal infeasibility was detected. For each candidate variable
  $e$ let $a_e$ be the column of the constraint matrix corresponding to $e$
  and solve the system $B z = a_e$. Let $z_b$ be the component of $z$
  corresponding to basis variable $b$. Activating $e$ ``reduces some
  infeasibility'' if one of the following holds, where $l_b$ and $u_b$
  are the local lower and upper bounds of variable $b$.

  \shortitem{--} $b$ is a structural variable (i.e., not a slack variable) and
  $$ x_b < l_b \hbox{\rm\ and } z_b < 0$$
  or
  $$ x_b > u_b \hbox{\rm\ and } z_b > 0$$

  \shortitem{--} $b$ is a slack variable and
  $$ x_b < 0 \hbox{\rm\ and } z_b < 0.$$

  We refer the reader to \ref{Pad95} for an excellent description of the
  dual simplex method.
  
@<emulate an iteration of the dual simplex method@>=
  bInvRow_ = new double[nCon()];

  status = lp_->getInfeas(infeasCon_, infeasVar_, bInvRow_);

  if (status) {
    master_->out() << "ABA_SUB::_makeFeasible(): lp_->getInfeas() failed" << endl;
    exit (Fatal);
  }

  status = makeFeasible();

  delete bInvRow_;
  bInvRow_ = 0;

  if (status) return 1;
  else        return 0;

@q m v makeFeasible @>
@ The default implementation of |makeFeasible()| does nothing.

  If there is an infeasible structural variable then it is stored 
  in |infeasVar_|, otherwise |infeasVar_| is |-1|. If there is 
  an infeasible slack variable, it is stored in |infeasCon_|, otherwise
  it is |-1|. At most one of the two members |infeasVar_| and |infeasCon_|
  can be nonnegative.
  A reimplementation in a derived class should  generate variables
  to restore feasibility or confirm that the subproblem is infeasible.

  The strategy for the generation of inactive variables is completely
  problem and user specific.
  For testing if a variable might restore again the feasibility the functions
  |ABA_VARIABLE::useful()| and |ABA_SUB::goodCol()| might be helpful.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If feasibility can be restored,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>
@(sub.cc@>=
@q m b f @>
  int ABA_SUB::makeFeasible()
@q m e f @>
  {
    return 1;
  }

@q m v goodCol @>
@q m e t @>
@ The function |goodCol()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the column |col| might restore feasibiblity if
                  the variable with value |x| turns out to be
                  infeasible,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|col|}{The column of the variable.}
  \docarg{|row|}{The row of the basis inverse associated with the infeasible
                 variable.}
  \docarg{|x|}{The LP-value of the infeasible variable.}
  \docarg{|lb|}{The lower bound of the infeasible variable.}
  \docarg{|ub|}{The upper bound of the infeasible variable.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::goodCol(ABA_COLUMN &col, 
                        ABA_ARRAY<double> &row,
                        double x, 
                        double lb, 
                        double ub)
@q m e f @>
  {
    double p = 0.0;

    const int nnz = col.nnz();

    for (int i = 0; i < nnz; i++)
      p += col.coeff(i) * row[col.support(i)];

    if (x < lb) {
      if (p < -master_->eps()) return true;
      else                  return false;
    }
    else if (x > ub) {
      if (p > master_->eps()) return true;
      else                 return false;
    }
    else {
      master_->err() << "ABA_SUB::goodCol(): variable is feasible!?" << endl;
      exit(Fatal);
      return false;  // to suppress warning of gcc
    }
  }

@q m v pricing @>
@ The function |pricing()| should generate inactive variables which
  do not price out correctly.
  The default implementation  does nothing and returns 0.
@q m e t @>
@q m b r @>
  \BeginReturn
   The number of new variables.
   \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::pricing()
@q m e f @>
  {
    return 0;
  }

@ The function |_pricing()|.

  If |doFixSet| is |true|, then we try to fix and set variables, if
  all inactive variables price out correctly. In this case |newValues|
  becomes |true| of a variable is set or fixed to a value different
  from its value in the last linear program.

  In a pricing step the reduced costs of inactive variables are
  computed and variables with positive (negative) reduced costs
  in a maximization (minimization) problem
  are activated.

  The function |_pricing()| returns the
  1 if no global optimality can be guaranteed, since variables have
  negative reduced costs, it returns 2 if before a pricing step can
  be performed, non-liftable constraints have to be removed,
  and 0 if the LP-solution is global dual feasible.

  Also if there are no inactive variables, this function is called
  since it will also try to fix and set variables.

  |true| is the default value
  of |doFixSet|. No variables should be fixed or set if |_pricing()|
  is called from |_makeFeasible()|.

@(sub.cc@>=
  int ABA_SUB::_pricing(bool &newValues, bool doFixSet)
  {
    int nNew = 0;

    newValues = false;

    if (master_->pricing()) {

      if (!removeNonLiftableCons()) return 2;
    
      master_->out() << endl;
      master_->out() << "Price out Inactive Variables" << endl;

      localTimer_.start(true);
      nNew = pricing();
      master_->pricingTime_.addCentiSeconds(localTimer_.centiSeconds());
    }

    if (nNew) {
      if(doFixSet && fixAndSetTime()) 
        fixing(newValues);  // only with old candidates
    } else {
      if (betterDual(lp_->value())) dualBound(dualRound(lp_->value()));
      if (doFixSet && fixAndSetTime()) {
        if (master_->primalViolated(dualBound()))
          fixing(newValues);
        else
          fixAndSet(newValues);
      }
    }
    
    if (nNew) return 1;
    else      return 0;
  }

@q m v primalSeparation @>
@ The function |primalSeparation()| is a virtual function which
  controls, if during the cutting plane phase a (primal) separation step
  or a pricing step (dual separation) should be performed.

  Per default a pure cutting plane algorithm performs always a primal
  separation step, a pure column generation algorithm never performs
  a primal separation, and a hybrid algorithm generates usually cutting
  planes but from time to time also inactive variables are priced
  out depending on the |pricingFrequency()|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{Then cutting planes are generated in this iteration.}
  \docarg{|false|}{Then columns are generated in this iteration.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::primalSeparation()
@q m e f @>
  {
    if (master_->cutting()) {
      if (master_->pricing()) {
        if (addConBuffer_->number()) return true;
        int pricingFrequency = master_->pricingFreq();

        if (pricingFrequency && nIter_ % pricingFrequency == 0)
          return false;
        else
          return true;
      }
      else return true;
    }
    else return false;
  }

@q m f xVal @>
@q m e t @>
@ The function |xVal()|

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable under consideration.}
  \EndArg
@q m e a @>
@q m b r @>
  \BeginReturn
  The value of the |i|-th variable in the last solved linear program.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_SUB::xVal(int i) const
@q m e f @>

@q m f yVal @>
@q m e t @>
@ The function |yVal()|

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable under consideration.}
  \EndArg
@q m e a @>
@q m b r @>
  \BeginReturn
  The value of the |i|-th dual variable in the last solved linear program.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_SUB::yVal(int i) const
@q m e f @>

@q m v dualRound @>
@q m e t @>
@ The function |dualRound()|.

@q m b r @>
  \BeginReturn
  If all objective function values of feasible solutions are integer
  the function |dualRound()| returns  |x| rounded up to the next integer
  if this is a minimization problem, or |x| rounded down to the next integer
  if this is a maximization problem, respectively. Otherwise, the return
  value is |x|.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{The value that should be rounded if possible.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  double ABA_SUB::dualRound(double x)
@q m e f @>
  {
    if (master_->objInteger()) {
      if (master_->optSense()->max()) return floor(x + master_->eps());
      else                         return ceil(x  - master_->eps());
    }
    else return x;
  }

@q m v guaranteed @>
@q m e t @>
@ The function |guaranteed()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the lower and the upper bound of the subproblem
                  satisfies the guarantee requirements,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::guaranteed()
@q m e f @>
  {
    double lb = lowerBound();

    if (fabs(lb) < master_->machineEps()){
      if (fabs(upperBound()) < master_->machineEps()) return true;
      else                                            return false;
    }
    
    if (guarantee() + master_->machineEps() < master_->requiredGuarantee()) {
      master_->out() << "Subproblem guarantee reached" << endl;
      master_->status(ABA_MASTER::Guaranteed);
      return true;
    }
    else
      return false;
  }

@q m v guarantee @>
@ The function |guarantee()| may not be called if 
  the lower bound is 0 and upper bound not equal to 0.
@q m e t @>

@q m b r @>
  \BeginReturn
  The guarantee that can be given for the subproblem. 
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  double ABA_SUB::guarantee()
@q m e f @>
  {
    double lb = lowerBound();

    if (fabs(lb) < master_->machineEps()) {
      if (fabs(upperBound()) < master_->machineEps()) return 0.0;
      else {
        master_->err() << "ABA_SUB::guarantee(): cannot compute guarantee ";
        master_->err() << "with lower bound 0" << endl;
        exit (Fatal);
      }
    }
    
    return fabs((upperBound() - lb)/lb * 100.0); 

  }

@q m f ancestor @>
@q m e t @>  
@ The function |ancestor()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |this| subproblem is an ancestor of the
                  subproblem |sub|. We define that a subproblem
                  is its own ancestor,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|sub|}{A pointer to a subproblem.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::ancestor(const ABA_SUB *sub) const
@q m e f @>
  {
#ifdef ABACUS_PARALLEL
    master_->out() << "WARNING: ABA_SUB::ancestor() not supported in parallel "
                   << "version. (returning false)" << endl;
#else
    const ABA_SUB *current = sub;

    do {
      if (this == current) return true;
      if (current == master_->root()) break;
      current = current->father();
    } while (1);
#endif

    return false;
  }

@q m v removeNonLiftableCons @>
@q m e t @>
@ The function |removeNonLiftableCons()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If all active constraints can be lifted.}
  \docarg{|false|}{otherwise.
@q m e r @>
  In this case the non-liftable constraints are removed and
  |genNonLiftCons_| is set to |false| to avoid the generation
  of non-liftable constraints in the next cutting plane iterations.
@q m b r @>
  }
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f  @>
  bool ABA_SUB::removeNonLiftableCons()
@q m e f @>
  {
    if (!genNonLiftCons_) return true;
    
    int nNonLiftable = 0;

    const int nConstraints = nCon();
    
    for (int i = 0; i < nConstraints; i++)
      if (!constraint(i)->liftable()) {
        removeCon(i);
        nNonLiftable++;
      }

    genNonLiftCons_ = false;

    if (nNonLiftable) {
      master_->out() << "Removing " << nNonLiftable << " non-liftable constraints" << endl;
      lpMethod_ = ABA_LP::Primal;
      return false;
    }
    return true;
      
  }

@q m v chooseLpMethod @>
@ The virtual function |chooseLpMethod()| controls the method used to
  solve a linear programming relaxation. The default implementation
  chooses the barrier method for the first linear program of the root
  node and for all other linear programs it tries to choose a method
  such that phase 1 of the simplex method is not required.
@q m e t @>
@q m b r @>
  \BeginReturn
  The method the next linear programming relaxation is solved with.
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|nVarRemoved|}{The number of removed variables.}
  \docarg{|nConRemoved|}{The number of removed constraints.}
  \docarg{|nVarAdded|}{The number of added variables.}
  \docarg{|nConAdded|}{The number of added constraint.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  ABA_LP::METHOD ABA_SUB::chooseLpMethod(int nVarRemoved, 
                                         int nConRemoved,
                                         int nVarAdded, 
                                         int nConAdded)
@q m e f @>
  {
    ABA_LP::METHOD lpMethod = ABA_LP::Primal;

    if (nIter_ == 0) {
#ifdef ABACUS_PARALLEL
      if (level_ == 1)
#else
      if (this == master_->root())
#endif
        lpMethod = ABA_LP::BarrierAndCrossover;
      else
        lpMethod = ABA_LP::Dual;
    }
      
    if (nConAdded)
      lpMethod = ABA_LP::Dual;
    else if (nConRemoved)
      lpMethod = ABA_LP::Primal;

    if (nVarAdded)
      lpMethod = ABA_LP::Primal;
    else if (nVarRemoved)
      lpMethod = ABA_LP::Dual;

    if (nConAdded && nVarAdded)
      lpMethod = ABA_LP::BarrierAndCrossover;

    return lpMethod;
  }

@q m f master @>
@q m e t @>
@ The function |master()|.

@q m b r @>
  \BeginReturn
  A pointer to the master of the optimization.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_MASTER *ABA_SUB::master()
@q m e f @>

@q m v removeVars @>
@ With function |removeVars()| variables can be removed from the
  set of active variables. The variables are not removed when this function
  is called, but are buffered and removed at the beginning of the next
  iteration.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|remove|}{The variables which should be removed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::removeVars(ABA_BUFFER<int> &remove)
@q m e f @>
  {
    const int nRemove = remove.number();

    for (int i = 0; i < nRemove; i++)
      removeVarBuffer_->push(remove[i]);
  }

@q m v removeVar @>
@ The function |removeVar()| can be used to remove a single variable
  from the set of active variables. Like in the function |removeVars()|
  the variable is buffered and removed at the beginning of the next
  iteration.
@q m e t @>
@q m b a @>
  \BeginArg
  \docarg{|i|}{The variable which should be removed.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_SUB::removeVar(int i)
@q m e f @>

@ The function |_selectVars()| selects the |master_->maxVarAdd()| best
  variables from the buffered variables.

  \BeginArg
  \docarg{|newVars|}{Holds the selected variables after the call.}
  \EndArg

@(sub.cc@>=
  void ABA_SUB::_selectVars(ABA_BUFFER<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT>*> &newVars)
  {
    selectVars();
    addVarBuffer_->sort(master_->maxVarAdd());
    addVarBuffer_->extract(master_->maxVarAdd(), newVars);
  }

@q m v selectVars @>
@ The virtual dummy function |selectVars()| is called before variables
  are selected from the variable buffer. It can be redefined in a
  derived class e.g., to remove multiply inserted variables from the
  buffer.
@q m e t @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::selectVars()
@q m e f @>
  { }

@ The function |_selectCons()| selects the |master_->maxConAdd()| best
  constraints from the buffered constraints and stores them in
  |newCons|.

@(sub.cc@>=
  void ABA_SUB::_selectCons(ABA_BUFFER<ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *> &newCons)
  {
    selectCons();
    addConBuffer_->sort(master_->maxConAdd());
    addConBuffer_->extract(master_->maxConAdd(), newCons);
  }

@q m v selectCons @>
@ The virtual dummy function |selectCons()| is called before constraint
  are selected from the constraint buffer. It can be redefined in a
  derived class e.g., to remove multiply inserted constraints from the
  buffer.
@q m e t @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::selectCons()
@q m e f @>
  { }

@q m f nnzReserve @>
@q m e t @>
@ The function |nnzReserve()|.

@q m b r @>
  \BeginReturn
  The additional space for nonzero elements of the constraint matrix when
  it is passed to the LP-solver.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_SUB::nnzReserve() const
@q m e f @>

@q m f relativeReserve @>
@q m e t @>
@ The function |relativeReserve()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the reserve space for variables, constraints, 
                   and nonzeros is given in percent of the
                   original space, and |false| if its given as absolute value,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b f @>
  bool ABA_SUB::relativeReserve() const
@q m e f @>
  
@q m f branchRule @>
@q m e t @>
@ The function |branchRule()|.

@q m b r @>
  \BeginReturn
  A pointer to the branching rule of the subproblem.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_BRANCHRULE *ABA_SUB::branchRule()
@q m e f @>

@q m f addBranchingConstraint @>
@ The function |addBranchingConstraint()| adds a branching constraint
  to the constraint buffer such that it is
  automatically added at the beginning of the cutting plane algorithm.
  It should be used in definitions of the pure virtual function
  |BRANCHRULE::extract()|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the constraint could be added,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|slot|}{A pointer to the pools slot containing the branching
                  constraint.}
  \EndArg
@q m e a @>

@q m b f @>
  int ABA_SUB::addBranchingConstraint(ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *slot)
@q m e f @>

@q m v addCons @>
@ The function |addCons()| tries to add new constraints to the constraint
  buffer and a pool. The memory management of added constraints is
  passed to \ABACUS\ by calling this function.
@q m e t @>

@q m b r @>
  \BeginReturn
  The number of added constraints.
  \EndReturn
@q m e r @>
@q m b a @>
    \BeginArg
    \docarg{|constraints|}{The new constraints.}
    \docarg{|pool|}{The pool in which the new constraints are inserted.
                  If the value of this argument is 0, then the
                  cut pool of the master is selected. Its default value
                  is 0.}
    \docarg{|keepInPool|}{If |(*keepInPool)[i]| is |true|, then the constraint
                        stays in the pool even if it is not activated.
                        The default value is a 0-pointer.}
  \docarg{|rank|}{If this pointer to a buffer is nonzero, this buffer
                  should store a rank for each constraint. 
                  The greater the rank, the better the variable.
                  The default value of |rank| is 0.}
   \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::addCons(ABA_BUFFER<ABA_CONSTRAINT*> &constraints,
                       ABA_POOL<ABA_CONSTRAINT, ABA_VARIABLE> *pool,
                       ABA_BUFFER<bool> *keepInPool,
                       ABA_BUFFER<double> *rank)
@q m e f @>
  {
    int       status;
    int       nAdded = 0;
    ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE> *slot;
    bool      keepIt;
    const int nConstraints = constraints.number();

    int       lastInserted = nConstraints;

    if (pool == 0) pool = master_->cutPool();

    for (int i = 0; i < nConstraints; i++) {
      slot = pool->insert(constraints[i]);
      if (slot == 0) {
        lastInserted = i - 1;
        break;
      }
      else {
        if (keepInPool) 
          keepIt = (*keepInPool)[i];
        else
          keepIt = false;
        if (rank) 
          status = addConBuffer_->insert(slot, keepIt, (*rank)[i]);
        else
          status = addConBuffer_->insert(slot, keepIt);
        if (status) {
          if (!keepIt && slot->conVar()->deletable()) 
            slot->removeConVarFromPool();
        }
        else
          nAdded++;
      }
    }
    @<delete the constraints that could be not inserted into the pool@>;

    return nAdded;
  }

@ @<delete the constraints that could be not inserted into the pool@>=
  if (lastInserted < nConstraints) {
    master_->out() << "ABA_SUB::addCons(): pool too small,";
    master_->out() << " deleting " << nConstraints - lastInserted;
    master_->out() << " constraints." << endl;

    for (int i = lastInserted + 1; i < nConstraints; i++)
      delete constraints[i];
  }



@q m v addCons @>
@ The function |addCons()| adds constraints to the active constraints
  and the linear program.
@q m e t @>

@q m b r @>
  \BeginReturn
  The number of added constraints.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|newCons|}{A buffer storing the pool slots of the new
                     constraints.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::addCons(
                 ABA_BUFFER<ABA_POOLSLOT<ABA_CONSTRAINT, ABA_VARIABLE>*> &newCons)
@q m e f @>
  {
    @<|ABA_SUB::addCons()|: local variables@>;
    @<require the new constraints a reallocation?@>;
    @<get the constraints from the pool slots@>;
    @<compute the average distance of the added cuts@>;
    @<add the constraints to the active constraints and the LP@>;
  }

@ @<|ABA_SUB::addCons()|: local variables@>=
  const int nNewCons = newCons.number();

  ABA_BUFFER<ABA_CONSTRAINT*> cons(master_, nNewCons);
  int i;

@ @<require the new constraints a reallocation?@>=
  if (nCon() + nNewCons >= maxCon()) {
    int newMax = ((maxCon() + nNewCons)*11)/10 + 1;
    conRealloc(newMax);
  }

@ @<get the constraints from the pool slots@>=
  for (i = 0; i < nNewCons; i++) {
    newCons[i]->conVar()->activate();
    cons.push((ABA_CONSTRAINT*) newCons[i]->conVar());
  }

@ @<compute the average distance of the added cuts@>=
  if (master_->showAverageCutDistance()) {
    double averageDistance;

    averageDistance = 0.0;

    for (i = 0; i < nNewCons; i++)
      averageDistance += cons[i]->distance(xVal_, actVar_);

    averageDistance /= nNewCons;

    master_->out(1) << "average distance of cuts: " << averageDistance << endl;
  }

@ @<add the constraints to the active constraints and the LP@>=
  for (i = 0; i < nNewCons; i++)
    (*slackStat_)[nCon() + i] = new ABA_SLACKSTAT(master_, ABA_SLACKSTAT::Unknown);
  actCon_->insert(newCons);

  localTimer_.start(true);
  lp_->addCons(cons);
  master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());

  master_->addCons(nNewCons);

  return nNewCons;



@q m v addVars @>
@ The function |addVars()| tries to add new variables to the variable
  buffer and a pool. The memory management of added variables is
  passed to \ABACUS\ by calling this function.
@q m e t @>

@q m b r @>
  \BeginReturn
   The number of added variables.
  \EndReturn
@q m e r @>
@q m b a @>
    \BeginArg
    \docarg{|variable|}{The new variables.}
    \docarg{|pool|}{The pool in which the new variables are inserted.
                  If the value of this argument is 0, then the default
                  variable pool is taken. The default value is 0.}
    \docarg{|keepInPool|}{If |(*keepInPool)[i]| is |true|, then the variable
                        stays in the pool even if it is not activated.
                        The default value is a 0-pointer.}
  \docarg{|rank|}{If this pointer to a buffer is nonzero, this buffer
                  should store a rank for each variable. 
                  The greater the rank, the better the variable.
                  The default
                  value of |rank| is 0.}
    \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::addVars(ABA_BUFFER<ABA_VARIABLE*> &variables,
                       ABA_POOL<ABA_VARIABLE, ABA_CONSTRAINT> *pool,
                       ABA_BUFFER<bool> *keepInPool,
                       ABA_BUFFER<double> *rank)
@q m e f @>
  {
      
    int       status;
    int       nAdded = 0;
    ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *slot;
    bool      keepIt;
    const int nVariables = variables.number();
    int       lastInserted = nVariables;


    if (pool == 0) pool = master_->varPool();

    for (int i = 0; i < nVariables; i++) {
      slot = pool->insert(variables[i]);
      if (slot == 0) {
        lastInserted = i - 1;
        break;
      }
      else {
        if (keepInPool) 
          keepIt = (*keepInPool)[i];
        else
          keepIt = false;
        if (rank) 
          status = addVarBuffer_->insert(slot, keepIt, (*rank)[i]);
        else
          status = addVarBuffer_->insert(slot, keepIt);
        if (status) {
          if (!keepIt && slot->conVar()->deletable()) 
            slot->removeConVarFromPool();
        }
        else
          nAdded++;
      }
    }
    @<delete the variables that could be not inserted into the pool@>;

    return nAdded;
  }

@ @<delete the variables that could be not inserted into the pool@>=
  if (lastInserted < nVariables) {
    master_->out() << "ABA_SUB::addVars(): pool too small,";
    master_->out() << " deleting " << nVariables - lastInserted;
    master_->out() << " variables." << endl;

    for (int i = lastInserted + 1; i < nVariables; i++)
      delete variables[i];
  }

@q m v addVars @>
@ The function |addVars()| adds both the variables in |newVars| to the
  set of active variables and to the linear program of the subproblem.
  If the new number of variables exceeds the maximal number of  variables
  an automatic reallocation is performed.
@q m e t @>

@q m b r @>
  \BeginReturn
  The number of added variables.
@q m e r @>
  We require this
  feature in derived classes if variables of |newVars| can be discarded
  if they are already active.
@q m b r @>
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|newVars|}{A buffer storing the pool slots of the new
                     variables.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::addVars(
                 ABA_BUFFER<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *> &newVars)
@q m e f @>
  {
    activateVars(newVars);
    addVarsToLp(newVars);
    
    tailOff_->reset();

    return newVars.number();
  }


@q m v variablePoolSeparation @>
@ The function |variablePoolSeparation()| tries to generate inactive
  variables from a pool.
@q m e t @>
@q m b r @>
  \BeginReturn
  The number of generated variables.
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|ranking|}{This parameter indicates how the ranks of
                     geneated variables should be computed
                       (0: no ranking; 1: violation is rank, 
                        2: absolute value of violation is rank
                        3: rank determined by |ABA_CONVAR::rank()|).
                      The default value is 0.
                     }
  \docarg{|pool|}{The pool the variables are generated from.
                  If |pool| is 0, then the default variable pool
                  is used. The default value of |pool| is 0.}
  \docarg{|minAbsViolation|}{A violated constraint/variable is only
                             added if the absolute
                             value of its violation is at least
                             |minAbsViolation|. The default value is 0.001.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::variablePoolSeparation(int ranking, 
                                      ABA_POOL<ABA_VARIABLE, ABA_CONSTRAINT> *pool,
                                      double minAbsViolation)
@q m e f @>
  {
    if (pool)
      return pool->separate(yVal_, actCon_, this, addVarBuffer_, 
                            minAbsViolation, ranking);
    else
      return master_->varPool()->separate(yVal_, actCon_, this, addVarBuffer_,
                                          minAbsViolation, ranking);
  }

@q m v constraintPoolSeparation @>
@ The function |constraintPoolSeparation()| tries to generate inactive
  constraints from a pool.
@q m e t @>
@q m b r @>
  \BeginReturn
  The number of generated constraints.
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|ranking|}{This parameter indicates how the ranks of
                     violated constraints should be computed
                       (0: no ranking; 1: violation is rank, 
                        2: absolute value of violation is rank,
                        3: rank determined by |ABA_CONVAR::rank()|).
                      The default value is 0.
                     }
  \docarg{|pool|}{The pool the constraints are generated from.
                  If |pool| is 0, then the default constraint pool
                  is used. The default value of |pool| is 0.}
  \docarg{|minAbsViolation|}{A violated constraint/variable is only
                             added if the absolute
                             value of its violation is at least
                             |minAbsViolation|.
                             The default value is |0.001|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::constraintPoolSeparation(int ranking,
                                        ABA_POOL<ABA_CONSTRAINT, ABA_VARIABLE> *pool,
                                        double minViolation)
@q m e f @>
  {
    if (pool)
      return pool->separate(xVal_, actVar_, this, addConBuffer_,
                            minViolation, ranking);
    else
      return master_->cutPool()->separate(xVal_, actVar_, this, addConBuffer_,
                                          minViolation, ranking);
  }

@q m f addConBufferSpace @>
@ The function |addConBufferSpace()| can be used to determine the maximal
  number of the constraints which still can be added to the constraint
  buffer. A separation algorithm should stop as soon as the number of
  generated constraints reaches this number because further work is
  useless.
@q m e t @>
@q m b r @>
  \BeginReturn
  The number of constraints
  which still can be inserted into the constraint buffer.
\EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::addConBufferSpace() const
@q m e f @>

@q m f addVarBufferSpace @>
@ The function |addVarBufferSpace()| can be used to determine the maximal
  number of the variables which still can be added to the variable
  buffer. A pricing algorithm should stop as soon as the number of
  generated variables reaches this number because further work is
  useless.
@q m e t @>
@q m b r @>
  \BeginReturn
  The number of variables
  which still can be inserted into the variable buffer.
\EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::addVarBufferSpace() const
@q m e f @>

@q m f objAllInteger @>
@ If all variables are |Binary| or |Integer| and all objective function
  coefficients are integral, then all objective function values of
  feasible solutions are integral. The function |objAllInteger()| tests
  this condition for the current set of active variables.

  Note, the result of this function can only be used to set the
  global parameter if |actVar| contains all variables of the problem
  formulation.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If this condition is satisfied by the currently active
                  variable set,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::objAllInteger()
@q m e f @>
  {
    ABA_VARIABLE *v;
    double x;

    const int nVariables = nVar();

    for (int i = 0; i < nVariables; i++) {
      v = variable(i);
      if (v->discrete()) {
        x = v->obj();
        if (x - floor(x) > master_->machineEps()) {
          return false;
        }
      }
      else
        return false;
    }
    
    master_->out() << "objective function values of feasible solutions are integer" << endl;

    return true;
  }

@q m f integerFeasible @>
@ The function |integerFeasible()| can be used to check if the solution
  of the LP-relaxation is primally feasible if for feasibility an
  integral value for all binary and integer variables is sufficient.
  This function can be called from the function |feasible()| in derived
  classes.
@q m e t @>
@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the LP-value of all binary and integer variables is
                  integral,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::integerFeasible()
@q m e f @>
  {
    double frac;

    const int nVariables = nVar();
    
    for (int i = 0; i < nVariables; i++) 
      if (variable(i)->discrete()) {
        frac = fracPart(xVal_[i]);
        if ((frac > master_->machineEps()) &&
            (frac < 1.0 - master_->machineEps()))
            return false;
      }

    return true;
  }

@q m f nDormantRounds @>
@q m e t @>
@ The function |nDormantRounds()|.

@q m b r @>
  \BeginReturn
  The number of subproblem optimization the subproblem is already dormant.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::nDormantRounds() const
@q m e f @>

@ The function |newDormantRound()| increments the counter for the
  number of rounds the subproblem is dormant. This function is called,
  when the set of open subproblems is scanned for the selection of
  the next subproblem.

  void ABA_SUB::newDormantRound()

@q m f ignoreInTailingOff @>
@ The function |ignoreInTailingOff()| can be used to control better
  the tailing-off effect. If this function is called, the next
  LP-solution is ignored in the tailing-off control. Calling 
  |ignoreInTailingOff()| can e.g.\ be considered in the following
  situation: If only constraints that are required for the integer
  programming formulation of the optimization problem are added then
  the next LP-value could be ignored in the tailing-off control. Only
  ``real'' cutting planes should be considered in the tailing-off
  control (this is only an example strategy that might not be
  practical in many situations, but sometimes turned out to be
  efficient).
@q m e t @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::ignoreInTailingOff()
@q m e f @>
  {
    master_->out(1) << "next LP solution ignored in tailing off" << endl;
    ignoreInTailingOff_ = true;
  }

@q m v branching @>
@ The function |branching()| is called if the global lower bound of a
  \bac\ node is still strictly less than the local upper bound, but
  either no violated cutting planes or variables are found, or we abort the
  cutting phase for some other strategic reason (e.g.,
  observation of a tailing off effect, or branch pausing).

  Usually, two new subproblems are generated. However,
  our implementation of |branching()| is more sophisticated that
  allows different branching. Moreover,
  we also check if this node is only paused. If this is the
  case the node is put back into the list of open \bac\
  nodes without generating sons of this node. 

  Finally if none of the previous conditions is satisfied
  we generate new subproblems.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|Done|}{If sons of the subproblem could be generated,}
  \docarg{|Fathoming|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  ABA_SUB::PHASE ABA_SUB::branching()
@q m e f @>
  {
    master_->out() << endl << "Branching Phase" << endl << endl;

    @<check if the maximum enumeration level is reached@>;
    @<check if the subproblem becomes dormant without branching@>;
    @<generate the branching rules@>;
    @<generate the sons@>;

    status_ = Processed;

    return Done;
  }

@ @<check if the maximum enumeration level is reached@>=
  if (level_ == master_->maxLevel()) {
    master_->out() << "Maximum enumeration level " << master_->maxLevel();
    master_->out() << " reached, no branching" << endl;
    master_->status(ABA_MASTER::MaxLevel);
    return Fathoming;
  }

@ Sometimes it turns out to be appropriate to stop the optimization
  of a specific subproblem without creating any sons but putting
  the node back into the list of open subproblems. Per default no
  pausing is performed but the virtual function |pausing()| can
  be redefined in derived classes.

  Then we check the parameter if only after processing a node
  several times its sons should be generated |(delayedBranching(nOpt_)|.
  This idea
  is motivated by the pool separation. When such a dormant
  node is awaked in the meantime pool constraints might have
  become available which are violated by the last |LP|-solution.

  A subproblem can be only inserted in the set of open subproblems without
  branching if there are other subproblems for further processing.

  The statuses of the variables (|fsVarStat|, |lpVarStat|) are
  not deleted, when a subproblem becomes |Dormant|.

@<check if the subproblem becomes dormant without branching@>=
  if (pausing() || master_->delayedBranching(nOpt_))
    if (!master_->openSub()->empty()) {
      master_->out() << "making node dormant" << endl;
      master_->openSub()->insert(this);
      status_ = Dormant;
      nDormantRounds_ = 0;
      return Done;
    }

@ If no branching rule is found we can fathom the subproblem.
  A branch rule defines the modifications of the current subproblem for
  a new subproblem. 

@<generate the branching rules@>=

  ABA_BUFFER<ABA_BRANCHRULE*> rules(master_, nVar());

  localTimer_.start(true);
  int status = generateBranchRules(rules);
  master_->branchingTime_.addCentiSeconds( localTimer_.centiSeconds() );

  if (status)
    return Fathoming;

@ For each branch rule a new subproblem is generated.

@<generate the sons@>=
  const int nRules = rules.number();

  master_->out() << "Number of new problems : " << nRules << endl;

  sons_ = new @,@[ABA_BUFFER<ABA_SUB*>(master_, nRules)@];

  ABA_SUB *newSub;

  for (int i = 0; i < nRules; i++) {
    newSub = generateSon(rules[i]);
#ifdef ABACUS_PARALLEL
    // At this point the new subproblem might also be sent to a subproblem
    // server. Here this is not done. Instead we insert it in the local
    // list of subproblems.
    //
    // Load balancing is done by the balancer server which can be queried
    // for subproblems of some quality.

    master_->openSub()->insert(newSub);

    // in the parallel version the subproblem is fathomed after its
    // sons are generated.
    // 
#else
    master_->openSub()->insert(newSub);
    sons_->push(newSub);
    master_->treeInterfaceNewNode(newSub);
#endif
  }

@q m v generateBranchRules @>
@ The function |generateBranchRules()| tries to find rules for splitting
  the current subproblem in further subproblems.
  Per default we generate rules for branching
  on variables (|branchingOnVariable()|). 
  But by redefining this function in a derived class
  any other branching strategy can be implemented.
@q m e t @>
@q m b r @>
  \BeginReturn
    \docarg{|0|}{If branching rules could be found,}
    \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
    \docarg{|rules|}{If branching rules are found, then they are stored
                     in this buffer.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::generateBranchRules(ABA_BUFFER<ABA_BRANCHRULE*> &rules)
@q m e f @>
  {
    return branchingOnVariable(rules);
  }

@q m v branchingOnVariable @>
@ The function |branchingOnVariable()| generates branching rules
  for two new subproblems by selecting a branching variable with
  the function |selectBranchingVariable()|. If a new branching variable
  selection strategy should be used the function
  |selectBranchingVariable()| should be redefined.
@q m e t @>
@q m b r @>
  \BeginReturn
    \docarg{|0|}{If branching rules could be found,}
    \docarg{|1|}{otherwise}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
    \docarg{|rules|}{If branching rules are found, then they are stored
                     in this buffer. The length of this buffer is the
                     number of active variables of the subproblem. If
                     more branching rules are generated a reallocation
                     has to be performed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::branchingOnVariable(ABA_BUFFER<ABA_BRANCHRULE*> &rules)
@q m e f @>
  {
    @<select the branching variable@>;
    @<generate the two rules for the branching variable@>;
    return 0;
  }

@ @<select the branching variable@>=
    int branchVar;
    
    int status = selectBranchingVariable(branchVar);

    if (status) {
      master_->out() << "no branching variable found" << endl;
      return 1;
    }
    
    master_->out() << endl;
    if (variable(branchVar)->binary()) master_->out() << "Binary ";
    else                               master_->out() << "Integer ";
    
    master_->out() << "Branching Variable     : ";
    master_->out() << branchVar << " (value: " << xVal_[branchVar];
    master_->out() << ", cost: " << variable(branchVar)->obj() << ") ";
    master_->out () << endl;

@ A binary branching variable is set to 0 in one of the
  two subproblems, and set to 1 in the other subproblem. For an
  integer branching variable we have to modify its lower and upper bound.
  
@<generate the two rules for the branching variable@>=
  if (variable(branchVar)->binary()) {
    rules.push(new @,@[ABA_SETBRANCHRULE(master_, branchVar,
                                 ABA_FSVARSTAT::SetToUpperBound)@]);
    rules.push(new @,@[ABA_SETBRANCHRULE(master_, branchVar,
                                 ABA_FSVARSTAT::SetToLowerBound)@]);
  }
  else {
    rules.push(new @,@[ABA_BOUNDBRANCHRULE(master_, branchVar, 
                                           floor(xVal_[branchVar]+1.0),
                                           uBound(branchVar))@]);
    rules.push(new @,@[ABA_BOUNDBRANCHRULE(master_, branchVar, lBound(branchVar),
                                   floor(xVal_[branchVar]))@]);
  }
  

@q m v selectBranchingVariable @>
@ The function |selectBranchingVariable()| chooses a branching variable.
  
  The function |selectBranchingVariableCandidates()| is asked to generate
  depending in the parameter {\tt NBranchingVariableCandidates} of the
  file {\tt .abacus} candidates for branching variables. If only
  one candidate is generate, this one becomes the branching variable.
  Otherwise, the pairs of branching rules are generated for all
  candidates and the ``best'' branching variables is determined with the
  function |selectBestBranchingSample()|.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If a branching variable is found,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>
@q m b a @>
  \BeginArg
  \docarg{|variable|}{Holds the branching variable if one is found.}
  \EndArg
@q m e a @>

@(sub.cc@>=
#ifdef TTT
extern "C" {
#include "cplex.h"
}
#endif
@q m b f @>
  int ABA_SUB::selectBranchingVariable(int &variable)
@q m e f @>
  {
    @<select the candidates for branching variables@>;
    @<generate the two branching rules for each candidate@>;
    @<evaluate the candidates and select the best ones@>;
  }

@ @<select the candidates for branching variables@>=
  ABA_BUFFER<int> candidates(master_, master_->nBranchingVariableCandidates());

  int status = selectBranchingVariableCandidates(candidates);

  if (status) return 1;

  if (candidates.number() == 1) {
    variable = candidates[0];
    return 0;
  }

#ifdef TTT
  @<perform strong branching with help of Cplex internals@>;
#endif

@ The call of |strongbranching()| is only correct if no variables are
  eliminated. 

@<perform strong branching with help of Cplex internals@>=
  int *goodlist = new int[candidates.number()];
  double *down  = new double[candidates.number()];
  double *up    = new double[candidates.number()];

  master_->out(1) << candidates.number() << " candidates" << endl;

  for (int i = 0; i < candidates.number(); i++)
    goodlist[i] = candidates[i];
 
  status = strongbranch(((ABA_CPLEXIF*) lp_)->cplexLp(), goodlist, 
                        candidates.number(), down, up, 100000);

  if (status) {
    master_->err() << "strong branching failed" << endl;
    exit(Fatal);
  }

  for (int i = 0; i < candidates.number(); i++) 
    master_->out(1) << down[i] << ' ' << up[i] << endl;

  int bestCand = 0;
  double bestVal;
  double val;

  if (master_->optSense()->max()) {
    if (down[0] < up[0]) bestVal = up[0];
    else                 bestVal = down[0];
  }
  else {
    if (down[0] > up[0]) bestVal = up[0];
    else                 bestVal = down[0];
  }

  for (int i = 1; i < candidates.number(); i++) 
    if (master_->optSense()->max()) {
      if (down[i] < up[i]) val = up[i];
      else                 val = down[i];
      if (val < bestVal) {
        bestCand = i;
        bestVal  = val;
      }
    }
    else {
      if (down[i] > up[i]) val = up[i];
      else                 val = down[i];
      if (val > bestVal) {
        bestCand = i;
        bestVal  = val;
      }
    }

  delete [] goodlist;
  delete [] up;
  delete [] down;

  master_->out(1) << "selecting sample " << bestCand << endl;

  variable = candidates[bestCand];
  return 0;

  

@ @<generate the two branching rules for each candidate@>=
  const int nCandidates = candidates.number();

#ifdef ABACUS_OLD_NEW
  typedef ABA_BUFFER<ABA_BRANCHRULE*>* pBufferPBranchRule;
  ABA_BUFFER<ABA_BRANCHRULE*> **samples = new pBufferPBranchRule[nCandidates];
#else
  ABA_BUFFER<ABA_BRANCHRULE*> **samples = new (ABA_BUFFER<ABA_BRANCHRULE*>*)[nCandidates];
#endif

  for (int i = 0; i < nCandidates; i++) {
    samples[i] = new ABA_BUFFER<ABA_BRANCHRULE*>(master_, 2);
    samples[i]->push(new ABA_SETBRANCHRULE(master_, candidates[i], 
                                       ABA_FSVARSTAT::SetToUpperBound));
    samples[i]->push(new ABA_SETBRANCHRULE(master_, candidates[i], 
                                       ABA_FSVARSTAT::SetToLowerBound));
  }

@ @<evaluate the candidates and select the best ones@>=
  int best = selectBestBranchingSample(nCandidates, samples);

  if (best == -1) {
    master_->err() << "ABA_SUB::selectBranchingVariable(): internal error, ";
    master_->err() << "selectBestBranchingSample returned -1" << endl;
    exit(Fatal);
  }

  variable = candidates[best];

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nCandidates; i++) {
#else
  for (int i = 0; i < nCandidates; i++) {
#endif
    delete (*samples[i])[0];
    delete (*samples[i])[1];
    delete samples[i];
  }
  delete [] samples;

  return 0;

@q m f selectBranchingVariableCandidates @>
@ The function |selectBranchingVariableCandidates()| selects depending
  on the branching variable strategy given by the parameter
  {\tt BranchingStrategy} in the file {\tt .abacus} candidates that
  for branching variables.

  Currently two branching variable selection strategies are
  implemented. The first one (|CloseHalf|) first searches the binary
  variables with fractional part closest to $0.5$. If there is
  no fractional binary variable it repeats this process with the
  integer variables.
  
  The second strategy (|CloseHalfExpensive|) first tries to find binary
  variables with fraction close to $0.5$ and high absolute objective function
  coefficient. If this fails, it tries to find an integer variable with
  fractional part close to $0.5$ and high absolute objective function
  coefficient.

  If neither a binary nor an integer variable with fractional value
  is found then for both strategies we try to find non-fixed
  and non-set binary variables. If this fails we repeat this process with
  the integer variables.

  Other branching variable selection strategies can be implemented by
  redefining this virtual function in a derived class.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If a candidate is found,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|candidates|}{The candidates for branching variables are
                        stored in this buffer. We try to find as
                        many variables as fit into the buffer.}
  \EndArg
@q m e a @>
                          
@(sub.cc@>=
@q m b f @>
  int ABA_SUB::selectBranchingVariableCandidates(ABA_BUFFER<int> &candidates)
@q m e f @>
  {
    int status;

    if (master_->branchingStrategy() == ABA_MASTER::CloseHalf) {
      status = closeHalf(candidates, ABA_VARTYPE::Binary);
      if (status) 
        status = closeHalf(candidates, ABA_VARTYPE::Integer);
      if (status)
        status = findNonFixedSet(candidates, ABA_VARTYPE::Binary);
      if (status)
        status = findNonFixedSet(candidates, ABA_VARTYPE::Integer);
    }
    else if (master_->branchingStrategy() == ABA_MASTER::CloseHalfExpensive) {
     status = closeHalfExpensive(candidates, ABA_VARTYPE::Binary);
     if (status) 
        status = closeHalfExpensive(candidates, ABA_VARTYPE::Integer);
      if (status)
        status = findNonFixedSet(candidates, ABA_VARTYPE::Binary);
      if (status)
        status = findNonFixedSet(candidates, ABA_VARTYPE::Integer);
    }
    else {
      master_->err() << "ABA_SUB::selectBranchingVariable(): unknown strategy" << endl;
      exit(Fatal);
    }

    return status;
  }

@q m f closeHalf @>
@ The function |closeHalf()| searches a branching variable of type
  |branchVarType|, with fraction as close to $0.5$ as possible. 
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If a branching variable is found,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|branchVar|}{Holds the branching variable if one is found.}
  \docarg{|branchVartype|}{The type of the branching variable can be
                           restricted either to |ABA_VARTYPE::Binary|
                           or |ABA_VARTYPE::Integer|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::closeHalf(int &branchVar, ABA_VARTYPE::TYPE branchVarType)
@q m e f @>
  {
    ABA_BUFFER<int> variables(master_, 1);

    int status = closeHalf(variables, branchVarType);

    if (status) 
      return 1;
    else {
      branchVar = variables[0];
      return 0;
    }

  }

@q m f closeHalf @>
@ The function |closeHalf()| searches searches several
  possible branching variable of type
  |branchVarType|, with fraction as close to $0.5$ as possible. 
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If at least one branching variable is found,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|variables|}{Stores the possible branching variables.}
  \docarg{|branchVartype|}{The type of the branching variable can be
                           restricted either to |ABA_VARTYPE::Binary|
                           or |ABA_VARTYPE::Integer|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::closeHalf(ABA_BUFFER<int> &variables, ABA_VARTYPE::TYPE branchVarType)
@q m e f @>
  {
     @<|ABA_SUB::closeHalf()|: check the branching variable type@>;
     @<search fractional variables closest to $0.5$@>;
     @<copy the best variables in the buffer |variables|@>;
  }
  
@ @<|ABA_SUB::closeHalf()|: check the branching variable type@>=
  if (branchVarType == ABA_VARTYPE::Continuous) {
    master_->err() << "ABA_SUB::closeHalf(): we cannot branch on a ";
    master_->err() << "continuous variable.";
    exit(Fatal);
  }
  
@ @<search fractional variables closest to $0.5$@>=
  ABA_BPRIOQUEUE<int, double> closest(master_, variables.size());

  double diff;
  double minKey;
  int    min;

  const int nVariables = nVar();
  for (int i = 0; i < nVariables; i++)
    if ((variable(i)->varType() == branchVarType)
        && !(*fsVarStat_)[i]->fixedOrSet()) {
      diff = fabs(fracPart(xVal_[i]) - 0.5);
      if (diff < 0.5 - master_->machineEps()) {
        if (closest.number() < closest.size())
          closest.insert(i, -diff);
        else {
          (void) closest.getMinKey(minKey);
          if (diff < -minKey) {
            (void) closest.extractMin(min);
            closest.insert(i, -diff);
          }
        }
      }
    }

@ @<copy the best variables in the buffer |variables|@>=
  while(!closest.extractMin(min))
    variables.push(min);

  if (variables.number()) return 0;
  else                    return 1;
  
@q m f closeHalfExpensive @>
@ The function |closeHalfExpensive()| selects a single branching variable
  of type |branchVarType|, with
  fractional part close to $0.5$ and high absolute objective function
  coefficient. 
@q m e t @>

  This is the default strategy from the TSP project \ref{JRT94}.

@q m b r @>
  \BeginReturn
    \docarg{|0|}{If a branching variable is found,}
    \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|branchVar|}{Holds the number of the branching variable if
                         one is found.}
    \docarg{|branchVartype|}{The type of the branching variable can be
                           restricted either to |ABA_VARTYPE::Binary|
                           or |ABA_VARTYPE::Integer|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::closeHalfExpensive(int &branchVar, ABA_VARTYPE::TYPE branchVarType)
@q m e f @>
  {
    ABA_BUFFER<int> branchVarBuffer(master_, 1);

    int status = closeHalfExpensive(branchVarBuffer, branchVarType);

    if (!status) branchVar = branchVarBuffer[0];

    return status;
}

@q m f closeHalfExpensive @>
@ This version of the 
  function |closeHalfExpensive()| selects several candidates
  for branching variables
  of type |branchVarType|. Thos variables with
  fractional part close to $0.5$ and high absolute objective function
  coefficient are selected.. 
@q m e t @>

@q m b r @>
  \BeginReturn
    \docarg{|0|}{If at least one branching variable is found,}
    \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|branchVar|}{Holds the numbers of possible branching variables if
                         at least one is found. We try to find as many
                         candidates as fit into this buffer. We abort
                         the function with a fatal error if the size
                         of the buffer is 0.}
    \docarg{|branchVartype|}{The type of the branching variable can be
                           restricted either to |ABA_VARTYPE::Binary|
                           or |ABA_VARTYPE::Integer|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::closeHalfExpensive(ABA_BUFFER<int> &branchVar, 
                                  ABA_VARTYPE::TYPE branchVarType)
@q m e f @>
  {
    @<local variables (|ABA_SUB::closeHalfExpensive()|)@>;
    @<check the selected branching variable type@>;
    @<determine interval for fraction of candidates@>;
    @<select the most expensive variables from interval@>;
    @<copy the ``best'' variables to |branchVar|@>;
  }

@ @<local variables (|ABA_SUB::closeHalfExpensive()|)@>=
  ABA_BPRIOQUEUE<int, double> candidates(master_, branchVar.size());
  int    i;              /* loop index */
  double fraction;       /* fraction of x-value of a variable */
  double eps         = master_->machineEps();
  double oneMinusEps = 1.0 - eps;
     
@ @<check the selected branching variable type@>=
  if (branchVarType == ABA_VARTYPE::Continuous) {
    master_->err() << "ABA_SUB::closeHalfExpensive(): we cannot branch on a ";
    master_->err() << "continuous variable.";
    exit(Fatal);
  }
  
@ First we determine |lower| the maximal LP-value of a variable less than
  $0.5$ and |upper| the minimal LP-value of a variable greater than $0.5$.
  Finally, |lower| and |upper| are scaled. 
  
@<determine interval for fraction of candidates@>=
  double lower = eps;
  double upper = oneMinusEps;

  const int nVariables = nVar();

  for (i = 0; i < nVariables; i++)
    if ((variable(i)->varType() == branchVarType) 
        && !(*fsVarStat_)[i]->fixedOrSet()) {
      fraction = fracPart(xVal_[i]);

      if (fraction <= 0.5 && fraction > lower) lower = fraction;
      if (fraction >= 0.5 && fraction < upper) upper = fraction;
    }  

  if (lower == eps && upper == oneMinusEps) return 1;

  double scale = 0.25;

  lower   = (1.0 - scale) * lower;
  upper = upper + scale * (1.0-upper);

@ Under |cost| in this context we understand the absolute value of the
  objective function coefficient.
  
@<select the most expensive variables from interval@>=
  double cost;               // cost of current variable
  double minCostCandidate;   // cost of worst variable in priority queue
  int    dummy;              // for extracting item of priority queue

  for (i = 0; i < nVariables; i++)
    if ((variable(i)->varType() == branchVarType)
        && !(*fsVarStat_)[i]->fixedOrSet()) {

    @<check if this variable might a candidate@>;
  }

@ We select the variable either if there are not enough candidates,
  otherwise, we check if its cost are higher than those of the worst
  element of |candidate|. In this case we replace this element with the
  variable |i|.
  
@<check if this variable might a candidate@>=
  fraction = fracPart(xVal_[i]);

  if (lower <= fraction && fraction <= upper) {
    cost     = fabs(variable(i)->obj());
    if (candidates.number() < candidates.size())
      candidates.insert(i, cost);
    else {
      if (candidates.getMinKey(minCostCandidate)) {
        master_->err() << "ABA_SUB::CloseHalfExpensive(): internal error: ";
        master_->err() << "candidate priorirty queue is empty." << endl;
        exit(Fatal);
      }
      if (cost > minCostCandidate) {
        (void) candidates.extractMin(dummy);
        candidates.insert(i, cost);
      }
    }
  }

@ @<copy the ``best'' variables to |branchVar|@>=
  if (candidates.number() == 0) {
    master_->err() << "ABA_SUB::closeHalfExpensive(): where is the fractional variable?" << endl;
    exit(Fatal);
  }

  while (!candidates.extractMin(dummy))
    branchVar.push(dummy);

  return 0;

@q m f findNonFixedSet @>
@ The function |findNonFixedSet()| selects the first variable that is neither
  fixed nor set.
@q m e t @>

@q m b r @>
  \BeginReturn
    \docarg{|0|}{If a variable neither fixed nor set is found,}
    \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|branchVar|}{Holds the number of the branching variable if
                         one is found.}
    \docarg{|branchVarType|}{The type of the branching
                             have (|ABA_VARTYPE::Binary| or 
                             |ABA_VARTYPE::Integer|).}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::findNonFixedSet(int &branchVar, ABA_VARTYPE::TYPE branchVarType)
@q m e f @>
  {
    ABA_BUFFER<int> variables(master_, 1);

    int status = findNonFixedSet(variables, branchVarType);
    
    if (status)
      return 1;
    else {
      branchVar = variables[0];
      return 0;
    }
  }

@q m f findNonFixedSet @>
@ The function |findNonFixedSet()| selects the first variables that are neither
  fixed nor set.
@q m e t @>

@q m b r @>
  \BeginReturn
    \docarg{|0|}{If at least one variable neither fixed nor set is found,}
    \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|branchVar|}{Holds the number of the possible branching
                         variables if one is found.}
    \docarg{|branchVartype|}{The type of the branching variable can be
                           restricted either to |ABA_VARTYPE::Binary|
                           or |ABA_VARTYPE::Integer|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::findNonFixedSet(ABA_BUFFER<int> &branchVar, 
                               ABA_VARTYPE::TYPE branchVarType)
@q m e f @>
  {
    @<|ABA_SUB::findNonFixedSet|: check the selected branching variable type@>;

    const int nVariables = nVar();
    
    for (int i = 0; i < nVariables; i++)
      if ((variable(i)->varType() == branchVarType)
          && !(*fsVarStat_)[i]->fixedOrSet()) {
        branchVar.push(i);
        if (branchVar.full()) return 0;
      }

    if (branchVar.number()) return 0;
    else                    return 1;
  }
      
@ @<|ABA_SUB::findNonFixedSet|: check the selected branching variable type@>=
  if (branchVarType == ABA_VARTYPE::Continuous) {
    master_->err() << "ABA_SUB::findNonFixedSet(): we cannot branch on a ";
    master_->err() << "continuous variable.";
    exit(Fatal);
  }

@q m v selectBestBranchingSample @>
@ The function |selectBestBranchingSample()| evaluates branching
  samples (we denote a branching sample the set of rules defining
  all sons of a subproblem in the enumeration tree). For each sample
  the ranks are determined with the function |rankBranchingSample()|.
  The ranks of the various samples are compared with the function
  |compareBranchingSample()|.
@q m e t @>

@q m b r @>
  \BeginReturn
  The number of the best branching sample, or |-1| in case of an
  internal error.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|nSamples|}{The number of branching samples.}
  \docarg{|samples|}{An array of pointer to buffers storing the
                     branching rules of each sample.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::selectBestBranchingSample(int nSamples,
                                         ABA_BUFFER<ABA_BRANCHRULE*> **samples)
@q m e f @>
  {
    @<allocate memory (|ABA_SUB::selectBestBranchingSample()|)@>;
    @<compute the ranks and select the best sample@>;
    @<delete memory (|ABA_SUB::selectBestBranchingSample()|)@>;
  }

@ @<allocate memory (|ABA_SUB::selectBestBranchingSample()|)@>=
#ifdef ABACUS_OLD_NEW
  typedef ABA_ARRAY<double>* pABA_ARRAYpDouble;
  ABA_ARRAY<double> **rank = new pABA_ARRAYpDouble[nSamples];
#else
  ABA_ARRAY<double> **rank = new (ABA_ARRAY<double>*)[nSamples];
#endif

  for (int i = 0; i < nSamples; i++)
    rank[i] = new ABA_ARRAY<double>(master_, samples[i]->number());

@ @<compute the ranks and select the best sample@>=
  master_->out() << "Computing ranks of branching samples: "<< endl;
  int best = 0;

#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nSamples; i++) {
#else
  for (int i = 0; i < nSamples; i++) {
#endif
    rankBranchingSample(*(samples[i]), *(rank[i]));
    master_->out(1) << "Sample " << i << ": ";
    for (int j = 0; j < samples[i]->number(); j++)
        master_->out() << (*(rank[i]))[j] << ' ';
    master_->out() << endl;
    if (i > 0 &&
        compareBranchingSampleRanks(*(rank[best]), *(rank[i])) == -1) 
      best = i;
  }

  master_->out() << endl;
  master_->out() << "Selecting branching sample " << best << "." << endl;

@ @<delete memory (|ABA_SUB::selectBestBranchingSample()|)@>=
#ifdef ABACUS_NO_FOR_SCOPE
for (i = 0; i < nSamples; i++)
#else
for (int i = 0; i < nSamples; i++)
#endif
    delete rank[i];
  delete [] rank;

  return best;

@q m v rankBranchingSample @>
@ The function |rankBranchingSample()| computes for each branching
  rule of a branching sample a rank with the function 
  |rankBranchingRule()|.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|sample|}{A branching sample.}
  \docarg{|rank|}{An array storing the rank for each branching rule in the
                  sample after the function call.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::rankBranchingSample(ABA_BUFFER<ABA_BRANCHRULE*> &sample,
                                    ABA_ARRAY<double> &rank) 
@q m e f @>
  {
    const int nSample = sample.number();

    for (int i = 0; i < nSample; i++)
      rank[i] = rankBranchingRule(sample[i]);
  }

@q m v rankBranchingRule @>
@ The function |rankBranchingRule()| computes the rank of a branching
  rule. This default implementation computes the rank 
  with the function |lpRankBranchingRule()|. By redefining this virtual
  function the rank for a branching rule can be computed differently.
@q m e t @>

@q m b r @>
  \BeginReturn
  The rank of the branching rule.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|branchRule|}{A pointer to a branching rule.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  double ABA_SUB::rankBranchingRule(ABA_BRANCHRULE *branchRule)
@q m e f @>
  {
    return lpRankBranchingRule(branchRule, 50);
  }

@q m f lpRankBranchingRule @>
@ The function |lpRankBranchingRule()| computes the rank of a branching rule
  by modifying the linear programming relaxation of the subproblem
  according to the branching rule and solving it. This modifiction
  is undone after the solution of the linear program.

  It is useless, but no error, to call this function for branching rules
  for which the virtual dummy functions |extract(ABA_LPSUB*)|  and 
  |unExtract(ABA_LPSUB*)| of the base class |ABA_BRANCHRULE| are not redefined.
@q m e t @>

@q m e r @>
  \BeginReturn
  The value of he linear programming relaxation of the subproblem modified
  by the branching rule.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|branchRule|}{A pointer to a branching rule.}
  \docarg{|iterLimit|}{The maximal number of iterations that should be
                       performed by the simplex method. If this number
                       is negative there is no  iteration limit
                       (besides internal limits of the LP-solver). The default
                       value is |-1|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  double ABA_SUB::lpRankBranchingRule(ABA_BRANCHRULE *branchRule, int iterLimit)
@q m e f @>
  {
    @<add the branching rule and solve the linear program@>;
    @<get the |value| of the linear program@>;
    @<remove the branching rule@>;
    return value;
  }

@ @<add the branching rule and solve the linear program@>=
  @<set the new iteration limit@>;
  @<load the final basis of the subproblem optimization@>;

  branchRule->extract(lp_);
  localTimer_.start(true);
  lp_->optimize(ABA_LP::Dual);
  master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());

@ @<set the new iteration limit@>=
  int oldIterLimit;

  if (iterLimit >= 0) {
    if (lp_->getSimplexIterationLimit(oldIterLimit)) {
      master_->err() << "WARNING: ";
      master_->err() << "ABA_SUB::lpRankBranchingRule(): ";
      master_->err() << "getting the iteration limit of the LP-solver failed.";
      master_->err() << endl;
      oldIterLimit = -1;
    }
    else {
      if (lp_->setSimplexIterationLimit(iterLimit)) {
        master_->err() << "WARNING: ";
        master_->err() << "ABA_SUB::lpRankBranchingRule(): ";
        master_->err() << "setting the iteration limit of the LP-solver failed.";
        master_->err() << endl;
        oldIterLimit = -1;
      }
    }      
  }
  
@ @<load the final basis of the subproblem optimization@>=
  ABA_ARRAY<ABA_LPVARSTAT::STATUS> vStat(master_, nVar());
  ABA_ARRAY<ABA_SLACKSTAT::STATUS> sStat(master_, nCon());

  const int nVariables = nVar();

  for (int i = 0; i < nVariables; i++) 
    vStat[i] = lpVarStat(i)->status();

  const int nConstraints = nCon();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nConstraints; i++)
#else
  for (int i = 0; i < nConstraints; i++)
#endif
    sStat[i] = slackStat(i)->status();

  lp_->loadBasis(vStat, sStat);

@ @<get the |value| of the linear program@>=
  double value;
  if (lp_->infeasible()) {
    if (master_->optSense()->max()) value = -master_->infinity();
    else                            value =  master_->infinity();
  }
  else 
    value = lp_->value();

@ @<remove the branching rule@>=
  @<set the iteration limit back to its old value@>;
  branchRule->unExtract(lp_);

@ @<set the iteration limit back to its old value@>=
  if (iterLimit >= 0 && oldIterLimit >=0)
    if (lp_->setSimplexIterationLimit(oldIterLimit)) {
      master_->err() << "ABA_SUB::lpRankBranchingRule(): ";
      master_->err() << "setting the iteration limit of LP-solver ";
      master_->err() << "failed." << endl;
      exit(Fatal);
    }

@q m v compareBranchingSampleRanks @>
@ The function |compareBranchingSampleRanks()| compares the ranks of
  two branching samples. For maximimization problem that rank
  is better for which the maximal rank of a rule is minimal, while for
  minimization problem the rank is better for which the minimal rank of
  a rule is maximal. If this value equals for both ranks we continue
  with the secand greatest value, etc.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If |rank1| is better.}
  \docarg{|0|}{If both ranks are equal.}
  \docarg{|-1|}{If |rank2| is better.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::compareBranchingSampleRanks(ABA_ARRAY<double> &rank1,
                                           ABA_ARRAY<double> &rank2)
@q m e f @>
  {
    @<build up a priority queue for each rank@>;
    @<prefer the rank where the minimal change is maximal@>;
  }

@ @<build up a priority queue for each rank@>=
  ABA_BPRIOQUEUE<int, double> prio1(master_, rank1.size());
  ABA_BPRIOQUEUE<int, double> prio2(master_, rank2.size());

  const int s1 = rank1.size();
  const int s2 = rank2.size();
  
  if (master_->optSense()->max()) {
    for (int i = 0; i < s1; i++) 
      prio1.insert(i, -rank1[i]);
#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < s2; i++) 
#else
    for (int i = 0; i < s2; i++) 
#endif
      prio2.insert(i,-rank2[i]);
  }
  else {
    for (int i = 0; i < s1; i++) 
      prio1.insert(i, rank1[i]);
#ifdef ABACUS_NO_FOR_SCOPE
    for (i = 0; i < s2; i++) 
#else
    for (int i = 0; i < s2; i++) 
#endif
      prio2.insert(i, rank2[i]);
  }

@ @<prefer the rank where the minimal change is maximal@>=
  double minKey1;
  double minKey2;

  int min1;
  int min2;

  while(!prio1.getMinKey(minKey1) && !prio2.getMinKey(minKey2)) {
    if (!master_->equal(minKey1, minKey2)) {
      if (minKey1 > minKey2) return  1;
      else                   return -1;
    }
    else {
      (void) prio1.extractMin(min1);
      (void) prio2.extractMin(min2);
    }
  }
  
  return 0;

@q m v fathoming @>
@ The function |fathoming()| fathoms the node, and if certain conditions are
  satisfied, also its ancestor.

  The third central phase of the optimization of a subproblem is the
  |Fathoming| of a subproblem. A subproblem is fathomed if it can be
  guaranteed that this subproblem cannot contain a better solution
  than the best known one. This is the case if the global upper bound
  does not exceed the local lower bound (maximization problem assumed)
  or the subproblem cannot
  contain a feasible solution either if there is a fixing/setting
  contradiction or the |LP|-relaxation turns out to be infeasible.

  Note, use the function |ExceptionFathom()| for specifying problem
  specific fathoming criteria.
@q m e t @>

  The called function |fathom()| fathoms the subproblem itself
  and recursively also tries to fathom its father in the enumeration
  tree. The argument of |fathom()| is |true| as a possibly detected
  new root should be reoptimized in order to receive better criteria
  for fixing variables by reduced costs.

  In the parallel version, only the subproblem itself is fathomed.
  No processed unfathomed nodes are kept in memory (father_=0).

@q m b r @>
  \BeginReturn
   The function always returns |Done|.
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  ABA_SUB::PHASE ABA_SUB::fathoming()
@q m e f @>
  {
    master_->out() << endl;
    master_->out() << "Fathoming Phase" << endl;
    
    fathom(true);

    return Done;
  }

@q m v fathom @>
@ The function |fathom()| fathoms a node and recursively tries
  to fathom its father.
@q m e t @>

  If the root of the remaining \bac\ tree is fathomed we are done
  since the optimization problem has been solved.

  Otherwise, we count the number of unfathomed sons of the father
  of the subproblem being fathomed. If all sons of the father
  are fathomed it is recursively fathomed, too. If the father
  is the root of the remaining \bac\ tree and only one of its
  sons is unfathomed, then this unfathomed son becomes the new root
  of the remaining \bac\ tree.

  We could stop the recursive fathoming already at the root of
  the remaining \bac\ tree. But, we proceed until the root
  of the complete tree was visited to be really correct.

  Note, use the function |ExceptionFathom()| for specifying problem
  specific fathoming criteria.

@q m b a @>
  \BeginArg
  \docarg{|reoptimize|}
  {If |reoptimize| is |true|, then we perform a reoptimization in the new
  root. 
@q m e a @>
  This is controlled via a parameter since it might not be
  desirable when we find a new root during the fathoming
  of a complete subtree with the function |fathomTheSubTree()|.
@q m b a @>
  }
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::fathom(bool reoptimize)
@q m e f @>
  {
    @<|ABA_SUB::fathom()|: output some infos@>;
    @<reset the flags of the active variables and constraints@>;
#ifndef ABACUS_PARALLEL
    @<update the dual bound of the subproblem@>;
#endif
    @<|delete| allocated memory of the fathomed subproblem@>;
#ifndef ABACUS_PARALLEL
    @<check if the root node is fathomed@>;
    @<count the number of unfathomed sons of the father@>;
    @<process the father@>;
#endif
  }

@ @<|ABA_SUB::fathom()|: output some infos@>=
#ifdef ABACUS_PARALLEL
  master_->out(1) << "node ";
  master_->parmaster()->printId(id_);
  master_->out() << " fathomed" << endl;
#else
  master_->out(1) << "node " << id_ << " fathomed" << endl;
#endif

@ If an active subproblem is fathomed, then the active variables
  and constraints have to be deactivated. This can only be done
  if these sets are allocated already.

  Then, we can set the status of the subproblem to |Fathomed|.

@<reset the flags of the active variables and constraints@>=
  if (status_ == Active) {
    if (actVar_) {
      int nActVar = actVar_->number();
      for (int i = 0; i < nActVar; i++) (*actVar_)[i]->deactivate();
    }

    if (actCon_) {
      const int nActCon = actCon_->number();
      for (int i = 0; i < nActCon; i++) (*actCon_)[i]->deactivate();
    }
  }

#ifdef ABACUS_PARALLEL
  // master_->treeInterfacePaintNode(id_, 7+master_->parmaster()->hostId() );
#endif
  status_ = Fathomed;

@ If the subproblem is not a leaf of the enumeration tree, we
  update its dual bound with the minimal (maximal) dual bound
  of its sons if the optimization problem is a minimization
  (maximization) problem. 

  We update the dual bound only if the sons define a better dual
  bound. If heuristic separation methods are applied or a
  reoptimization of a subproblem has been performed, then it is
  possible that the dual bound defined by the sons is worse than the
  dual bound of the subproblem.

@<update the dual bound of the subproblem@>=
   if (sons_) {
     double newDualBound = (*sons_)[0]->dualBound();
   
     if (master_->optSense()->max()) {
       const int nSons = sons_->number();
       for (int i = 1; i < nSons; i++)
         if (newDualBound < (*sons_)[i]->dualBound())
           newDualBound = (*sons_)[i]->dualBound();
     }
     else {
       const int nSons = sons_->number();
       for (int i = 1; i < nSons; i++)
         if (newDualBound > (*sons_)[i]->dualBound())
           newDualBound = (*sons_)[i]->dualBound();
     }
     if (betterDual(newDualBound))
       dualBound(newDualBound);
   }
   
@ A fathomed subproblem will neither be required for the initialization of
  one of its sons nor in a reoptimization to determine the new candidates
  for fixing variables. Hence we can delete all local memory. 

@<|delete| allocated memory of the fathomed subproblem@>=

  if (fsVarStat_) {
    const int nVariables = nVar();
    for (int i = 0; i < nVariables; i++)
      delete (*fsVarStat_)[i];
  }

  if (lpVarStat_) {
    const int nVariables = nVar();
    for (int i = 0; i < nVariables; i++)
      delete (*lpVarStat_)[i];
  }

  delete fsVarStat_;
  fsVarStat_ = 0;
  delete lpVarStat_;
  lpVarStat_ = 0;
    
  delete lBound_;
  lBound_ = 0;
  delete uBound_;
  uBound_ = 0;

  if (slackStat_) {
    const int nConstraints = nCon();
    for (int i = 0; i < nConstraints; i++)
      delete (*slackStat_)[i];
    delete slackStat_;
    slackStat_ = 0;
  }
    
  delete actCon_;
  actCon_ = 0;
  delete actVar_;
  actVar_ = 0;

  delete branchRule_;
  branchRule_ = 0;
    
@ @<check if the root node is fathomed@>=
  if (this == master_->root()) {
    master_->out(2) << "root node fathomed" << endl;
    return;
  }
  
  
@ @<count the number of unfathomed sons of the father@>=
  int   nuf = 0;            // number of unfathomed sons of |father_|

  for (int i = 0; i < father_->sons_->number(); i++)
    if ((*(father_->sons_))[i]->status_ != Fathomed) ++nuf;

@ If all sons of the father are fathomed we can fathom the father
  recursively. If only one son of the father is unfathomed and the father
  is the root of the remaining \bac\ tree, then this unfathomed son
  becomes the new root of the remaining \bac\ tree.
  As in this case, there is exactly one unfathomed son, this
  is son |i| when the |for|-loop is left by the |break| instruction.

@<process the father@>=
    if (nuf==0) 
      father_->fathom(reoptimize);
    else if (nuf == 1 && father_ == master_->rRoot()) {
      int i;
      for (i = 0; i < father_->sons_->number(); i++)
        if ((*(father_->sons_))[i]->status_ != Fathomed) break;

      master_->rRoot((*(father_->sons_))[i], reoptimize);
    }

@q m v fixAndSet @>
@ The function |fixAndSet()| tries to fix and set variables both
  by logical implications and reduced cost criteria.
@q m e t @>
  Actually, variables fixed or set to 0 could be eliminated. However,
  this could lead to a loss of important structural information
  for fixing and setting further variables later, for the computation
  of feasible solutions, for the separation and for detecting contradictions.
  Therefore, we do not eliminate these variables per default.

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|newValues|}{If a variables is set or fixed to a value different
                       from the last LP-solution, |newValues| is set
                       to |true|, otherwise it is set to |false|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::fixAndSet(bool &newValues)
@q m e f @>
  {
    int  status;
    bool lNewValues;

    newValues = false;

    status = fixing(lNewValues, true);
    if (lNewValues) newValues = true;
    if (status)     return 1;
    
    status = setting(lNewValues);
    if (lNewValues) newValues = true;
    if (status)     return 1;

    return 0;
  }
  
@q m v fixing @>
@ The function |fixing()| tries to fix variables by reduced cost criteria
  and logical implications.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|newValues|}
  {The parameter |newValues| becomes |true| if variables are fixed
  to other values as in the current LP-solution.
  }
  \docarg{|saveCand|}
  {If the parameter |saveCand| is |true| a new candidate list of variables
  for fixing is generated.
  The default value of |saveCand| is false. Candidates should not be
  saved if fixing is performed after the addition of variables.
  }
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::fixing(bool &newValues, bool saveCand)
@q m e f @>
  {
    int  status;
    bool lNewValues = false;

    newValues = false;

    status = fixByRedCost(lNewValues, saveCand);
    if (lNewValues) newValues = true;
    if (status)     return 1;
    
    status = _fixByLogImp(lNewValues);
    if (lNewValues) newValues = true;
    if (status)     return 1;
    
    return 0;
  }

@q m v setting @>
@ The function |setting()| tries to set variables by reduced cost criteria
  and logical implications like |fixing()|, but instead of global
  conditions only locally valid conditions have to be satisfied.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction has been found,}
  \docarg{|0|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|newValues|}
  {The parameter |newValues| becomes |true| if variables are fixed
  to other values as in the current LP-solution (|setByRedCost()|
  cannot set variables to new values).}
  \EndArg
@q m e a  @>
  
@(sub.cc@>=
@q m b f @>
  int ABA_SUB::setting(bool &newValues)
@q m e f @>
  {
    int  status;
    bool lNewValues;

    newValues = false;

    status = setByRedCost();
    if (status) return 1;
    
    status = _setByLogImp(lNewValues);
    if (lNewValues) newValues = true;
    if (status)     return 1;
    
    return 0;
  }
  

@q m v fixByRedCost @>
@ The function |fixByRedCost()| tries to fix variables according to
  the reduced cost criterion. 
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|newVales|}{If variables are fixed to different values as
                      in the last solved linear program, then
                      |newValues| becomes true.}
  \docarg{|saveCand|}{If |saveCand| is |true|, then a new list of
                      candidates for later calls is compiled. This is 
                      only possible when the root of the remaining
                      \bab\ is processed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::fixByRedCost(bool &newValues, bool saveCand)
@q m e f @>
  {
    if (!master_->fixSetByRedCost()) return 0;
    
    newValues = false;

#ifdef ABACUS_PARALLEL
    if (level_ == 1 && saveCand) 
#else
    if (this == master_->rRoot() && saveCand) 
#endif
      master_->fixCand()->saveCandidates(this);

    master_->fixCand()->fixByRedCost(addVarBuffer_);

    @<update the global fixings also in the subproblem@>;
    
  }

@ If a contradiction between a global fixing a the local status of the
  variable is detected we immediately stop such that the subproblem can
  be fathomed.
  
@<update the global fixings also in the subproblem@>=
  ABA_FSVARSTAT *global;
  bool       lNewValues;

  const int nVariables = nVar();  

  for (int i = 0; i < nVariables; i++) {
    global = variable(i)->fsVarStat();
    if (global->fixed() && global->status() != (*fsVarStat_)[i]->status()) {
      if (fix(i, global, lNewValues)) return 1;
      if (lNewValues) newValues = true;
    }
  }
  return 0;

@ The function |_fixByLogImp()| returns 1, if a contradiction has been found,
  0 otherwise. The parameter |newValues| is set to |true| if a variable
  is fixed to value different from its value in the last solved linear
  program.

@(sub.cc@>=
  int ABA_SUB::_fixByLogImp(bool &newValues)
  {
    master_->out() << "Fixing Variables by Logical Implications:  " << flush;
    @<call the virtual function to fix variables by logical implications@>;
    @<check if |fixByLogImp()| caused contradictions or fixed variables to new values@>;
    @<|ABA_SUB::_fixByLogImp()|: clean up and return@>;
  }

@ @<call the virtual function to fix variables by logical implications@>=
  ABA_BUFFER<int>        variables(master_, nVar());
  ABA_BUFFER<ABA_FSVARSTAT*> status(master_, nVar());

  fixByLogImp(variables, status);

@ @<check if |fixByLogImp()| caused contradictions or fixed variables to new values@>=
  int contra = 0;
  int stat;
  bool lNewValues;

  newValues = false;

  const int nVariables = variables.number();
    
  for (int i = 0; i < nVariables; i++) {
    stat = fix(variables[i], status[i], lNewValues);
    if (stat)       contra    = 1;
    if (lNewValues) newValues = true;
  }

@ @<|ABA_SUB::_fixByLogImp()|: clean up and return@>=
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVariables; i++)
#else
  for (int i = 0; i < nVariables; i++)
#endif
    delete status[i];

  if (contra)
    master_->out() << "contradiction" << endl;
  else
    master_->out() << nVariables << " variables fixed" << endl;
    
  return contra;

@q m v fixByLogImp @>
@ The function |fixByLogImp()| should collect the numbers of the
  variables to be fixed in |variable| and the respective statuses
  in |status|.
  The default implementation of |fixByLogImp()| does nothing. This
  function has to be redefined if variables should be fixed by logical
  implications in derived classes.
@q m e t @>
@q m b a @>
  \BeginArg
     \docarg{|variables|}{The variables which should be fixed.}
     \docarg{|status|}{The statuses these variables should be fixed to.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::fixByLogImp(ABA_BUFFER<int> &variables, 
                            ABA_BUFFER<ABA_FSVARSTAT*> &status)
@q m e f @>
  { }

@q m v setByRedCost @>
@ The function |setByRedCost()| tries to set variables according
  to the reduced cost criterion.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::setByRedCost()
@q m e f @>
  {

    if (!master_->fixSetByRedCost()) return 0;

    @<|ABA_SUB::setByRedCost()|: local variables@>;

    master_->out() << "Setting Variables by Reduced Costs:        " << flush;

    if (master_->optSense()->max()) {
      @<set by reduced costs for maximization problems@>;
    }
    else {
      @<set by reduced costs for minimization problems@>;
    }

    master_->out() << nSet << " variables set" << endl;
    return 0;
  }

@ @<|ABA_SUB::setByRedCost()|: local variables@>=
  int  nSet = 0;  // number of variables set 
  bool dummy;     // required to call function |set()|, no new values possible here
  int i;          // loop index

@ In maximization problems the dual bound should not fall below the
  primal bound. Remember, the reduced cost of a (nonbasic) variable is the
  change of the object function if the variable becomes basic and
  changes one unit from its current value. As discrete variables can
  take only integer values, we analyzes what would happen if the
  value of the variable would decrease by one unit, if it is currently
  at its upper bound, or increase by one unit if it is currently at its
  lower bound.

  Even for integer objective function values we require a violation of
  at least |master_->eps()|, otherwise a variable might be set to the wrong
  value.
  
@<set by reduced costs for maximization problems@>=
  const int nVariables = nVar();

  for (i = 0; i < nVariables; i++) 
    if (variable(i)->discrete() && !variable(i)->fsVarStat()->fixed())
      if (lpVarStat(i)->status() == ABA_LPVARSTAT::AtUpperBound) {
        if (lp_->value() - lp_->reco(i) + master_->eps() 
            < master_->primalBound())
          if (set(i, ABA_FSVARSTAT::SetToUpperBound, dummy)) return 1;
          else 
            ++nSet;
      }
      else if (lpVarStat(i)->status() == ABA_LPVARSTAT::AtLowerBound) {
        if (lp_->value() + lp_->reco(i) + master_->eps() 
            < master_->primalBound())
          if (set(i, ABA_FSVARSTAT::SetToLowerBound, dummy)) return 1;
          else 
            ++nSet;
      }
      
@ In minimization problems the dual bound should not exceed the
  primal bound.
  
@<set by reduced costs for minimization problems@>=
  const int nVariables = nVar();

  for (i = 0; i < nVariables; i++) 
    if (variable(i)->discrete() && !variable(i)->fsVarStat()->fixed())
      if (lpVarStat(i)->status() == ABA_LPVARSTAT::AtUpperBound) {
        if (lp_->value() - lp_->reco(i) - master_->eps() 
            > master_->primalBound())
          if (set(i, ABA_FSVARSTAT::SetToUpperBound, dummy)) return 1;
          else
            ++nSet;
      }
      else if (lpVarStat(i)->status() == ABA_LPVARSTAT::AtLowerBound) {
        if (lp_->value() + lp_->reco(i) - master_->eps() 
            > master_->primalBound())
          if (set(i, ABA_FSVARSTAT::SetToLowerBound, dummy)) return 1;
          else 
            ++nSet;
      }

@q m f constraint @>
@q m e t @>
@ The function |constraint()|.

@q m b r @>
 \BeginReturn
  A pointer to the |i|-th active constraint.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The constraint being accessed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  ABA_CONSTRAINT *ABA_SUB::constraint(int i) const
@q m e f @>
  {
    return (*actCon_)[i];
  }

@q m f variable @>
@q m e t @>
@ The function |variable()|.

@q m b r @>
  \BeginReturn
    A pointer to the |i|-th active variable.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|i|}{The number of the variable being accessed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  ABA_VARIABLE *ABA_SUB::variable(int i) const
@q m e f @>
  {
    return (*actVar_)[i];
  }

@q m f lBound @>
@ The function |lBound()| can be used to access the lower of an active
  variable of the subproblem.
  {\bf Warning:} This is the lower bound of the variable within
  the current subproblem which can differ from its global lower bound.
@q m e t @>

@q m b r @>
  \BeginReturn
   The lower bound of the |i|-th variable.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|i|}{The number of the variable.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_SUB::lBound(int i) const
@q m e f @>

@q m f lBound @>
@ This version of the function |lBound()| sets the local lower bound
  of a variable. It does not change the global lower bound of the
  variable. The bound of a fixed or set variable should not be changed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable.}
  \docarg{|x|}{The new value of the lower bound.}
  \EndArg
@q m e a @>

@q m b f @>
  void ABA_SUB::lBound(int i, double x)
@q m e f @>

@q m f uBound @>
@ The function |uBound()| can be used to access the upper of an active
  variable of the subproblem.
  {\bf Warning:} This is the upper bound of the variable within
  the current subproblem which can differ from its global upper bound.
@q m e t @>

@q m b r @>
  \BeginReturn
   The upper bound of the |i|-th variable.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|i|}{The number of the variable.}
  \EndArg
@q m e a @>

@q m b f @>
  double ABA_SUB::uBound(int i) const
@q m e f @>

@q m f uBound @>
@ This version of the  function |uBound()| sets thef local upper bound
  of a variable. This does not change the global lower bound of the
  variable. The bound of a fixed or set variable should not be changed.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable.}
  \docarg{|x|}{The new value of the upper bound.}
  \EndArg
@q m e a @>
@q m b f @>
  void ABA_SUB::uBound(int i, double x)
@q m e f @>

@q m f fsVarStat @>
@q m e t @>
@ The function |fsVarStat(i)|.

  In a \pbac\ algorithm we also would have to refer to the global variable
  status. While this subproblem is processed another subproblem could change
  the global status.

@q m b r @>
  \BeginReturn
    A pointer to the status of fixing/setting of the |i|-th variable.
    Note, this is the local status of fixing/setting that might differ
    from the global status of fixing/setting of the variable 
    (|variable(i)->fsVarStat()|).
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
    \docarg{|i|}{The number of the variable.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_FSVARSTAT *ABA_SUB::fsVarStat(int i)
@q m e f @>

@q m f lpVarStat @>
@q m e t @>
@ The function |lpVarStat()|.

@q m b r @>
  \BeginReturn
    A pointer to the status of the variable |i| in the last solved
    linear program.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
     \docarg{|i|}{The number of the variable.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_LPVARSTAT *ABA_SUB::lpVarStat(int i)
@q m e f @>

@q m f slackStat @>
@q m e t @>
@ The function |slackStat()|.

@q m b r @>
  \BeginReturn
    A pointer to the status of the slack variable |i| in the last solved
    linear program.
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
     \docarg{|i|}{The number of the slack variable.}
  \EndArg
@q m e a @>

@q m b f @>
  ABA_SLACKSTAT *ABA_SUB::slackStat(int i)
@q m e f @>

@q m v generateLp @>
@ The virtual function |generateLp()| instantiates an |LP| for the solution
  of the |LP|-relaxation in this subproblem. 
  The generated |LP| is solved with |CPLEX|.
  By redefining this function in a derived class other |LP|-solvers
  can be used.

  This function is defined in the file |lpif.cc|.
@q m e t @>

@q m b r @>
  \BeginReturn
  A pointer to an object of type |ABA_LPSUBCPLEX|.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_LPSUB *ABA_SUB::generateLp()
@q m e f @>

@q m v reoptimize @>
@ The function |reoptimize()| repeats the optimization of an already
  optimized subproblem. This function is used to determine the
  reduced costs for fixing variables of a new root of the remaining
  \bab\ tree.
@q m e t @>

  As the subproblem has been processed already earlier it is sufficient
  to perform the cutting plane algorithm. If the subproblem is
  fathomed the complete subtree rooted at this subproblem can be fathomed,
  too. Since this function is usually only called for the root of the
  remaining \bab\ tree, we are done in this case.

  It is not guaranteed that all constraints and variables of this subproblem
  can be regenerated in |_activate()|. Therefore, the result of a call
  to |reoptimize()| can differ from the result obtained by the cutting
  plane algorithm in |optimize()|.
  
@(sub.cc@>=
@q m b f @>
  void ABA_SUB::reoptimize ()
@q m e f @>
  {
    PHASE phase;  // current phase of the subproblem optimization

    @<output a banner for the subproblem@>;

    master_->out() << "reoptimization starts" << endl;

    phase = _activate ();
    
    if (phase == Fathoming) fathomTheSubTree ();
    else {
      phase = cutting ();
      if (phase == Fathoming) fathomTheSubTree ();
    }

    _deactivate ();

    status_ = Processed;

  }

@q m f level @>
@q m e t @>
@ The function |level()|.

@q m b r @>
  \BeginReturn
  The level of the subproblem in  the \bab\ tree.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::level() const
@q m e f @>

@q m f id @>
@q m e t @>
@ The function |id()|.

@q m b r @>
  \BeginReturn
  The identity number of the subproblem.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::id() const
@q m e f @>

@q m f lowerBound @>
@q m e t @>
@ The function |lowerBound()|.

@q m b r @>
  \BeginReturn
  A lower bound on the optimal solution of the subproblem.
  \EndReturn
@q m e r @>
  
@(sub.cc@>=
@q m b f @>
  double ABA_SUB::lowerBound() const
@q m e f @>
  {
    if (master_->optSense()->max()) return master_->primalBound();
    else                        return dualBound_;
  }
  
@q m f upperBound @>
@q m e t @>
@ The function |upperBound()|.

@q m b r @>
  \BeginReturn
    An upper bound on the optimal solution of the subproblem.
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  double ABA_SUB::upperBound() const
@q m e f @>
  {
    if (master_->optSense()->min()) return master_->primalBound();
    else                         return dualBound_;
  }

@ Like in the class |ABA_MASTER| we work again with primal and dual
  bounds such that the code works both for minimization and maximization
  problems.

@q m f dualBound @>
@q m e t @>
@ The function |dualBound()|. 

@q m b r @>
  \BeginReturn
  A bound which is better
  than the optimal solution of the subproblem
  in respect to the sense of the optimization,
  i.e., an upper for a maximization problem or
  a lower bound for a minimization problem, respectively.
  \EndReturn
@q m e r @>

@q m b f @>
  double ABA_SUB::dualBound() const
@q m e f @>

@q m f dualBound @>
@ The function |dualBound()| sets the dual bound of the subproblem,
  and if the subproblem is the root node of the enumeration tree
  and the new value is better than its dual bound, 
  also the global dual bound is
  updated. It is an error if the dual bound gets worse.

  In normal applications it is not required to call this function
  explicitly. This is already done by \ABACUS\ during the subproblem
  optimization.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|x|}{The new value of the dual bound.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::dualBound(double x)
@q m e f @>
  {
    if (master_->optSense()->max()) {
      if (x > dualBound_) {
        master_->err() << "Warning: ABA_SUB::dualBound(): worse dual ";
        master_->err() << "bound " << x << "ignored." << endl;
        master_->err() << "Keeping old dual bound " << dualBound_ << "." << endl;
        return;
      }
    }
    else
      if (x < dualBound_) {
        master_->err() << "Warning: ABA_SUB::dualBound(): worse dual ";
        master_->err() << "bound " << x << "ignored." << endl;
        master_->err() << "Keeping old dual bound " << dualBound_ << "." << endl;
        return;
      }

    dualBound_ = x;

#ifdef ABACUS_PARALLEL
    // dualBound_ is the lower (upper) bound of this subproblem
    // newDual becomes the minimum (maximum) of all dual bounds of THIS host
    double newDual = dualBound_;
    
    if (master_->optSense()->max()) {
      if (master_->openSub()->dualBound() > newDual)
        newDual = master_->openSub()->dualBound();
    }
    else
      if (master_->openSub()->dualBound() < newDual)
        newDual = master_->openSub()->dualBound();

    master_->parmaster()->newHostDualBound(newDual);
    // parmaster will change the global dual bound, if needed
#else
    if (this == master_->root() && master_->betterDual(dualBound_)) 
      master_->dualBound(dualBound_);
#endif

    if (status_ == Active) {
      if (master_->optSense()->max())
        master_->treeInterfaceNodeBounds(id_, master_->primalBound(), 
                                         dualBound_);
      else
        master_->treeInterfaceNodeBounds(id_, dualBound_, 
                                         master_->primalBound());
    }
    
  }

@q m f betterDual @>
@q m e t @>
@ The function |betterDual()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If |x| is better than the best known dual bound of
                  the subproblem,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::betterDual(double x) const
@q m e f @>
  {
    if (master_->optSense()->max()) return x < dualBound_ ? true : false;
    else                            return x > dualBound_ ? true : false;
  }

@q m f father @>
@q m e t @>
@ The function |father()|. 

@q m b r @>
  \BeginReturn
  A pointer to the father of the subproblem in the \bab\ tree.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_SUB *ABA_SUB::father()
@q m e f @>

@q m f lp @>
@q m e t @>
@ The function |lp()|.

@q m b r @>
  \BeginReturn
  A pointer to the linear program of the subproblem.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_LPSUB *ABA_SUB::lp()
@q m e f @>

@q m f boundCrash @>
@q m e t @>
@ The function |boundCrash()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the dual bound is worse than the best known
                  primal bound,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::boundCrash() const
@q m e f @>
  {
    return(master_->primalViolated(dualBound_));
  }


@q m f status @>
@q m e t @>
@ The function |status()|.

@q m b r @>
  \BeginReturn
    The status of the subproblem optimization.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_SUB::STATUS ABA_SUB::status() const
@q m e f @>
  
@q m f maxIterations @>
@ The function |maxIterations()| sets the maximal number of iterations in
  the cutting plane phase. Setting this value to 1 implies that no
  cuts are generated in the optimization process of the subproblem.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|max|}{The maximal number of iterations.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::maxIterations(int max)
@q m e f @>
  {
    master_->out() << "Setting maximal number of iterations in the ";
    master_->out() << "cutting plane phase to " << max << endl;
    maxIterations_ = max;
  }

@ The function |getBase()| updates the status of the variables and the
  slack variables.

@(sub.cc@>=
  void ABA_SUB::getBase()
  {
    if (lp_->basisStatus() == ABA_LP::Available) {
      @<get the LP status of the variables@>;
      @<get the LP status of the slack variables@>;
    }
  }

@ There may be variables which are fixed or set but which could
  not be eliminated since their previous |ABA_LPVARSTAT| has been
  |Basic|. The |ABA_LPVARSTAT| of these variables after the solution
  of the linear program could cause a wrong fixing or setting.
  Hence we assign to them the status |Unknown| if the status
  is not |Basic|.
  
@<get the LP status of the variables@>=
  ABA_LPVARSTAT::STATUS newStat;

  const int nVariables = nVar();

  for (int i = 0; i < nVariables; i++) {
    newStat = lp_->lpVarStat(i);
    if (newStat != ABA_LPVARSTAT::Eliminated) {
      if ((*fsVarStat_)[i]->fixedOrSet() && newStat != ABA_LPVARSTAT::Basic)
        (*lpVarStat_)[i]->status(ABA_LPVARSTAT::Unknown);
      else
        (*lpVarStat_)[i]->status(newStat);
    }
    else
      (*lpVarStat_)[i]->status(ABA_LPVARSTAT::Eliminated);
  }

@ @<get the LP status of the slack variables@>=
  const int nActCon = actCon_->number();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nActCon; i++)
#else
  for (int i = 0; i < nActCon; i++)
#endif
    (*slackStat_)[i]->status(lp_->slackStat(i));

@q m v fix @>
@ The function |fix()| fixes a variable.
@q m e t @>
  If the variable which is currently fixed is already set then
  we must not change its bounds in the LP since it might be
  eliminated.

@q m b r @>
  \BeginReturn 
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable being fixed.}
  \docarg{|newStat|}{A pointer to an object storing the new status of
                     the variable.}
  \docarg{|newValue|}{If the variable is fixed to a value different
                     from the one of the last LP-solution, the
                     argument |newValue| is set to |true|. Otherwise,
                     it is set to |false|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::fix(int i, ABA_FSVARSTAT *newStat, bool &newValue)
@q m e f @>
  {
    ABA_VARIABLE *v = variable(i);

    int contra = 0;

    if (fsVarStat(i)->contradiction(newStat)) contra = 1;
    else fsVarStat(i)->status(newStat);
      
    if (!v->fsVarStat()->fixed())    
      master_->newFixed(1);

    v->fsVarStat()->status(newStat);

    @<is variable fixed to a new value@>;
    @<update the bounds of the fixed variable@>;

    return contra;
  }

@ @<is variable fixed to a new value@>=
  double x = xVal_[i];
  if ((newStat->status() == ABA_FSVARSTAT::FixedToLowerBound
                                       && x > lBound(i) + master_->eps()) ||
      (newStat->status() == ABA_FSVARSTAT::FixedToUpperBound
                                       && x < uBound(i) - master_->eps()) ||
      (newStat->status() == ABA_FSVARSTAT::Fixed
                                       && !(master_->equal(x, newStat->value())))
      )
        newValue = true;
      else
        newValue = false;
        
@ @<update the bounds of the fixed variable@>=
  double newBound = fixSetNewBound(i);
    
  (*lBound_)[i] = newBound;
  (*uBound_)[i] = newBound;

  variable(i)->lBound(newBound);
  variable(i)->uBound(newBound);
    
  updateBoundInLp(i);
    

@q m v set @>
@ The function |set()| sets a variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndArg
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable being set.}
  \docarg{|newStat|}{A pointer to the object storing the new status of the
                     the variable.}
  \docarg{|newValue|}{If the variable is set to a value different from
                      the one of the last LP-solution, |newValue|
                      is set to |true|. Otherwise, it is set to
                      |false|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::set(int i, ABA_FSVARSTAT *newStat, bool &newValue)
@q m e f @>
  {
    return set(i, newStat->status(), newStat->value(), newValue);
  }

@q m v set @>
@ The function |set()| sets a variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndArg
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable being set.}
  \docarg{|newStat|}{The new status of the variable.}
  \docarg{|newValue|}{If the variable is set to a value different from
                      the one of the last LP-solution, |newValue|
                      is set to |true|. Otherwise, it is set to
                      |false|.}
  \EndArg
@q m e a @>

@(sub.cc@>=  
@q m b f @>
  int ABA_SUB::set(int i, ABA_FSVARSTAT::STATUS newStat, bool &newValue)
@q m e f @>
  {
    if (newStat == ABA_FSVARSTAT::Set) {
      master_->err() << "ABA_SUB::set() no value specified for status Set" << endl;
      master_->err() << "setting to value not implemented" << endl;
      exit (Fatal);
    }

    return set (i, newStat, 0.0, newValue);
  }


@q m v set @>
@ The function |set()| sets a variable.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|1|}{If a contradiction is found,}
  \docarg{|0|}{otherwise.}
  \EndArg
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|i|}{The number of the variable being set.}
  \docarg{|newStat|}{The new status of the variable.}
  \docarg{|value|}{The value the variable is set to.}
  \docarg{|newValue|}{If the variable is set to a value different from
                      the one of the last LP-solution, |newValue|
                      is set to |true|. Otherwise, it is set to
                      |false|.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::set(int i, ABA_FSVARSTAT::STATUS newStat, double value, bool &newValue)
@q m e f @>
  {
    ABA_VARIABLE *v = variable(i);

    if (v->fsVarStat()->contradiction(newStat, value)) return 1;
    else {
      (*fsVarStat_)[i]->status(newStat, value);

      @<is variable set to a new value@>;
      @<update the bounds of the set variables@>;

      return 0;
    }
  }

@ If a variable is fixed according to logical implications before
  the subproblem is processed, then no |lp_| is available.
  
@<is variable set to a new value@>=
  if (lp_ == 0) newValue = false;
  else {
    double x = xVal_[i];
    if ((newStat == ABA_FSVARSTAT::SetToLowerBound
         && x > (*lBound_)[i] + master_->eps())   ||
        (newStat == ABA_FSVARSTAT::SetToUpperBound
         && x < (*uBound_)[i] - master_->eps())   ||
        (newStat == ABA_FSVARSTAT::Set
         && !(master_->equal(x, value)))            )
            newValue = true;
        else
          newValue = false;
  }
        
@ The function |updateBoundInLp()| checks if a linear program is present
  in the subproblem. The bounds in the linear program can only be
  changed if the variable is not eliminated. However if the variable
  is set to a different bound, then we would have detected a contradiction.
  
@<update the bounds of the set variables@>=
  double newBound = fixSetNewBound(i);
    
  (*lBound_)[i] = newBound;
  (*uBound_)[i] = newBound;

  updateBoundInLp(i);

@ The function |updateBoundInLp()| adapts the bound of a fixed or set
  variable |i| also in the linear program. This can be only done if a linear
  program is available and the variable is not eliminated.

@(sub.cc@>=
  void ABA_SUB::updateBoundInLp(int i)
  {
    if (lp_ == 0 || lp_->eliminated(i)) return;

    double newBound = (*lBound_)[i];

    lp_->changeLBound(i, newBound);
    lp_->changeUBound(i, newBound);
    
  }

@ The function |fixSetNewBound()| returns the value which the upper and
  lower bounds of a variable should take after it is fixed or set.

@(sub.cc@>=
  double ABA_SUB::fixSetNewBound(int i)
  {
    switch ((*fsVarStat_)[i]->status()) {
      case ABA_FSVARSTAT::SetToLowerBound:
        return (*lBound_)[i];
      case ABA_FSVARSTAT::FixedToLowerBound:
        return variable(i)->lBound();
      case ABA_FSVARSTAT::SetToUpperBound:
        return (*uBound_)[i];
      case ABA_FSVARSTAT::FixedToUpperBound:
        return variable(i)->uBound();
      case ABA_FSVARSTAT::Set:
      case ABA_FSVARSTAT::Fixed:
        return (*fsVarStat_)[i]->value();
      default:
        master_->err() << "ABA_SUB::fixSetNewBound(): variable neither fixed nor set" << endl;
        exit (Fatal);
        return 0.0;
    }
  }
  
@q m v pausing @>
@ Sometimes it is appropriate to put a subproblem back into the list
  of open subproblems. This is called |pausing|.
  In the default implementation the virtual function |pausing()|
  always returns |false|.

  It could be useful to enforce pausing a node if a tailing off
  effect is observed during its first optimization.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{The function |pausing()| should return |true| if this 
                 condition is satisfied,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::pausing()
@q m e f @>
  {
    return false;
  }

@ The function |_conEliminate()| returns the number of eliminated
  constraints.

  Only dynamic constraints are eliminated from the |LP|.

  It might be worth to implement problem specific versions of
  this function.

@(sub.cc@>=
  int ABA_SUB::_conEliminate()
  {
    ABA_BUFFER<int> eliminate(master_, nCon());
    
    conEliminate(eliminate);

    removeCons(eliminate);

    master_->out() << eliminate.number() << " constraints eliminated" << endl;

    return eliminate.number();
  }

@q m v conEliminate @>
@ The function |conEliminate()| can be used as an entry point for
  application specific elimination of constraints by redefinig it
  in derived classes. 

  The default implementation of this function calls either the function
  |nonBindingConEliminate()| or the function |basicConEliminate()| depending
  on the constraint elimination mode of the master that is initialized
  via the parameter file.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|remove|}{The constraints that should be eliminated must be
                    inserted in this buffer.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::conEliminate(ABA_BUFFER<int> &remove)
@q m e f @>
  {
    switch (master_->conElimMode()) {
      case ABA_MASTER::NonBinding: nonBindingConEliminate(remove);
                               break;
      case ABA_MASTER::Basic:      basicConEliminate(remove);
                               break;
      default:                 break;
    }
  }

@q m v nonBindingConEliminate @>
@ The function |nonBindingConEliminate()| retrieves the
  dynamic constraints with slack exceeding the value given by
  the parameter {\tt ConElimEps}.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|remove|}{Stores the nonbinding constraints.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::nonBindingConEliminate(ABA_BUFFER<int> &remove)
@q m e f @>
  {
    const int conElimAge = master_->conElimAge() - 1;

    const int nConstraints = nCon();

    for (int i = 0; i < nConstraints; i++) 
      if ((*actCon_)[i]->dynamic()) {
        if (fabs(lp_->slack(i)) > master_->conElimEps()) {
          if (actCon_->redundantAge(i) >= conElimAge)
            remove.push(i);
          else
            actCon_->incrementRedundantAge(i);
        }
        else
          actCon_->resetRedundantAge(i);
      }
  }

@q m v basicConEliminate @>
@ The function |basicConEliminate()| retrieves all 
  dynamic constraints having basic slack variable.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|remove|}{Stores the nonbinding constraints.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::basicConEliminate(ABA_BUFFER<int> &remove)
@q m e f @>
  {
    const int conElimAge   = master_->conElimAge() - 1;
    const int nConstraints = nCon();

    for (int i = 0; i < nConstraints; i++) 
      if ((*actCon_)[i]->dynamic()) {
        if ((*slackStat_)[i]->status() == ABA_SLACKSTAT::Basic) {
          if (actCon_->redundantAge(i) >= conElimAge)
            remove.push(i);
          else
            actCon_->incrementRedundantAge(i);
        }
        else
          actCon_->resetRedundantAge(i);
      }
  }

@ The function |_varEliminate()| returns the number of eliminated
  variables. Only dynamic variables can be eliminated.

@(sub.cc@>=
  int ABA_SUB::_varEliminate()
  {
    ABA_BUFFER<int> eliminate(master_, nVar());
    
    varEliminate(eliminate);

    removeVars(eliminate);

    master_->out() << eliminate.number() << " variables eliminated" << endl;

    return eliminate.number();
  }

@q m v varEliminate @>
@ The function |varEliminate()| provides an entry point for application
  specific variable elimination that can be implemented by redefining
  this function in a derived class.

  The default implementation selects the variables
  with the function |redCostVarEliminate()|.
@q m e t @>

@q m b a @>
  \BeginArg
   \docarg{|remove|}{The variables that should be removed have to be 
                     stored in this buffer.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::varEliminate(ABA_BUFFER<int> &remove)
@q m e f @>
  {
    if (master_->varElimMode() == ABA_MASTER::ReducedCost)
      redCostVarEliminate(remove);
  }

@q m f redCostVarEliminate @>
@ The function |redCostVarEliminate()| retrieves all variables with
  ``wrong'' reduced costs.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|remove|}{The variables with ``wrong'' reduced cost are stored
                    in this buffer.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::redCostVarEliminate(ABA_BUFFER<int> &remove)
@q m e f @>
  {
    bool max = master_->optSense()->max();
    const int  varElimAge = master_->varElimAge() - 1;
    const double eps = master_->machineEps();
    
    const int nVariables = nVar();
    for (int i = 0; i < nVariables; i++) 
      if (variable(i)->dynamic() && !(*fsVarStat_)[i]->fixedOrSet()
          && fabs(xVal_[i]) < eps) {

        bool bad = false;

        if (!lpVarStat(i)->basic()) {
          if (max) {
            if (lp_->reco(i) < -master_->varElimEps()) bad = true;
          }
          else {
            if (lp_->reco(i) > master_->varElimEps()) bad = true;
          }
        }

        if (bad) {
          if (actVar_->redundantAge(i) >= varElimAge)
            remove.push(i);
          else
            actVar_->incrementRedundantAge(i);
        }
        else
          actVar_->resetRedundantAge(i);
      }
  } 
  
@q m v fathomTheSubTree @>
@ The function |fathomTheSubTree()| fathoms all nodes in the
  subtree rooted at this subproblem. |Dormant| and |Unprocessed| nodes are also
  removed from the set of open subproblems.
@q m e t @>

  If the subproblem is already |Fathomed| we do not have to
  proceed in this branch. Otherwise, we fathom the node and
  continue with all its sons.  
  The actual fathoming starts at the unfathomed
  leaves of the subtree and recursively goes up through
  the tree.

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::fathomTheSubTree()
@q m e f @>
  {
#ifdef ABACUS_PARALLEL
    master_->err() << "ABA_SUB::fathomTheSubTree(): not available in "
                      "the parallel version!" << endl;
    exit(Fatal);
#else
    master_->out() << "fathom complete subtree" << endl;

    if (status_ != Fathomed) {
      if (status_ == Dormant || status_ == Unprocessed)
        master_->openSub()->remove(this);
     
      if (sons_) for (int i = 0; i < sons_->number(); i++)
        (*sons_)[i]->fathomTheSubTree();
      else
        fathom(false); // no reoptimization desired
    }
#endif
  }

@q m f actCon @>
@q m e t @>
@ The function |actCon()|.

@q m b r @>
  \BeginReturn
  A pointer to the currently active constraints.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_ACTIVE<ABA_CONSTRAINT, ABA_VARIABLE> *ABA_SUB::actCon()
@q m e f @>

@q m f actVar @>
@q m e t @>
@ The function |actVar()|.

@q m b r @>
  \BeginReturn
  A pointer to the currently active variables.
  \EndReturn
@q m e r @>

@q m b f @>
  ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *ABA_SUB::actVar()
@q m e f @>
  
@ The function |_separate()| returns the number of generated cutting
  planes. 

@(sub.cc@>=
  int ABA_SUB::_separate()
  {
    @<should we separate cuts at all?@>;
    @<separate cuts@>;
  }
 
@ @<should we separate cuts at all?@>=
  if (!master_->cutting()) return 0;

@ @<separate cuts@>=
  int nCuts;

  master_->out() << endl;
  master_->out() << "Separation of Cutting Planes" << endl;

  localTimer_.start(true);
  nCuts = separate();
  master_->separationTime_.addCentiSeconds( localTimer_.centiSeconds() );

  return nCuts;

@q m v separate @>
@ The virtual dummy function |separate()| must be redefined in derived
  classes for the generation of cutting planes. The default
  implementation does nothing.
@q m e t @>

@q m b r @>
  \BeginReturn
    The number of generated cutting planes.
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::separate()
@q m e f @>
  {
    master_->out() << endl << "no separation implemented" << endl;
    return 0;
  }

@ The function |_improve()| tries to find a better feasible solution.
  If a better solution is found its value is stored in |primalValue|
  and we return 1, otherwise we return 0.

  If the upper bound has been initialized with the optimum solution
  or with the optimum solution plus/minus one these primal heuristics
  are skipped.

  The primal bound, if improved, is either updated in the function
  |cutting()|, from which |_improved()| is called, are can be
  updated in the function |improve()| of an application in a derived class.

@(sub.cc@>=
  int ABA_SUB::_improve(double &primalValue)
  {
    if (master_->pbMode() != ABA_MASTER::NoPrimalBound) return 0;
    master_->out() << endl;
    master_->out() << "Apply Primal Heuristic" << endl;
    
    localTimer_.start(true);

    int status = improve(primalValue);

    master_->improveTime_.addCentiSeconds( localTimer_.centiSeconds() );

    return status;
  }

@q m v improve @>
@ The function |improve()| can be redefined in derived classes in
  order to implement primal heuristics for finding feasible solutions.
  The default implementation does nothing.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If no better solution could be found,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|primalValue|}{Should hold the value of the feasible solution,
                         if a better one is found.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::improve(double &primalValue)
@q m e f @>
  {
    master_->out() << endl << "no primal heuristic implemented" << endl;
    return 0;
  }

@ The function |infeasibleSub()| should be called if a subproblem turns
  out to be infeasible. It sets the dual bound of the subproblem
  correctly.

@(sub.cc@>=
  void ABA_SUB::infeasibleSub()
  {
    master_->out() << "infeasible subproblem" << endl;
    
    if (master_->optSense()->max()) dualBound_ = -master_->infinity();
    else                            dualBound_ =  master_->infinity();

    master_->treeInterfaceNodeBounds(id_, lowerBound(), upperBound());
  }

@q m f infeasible @>
@q m e t @>
@ The function |infeasible()|.

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If the subproblem does not contain a feasible solution,}
  \docarg{|false|}{otherwise.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::infeasible()
@q m e f @>
  {
    if (master_->optSense()->max()) {
      if (dualBound_ == -master_->infinity()) return true;
      else                                    return false;
    }
    else {
      if (dualBound_ ==  master_->infinity()) return true;
      else                                    return false;
    }
  }
  

@ The function |activateVars()| adds the variables stored in
  the pool slots of |newVars| to the set of active variables, but
  not to the linear program.
  If the new number of variables exceeds the maximal number of  variables
  an automatic reallocation is performed.

@(sub.cc@>=
  void ABA_SUB::activateVars(ABA_BUFFER<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *> &newVars)
  {
    @<perform a reallocation if required by the new variables@>;
    @<add the new variables to the data structures of the subproblem@>;
  }

@ @<perform a reallocation if required by the new variables@>=
  int n = nVar();
    
  if (n + newVars.number() >= maxVar()) {
    int newMax = ((maxVar() + newVars.number())*11)/10 + 1;
    varRealloc(newMax);
  }

@ @<add the new variables to the data structures of the subproblem@>=
  ABA_VARIABLE *v;

  const int nNewVars = newVars.number();
    
  for (int i = 0; i < nNewVars; i++) {
    v = (ABA_VARIABLE *) newVars[i]->conVar();
      
    (*fsVarStat_)[n + i] = new ABA_FSVARSTAT(v->fsVarStat());
    (*lpVarStat_)[n + i] = new ABA_LPVARSTAT(master_, ABA_LPVARSTAT::Unknown);
    (*lBound_)[n + i]    = v->lBound();
    (*uBound_)[n + i]    = v->uBound();

    v->activate();
  }

  actVar_->insert(newVars);

  master_->addVars(nNewVars);

@ The function |addVarsToLp()| adds the variables stored in the pool slots
  of |newVars| to the linear program. |localStatus| can specify a local status
  of fixing and setting.

  If the local |ABA_FSVARSTAT| of the added variables differs from their
  global status, then this local status can be stated in |localStatus|.
  Per default the value of |localStatus| is 0.

@(sub.cc@>=
  void ABA_SUB::addVarsToLp(ABA_BUFFER<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT>*> &newVars,
                        ABA_BUFFER<ABA_FSVARSTAT*> *localStatus)
  {
    @<|ABA_SUB::addVarsToLp()|: local variables@>;
    @<get the new variables together with their status and bounds@>;
    @<add the new variables to the linear program@>;
  }

@ @<|ABA_SUB::addVarsToLp()|: local variables@>=
  const int nNewVars = newVars.number();
  ABA_BUFFER<ABA_VARIABLE*>  vars(master_, nNewVars);
  ABA_BUFFER<ABA_FSVARSTAT*> stat(master_, nNewVars);
  ABA_BUFFER<double>     lb(master_, nNewVars);
  ABA_BUFFER<double>     ub(master_, nNewVars);
  ABA_VARIABLE          *v;

@ @<get the new variables together with their status and bounds@>=
  for (int i = 0; i < nNewVars; i++) {
    v = (ABA_VARIABLE *) newVars[i]->conVar();

    vars.push(v);
      
    if (localStatus) {
      if (v->fsVarStat()->contradiction((*localStatus)[i])) {
        master_->err() << "ABA_SUB::addVarsToLp(): local status contradicts global status" << endl;
        exit(Fatal);
      }
      stat.push((*localStatus)[i]);
    }
    else stat.push(v->fsVarStat());
      
    lb.push(v->lBound());
    ub.push(v->uBound());
  }

@ If more than one variable is added we generate the expanded format
  of the constraints, such that the columns can be determined more
  efficiently. However, this threshold is completely experimental
  and problem specific!
  
@<add the new variables to the linear program@>=
  const int nConstraints = nCon();
  if (vars.number() > 1)
    for (int i = 0; i < nConstraints; i++) constraint(i)->_expand();
  
  localTimer_.start(true);
    
  lp_->addVars(vars, stat, lb, ub);

  master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());

  if (vars.number() > 1)
    for (int i = 0; i < nConstraints; i++) constraint(i)->_compress();

@ The function |_removeVars()| removes the variables with numbers |remove|
  from the set of active variables.

@(sub.cc@>=
  int ABA_SUB::_removeVars(ABA_BUFFER<int> &remove)
  {
    const int nRemove = remove.number();

    if (nRemove) {
      @<sort the variables which are removed@>;
      @<remove the variables from the subproblem@>;
    }
    return remove.number();
  }

@ The following functions removing the variables from the data structures
  of the subproblem require the variables sorted in increasing order.
  This sorting can be performed in linear time.
  
@<sort the variables which are removed@>=
  ABA_ARRAY<bool> marked(master_, nVar(), false);

  for (int i = 0; i < nRemove; i++)
    marked[remove[i]] = true;

  ABA_BUFFER<int> removeSorted(master_, nRemove);

  const int nVariables = nVar();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nVariables; i++)
#else
  for (int i = 0; i < nVariables; i++)
#endif
    if (marked[i]) removeSorted.push(i);
    
@ @<remove the variables from the subproblem@>=
  const int nRemoveSorted = removeSorted.number();

  localTimer_.start(true);
  lp_->removeVars(removeSorted);
  master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());
      
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRemoveSorted; i++) {
#else
  for (int i = 0; i < nRemoveSorted; i++) {
#endif
    delete (*fsVarStat_)[removeSorted[i]];
    delete (*lpVarStat_)[removeSorted[i]];
  }
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRemoveSorted; i++)
#else
  for (int i = 0; i < nRemoveSorted; i++)
#endif
    (*actVar_)[removeSorted[i]]->deactivate();
      
  actVar_->remove(removeSorted);

  fsVarStat_->leftShift(removeSorted);
  lpVarStat_->leftShift(removeSorted);
  uBound_->leftShift(removeSorted);
  lBound_->leftShift(removeSorted);
      
  master_->removeVars(nRemoveSorted);


@q m v removeCons @>
@ The function |removeCons()| adds constraints 
  to the buffer of the removed constraints, which will be removed at
  the beginning of the next iteration of the cutting plane algorithm.
@q m e t @>

@q m b a @>
  \BeginArg
   \docarg{|remove|}{The constraints which should be removed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::removeCons(ABA_BUFFER<int> &remove)
@q m e f @>
  {
    const int nRemove = remove.number();

    for (int i = 0; i < nRemove; i++)
      removeConBuffer_->push(remove[i]);
  }

@q m v removeCon @>
@ The following version of the 
  function |removeCon()| adds a single constraint to the set of
  constraints which are removed from the active set at the beginning of
  the next iteration.
@q m e t @>

@q m b a @>
  \BeginArg
   \docarg{|i|}{The number of the constraint being removed.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::removeCon(int i)
@q m e f @>
  {
    removeConBuffer_->push(i);
  }


@ The function |_removeCons()| removes the constraints with numbers |remove|
  from the set of active constraints.

@(sub.cc@>=
  int ABA_SUB::_removeCons(ABA_BUFFER<int> &remove)
  {
    const int nRemove = remove.number();
    if (nRemove) {
      @<sort the constraints which are removed@>;
      @<remove the constraints from the subproblem@>;
    }
    return nRemove;
  }
  
@ The following functions to remove the variables from the data structures
  in the subproblem require the constraints sorted in increasing order.
  This sorting is performed in linear time.
  
@<sort the constraints which are removed@>=
  ABA_ARRAY<bool> marked(master_, nCon(), false);

  for (int i = 0; i < nRemove; i++)
    marked[remove[i]] = true;

  ABA_BUFFER<int> removeSorted(master_, nRemove);

  const int nConstraints = nCon();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nConstraints; i++)
#else
  for (int i = 0; i < nConstraints; i++)
#endif
    if (marked[i]) removeSorted.push(i);
    
@ @<remove the constraints from the subproblem@>=
  const int nRemoveSorted = removeSorted.number();
#ifdef ABACUS_NO_FOR_SCOPE
  for (i = 0; i < nRemoveSorted; i++) {
#else
  for (int i = 0; i < nRemoveSorted; i++) {
#endif
    (*actCon_)[removeSorted[i]]->deactivate();
    delete (*slackStat_)[removeSorted[i]];
  }

  actCon_->remove(removeSorted);
  slackStat_->leftShift(removeSorted);
  localTimer_.start(true);
  lp_->removeCons(removeSorted);
  master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());

  master_->removeCons(nRemoveSorted);

@q m v varRealloc @>
@ The function |varRealloc()| reallocates memory that at most |newSize|
  variables can be handled in the subproblem.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newSize|}{The new maximal number of variables in the
                     subproblem.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::varRealloc(int newSize)
@q m e f @>
  {
    actVar_->realloc(newSize);
    fsVarStat_->realloc(newSize);
    lpVarStat_->realloc(newSize);
    lBound_->realloc(newSize);
    uBound_->realloc(newSize);
    removeVarBuffer_->realloc(newSize);

    double *xValNew = new double[newSize];

    const int nVariables = nVar();

    for (int i = 0; i < nVariables; i++)
      xValNew[i] = xVal_[i];

    delete [] xVal_;
    xVal_ = xValNew;

    localTimer_.start(true);
    lp_->varRealloc(newSize);
    master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());
  }

@q m v conRealloc @>
@ The function |conRealloc()| reallocates memory that at most |newSize|
  constraints can be handled in the subproblem.
@q m e t @>

@q m b a @>
  \BeginArg
  \docarg{|newSize|}{The new maximal number of constraints of the
                     subproblem.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  void ABA_SUB::conRealloc(int newSize)
@q m e f @>
  {
    actCon_->realloc(newSize);
    slackStat_->realloc(newSize);
    removeConBuffer_->realloc(newSize);
    localTimer_.start(true);
    lp_->conRealloc(newSize);
    master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());

    double *yValNew = new double[newSize];

    const int nConstraints = nCon();

    for (int i = 0; i < nConstraints; i++)
      yValNew[i] = yVal_[i];

    delete [] yVal_;
    yVal_ = yValNew;

  }

@q m f nVar @>
@q m e t @>
@ The function |nVar()|.

@q m b r @>
  \BeginReturn
    The number of active variables.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::nVar() const
@q m e f @>

@q m f nCon @>
@q m e t @>
@ The function |nCon()| .

@q m b r @>
 \BeginReturn
 The number of active constraints.
 \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::nCon() const
@q m e f @>
  
@q m f maxVar @>
@q m e t @>

@ The function |maxVar()|.

@q m b r @>
  \BeginReturn
   The maximum number of variables which can be handled without reallocation.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::maxVar() const
@q m e f @>

@q m f maxCon @>
@q m e t @>
@ The function |maxCon()|.

@q m b r @>
  \BeginReturn
   The maximum number of constraints which can be handled without reallocation.
  \EndReturn
@q m e r @>

@q m b f @>
  int ABA_SUB::maxCon() const
@q m e f @>

@q m v initializeLp @>
@ The function |initializeLp()| initializes the linear program.
@q m e t @>
  Since not all variables might be active we
  first have to try making the |LP| feasible again by
  the addition of variables. If this fails, i.e., |_initMakeFeas()|
  has a nonzero return value, we return 1 in order to
  indicate that the corresponding subproblem can be fathomed.
  Otherwise, we continue with the initialization of the |LP|.

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the linear program could be initialized
               successfully.}
  \docarg{|1|}{If the linear program turns out to be infeasible.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::initializeLp()
@q m e f @>
  {
    while (1) {
      localTimer_.start(true);
      lp_ = generateLp();
      master_->lpTime_.addCentiSeconds(localTimer_.centiSeconds());
      
      if (lp_->infeasible()) {
        if (_initMakeFeas()) return 1;
        else                 delete lp_;
      }
      else return 0;
    }
  }

@ The virtual function |_initMakeFeas()| tries to add variables
  to restore infeasibilities detected at initialization time.
  It returns 0 if variables could be activated which might
  restore feasibility, otherwise it returns 1.

  The function should analyse the constraints stored in
  |ABA_LPSUB::infeasCons_| and try to add inactive variables
  which could restore the infeasibility.

  The new variables are only added to the set of active variables
  but not to the linear program since no linear program exists
  when this function is called.

@(sub.cc@>=
  int ABA_SUB::_initMakeFeas()
  {
    if (!master_->pricing()) return 1;

    @<find variables which could make initial LP feasible@>;
    @<insert the variables in a pool and determine the pool slots |newSlots|@>;

    activateVars(newSlots);

    return 0;
  }

@ The determination of useful variables has be implemented
  problem specifically in the function |initMakeFeas()|.
  
@<find variables which could make initial LP feasible@>=
  ABA_BUFFER<ABA_INFEASCON*>   *infeasCon = lp_->infeasCon();
  ABA_BUFFER<ABA_VARIABLE*>  newVars(master_, infeasCon->number());
  ABA_POOL<ABA_VARIABLE, ABA_CONSTRAINT> *pool = 0;

  int status = initMakeFeas(*infeasCon, newVars, &pool);

  if (status) return 1;

@ If not differently specified with the help of the function
  |initMakeFeas()| we use the default variable pool for the new variables.
  
@<insert the variables in a pool and determine the pool slots |newSlots|@>=  
  const int nNewVars = newVars.number();

  ABA_BUFFER<ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *> newSlots(master_, nNewVars);
  ABA_POOLSLOT<ABA_VARIABLE, ABA_CONSTRAINT> *slot;

  if (pool == 0) pool = master_->varPool();
    
  for (int i = 0; i < nNewVars; i++) {
    slot = pool->insert(newVars[i]);
  
    if (slot == 0) {
      master_->err() << "ABA_SUB::_initMakeFeas(): pool too small ";
      master_->err() << "to insert all constraints" << endl;
      exit (Fatal);
    }

    newSlots.push(slot);
  }


@q m v initMakeFeas @>
@ The default implementation of the virtual |initMakeFeas()| does nothing.
  A reimplementation of this function should generate inactive
  variables until at least one variable |v| which satisfies the function
  |ABA_INFEASCON::goodVar(v)| for each infeasible constraint
  is found.
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|0|}{If the feasibility might have been restored,}
  \docarg{|1|}{otherwise.}
  \EndReturn
@q m e r @>

@q m b a @>
  \BeginArg
  \docarg{|infeasCons|}{The infeasible constraints.}
  \docarg{|newVars|}{The variables that might restore feasibility should be
                     added here.}
  \docarg{|pool|}{A pointer to the pool to which the new variables should
                  be added. If this is a 0-pointer the variables are added
                  to the default variable pool. The default value is 0.}
  \EndArg
@q m e a @>

@(sub.cc@>=
@q m b f @>
  int ABA_SUB::initMakeFeas(ABA_BUFFER<ABA_INFEASCON*> &infeasCons,
                            ABA_BUFFER<ABA_VARIABLE*> &newVars,
                            ABA_POOL<ABA_VARIABLE, ABA_CONSTRAINT> **pool)
@q m e f @>
  {
    return 1;
  }

@q m v tailingOff @>
@ The function |tailingOff()| is called when a tailing off effect 
  according to the parameters {\tt TailOffPercent} and
  {\tt TailOffNLps} of the parameter file is observed.
  This function can be redefined in derived classes in order to 
  perform actions to resolve the tailing off (e.g., switching on
  an enhanced separation strategy).
@q m e t @>

@q m b r @>
  \BeginReturn
  \docarg{|true|}{If a branching step should be enforced. But before
                  branching a pricing operation is perfored. The
                  branching step is only performed if no variables are
                  added. Otherwise, the cutting plane algorithm is continued.}
  \docarg{|false|}{If the cutting plane algorithm should be continued.}
  \EndReturn
@q m e r @>

@(sub.cc@>=
@q m b f @>
  bool ABA_SUB::tailingOff()
@q m e f @>
  {
    return true;
  }
